unit Utils3D;

interface

uses Windows, Classes, SysUtils, Types3D,WinUtils;


//  POINT functions
function Point3S(X, Y, Z: single): TPoint3S;
function Point2S(X, Y: single): TPoint2S;
function Point3I(X, Y, Z: TInt): TPoint3I;
function Point2I(X, Y: TInt): TPoint2I;


function SamePoints(P1, P2: TPoint3S): boolean;
procedure SwapPoints(var P1,P2:TPoint3S);

function SubPoint(P1, P2: TPoint3S): TPoint3S;
function PointToAngle(P: TPoint3S): TAngle3S;
function VectorLength(P1: TPoint3S): Single;
function NormalizePoint(P:TPoint3S):TPoint3S;overload;
procedure Normalize(var P : TPoint3S);overload;
function AddPoint(P1, P2 : TPoint3S) : TPoint3S;
function VectorDot(P1, P2 : TPoint3S): Single;
function VectorCross(P1, P2: TPoint3S): TPoint3S;
procedure Negate(var P : TPoint3S); overload;
function NegatePoint(P:TPoint3S):TPoint3S;overload;
function ScalePoint(P : TPoint3S; Scale : Single) : TPoint3S;
function VisibleDotProduct(P1,P2: TAngle3S): LongBool;
procedure CombinePoint(var vr : TPoint3S;v : TPoint3S; var f : Single);
function MulPoint(P1,P2:TPoint3S):TPoint3S;
function PointDistance(const P1, P2 : TPoint3S) : Single;
function DivPoint(A, B: TPoint3S): TPoint3S;
function VectorAngle(const V1, V2: TPoint3S): Single;

// TEXTURE COORDS functions
function Coord(X, Y: single): TTexCoord;


// MATRIX functions
function Identity: TMatrix; overload;
procedure Identity(var Mat: TMatrix); overload;
procedure ZeroMatrix(var Mat: TMatrix);
function MatrixTransform( P: TPoint3S; M: TMatrix): TPoint3S;
function CreateRotationMatrix( Ang: TAngle3S): TMatrix;
function CreateRotationMatrixX( Sine, Cosine: Single): TMatrix;
function CreateRotationMatrixY( Sine, Cosine: Single): TMatrix;
function CreateRotationMatrixZ( Sine, Cosine: Single): TMatrix;
function MatrixMultiply(M1, M2: TMatrix): TMatrix;
function CreateScaleMatrix(const P: TPoint3S): TMatrix;
function CreateTranslationMatrix(const P: TPoint3S): TMatrix;
procedure CreateViewMatrix(var Mat: TMatrix; From, At, Worldup: TPoint3S);
procedure CreateProjectionMatrix(var Mat: TMatrix; FOV, Aspect, NearPlane, FarPlane: Single);

// plane functions
function Plane(P1, P2, P3 : TPoint3S) : TPlane;
procedure NormalizePlane(var p : TPlane);
function CalcPlaneNormal(p1, p2, p3 : TPoint3S) : TPoint3S;overload;
procedure CalcPlaneNormal(p1, p2, p3 : TPoint3S; var vr : TPoint3S);overload;
function PlaneEvaluatePoint(pl : TPlane;p : TPoint3S) : Single;

// OTHER functions
function RadToDeg(const Radians: Single): Single;
function DegToRad(const Degrees: Single): Single;
procedure SinCos(Theta: Single; var Sin, Cos: Single);
function ClampValue(const aValue, aMin, aMax : Single) : Single;overload;
function ClampValue(const aValue, aMin : Single) : Single;overload;
function Tan(const x : Single) : Single;
function CoTan(const x : Single) : Single;
function BoundInfo(Min,Max:TPoint3S):TBoundingInfo;
function ArcCos(const x : Single): Single;
function ArcSin(const x : Single) : Single;


// INTERSECTION functions





implementation

const
 cOne : Single = 1.0;

function DegToRad(const Degrees: Single): Single; register;
asm
      FLD  DWORD PTR [EBP+8]
      FMUL cPIdiv180
end;

function RadToDeg(const Radians: Single): Single; register;
asm
      FLD  DWORD PTR [EBP+8]
      FMUL c180divPI
end;

procedure SinCos(Theta: Single; var Sin, Cos: Single); register;
asm
   FLD  Theta
   FSINCOS
   FSTP DWORD PTR [EDX]    // cosine
   FSTP DWORD PTR [EAX]    // sine
end;

function ArcCos(const x : Single): Single;
asm
      FLD   X
      FMUL  ST, ST
      FSUBR cOne
      FSQRT
      FLD   X
      FPATAN
end;

function ArcSin(const x : Single) : Single;
asm
      FLD   X
      FLD   ST
      FMUL  ST, ST
      FSUBR cOne
      FSQRT
      FPATAN
end;

function Tan(const x : Single) : Single;
asm
      FLD  X
      FPTAN
      FSTP ST(0)
end;

function CoTan(const x : Single) : Single;
asm
      FLD  X
      FPTAN
      FDIVRP
end;

function ClampValue(const aValue, aMin, aMax : Single) : Single;
begin
   if aValue<aMin then
      Result:=aMin
   else if aValue>aMax then
      Result:=aMax
   else Result:=aValue;
end;

function ClampValue(const aValue, aMin : Single) : Single;
begin
   if aValue<aMin then
      Result:=aMin
   else Result:=aValue;
end;

function SamePoints(P1, P2: TPoint3S): boolean;
begin
  Result:=false;
  if P1.X<>P2.X then Exit;
  if P1.Y<>P2.Y then Exit;
  if P1.Z<>P2.Z then Exit;
  Result:=true;

end;

procedure SwapPoints(var P1,P2:TPoint3S);
var Temp:TPoint3S;
begin
 Temp:=P2;
 P2:=P1;
 P1:=Temp;
end;

function Point3S(X, Y, Z: single): TPoint3S;
begin
  Result.X := X;
  Result.Y := Y;
  Result.Z := Z;
end;



function Point2S(X, Y: single): TPoint2S;
begin
  Result.X := X;
  Result.Y := Y;
end;

function Point3I(X, Y, Z: TInt): TPoint3I;
begin
  Result.X := X;
  Result.Y := Y;
  Result.Z := Z;
end;

function Point2I(X, Y: TInt): TPoint2I;
begin
  Result.X := X;
  Result.Y := Y;
end;



function Coord(X, Y: single): TTexCoord;
begin
  Result.X := X;
  Result.Y := Y;
end;

function BoundInfo(Min,Max:TPoint3S):TBoundingInfo;
 begin
  Result.Min:=Min;
  Result.Max:=Max;
  Result.Rad:=PointDistance(Result.Min,Result.Max);
  Result.Center.X:=Result.Min.X+Result.Rad;
  Result.Center.Y:=Result.Min.Y+Result.Rad;
  Result.Center.Z:=Result.Min.Z+Result.Rad;
 end;

function Identity: TMatrix;
begin
  ZeroMemory(@Result, SizeOf(TMatrix));
  Result._11 := 1;
  Result._22 := 1;
  Result._33 := 1;
  Result._44 := 1;

end;


procedure Identity(var Mat: TMatrix);
begin
  ZeroMemory(@Mat, SizeOf(Mat));
  Mat._11 := 1;
  Mat._22 := 1;
  Mat._33 := 1;
  Mat._44 := 1;
end;


procedure ZeroMatrix(var Mat: TMatrix);
begin
  ZeroMemory(@Mat, SizeOf(Mat));
end;

function CreateScaleMatrix(const P: TPoint3S): TMatrix; register;
begin
   Result:=IdentityMatrix;
   Result.m[0, 0]:=P.X;
   Result.m[1, 1]:=P.Y;
   Result.m[2, 2]:=P.Z;
end;

function CreateTranslationMatrix(const P: TPoint3S): TMatrix; register;
begin
   Result:=IdentityMatrix;
   Result.m[3, 0]:=P.X;
   Result.m[3, 1]:=P.Y;
   Result.m[3, 2]:=P.Z;
end;


function MatrixTransform(P: TPoint3S; M: TMatrix): TPoint3S;
  register;
begin
  Result.X := P.X * M.m[0, 0] + P.Y * M.m[1, 0] + P.Z * M.m[2, 0];
  Result.Y := P.X * M.m[0, 1] + P.Y * M.m[1, 1] + P.Z * M.m[2, 1];
  Result.Z := P.X * M.m[0, 2] + P.Y * M.m[1, 2] + P.Z * M.m[2, 2];
end;

function CreateRotationMatrixX( Sine, Cosine: Single): TMatrix; register;
begin
  Result := NullMatrix;
  Result.m[0, 0] := 1;
  Result.m[1, 1] := Cosine;
  Result.m[1, 2] := Sine;
  Result.m[2, 1] := -Sine;
  Result.m[2, 2] := Cosine;
  Result.m[3, 3] := 1;
end;

function CreateRotationMatrixY(Sine, Cosine: Single): TMatrix; register;
begin
  Result := NullMatrix;
  Result.m[0, 0] := Cosine;
  Result.m[0, 2] := -Sine;
  Result.m[1, 1] := 1;
  Result.m[2, 0] := Sine;
  Result.m[2, 2] := Cosine;
  Result.m[3, 3] := 1;
end;

function CreateRotationMatrixZ(Sine, Cosine: Single): TMatrix; register;
begin
  Result := NullMatrix;
  Result.m[0, 0] := Cosine;
  Result.m[0, 1] := Sine;
  Result.m[1, 0] := -Sine;
  Result.m[1, 1] := Cosine;
  Result.m[2, 2] := 1;
  Result.m[3, 3] := 1;
end;

function CreateRotationMatrix(Ang: TAngle3S): TMatrix;
var
  AX, AY, AZ: Single;
  sx, sy, sz,
    cx, cy, cz: Single;
  mx, my, mz: TMatrix;
  s: TMatrix;
begin
  AX := DegToRad(Ang.X);
  AY := DegToRad(Ang.Y);
  AZ := DegToRad(Ang.Z);

  sx := Sin(AX);
  sy := Sin(AY);
  sz := Sin(AZ);

  cx := Cos(AX);
  cy := Cos(AY);
  cz := Cos(AZ);

  mx := CreateRotationMatrixX(sx, cx);
  my := CreateRotationMatrixY(sy, cy);
  mz := CreateRotationMatrixZ(sz, cz);
  s := MatrixMultiply(mz, my);
  Result := MatrixMultiply(mx, s);
end;

function MatrixMultiply(M1, M2: TMatrix): TMatrix; register;
begin
  Result.m[0, 0] := M1.m[0, 0] * M2.m[0, 0] + M1.m[0, 1] * M2.m[1, 0] + M1.m[0, 2] * M2.m[2, 0]
    + M1.m[0, 3] * M2.m[3, 0];
  Result.m[0, 1] := M1.m[0, 0] * M2.m[0, 1] + M1.m[0, 1] * M2.m[1, 1] + M1.m[0, 2] * M2.m[2, 1]
    + M1.m[0, 3] * M2.m[3, 1];
  Result.m[0, 2] := M1.m[0, 0] * M2.m[0, 2] + M1.m[0, 1] * M2.m[1, 2] + M1.m[0, 2] * M2.m[2, 2]
    + M1.m[0, 3] * M2.m[3, 2];
  Result.m[0, 3] := M1.m[0, 0] * M2.m[0, 3] + M1.m[0, 1] * M2.m[1, 3] + M1.m[0, 2] * M2.m[2, 3]
    + M1.m[0, 3] * M2.m[3, 3];
  Result.m[1, 0] := M1.m[1, 0] * M2.m[0, 0] + M1.m[1, 1] * M2.m[1, 0] + M1.m[1, 2] * M2.m[2, 0]
    + M1.m[1, 3] * M2.m[3, 0];
  Result.m[1, 1] := M1.m[1, 0] * M2.m[0, 1] + M1.m[1, 1] * M2.m[1, 1] + M1.m[1, 2] * M2.m[2, 1]
    + M1.m[1, 3] * M2.m[3, 1];
  Result.m[1, 2] := M1.m[1, 0] * M2.m[0, 2] + M1.m[1, 1] * M2.m[1, 2] + M1.m[1, 2] * M2.m[2, 2]
    + M1.m[1, 3] * M2.m[3, 2];
  Result.m[1, 3] := M1.m[1, 0] * M2.m[0, 3] + M1.m[1, 1] * M2.m[1, 3] + M1.m[1, 2] * M2.m[2, 3]
    + M1.m[1, 3] * M2.m[3, 3];
  Result.m[2, 0] := M1.m[2, 0] * M2.m[0, 0] + M1.m[2, 1] * M2.m[1, 0] + M1.m[2, 2] * M2.m[2, 0]
    + M1.m[2, 3] * M2.m[3, 0];
  Result.m[2, 1] := M1.m[2, 0] * M2.m[0, 1] + M1.m[2, 1] * M2.m[1, 1] + M1.m[2, 2] * M2.m[2, 1]
    + M1.m[2, 3] * M2.m[3, 1];
  Result.m[2, 2] := M1.m[2, 0] * M2.m[0, 2] + M1.m[2, 1] * M2.m[1, 2] + M1.m[2, 2] * M2.m[2, 2]
    + M1.m[2, 3] * M2.m[3, 2];
  Result.m[2, 3] := M1.m[2, 0] * M2.m[0, 3] + M1.m[2, 1] * M2.m[1, 3] + M1.m[2, 2] * M2.m[2, 3]
    + M1.m[2, 3] * M2.m[3, 3];
  Result.m[3, 0] := M1.m[3, 0] * M2.m[0, 0] + M1.m[3, 1] * M2.m[1, 0] + M1.m[3, 2] * M2.m[2, 0]
    + M1.m[3, 3] * M2.m[3, 0];
  Result.m[3, 1] := M1.m[3, 0] * M2.m[0, 1] + M1.m[3, 1] * M2.m[1, 1] + M1.m[3, 2] * M2.m[2, 1]
    + M1.m[3, 3] * M2.m[3, 1];
  Result.m[3, 2] := M1.m[3, 0] * M2.m[0, 2] + M1.m[3, 1] * M2.m[1, 2] + M1.m[3, 2] * M2.m[2, 2]
    + M1.m[3, 3] * M2.m[3, 2];
  Result.m[3, 3] := M1.m[3, 0] * M2.m[0, 3] + M1.m[3, 1] * M2.m[1, 3] + M1.m[3, 2] * M2.m[2, 3]
    + M1.m[3, 3] * M2.m[3, 3];
end;

procedure CreateViewMatrix(var Mat: TMatrix; From, At, Worldup: TPoint3S);
var
  View: TPoint3S;
  Length: Single;
  DotProduct: Single;
  Up: TPoint3S;
  Right: TPoint3S;
begin
  Mat:=IdentityMatrix;
  View := SubPoint(At,From);
  Length := VectorLength(View);
  if (Length < 1e-6) then
  begin
    Exit;
  end;
  View := ScalePoint(View,1/Length);
  DotProduct := VectorDot(WorldUp,View);
  Up := SubPoint(WorldUp,ScalePoint(View,DotProduct));
  Length := VectorLength(Up);
  if (Length < 1e-6) then
  begin
    Up := SubPoint(Point3S(0,1,0),ScalePoint(View,View.y));
    Length := VectorLength(Up);
    if (Length < 1e-6) then
    begin
      Up := SubPoint(Point3S(0,0,1),ScalePoint(View,View.z));
      Length := VectorLength(Up);
      if (Length < 1e-6) then
      begin
        Exit;
      end;
    end;
  end;
  Up := ScalePoint(Up,1/Length);
  Right := VectorCross(Up,View);
  Mat._11 := Right.x;  Mat._12 := Up.x;  Mat._13 := View.x;  Mat._14 := 0;
  Mat._21 := Right.y;  Mat._22 := Up.y;  Mat._23 := View.y;  Mat._24 := 0;
  Mat._31 := Right.z;  Mat._32 := Up.z;  Mat._33 := View.z;  Mat._34 := 0;
  Mat._41 := - VectorDot(From,Right );
  Mat._42 := - VectorDot(From,Up );
  Mat._43 := - VectorDot(From,View );
  Mat._44 := 1;
end;

procedure CreateProjectionMatrix(var Mat: TMatrix; FOV, Aspect, NearPlane, FarPlane: Single);
var
  w, h, Q: Single;
begin
  h := (Cos(FOV/2)/Sin(FOV/2));
  w := Aspect * h;
  Q := FarPlane / (FarPlane - NearPlane);
  ZeroMemory(@Mat, SizeOf(Mat));
  Mat._11 := w;
  Mat._22 := h;
  Mat._33 := Q;
  Mat._34 := 1.0;
  Mat._43 := -Q*NearPlane;
end;

function SubPoint(P1, P2: TPoint3S): TPoint3S; register;
asm
  FLD  DWORD PTR [EAX]
  FSUB DWORD PTR [EDX]
  FSTP DWORD PTR [ECX]
  FLD  DWORD PTR [EAX+4]
  FSUB DWORD PTR [EDX+4]
  FSTP DWORD PTR [ECX+4]
  FLD  DWORD PTR [EAX+8]
  FSUB DWORD PTR [EDX+8]
  FSTP DWORD PTR [ECX+8]
end;

function MulPoint(P1, P2: TPoint3S): TPoint3S; register;
asm
  FLD P1.X
  FMUL P2.X
  FSTP Result.X

  FLD P1.Y
  FMUL P2.Y
  FSTP Result.Y

  FLD P1.Z
  FMUL P2.Z
  FSTP Result.Z
end;

function DivPoint(A, B: TPoint3S): TPoint3S;
begin
  if B.X <> 0 then
    Result.X := A.X / B.X
  else
    Result.X := 0;

  if B.Y <> 0 then
    Result.Y := A.Y / B.Y
  else
    Result.Y := 0;

  if B.Z <> 0 then
    Result.Z := A.Z / B.Z
  else
    Result.Z := 0;
end;

function PointToAngle(P: TPoint3S): TAngle3S;

  function ArcSin(X: Single): Single;
  begin
    Result := 0;
    if abs(x) < 1.0 then
      Result := ArcTan(x / Sqrt(1 - Sqr(x)))
    else
      if x = 1.0 then
        Result := PIdiv2
      else
        if x = -1.0 then
          Result := -PIdiv2;

  end;

  function ArcCos(X: Single): Single;
  begin
    Result := PIdiv2 - arcsin(x);
  end;

  function Arcus(const s, c: Single): Single;
  begin
    if (s >= 0.0) then
    begin
      if (c > 0.0) then
        result := arctan(s / c)
      else
        if (c < 0.0) then
          result := pi - arctan(s / -c)
        else
          result := PIdiv2
    end
    else
    begin
      if (c > 0.0) then
        result := 2 * pi + arctan(s / c)
      else
        if (c < 0.0) then
          result := pi + arctan(s / c)
        else
          result := 1.5 * pi;
    end;
  end;
begin
  Result.x := -RadToDeg(arcsin(-P.z / VectorLength(P)));
  Result.z :=-RadToDeg(Arcus(P.X, P.y));

  Result.y := 0;
end;

function VectorLength(P1: TPoint3S): Single; register;
asm
 FLD  DWORD PTR [EAX]
 FMUL ST, ST
 FLD  DWORD PTR [EAX+4]
 FMUL ST, ST
 FADDP
 FLD  DWORD PTR [EAX+8]
 FMUL ST, ST
 FADDP
 FSQRT
end;

procedure Normalize(var P : TPoint3S); register;
asm
  FLD  DWORD PTR [EAX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FMUL ST, ST
      FADD
      FLD  DWORD PTR [EAX+8]
      FMUL ST, ST
      FADD
      FSQRT
      FLD1
      FDIVR
      FLD  ST
      FMUL DWORD PTR [EAX]
      FSTP DWORD PTR [EAX]
      FLD  ST
      FMUL DWORD PTR [EAX+4]
      FSTP DWORD PTR [EAX+4]
      FMUL DWORD PTR [EAX+8]
      FSTP DWORD PTR [EAX+8]
end;

function NormalizePoint(P:TPoint3S):TPoint3S;
begin
 Result:=P;
 Normalize(Result);
end;

procedure CombinePoint(var vr : TPoint3S;v : TPoint3S; var f : Single); register;
asm
         FLD  DWORD PTR [EDX]
         FMUL DWORD PTR [ECX]
         FADD DWORD PTR [EAX]
         FSTP DWORD PTR [EAX]
         FLD  DWORD PTR [EDX+4]
         FMUL DWORD PTR [ECX]
         FADD DWORD PTR [EAX+4]
         FSTP DWORD PTR [EAX+4]
         FLD  DWORD PTR [EDX+8]
         FMUL DWORD PTR [ECX]
         FADD DWORD PTR [EAX+8]
         FSTP DWORD PTR [EAX+8]
end;

function AddPoint(P1, P2 : TPoint3S) : TPoint3S; register;
asm
         FLD  DWORD PTR [EAX]
         FADD DWORD PTR [EDX]
         FSTP DWORD PTR [ECX]
         FLD  DWORD PTR [EAX+4]
         FADD DWORD PTR [EDX+4]
         FSTP DWORD PTR [ECX+4]
         FLD  DWORD PTR [EAX+8]
         FADD DWORD PTR [EDX+8]
         FSTP DWORD PTR [ECX+8]
end;

function VectorDot(P1, P2 : TPoint3S): Single; register;
asm
         FLD DWORD PTR [EAX]
         FMUL DWORD PTR [EDX]
         FLD DWORD PTR [EAX + 4]
         FMUL DWORD PTR [EDX + 4]
         FADDP
         FLD DWORD PTR [EAX + 8]
         FMUL DWORD PTR [EDX + 8]
         FADDP
end;


function VectorCross(P1, P2: TPoint3S): TPoint3S;register;
var
   Temp: TPoint3S;
asm
   PUSH EBX
   LEA EBX, [Temp]
   FLD DWORD PTR [EDX + 8]
   FLD DWORD PTR [EDX + 4]
   FLD DWORD PTR [EDX + 0]
   FLD DWORD PTR [EAX + 8]
   FLD DWORD PTR [EAX + 4]
   FLD DWORD PTR [EAX + 0]

   FLD ST(1)
   FMUL ST, ST(6)
   FLD ST(3)
   FMUL ST, ST(6)
   FSUBP ST(1), ST
   FSTP DWORD [EBX]
   FLD ST(2)
   FMUL ST, ST(4)
   FLD ST(1)
   FMUL ST, ST(7)
   FSUBP ST(1), ST
   FSTP DWORD [EBX + 4]
   FLD ST
   FMUL ST, ST(5)
   FLD ST(2)
   FMUL ST, ST(5)
   FSUBP ST(1), ST
   FSTP DWORD [EBX + 8]
   FSTP ST(0)
   FSTP ST(0)
   FSTP ST(0)
   FSTP ST(0)
   FSTP ST(0)
   FSTP ST(0)
   MOV EAX, [EBX]
   MOV [ECX], EAX
   MOV EAX, [EBX + 4]
   MOV [ECX + 4], EAX
   MOV EAX, [EBX + 8]
   MOV [ECX + 8], EAX
   POP EBX
end;

procedure Negate(var P : TPoint3S); register;
asm
      FLD DWORD PTR [EAX]
      FCHS
      FSTP DWORD PTR [EAX]
      FLD DWORD PTR [EAX+4]
      FCHS
      FSTP DWORD PTR [EAX+4]
      FLD DWORD PTR [EAX+8]
      FCHS
      FSTP DWORD PTR [EAX+8]
end;

function NegatePoint(P:TPoint3S):TPoint3S;
begin
 Result:=P;
 Negate(Result);
end;

function ScalePoint(P : TPoint3S; Scale : Single) : TPoint3S;register;
asm
      FLD  DWORD PTR [EAX]
      FMUL DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX]
      FLD  DWORD PTR [EAX+4]
      FMUL DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX+4]
      FLD  DWORD PTR [EAX+8]
      FMUL DWORD PTR [EBP+8]
      FSTP DWORD PTR [EDX+8]
end;

function PointDistance(const P1, P2 : TPoint3S) : Single; register;
asm
      FLD  DWORD PTR [EAX]
      FSUB DWORD PTR [EDX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FSUB DWORD PTR [EDX+4]
      FMUL ST, ST
      FADD
      FLD  DWORD PTR [EAX+8]
      FSUB DWORD PTR [EDX+8]
      FMUL ST, ST
      FADD
      FSQRT
end;

function VisibleDotProduct(P1,P2: TAngle3S): LongBool;register;
var
  L: LongInt;
asm
  FLD DWORD PTR [EAX]
  FMUL DWORD PTR [EDX]
  FLD DWORD PTR [EAX + 4]
  FMUL DWORD PTR [EDX + 4]
  FADDP
  FLD DWORD PTR [EAX + 8]
  FMUL DWORD PTR [EDX + 8]
  FADDP

  FSTP L
  TEST DWORD PTR L, $80000000
  SETE AL
  NEG AL
  SBB EAX, EAX
end;

function VectorAngle(const V1, V2: TPoint3S): Single;
//   Result = DotProduct(V1, V2) / (Length(V1) * Length(V2)) }
asm
      FLD DWORD PTR [EAX]
      FLD ST
      FMUL ST, ST
      FLD DWORD PTR [EDX]
            FMUL ST(2), ST
      FMUL ST, ST
      FLD DWORD PTR [EAX + 4]
      FLD ST
      FMUL ST, ST
      FADDP ST(3), ST
      FLD DWORD PTR [EDX + 4]
      FMUL ST(1), ST
      FMUL ST, ST
      FADDP ST(2), ST
      FADDP ST(3), ST
      FLD DWORD PTR [EAX + 8]
      FLD ST
            FMUL ST, ST
      FADDP ST(3), ST
      FLD DWORD PTR [EDX + 8]
      FMUL ST(1), ST
      FMUL ST, ST
      FADDP ST(2), ST
      FADDP ST(3), ST
      FMULP

      FSQRT
      FDIVP
end;

function Plane(P1, P2, P3 : TPoint3S) : TPlane;
begin
   CalcPlaneNormal(p1, p2, p3, PPoint3S(@Result)^);
   Result.Arr[3]:=-VectorDot(p1, PPoint3S(@Result)^);
end;

procedure NormalizePlane(var P : TPlane);
var t : single;
begin
 t := sqrt(p.a * p.a + p.b * p.b + p.c * p.c);
 p.a := p.a / t;
 p.b := p.b / t;
 p.c := p.c / t;
 p.d := p.d / t;
  end;

function CalcPlaneNormal(p1, p2, p3 : TPoint3S) : TPoint3S;
var
   v1, v2 : TPoint3S;
begin
   v1:=SubPoint(p2,p1);
   v2:=SubPoint(p2,p1);
   Result:=VectorCross(v1, v2);
   Normalize(Result);
end;

procedure CalcPlaneNormal(p1, p2, p3 : TPoint3S; var vr : TPoint3S);
var
   v1, v2 : TPoint3S;
begin
   v1:=SubPoint(p2, p1);
   v2:=SubPoint(p3, p1);
   vr:=VectorCross(v1, v2);
   Normalize(vr);
end;

function PlaneEvaluatePoint(pl : TPlane;p : TPoint3S) : Single;
asm
      FLD DWORD PTR [EAX]
      FMUL DWORD PTR [EDX]
      FLD DWORD PTR [EAX + 4]
      FMUL DWORD PTR [EDX + 4]
      FADDP
      FLD DWORD PTR [EAX + 8]
      FMUL DWORD PTR [EDX + 8]
      FADDP
      FLD DWORD PTR [EAX + 12]
      FADDP
end;





end.

