unit Colors;

interface

uses
 Windows,SysUtils;

type

  TBGR=DWord;      // color format used in GDI
                   // bit3=0 bit2=B bit1=G bit0=R

  TARGB=DWord;     // color format used in D3D
                   // bit3=A bit2=R bit1=G bit0=B

  TARGBRec=record
         case Integer of
         0:(Color:TARGB);
         1:(B,G,R,A:byte);
        end ;

  TRGBRec=record
         B,G,R:byte;
        end ;

  TBGRRec=record
         R,G,B:byte;
        end ;

  PBGR=^TBGR;
  PARGB=^TARGB;
  PARGBRec=^TARGBRec;
  PRGBRec=^TRGBRec;
  PBGRRec=^TBGRRec;

  TARGBRecArray=array[0..0]of TARGBRec;
  PARGBRecArray=^TARGBRecArray;

const
   // std Colors
   // uses colXXX prefix to avoid conflicts with Graphics.pas
  colNone:TARGB=$00000000;
  colBlack:TARGB=$FF000000;
  colWhite:TARGB=$FFFFFFFF;
  colRed:TARGB=$FFFF0000;
  colLime:TARGB=$FF00FF00;
  colBlue:TARGB=$FF0000FF;
  colMaroon:TARGB=$FF800000;
  colGreen:TARGB=$FF008000;
  colNavy:TARGB=$FF000080;
  colYellow:TARGB=$FFFFFF00;
  colOlive:TARGB=$FF808000;
  colFuchsia:TARGB=$FFFF00FF;
  colPurple:TARGB=$FF800080;
  colAqua:TARGB=$FF00FFFF;
  colTeal:TARGB=$FF008080;
  colSilver:TARGB=$FFC0C0C0;
  colGray:TARGB=$FF808080;
  colSkyBlue:TARGB=$FFA6CAF0;
  colMoneyGreen:TARGB=$FFC0DC0;
  colCream:TARGB=$FFFFFBF0;




function ARGB(A,R,G,B:byte):TARGB;
procedure GetARGB(ARGB:TARGB;var A,R,G,B:byte);
function BGR(R,G,B:byte):TBGR;
function GetA(ARGB:TARGB):byte;
function GetR(ARGB:TARGB):byte;
function GetG(ARGB:TARGB):byte;
function GetB(ARGB:TARGB):byte;
function ARGBToBGR(ARGB:TARGB):TBGR;
function BGRToARGB(BGR:TBGR;Alpha:byte=$FF):TARGB;

function ARGBRec(ARGB:TARGB):TARGBRec;
function RGBRec(ARGB:TARGB):TRGBRec;
function BGRRec(ARGB:TARGB):TBGRRec;

function SetA(ARGB:TARGB;A:byte):TARGB;
function SetR(ARGB:TARGB;R:byte):TARGB;
function SetG(ARGB:TARGB;G:byte):TARGB;
function SetB(ARGB:TARGB;B:byte):TARGB;


// to get single color values for OpenGL and D3D
function GetAS(ARGB:TARGB):single;
function GetRS(ARGB:TARGB):single;
function GetGS(ARGB:TARGB):single;
function GetBS(ARGB:TARGB):single;

function AvgColor(C1,C2:TARGB):TARGB;


type
  TARGBStrFmt=(afHex,afInt,afBytes,afDescBytes,afFullDesc);

function ARGBToStr(ARGB:TARGB):string;
function ARGBToStrEx(ARGB:TARGB;Format:TARGBStrFmt):string;


implementation


function ARGB(A,R,G,B:byte):TARGB;
begin
Result := (A shl 24) or (R shl 16) or (G shl 8) or B;
end;

function BGR(R,G,B:byte):TBGR;
begin
Result := (B shl 16) or (G shl 8) or R;
end;

function GetA(ARGB:TARGB):byte;
begin
 Result := ARGB shr 24;
end;

function GetR(ARGB:TARGB):byte;
begin
 Result := ARGB shr 16;
end;

function GetG(ARGB:TARGB):byte;
begin
 Result := ARGB shr 8;
end;

function GetB(ARGB:TARGB):byte;
begin
 Result := ARGB;
end;

function ARGBToBGR(ARGB:TARGB):TBGR;
begin
 Result:=RGB(GetR(ARGB),GetG(ARGB),GetB(ARGB));
end;

function BGRToARGB(BGR:TBGR;Alpha:byte):TARGB;
begin
 Result:=ARGB(Alpha,GetB(BGR),GetG(BGR),GetR(BGR));
end;

function ARGBRec(ARGB:TARGB):TARGBRec;
begin
 Result.Color:=ARGB;
end;


function GetAS(ARGB:TARGB):single;
begin
 Result:=GetA(ARGB)/255;
end;

function GetRS(ARGB:TARGB):single;
begin
Result:=GetR(ARGB)/255;
end;

function GetGS(ARGB:TARGB):single;
begin
 Result:=GetG(ARGB)/255;
end;

function GetBS(ARGB:TARGB):single;
begin
 Result:=GetB(ARGB)/255;
end;

function ARGBToStr(ARGB:TARGB):string;
begin
 ARGBTostrEx(ARGB,afHex);
end;


function ARGBToStrEx(ARGB:TARGB;Format:TARGBStrFmt):string;
begin
Result:='';
case Format of
 afHex:Result:=SysUtils.Format('$%.8x',[ARGB]);
 afInt:Result:=IntToStr(ARGB);
 afBytes:Result:=SysUtils.Format('%d %d %d %d',[GetA(ARGB),GetR(ARGB),GetG(ARGB),GetB(ARGB)]);
 afDescBytes:Result:=SysUtils.Format('A:%d R:%d G:%d B:%d',[GetA(ARGB),GetR(ARGB),GetG(ARGB),GetB(ARGB)]);
 afFullDesc:Result:=SysUtils.Format('Alpha: %d  Red: %d  Green: %d  Blue: %d',[GetA(ARGB),GetR(ARGB),GetG(ARGB),GetB(ARGB)]);
end;

end;

function RGBRec(ARGB:TARGB):TRGBRec;
begin
Result.B:=ARGB;
Result.G:=ARGB shr 8;
Result.R:=ARGB shr 16;
end;

function BGRRec(ARGB:TARGB):TBGRRec;
begin
Result.B:=ARGB;
Result.G:=ARGB shr 8;
Result.R:=ARGB shr 16;
end;

procedure GetARGB(ARGB:TARGB;var A,R,G,B:byte);
begin
 A:=ARGB shr 24;
 R:=ARGB shr 16;
 G:=ARGB shr 8;
 B:=ARGB;
end;

function SetA(ARGB:TARGB;A:byte):TARGB;
begin
 Result:=(A shl 24)or(Byte(ARGB shr 16)shl 16)or(Byte(ARGB shr 8)shl 8)or Byte(ARGB);
end;

function SetR(ARGB:TARGB;R:byte):TARGB;
begin
Result:=(Byte(ARGB shr 24) shl 24)or(R shl 16)or(Byte(ARGB shr 8)shl 8)or Byte(ARGB);
end;

function SetG(ARGB:TARGB;G:byte):TARGB;
begin
 Result:=(Byte(ARGB shr 24) shl 24)or(Byte(ARGB shr 16)shl 16)or(G shl 8)or Byte(ARGB);
end;

function SetB(ARGB:TARGB;B:byte):TARGB;
begin
Result:=(Byte(ARGB shr 24) shl 24)or(Byte(ARGB shr 16)shl 16)or(Byte(ARGB shr 8)shl 8)or B;
end;

function AvgColor(C1,C2:TARGB):TARGB;
begin
 Result:=ARGB(
  (GetA(C1)+GetA(C2))div 2,
  (GetR(C1)+GetR(C2))div 2,
  (GetG(C1)+GetG(C2))div 2,
  (GetB(C1)+GetB(C2))div 2);
end;



end.
