unit BaseMgrs;

interface

uses Windows, Classes, SysUtils, WinUtils, BaseObjs, ContNrs;

type

  TBaseMgr = class(TBaseObj)
  private
    FObjects: TObjectList;
    FOwns: boolean;
    FUnique: boolean; //Same object can not be contained twice

  protected
    function GetCount: Longint; virtual;
    function GetObj(Idx: LongInt): TBaseObj; virtual;
    procedure SetObj(Idx: LongInt; Value: TBaseObj); virtual;

    procedure SetOwns(Value: boolean); virtual;
    procedure SetUnique(Value: boolean); virtual;

    procedure CheckUnique; virtual;

  public
    constructor Create(Owns: boolean = true); reintroduce; virtual;
    destructor Destroy; override;

    function Add(Obj: TBaseObj): LongInt; virtual;
    function Remove(Obj: TBaseObj): boolean; virtual;
    function Extract(Obj: TBaseObj): TBaseObj; virtual;
    function Clear: LongInt; virtual;
    function Delete(Index: LongInt): boolean; virtual;
    function IndexOf(Item: TBaseObj): LongInt; virtual;
    function Insert(Index: LOngInt; Item: TBaseObj): boolean; virtual;
    function Find(AName:string):TBaseObj;virtual;

    property Objects[Index: LongInt]: TBaseObj read GetObj write SetObj;default;
    property Count: LongInt read GetCount;
    property ObjectList:TObjectList read FObjects;

  published

    property OwnsObjects: boolean read FOwns write SetOwns;
    property UniqueObjects: boolean read FUnique write SetUnique;

  end;

  TSortType = (stAscending, stDescending);
     //  Ascending - Objs with lower Order values have lower indexes
     //  Descending - Objs with lower Order have higher indexes

  TSortedMgr = class(TBaseMgr)
  private
    FSorting: TSortType;

  protected
    procedure SetObj(Idx: LongInt; Value: TBaseObj); override;
    procedure SetSorting(Value: TSortType); virtual;

  public
    function Add(Obj: TBaseObj): LongInt; override;
    function Insert(Index: LOngInt; Item: TBaseObj): boolean; override;

  published
    property Sorting: TSortType read FSorting write SetSorting;

  end;

 TNamedMgr=class(TBaseMgr)
  private
  protected
  public

    function Add(Obj: TBaseObj): LongInt; override; // do not add obj with same name as another in list

  published

  end;



implementation


constructor TBaseMgr.Create(Owns: boolean);
begin
  inherited Create;
  FObjects := TObjectList.Create(Owns);
  FUnique := true;
end;

destructor TBaseMgr.Destroy;
begin

  FObjects.Free;
  inherited Destroy;
end;

function TBaseMgr.Add(Obj: TBaseObj): LongInt;
begin
  Result := FObjects.Add(Obj);
  CheckUnique;
end;

function TBaseMgr.Remove(Obj: TBaseObj): boolean;
begin
  Result := Iff(FObjects.Remove(Obj) >= 0, true, false);
end;

function TBaseMgr.Delete(Index: LongInt): boolean;
begin
  if (Index>-1)and(Index<Count) then

    begin
    Result := true;
    FObjects.Delete(Index);
  end
  else
    Result := false;

end;

function TBaseMgr.Find(AName:string):TBaseObj;
var I:LongInt;
begin
 Result:=nil;
 for I:=0 to Count-1 do
  if CompStr(AName,Objects[I].Name)then
   begin
    Result:=Objects[I];
    Exit;
   end;
end;


function TBaseMgr.Extract(Obj: TBaseObj): TBaseObj;
begin
  Result := TBaseObj(FObjects.Extract(Obj));
end;

function TBaseMgr.IndexOf(Item: TBaseObj): LongInt;
begin
  Result := FObjects.IndexOf(Item);
end;

function TBaseMgr.Insert(Index: LOngInt; Item: TBaseObj): boolean;
begin
  if (Index>-1)and(Index<Count) then
  begin
    Result := true;
    FObjects.Insert(Index, Item);
    CheckUnique;
  end
  else
    Result := false;

    end;


procedure TBaseMgr.SetOwns(Value: boolean);
begin
  FOwns := Value;
  FObjects.OwnsObjects := Value;
end;

procedure TBaseMgr.SetUnique(Value: boolean);
begin
  FUnique := Value;
  CheckUnique;
end;

function TBaseMgr.GetObj(Idx: LongInt): TBaseObj;
begin
  Result := nil;
  if (Idx>-1)and(Idx<Count) then
     Result := TBaseObj(FObjects[Idx]);

end;

procedure TBaseMgr.SetObj(Idx: LongInt; Value: TBaseObj);
begin
 if (Idx>-1)and(Idx<Count) then
    FObjects[Idx] := Value;
end;

function TBaseMgr.GetCount: Longint;
begin
  Result := FObjects.Count;
end;


procedure TBaseMgr.CheckUnique;
var Max, I, J: LongInt;
  Obj: TObject;
begin
  if not FUnique then Exit;

  Max := FObjects.Count - 1;
  for I := 0 to Max do
  begin
    if I > FObjects.Count - 1 then Break;
    Obj := FObjects[I];
    for J := 0 to Max do
    begin
      if J > FObjects.Count - 1 then Break;
      if (Obj = FObjects[J]) and (I <> J) then FObjects.Extract(FObjects[J]);
     end;
  end;


  Max := FObjects.Count - 1;
  for I := 0 to Max do
  begin
    if I > FObjects.Count - 1 then Break;
    if FObjects[I]=nil then
     FObjects.Delete(I);
  end;


end;

function TBaseMgr.Clear: LongInt;
begin
  Result := FObjects.Count;
  FObjects.Clear;
end;


////////////////////////////////

function TSortedMgr.Add(Obj: TBaseObj): LongInt;
var
  L, H, I, C: Integer;
  IsC: boolean;
begin

  L := 0;
  H := FObjects.Count - 1;
  while L <= H do
  begin
    I := (L + H) div 2;
    C := TBaseObj(FObjects[I]).Order - Obj.Order;
    if FSorting = stAscending then
      IsC := C < 0
    else
      IsC := C > 0;
    if IsC then
      L := I + 1
    else
      H := I - 1;
  end;

  FObjects.Insert(L, Obj);
  CheckUnique;
  Result := L;
end;

procedure TSortedMgr.SetObj(Idx: LongInt; Value: TBaseObj);
begin
 //do nothing for sorted list

end;

function TSortedMgr.Insert(Index: LOngInt; Item: TBaseObj): boolean;
begin
  Result:=true; 
  Add(Item);
end;


procedure TSortedMgr.SetSorting(Value: TSortType);
var Imax, J: LongInt;
  Obj: TBaseObj;

begin
  FSorting := Value;
  Imax := Count - 1;
  for j := 0 to Imax do
  begin
    Obj := Objects[0];
    FObjects.Extract(Obj);
    Add(Obj);
  end;
end;


function TNamedMgr.Add(Obj: TBaseObj): LongInt;
var I:LongInt;
    Found:boolean;
begin
 Result:=0;
 if Obj=nil then Exit;
 Found:=false;

 for I:=0 to Count-1 do
  if (Objects[I].Name=Obj.Name)and
     (Objects[I]<>Obj)then
    Found:=true;

 if not Found then
  Result:= inherited Add(Obj)
 else
  begin
  Obj.Free;        // be careful - don't use   HuhlyManager.Add(Self);
  Obj:=nil;
  end;


end;



end.

