unit BaseObjs;

interface

uses
  Windows,Timing,WinUtils;

type

  TNamedObj = class(TObject)
  private
    FName: string;
  protected
    procedure SetName(Value: string); virtual;
  published
    property Name: string read FName write SetName;
  end;



  TTimedObj = class(TNamedObj)
  private
    FInited:boolean;
    FPeriod: LongInt; //Minimal period of updates in miliseconds
    FAge: LongInt; //Age of object in miliseconds
    FLastUpdate: LongInt; //Time of last update
    FElapsed: LongInt; //Time between updates
    FBirth:LongInt;
  protected

  public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure Init; virtual;
    procedure Done;virtual;
    procedure Update; virtual;
    procedure TryUpdate(Time: LongInt=0); virtual;

    procedure SetPeriod(MiliSec: LongInt); virtual;

    property LastUpdate: LongInt read FLastUpdate;
    property Age: LongInt read FAge;
    property Elapsed: LongInt read FElapsed;
    property Birth: LongInt read FBirth;
    property Inited:boolean read FInited;

  published

    property UpdatePeriod: LongInt read FPeriod write SetPeriod;


  end;


  TBaseObj=class(TTimedObj)
  private
   FOrder:LongInt;
   FTag:LongInt;

  protected
   procedure SetOrder(Value:LongInt);virtual;
   procedure SetTag(Value:LongInt);virtual;

  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Init;override;
    procedure Done;override;

    procedure Assign(Source:TBaseObj);virtual;
  published
    property Order:LongInt read FOrder write SetOrder;
    property Tag:LongInt read FTag write SetTag;

  end;




implementation


//////////////////////////////
///////  TNAMED OBJ
//////////////////////////////


procedure TNamedObj.SetName(Value: string);
begin
  FName := Value;
end;

//////////////////////////////
///////  TTIMED OBJ
//////////////////////////////


constructor TTimedObj.Create;
begin
  inherited Create;
  FBirth:=Timing.GetWindowsTimeInt;
  FAge := 0;
  FLastUpdate := 0;
  FPeriod := 1000;
  FName:='';

end;

destructor TTimedObj.Destroy;
begin

  inherited Destroy;
end;

procedure TTimedObj.Init;
begin
FInited:=true;
end;

procedure TTimedObj.Done;
begin
FInited:=false;
end;


procedure TTimedObj.Update;
begin

end;

procedure TTimedObj.SetPeriod(MiliSec: LongInt);
begin
  FPeriod := MiliSec;
end;

procedure TTimedObj.TryUpdate(Time: LongInt);
begin
  if Time<0 then Exit;
  if Time>0 then
   FAge := FAge + Time
  else
   FAge:=LongInt(Timing.GetWindowsTimeInt)-FBirth;

  if (FAge - FLastUpdate >= FPeriod) then
  begin
    FElapsed := FAge - FLastUpdate;
    FLastUpdate := FAge;
    Update;
  end;


end;


//////////////////////////////
///////  TBASE OBJ
//////////////////////////////

constructor TBaseObj.Create;
begin
  inherited Create;
  FOrder:=0;
end;

destructor TBaseObj.Destroy;
begin

  inherited Destroy;
end;

procedure TBaseObj.SetOrder(Value:LongInt);
begin
 FOrder:=Value;
end;

procedure TBaseObj.SetTag(Value:LongInt);
begin
 FTag:=Value;
end;

procedure TBaseObj.Assign(Source:TBaseObj);
begin
 Name:=Source.Name;
 Order:=Source.Order;
 
end;

procedure TBaseObj.Init;
begin

 inherited Init;
end;

procedure TBaseObj.Done;
begin
 if not Inited then Exit;
 inherited Done;
end;

end.

