{*****************************************************************************}
{  								                                                            }
{	Copyright (c) 2002  Vampyre SoftWorks			                                  }
{								                                                              }
{								                                                              }
{                                                                             }
{*****************************************************************************}
{								                                                              }
{	WINUTILS.PAS						                                                    }
{	Miscellaneous routines		                                                  }
{								                                                              }
{	Created by:						                                                      }
{	Marek "Mar" "Galfar"  Mauder  [pentar@seznam.cz]	                          }
{								                                                              }
{	Last Modified:						                                                  }
{	22.06.2002						                                                      }
{								                                                              }
{*****************************************************************************}



unit WinUtils;

interface


uses Classes, Windows, SysUtils, Graphics;

type

  TFileListOption  = (flFullNames, flRecursive, flMaskedSubfolders);
  TFileListOptions = set of TFileListOption;

function CompStr(S1, S2: string): boolean;
function CompStrArray(S1: string; Arr: array of string): boolean;
function ExeDir: string;
function StrInList(S: string; List: TStrings): boolean;
procedure WriteStr(Stream: TStream; S: string);
function ReadStr(Stream: TStream): string;
function ForceDirectories(Dir: string): Boolean;
function DirectoryExists(const Name: string): Boolean;
function CurPos: TPoint;
function MsgBox(S: string): LongInt; overload;
function MsgBox(L: LongInt): LongInt; overload;
procedure ExitApp;
function Max(A, B: Integer): Integer; overload;
function Min(A, B: Integer): Integer; overload;
function Max(A, B: single): single; overload;
function Min(A, B: single): single; overload;
procedure Swap(var a, b: LongInt); overload;
procedure Swap(var a, b: single); overload;
function ClipRect(var DestRect: TRect; const DestRect2: TRect): Boolean;
function ClipRect2(var DestRect, SrcRect: TRect; const DestRect2, SrcRect2:
  TRect): Boolean;
function Sgn(a: LongInt): LongInt;
function DegToRad(Degrees: Extended): Extended;
function SubRect(R1, R2: TRect): TRect;
function Iff(const Condition: Boolean; const TruePart, FalsePart: string):
string; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Char): Char;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Byte): Byte;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Integer):
Integer; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Cardinal):
Cardinal; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Single):
Single; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Boolean):
Boolean; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Pointer):
Pointer; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Int64): Int64;
overload;
function IntInArray(Num: LongInt; Arr: array of LongInt): boolean;
function RPos(C: ShortString; Src: string): Integer;
function RightStr(S: string; ToPos: Integer): string;
function LastDir(Dir: string): string;
function DirUp(Directory: string): string;
function StrListFromArr(Arr: array of string): TStringList;
procedure FillMem(Mem: Pointer; Size, Value: Integer);
function RandSignI(Num: LongInt): LongInt;
function RandSignF(Num: single): single;
procedure Switch(var Value: boolean);
function StrToken(var S: string; Separator: Char): string;
procedure StrTokenToStrings(S: string; Separator: Char; List: TStrings);
function ComposeString(List: TStrings; Separator: char): string;
function StrMatches(Substr, S: string; Index: LongInt = 1 ): Boolean;
function AdvBuildFileList(Path: string;  Attr: Integer;
   Files: TStrings; Options: TFileListOptions = [] ;
   SubfoldersMask: string  = '' ): Boolean;
function FileSize(FileName: string): Int64;
// returns index of found substring
function StrArrPos(S:string;Subs:array of string;Off:LongInt=0):LongInt;
function StrListPos(S:string;Subs:TStrings;Off:LongInt=0):LongInt;
function CompFonts(F1,F2:TFont):boolean;


implementation

procedure FillMem(Mem: Pointer; Size, Value: Integer);
asm
  push edi
  push ebx
  mov ebx,edx
  mov edi,eax
  mov eax,ecx
  mov ecx,edx
  shr ecx,2
  jz  @word
  rep stosd
  @word:
  mov ecx,ebx
  and ecx,2
  jz  @byte
  mov [edi],ax
  add edi,2
  @byte:
  mov ecx,ebx
  and ecx,1
  jz  @exit
  mov [edi],al
  @exit:
  pop ebx
  pop edi
end;

function RPos(C: ShortString; Src: string): Integer;
var
  i: integer;
begin
  Result := 0;
  for i := Length(Src) downto 1 do
    if Src[i] = C then
    begin
      Result := i;
      Break;
    end;
end;

function RightStr(S: string; ToPos: Integer): string;
begin
  Result := Copy(S, ToPos, Length(S) - ToPos + 1);
end;



function LastDir(Dir: string): string;
begin
  if Dir[Length(Dir)] = '\' then
    Delete(Dir, Length(Dir), 1);
  Result := RightStr(Dir, RPos('\', Dir) + 1);
end;


function DirUp(Directory: string): string;
begin
  Result := Directory;
  if Result[Length(Result)] = '\' then
    Delete(Result, Length(Result), 1);
  Delete(Result, Length(Result) - Length(LastDir(Result)), Length(Result));
end;



function DegToRad(Degrees: Extended): Extended;
begin
  Result := Degrees * (PI / 180);
end;


function Sgn(a: LongInt): LongInt;
begin
  if a > 0 then
    result := +1
  else
    if a < 0 then
      result := -1
    else
      result := 0;
end;



function ClipRect(var DestRect: TRect; const DestRect2: TRect): Boolean;
begin
  with DestRect do
  begin
    Left := Max(Left, DestRect2.Left);
    Right := Min(Right, DestRect2.Right);
    Top := Max(Top, DestRect2.Top);
    Bottom := Min(Bottom, DestRect2.Bottom);

    Result := (Left < Right) and (Top < Bottom);
  end;
end;


function ClipRect2(var DestRect, SrcRect: TRect; const DestRect2, SrcRect2:
  TRect): Boolean;
begin
  if DestRect.Left < DestRect2.Left then
  begin
    SrcRect.Left := SrcRect.Left + (DestRect2.Left - DestRect.Left);
    DestRect.Left := DestRect2.Left;
  end;

  if DestRect.Top < DestRect2.Top then
  begin
    SrcRect.Top := SrcRect.Top + (DestRect2.Top - DestRect.Top);
    DestRect.Top := DestRect2.Top;
  end;

  if SrcRect.Left < SrcRect2.Left then
  begin
    DestRect.Left := DestRect.Left + (SrcRect2.Left - SrcRect.Left);
    SrcRect.Left := SrcRect2.Left;
  end;

  if SrcRect.Top < SrcRect2.Top then
  begin
    DestRect.Top := DestRect.Top + (SrcRect2.Top - SrcRect.Top);
    SrcRect.Top := SrcRect2.Top;
  end;

  if DestRect.Right > DestRect2.Right then
  begin
    SrcRect.Right := SrcRect.Right - (DestRect.Right - DestRect2.Right);
    DestRect.Right := DestRect2.Right;
  end;

  if DestRect.Bottom > DestRect2.Bottom then
  begin
    SrcRect.Bottom := SrcRect.Bottom - (DestRect.Bottom - DestRect2.Bottom);
    DestRect.Bottom := DestRect2.Bottom;
  end;

  if SrcRect.Right > SrcRect2.Right then
  begin
    DestRect.Right := DestRect.Right - (SrcRect.Right - SrcRect2.Right);
    SrcRect.Right := SrcRect2.Right;
  end;

  if SrcRect.Bottom > SrcRect2.Bottom then
  begin
    DestRect.Bottom := DestRect.Bottom - (SrcRect.Bottom - SrcRect2.Bottom);
    SrcRect.Bottom := SrcRect2.Bottom;
  end;

  Result := (DestRect.Left < DestRect.Right) and (DestRect.Top < DestRect.Bottom)
    and
    (SrcRect.Left < SrcRect.Right) and (SrcRect.Top < SrcRect.Bottom);
end;



procedure Swap(var a, b: LongInt);
var
  c: LongInt;
begin
  c := a;
  a := b;
  b := c;
end;

procedure Swap(var a, b: single);
var
  c: single;
begin
  c := a;
  a := b;
  b := c;
end;

function Max(A, B: LongInt): LongInt;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;


function Min(A, B: LongInt): LongInt;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;


function Max(A, B: single): single;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;


function Min(A, B: single): single;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;


function DirectoryExists(const Name: string): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Name));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;



function ForceDirectories(Dir: string): Boolean;
begin
  Result := True;
  if Length(Dir) = 0 then
    Exit;
  Dir := ExcludeTrailingBackslash(Dir);
  if (Length(Dir) < 3) or DirectoryExists(Dir)
    or (ExtractFilePath(Dir) = Dir) then Exit;
  Result := ForceDirectories(ExtractFilePath(Dir)) and CreateDir(Dir);
end;





function ExeDir: string;
begin
  Result := ExtractFileDir(ParamStr(0)) + '\';

end;

function CompStr(S1, S2: string): boolean;
begin
  if AnsiCompareText(S1, S2) = 0 then
    Result := true
  else
    Result := false;
end;

function CompStrArray(S1: string; Arr: array of string): boolean;
var
  i: LongInt;
begin
  Result := false;
  for i := 0 to High(Arr) do
    if CompStr(S1, Arr[i]) then Result := true;

end;



function StrInList(S: string; List: TStrings): boolean;
var
  i: LongInt;
begin
  Result := false;
  for i := 0 to List.Count - 1 do
    if CompStr(S, List.Strings[i]) then Result := true;

end;

procedure WriteStr(Stream: TStream; S: string);
var
  Writer: TWriter;
begin
  Writer := TWriter.Create(Stream, 8192);
  Writer.WriteString(S);
  Writer.Free;
end;

function ReadStr(Stream: TStream): string;
var
  Reader: TReader;
begin
  Reader := TReader.Create(Stream, 8192);
  Result := Reader.ReadString;
  Reader.Free;
end;


function CurPos: TPoint;
var
  P: TPoint;
begin
  GetCursorPos(P);
  Result := P;
end;


function MsgBox(S: string): LongInt;
begin
  Result := MessageBox(GetForegroundWindow, Pchar(S), 'Message', 0);
end;

function MsgBox(L: LongInt): LongInt;
begin
  Result := MessageBox(GetForegroundWindow, Pchar(IntToStr(L)), 'Message', 0);
end;


procedure ExitApp;
begin
  Halt(0);
end;

function SubRect(R1, R2: TRect): TRect;
begin
  Result := Rect(r1.left - r2.left, r1.top - r2.top, r1.right - r2.left,
    r1.bottom - r2.top);

end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: string):
string; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Char): Char;
  overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Byte): Byte;
  overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Integer):
Integer; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Cardinal):
Cardinal; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Single):
Single; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Boolean):
Boolean; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Pointer):
Pointer; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;


function Iff(const Condition: Boolean; const TruePart, FalsePart: Int64): Int64;
  overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function IntInArray(Num: LongInt; Arr: array of LongInt): boolean;
var I: LongInt;
begin
  Result := false;
  for I := Low(Arr) to High(Arr) do
    if Num = Arr[I] then Result := true;

end;

function StrListFromArr(Arr: array of string): TStringList;
var SL: TStringList;
  I: LongInt;
begin
  SL := TStringList.Create;
  for I := Low(Arr) to High(Arr) do
    SL.Add(Arr[I]);

  Result := SL;

end;

function RandSignI(Num: LongInt): LongInt;
begin
  if Random(2) = 0 then
    Result := Num
  else
    Result := -Num;
end;


function RandSignF(Num: single): single;
begin
  if Random(2) = 0 then
    Result := Num
  else
    Result := -Num;
end;

procedure Switch(var Value: boolean);
begin
  Value := not Value;
end;

function StrToken(var S: string; Separator: Char): string;
var
  I: LongInt;
begin
  I := Pos(Separator, S);
  if I <> 0 then
  begin
    Result := Copy(S, 1, I - 1);
    Delete(S, 1, I);
  end
  else
  begin
    Result := S;
    S := '';
  end;
end;


procedure StrTokenToStrings(S: string; Separator: Char; List: TStrings);
var
  Token: string;
begin
  List.Clear;
  while S <> '' do
  begin
    Token := StrToken(S, Separator);
    List.Add(Token);
  end;
end;


function ComposeString(List: TStrings; Separator: char): string;
var
  S: string;
  I: LongInt;
begin
  S := '';
  for I := 0 to List.Count - 1 do
    S := S + List[I] + Separator;
  Delete(S, Length(S), 1);
  Result := S;
end;


function StrMatches(Substr, S: string; Index: LongInt = 1 ): Boolean;
var
  StringPtr: PChar;
  PatternPtr: PChar;
  StringRes: PChar;
  PatternRes: PChar;

begin
  Result := False;
  StringPtr := PChar(@S[Index]);
  PatternPtr := PChar(SubStr);
  StringRes := nil;
  PatternRes := nil;

  if (S='') or (SubStr='') then
    Exit;

  repeat
    repeat
      case PatternPtr^ of
        #0: begin
              Result:=StringPtr^=#0;
              if Result or (StringRes=nil) or (PatternRes=nil) then
                Exit;

              StringPtr:=StringRes;
              PatternPtr:=PatternRes;
              Break;
            end;
        '*': begin
               inc(PatternPtr);
               PatternRes:=PatternPtr;
               Break;
             end;
        '?': begin
               if StringPtr^=#0 then
                 Exit;
               inc(StringPtr);
               inc(PatternPtr);
             end;
        else
        begin
          if StringPtr^=#0 then
            Exit;
          if StringPtr^<>PatternPtr^ then
          begin
            if (StringRes=nil) or (PatternRes=nil) then
              Exit;
            StringPtr:=StringRes;
            PatternPtr:=PatternRes;
            Break;
          end
          else
          begin
            inc(StringPtr);
            inc(PatternPtr);
          end;
        end;
      end;
    until false;

    repeat
      case PatternPtr^ of
        #0: begin
          Result:=true;
          Exit;
        end;
        '*': begin
          inc(PatternPtr);
          PatternRes:=PatternPtr;
        end;
        '?': begin
          if StringPtr^=#0 then
            Exit;
          inc(StringPtr);
          inc(PatternPtr);
        end;
        else begin
          repeat
            if StringPtr^=#0 then
              Exit;
            if StringPtr^=PatternPtr^ then
              Break;
            inc(StringPtr);
          until false;
          inc(StringPtr);
          StringRes:=StringPtr;
          inc(PatternPtr);
          Break;
        end;
      end;
    until false;
  until false;
end;


// use *.* wildcard for files ( if they have no atribs )
function AdvBuildFileList(Path: string;  Attr: Integer;
   Files: TStrings; Options: TFileListOptions = [] ;
   SubfoldersMask: string  = '' ): Boolean;
  var
  FileMask: string;
  RootDir: string;
  Folders: TStringList;
  CurrentItem: Integer;
  Counter: Integer;
  LocAttr: Integer;

  procedure BuildFolderList;
  var
    FindInfo: TSearchRec;
    Rslt: Integer;
  begin
    Counter := Folders.Count - 1;
    CurrentItem := 0;
    while CurrentItem <= Counter do
    begin
      // searching for subfolders
      Rslt := FindFirst(Folders[CurrentItem] + '*.*', faDirectory, FindInfo);
      try
        while Rslt = 0 do
        begin
          if (FindInfo.Name <> '.') and (FindInfo.Name <> '..') and
            (FindInfo.Attr and faDirectory = faDirectory) then
            Folders.Add(Folders[CurrentItem] + FindInfo.Name + '\');
          Rslt := FindNext(FindInfo);
        end;
      finally
        FindClose(FindInfo);
      end;
      Counter := Folders.Count - 1;
      Inc(CurrentItem);
    end;
  end;

  procedure FillFileList(CurrentCounter: Integer);
  var
    FindInfo: TSearchRec;
    Rslt: Integer;
    CurrentFolder: String;
  begin
    CurrentFolder := Folders[CurrentCounter];
    Rslt := FindFirst(CurrentFolder + FileMask, LocAttr, FindInfo);
    try
      while Rslt = 0 do
      begin
        //if (LocAttr and FindInfo.Attr) =FindInfo.Attr then
          if flFullNames in Options then
            Files.Add(CurrentFolder + FindInfo.Name)
          else
            Files.Add(FindInfo.Name);
        Rslt := FindNext(FindInfo);

      end;
    finally
      FindClose(FindInfo);
    end;
  end;

begin
  FileMask := ExtractFileName(Path);
  RootDir := ExtractFilePath(Path);

  Folders := TStringList.Create;
  Folders.Add(RootDir);

  Files.Clear;

  if Attr = faAnyFile then
    LocAttr := faReadOnly + faHidden + faSysFile + faArchive
  else
    LocAttr := Attr;

  // here's the recursive search for nested folders

  if flRecursive in Options then
    BuildFolderList;

  if Attr<>faDirectory then
  for Counter := 0 to Folders.Count - 1 do
  begin
    if (((flMaskedSubfolders in Options) and (StrMatches(SubfoldersMask, Folders[Counter],1))) or
      (not (flMaskedSubfolders in Options))) then
      FillFileList(Counter);
  end
  else
  Files.AddStrings(Folders);
  Folders.Free;
  Result := True;
end;

function FileSize(FileName: string): Int64;
var
  Handle: THandle;
  FindData: TWin32FindData;
begin
  Result := 0;
  Handle := FindFirstFile(PChar(FileName), FindData);
  if Handle <> INVALID_HANDLE_VALUE then
  begin
    Windows.FindClose(Handle);
    Result := (FindData.nFileSizeHigh shl 32) + FindData.nFileSizeLow;
  end
  else
    RaiseLastOSError;
end;

function StrArrPos(S:string;Subs:array of string;Off:LongInt):LongInt;
var LS:string;
    I:LongInt;
begin
 LS:=AnsiLowerCase(S);
 Delete(LS,1,Off);

 for I:=Low(Subs) to High(Subs) do
  begin
  Result:=AnsiPos(AnsiLowerCase(Subs[I]),LS);
  if Result>0 then
   begin
   Result:=Result+Off;
   Exit;
   end;
  end;
   Result:=0;
 end;

function StrListPos(S:string;Subs:TStrings;Off:LongInt=0):LongInt;
var LS:string;
    I:LongInt;
begin
 LS:=AnsiLowerCase(S);
 Delete(LS,1,Off);

 for I:=0 to Subs.Count-1 do
  begin
  Result:=AnsiPos(AnsiLowerCase(Subs[I]),LS);
  if Result>0 then
   begin
   Result:=Result+Off;
   Exit;
   end;
  end;
   Result:=0;
 end;

function CompFonts(F1,F2:TFont):boolean;
begin
 if F1=nil then Exit;
 if F2=nil then Exit;

 Result:=CompStr(F1.Name,F2.Name)and
 (F1.Size=F2.Size)and(F1.Color=F2.Color)and(F1.Style=F2.Style);


end;



end.

