unit Tables;

interface

uses Windows, Classes, SysUtils, FileVFormat, BaseObjs, TableCellMgr,
TableStyles,TablePages,BaseMgrs,TableCells;

type

  TTable = class(TBaseObj)
  private
    FPages:TBaseMgr;
    FComment: TStringList;
    FStyle: TTableStyle;
    FMaxCols: LongInt;
    FMaxRows: LongInt;
    FAPage:TTablePage;

  protected

    procedure SetComment(Value: TStringList);
    procedure SetMax(Idx: LongInt; Value: LongInt);

    function GetCells:TCellMgr;
    function GetPageCount:LongInt;
    function GetCell(C,R:LongInt):TCell;
    function GetPage(Idx:LongInt):TTablePage;
    function GetPageName(Name:string):TTablePage;


  public
    constructor Create(AName: string = '');
    destructor Destroy; override;

    property Pages:TBaseMgr read FPages;
    property Cells[C,R:LongInt]:TCell read GetCell;
    property CellMgr: TCellMgr read GetCells;
    property PagesIdx[Index:LongInt]:TTablePage read GetPage;
    property PagesName[Name:string]:TTablePage read GetPageName;
    property PageMgr:TBaseMgr read FPages;
    property PageCount:LongInt read GetPageCount;
    property Style:TTableStyle read FStyle;

  published

    property Comment: TStringList read FComment write SetComment;
    property MaxCols: LongInt index 0 read FMaxCols write SetMax;
    property MaxRows: LongInt index 1 read FMaxRows write SetMax;


  end;


implementation

constructor TTable.Create(AName: string);
begin
  inherited Create;
  if AName = '' then
    Name := 'Unnamed Table'
  else
    Name := AName;

  FPages:=TBaseMgr.Create;
  FPages.Add(TTablePage.Create(Self));
  FAPage:=TTablePage(Pages[0]);
  FComment := TStringList.Create;
  FMaxCols:=256;
  FMaxRows:=256;
  FStyle:=TTableStyle.Create(Self);
end;

destructor TTable.Destroy;
begin
  FComment.Free;
  FAPage:=nil;
  FPages.Free;
  FStyle.Free;
  inherited Destroy;
end;

procedure TTable.SetComment(Value: TStringList);
begin
  FComment.Assign(Value);
end;

procedure TTable.SetMax(Idx: LongInt; Value: LongInt);
begin
  case Idx of
    0: FMaxCols := Value;
    1: FMaxRows := Value;
  end;
end;

function TTable.GetCells:TCellMgr;
begin
 Result:=nil;
 if FAPage<>nil then
  Result:=FAPage.Cells;
end;

function TTable.GetPageCount:LongInt;
begin
 Result:=FPages.Count;
end;

function TTable.GetCell(C,R:LongInt):TCell;
begin
 Result:=nil;
 if FAPage<>nil then
  Result:=FAPage.Cells[C,R];
end;

function TTable.GetPage(Idx:LongInt):TTablePage;
begin
 Result:=TTablePage(FPages[Idx]);
end;

function TTable.GetPageName(Name:string):TTablePage;
begin
 Result:=TTablePage(FPages.Find(Name));
end;


end.

