unit FileVFormat;

interface

uses Windows, Classes, SysUtils, WinUtils,ContNRS,ZLIB;

type

  EBadFileID = class(Exception);
  EBadHeader = class(Exception);
  ESaveData=class(Exception);
  ELoadData=class(Exception);
  EAddData=class(Exception);
  EBadMode=class(Exception);
  EReadData=class(Exception);
  ESeekData=class(Exception);



  TReserved = array[0..127] of byte;
  TID = array[0..3] of char;
  TSplit = array[0..1] of char;
  TDesc=array[0..19]of char;

  TVFile = class;


  TVFileHeader = class(TPersistent)
  private
    FOwner: TVFile;
    FID: string;
    FDesc: string;
    FVerHI: word;
    FVerLO: word;
    FObjects:LongInt;
    FFSize:LongInt;
    FHSize:LongInt;
    FCompress:boolean;
    FCoded:boolean;

    FReserved: TReserved;


  protected

  public
    constructor Create(AOwner: TVFile); virtual;

    procedure SaveToStream(Stream: TStream);
    procedure LoadFRomStream(Stream: TStream);


  published
    property Owner: TVFile read FOwner;
    property ID: string read FID write FID;
    property Desc: string read FDesc write FDesc;
    property VerHI: word read FVerHI write FVerHI;
    property VerLO: word read FVerLO write FVerLO;
    property Compress:boolean read FCompress write FCompress;
    property Coded:boolean read FCoded write FCoded;

  end;

  TVDataObj = class(TPersistent)
  private
   FStream:TMemoryStream;
   FName:string;
   FSize:LongInt;
   FTag:Word;
   FOwner:TVFile;
   FReadRet:boolean;

   procedure WriteHdr(Stream:TStream);
   procedure ReadHdr(Stream:TStream);

   function Skip(Stream:TStream):boolean;

   function ReadSize(Stream:TStream):LongInt;
   function ReadName(Stream:TStream):string;
   function ReadTag(Stream:TStream):Word;
   function GetHdrSize(Stream:TStream):LongInt;

  protected
   function GetSize:LongInt;
  public
   constructor Create(AOwner:TVFile;AName:string);
   destructor Destroy;override;

   procedure SaveToStream(Stream:TStream);
   procedure LoadFromStream(Stream:TStream);


  published
   property Stream:TMemoryStream read FStream write FStream;
   property Name:string read FName write FName;
   property Tag:Word read FTag write FTag;

  end;



       
  TVFileMode=(fmUndef,fmRead,fmWrite);

  TVFile = class(TPersistent)
  private
    FName: string;
    FStream: TMemoryStream;
    FHeader: TVFileHeader;
    FObjects:TObjectList;

  protected
    function GetMode:TVFileMode;virtual;
    function GetSize:LongInt;virtual;

  public

    constructor Create(AName: string); overload; virtual;
    constructor Create(AName: string; Size: LongInt; Data: Pointer); overload;
      virtual;
    constructor Create(AName: string; Stream: TStream); overload; virtual;
    constructor Create; overload; virtual;

    destructor Destroy; override;

    procedure LoadFromFile(AName: string); virtual;
    procedure LoadFromMem(Size: LongInt; Data: Pointer; AName: string = '');
      virtual;
    procedure LoadFromStream(Stream: TStream; AName: string = ''); virtual;

    procedure SaveToFile(AName: string); virtual;
    procedure SaveToStream(Stream: TStream); virtual;

    function AddData(AName:string;Size:LongInt;Data:Pointer):TVDataObj;overload;virtual;
    function AddData(AName:string;Stream:TStream):TVDataObj;overload;virtual;
    function AddData(Obj:TVDataObj):TVDataObj;overload;virtual;

    function GetDataStart:LongInt;virtual;

    function GetData(AName:string):TVDataObj;overload;virtual;
    function GetData(AName:string;out Obj:TVDataObj):LongInt;overload;virtual;

    function SeekData(AName:string):LongInt;overload;virtual;
    function SeekData(Pos:LongInt):LongInt;overload;virtual;
    function ReadData:TVDataObj;virtual;
    //function



    property Objects:TObjectList read FObjects;
    property DataSize:LongInt read GetSize;

  published

    property Name: string read FName write FName;
    property Stream: TMemoryStream read FStream write FStream;
    property Header: TVFileHeader read FHeader write FHeader;
    property Mode:TVFileMode read GetMode;

  end;



const

  sSplitter: TSplit = #0'\';
  sVFID: TID = 'VFCF'; //Vampyre Format Custom File

  sBadID = 'Error when loading header - Bad file ID';
  sBadHeader = 'Error - Bad header';
  sSaveData = 'Error when saving data object';
  sLoadData = 'Error when loading data object';
  sAddData='Error when adding data object';
  sBadMode='Bad file mode - error in get/find';
  sReadData='Error when reading data';
  sSeekData='Error when seeking';


implementation



/////////////////  TVFILEHEADER

constructor TVFileHeader.Create(AOwner: TVFile);
begin
  inherited Create;
  FOwner := AOwner;

end;

procedure TVFileHeader.SaveToStream(Stream: TStream);
begin

  try
    Stream.WriteBuffer(sVFID, SizeOf(sVFID));
    Stream.WriteBuffer(sSplitter, SizeOf(sSplitter));

    WriteStr(Stream,ID);
    Stream.WriteBuffer(sSplitter, SizeOf(sSplitter));

    WriteStr(Stream,Desc);
    Stream.WriteBuffer(sSplitter, SizeOf(sSplitter));

    Stream.WriteBuffer(FVerHi, SizeOf(FVerHi));
    Stream.WriteBuffer(FVerLO, SizeOf(FVerLO));

    Stream.WriteBuffer(FHSize, SizeOf(FHSize));
    Stream.WriteBuffer(FFSize, SizeOf(FFSize));

    Stream.WriteBuffer(FCompress, SizeOf(FCompress));

    Stream.WriteBuffer(FCoded, SizeOf(FCoded));

    Stream.WriteBuffer(FReserved, SizeOf(FReserved));
    Stream.WriteBuffer(sSplitter, SizeOf(sSplitter));

    Stream.WriteBuffer(FObjects, SizeOf(FObjects));
    Stream.WriteBuffer(sSplitter, SizeOf(sSplitter));


  except
    raise EBadHeader.Create(sBadHeader);
  end;


end;

procedure TVFileHeader.LoadFromStream(Stream: TStream);
var sID: TID;
  sp: TSplit;
begin
  Stream.ReadBuffer(sID, SizeOf(sID));
  if sID <> sVFID then
    raise EBadFileID.Create(sBadID);
  try
    Stream.ReadBuffer(sp, SizeOf(sSplitter));

    ID := ReadStr(Stream);
    Stream.ReadBuffer(sp, SizeOf(sSplitter));

    Desc := ReadStr(Stream);
    Stream.ReadBuffer(sp, SizeOf(sSplitter));

    Stream.ReadBuffer(FVerHi, SizeOf(FVerHi));
    Stream.ReadBuffer(FVerLO, SizeOf(FVerLO));

    Stream.ReadBuffer(FHSize, SizeOf(FHSize));
    Stream.ReadBuffer(FFSize, SizeOf(FFSize));

    Stream.ReadBuffer(FCompress, SizeOf(FCompress));

    Stream.ReadBuffer(FCoded, SizeOf(FCoded));

    Stream.ReadBuffer(FReserved, SizeOf(FReserved));
    Stream.ReadBuffer(sp, SizeOf(sp));

    Stream.ReadBuffer(FObjects, SizeOf(FObjects));
    Stream.ReadBuffer(sp, SizeOf(sp));


  except
    raise EBadHeader.Create(sBadHeader);
  end;

end;




//////// TVDATA OBJ


constructor TVDataObj.Create(AOwner:TVFile;AName:string);
begin
 inherited Create;
 FOwner:=AOwner;
 FName:=AName;
 FStream:=TMemoryStream.Create;
 FReadRet:=true;
end;

destructor TVDataObj.Destroy;
begin

 FStream.Free;
 inherited Destroy;
end;


function TVDataObj.GetSize:LongInt;
begin
Result:=FStream.Size;
FSize:=Result;
end;


function TVDataObj.ReadSize(Stream:TStream):LongInt;
var Pos,Buff:LongInt;
    tg:Word;
begin
 Pos:=Stream.Position;
 ReadStr(Stream);
 Stream.ReadBuffer(Tg,SizeOf(Tg));
 Stream.ReadBuffer(Buff,SizeOf(Buff));
 Result:=Buff;
 if FReadRet then Stream.Position:=Pos;
end;

function TVDataObj.ReadName(Stream:TStream):string;
var Pos:LOngInt;
begin
 Pos:=Stream.Position;
 Result:=ReadStr(Stream);
 if FReadRet then Stream.Position:=Pos;
end;


function TVDataObj.ReadTag(Stream:TStream):Word;
var
 Pos:LongInt;
 Buff:Word;
begin
 Pos:=Stream.Position;
 ReadStr(Stream);
 Stream.ReadBuffer(Buff,SizeOf(Buff));
 Result:=Buff;
 if FReadRet then Stream.Position:=Pos;
end;

procedure TVDataObj.WriteHdr(Stream:TStream);
begin
 WriteStr(Stream,FName);
 Stream.WriteBuffer(FTag,SizeOf(FTag));
 Stream.WriteBuffer(FSize,SizeOf(FSize));
end;

procedure TVDataObj.ReadHdr(Stream:TStream);
begin
 FName:=ReadStr(Stream);
 Stream.ReadBuffer(FTag,SizeOf(FTag));
 Stream.ReadBuffer(FSize,SizeOf(FSize));

end;


procedure TVDataObj.SaveToStream(Stream:TStream);
var MS:TMemoryStream;
    cSize:LongInt;
    data:Pointer;
begin
 try

 MS:=TMemoryStream.Create;
 FStream.SaveToStream(MS);
 if FOwner.Header.Coded then
 begin

 end;

 if FOwner.Header.Compress then
 begin
  CompressBuf(MS.Memory,MS.Size,Data,cSize);
  MS.Clear;
  MS.WriteBuffer(Data^,cSize);
  FreeMem(Data);
 end;

 FSize:=MS.Size;
 WriteHdr(Stream);
 MS.SaveToStream(Stream);
 MS.Free;
 except
  raise ESaveData.Create(sSaveData);
 end;

end;


procedure TVDataObj.LoadFromStream(Stream:TStream);
var Data:Pointer;
    cSize:LongInt;
    MS:TMemoryStream;
begin
 try
 ReadHdr(Stream);

 FStream.Clear;
 MS:=TMemoryStream.Create;
 MS.CopyFrom(Stream,FSize);

 if FOwner.Header.Compress then
 begin
  ZLIB.DecompressBuf(MS.Memory,FSize,0,Data,cSize);
  FSize:=cSize;
  MS.Clear;
  MS.WriteBuffer(Data^,FSize);
  FreeMem(Data);
 end;

 MS.Position:=0;
 FStream.CopyFrom(MS,FSize);
 FStream.Position:=0;
 MS.Free;
 except
  raise ELoadData.Create(sLoadData);
 end;


end;


function TVDataObj.GetHdrSize(Stream:TStream):LongInt;
var Pos,int:LongInt;
    tg:Word;
begin
 Pos:=Stream.Position;
 ReadStr(Stream);
 Stream.ReadBuffer(Tg,SizeOf(Tg));
 Stream.ReadBuffer(int,SizeOf(int));
 Result:=Stream.Position-Pos+1;
 if FReadRet then Stream.Position:=Pos;


end;


function TVDataObj.Skip(Stream:TStream):boolean;
var Pos,DSize:LongInt;
begin
 Result:=false;
 Pos:=Stream.Position;
 DSize:=ReadSize(Stream);
 if Pos+DSize+GetHdrSize(Stream)>Stream.Size then Exit;
 Stream.Position:=Pos+DSize+GetHdrSize(Stream)-1;

 Result:=true;
end;


//////////////  TVFILE


constructor TVFile.Create(AName: string);
begin
  Create;
  LoadFromFile(AName);

end;

constructor TVFile.Create(AName: string; Stream: TStream);
begin
Create;
LoadFromStream(Stream,AName);
end;

constructor TVFile.Create(AName: string; Size: LongInt; Data: Pointer);
begin


end;


constructor TVFile.Create;
begin
  inherited Create;
  FStream := TMemoryStream.Create;
  FHeader := TVFileHeader.Create(Self);
  FName := 'deffile';
  FObjects:=TObjectList.Create;

end;


destructor TVFile.Destroy;
begin

  FObjects.Free;
  FHeader.Free;
  FStream.Free;
  inherited;
end;

procedure TVFile.LoadFromFile(AName: string);
var
  Stream: TStream;
begin
  if not FileExists(AName) then Exit;
  Stream := TFileStream.Create(AName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream, AName);
  finally
    Stream.Free;
  end;
end;


procedure TVFile.LoadFromMem(Size: LongInt; Data: Pointer; AName: string);
var Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  Stream.WriteBuffer(Data^, Size);
  try
    LoadFromStream(Stream, AName);
  finally
    Stream.Free;
  end;
end;


procedure TVFile.LoadFromStream(Stream: TStream; AName: string);
begin
  if AName <> '' then FName := AName;
  FStream.LoadFromStream(Stream);

  Header.LoadFRomStream(FStream);



end;

procedure TVFile.SaveToFile(AName: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AName, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TVFile.SaveToStream(Stream: TStream);
var i:LongInt;
    Pos1,Pos2:LongInt;
begin
  Pos1:=Stream.Position;
  Header.FObjects:=Objects.Count;
  Header.FHSize:=GetDataStart;
  Header.SaveToStream(Stream);

  for i:=0 to Objects.Count-1 do
   TVDataObj(Objects[i]).SaveToStream(Stream);

  Pos2:=Stream.Position;
  Header.FFSize:=Pos2-Pos1;
  Stream.Position:=Pos1;
  Header.SaveToStream(Stream);
  Stream.Position:=Pos2;

end;


function TVFile.GetMode:TVFileMode;
begin
 if FStream.Size>0 then
   Result:=fmRead
 else if FObjects.Count>0 then
     Result:=fmWrite
     else Result:=fmUndef;


end;

function TVFile.GetSize:LongInt;
begin
 if Mode<>fmRead then raise EBadMode.Create(sBadMode);
 Result:=Stream.Size;
end;

function TVFile.AddData(AName:string;Size:LongInt;Data:Pointer):TVDataObj;
var Stream:TMemoryStream;
begin
if (Size=0) or (Data=nil) then raise EAddData.Create(sAddData);
try
 Stream:=TMemoryStream.Create;
 Stream.WriteBuffer(Data^,Size);
 Result:=AddData(AName,Stream);
 Stream.Free;
except
 raise EAddData.Create(sAddData);
end;



end;

function TVFile.AddData(AName:string;Stream:TStream):TVDataObj;
var VDO:TVDataObj;
begin
 VDO:=TVDataObj.Create(Self,AName);
 try
  Stream.Position:=0;
  VDO.Stream.CopyFrom(Stream,Stream.Size);
  Result:=AddData(VDO);
 except
  raise EAddData.Create(sAddData);
 end;


end;

function TVFile.AddData(Obj:TVDataObj):TVDataObj;
begin
 if Obj=nil then raise EAddData.Create(sAddData);
 Obj.FOwner:=Self;
 Objects.Add(Obj);
 Result:=Obj;
end;


function TVFile.GetDataStart:LongInt;
var Temp:TMemoryStream;
begin
Temp:=TMemoryStream.Create;
Header.SaveToStream(Temp);
Result:=Temp.Size;
Temp.Free
end;

function TVFile.GetData(AName:string):TVDataObj;
begin
if Mode<>fmRead then raise EBadMode.Create(sBadMode);
Result:=nil;
if SeekData(AName)<0 then Exit;;
Result:=ReadData;
end;

function TVFile.GetData(AName:string;out Obj:TVDataObj):LongInt;
var Seek:LongInt;
begin
if Mode<>fmRead then raise EBadMode.Create(sBadMode);
Seek:=SeekData(ANAme);
if Seek<0 then
begin Result:=-1;Exit;end;
Obj:=ReadData;
Result:=Seek;

end;


function TVFile.SeekData(AName:string):LongInt;
var VDO:TVDataObj;
    DName:string;
    Idx:LOngInt;
    Found:boolean;
begin
Result:=0;
if Mode<>fmRead then raise EBadMode.Create(sBadMode);
VDO:=TVDataObj.Create(Self,AName);

Stream.Position:=GetDataStart;

Idx:=0;
Found:=false;
try
while not Found do
begin
 DName:=VDO.ReadName(Stream);
 if CompStr(AName,DName)then
  begin Result:=Idx;Found:=true;end
 else
  if not VDO.Skip(Stream) then
  begin Result:=-1;Found:=true;end;

 Inc(Idx);
end;
except
raise ESeekData.Create(sSeekData);
end;

 VDO.Free;
end;

function TVFile.SeekData(Pos:LongInt):LongInt;
var VDO:TVDataObj;
    Idx:LOngInt;
    Found:boolean;
begin
Result:=0;
if Mode<>fmRead then raise EBadMode.Create(sBadMode);
VDO:=TVDataObj.Create(Self,'');

Stream.Position:=GetDataStart;

Idx:=0;
Found:=false;
try
while not Found do
begin
 if Pos=Idx then
  begin Result:=Idx;Found:=true;end
 else
  if not VDO.Skip(Stream) then
  begin Result:=-1;Found:=true;end;

 Inc(Idx);
end;
except
raise ESeekData.Create(sSeekData);
end;
 VDO.Free;
end;

function TVFile.ReadData:TVDataObj;
var VDO:TVDataObj;
begin
if Mode<>fmRead then raise EBadMode.Create(sBadMode);
VDO:=TVDataObj.Create(Self,'');
try
VDO.LoadFromStream(Stream);
except
 raise EReadData.Create(sReadData);
end;
Result:=VDO;
end;

end.

