unit WindowCls;

interface

uses Windows, Classes, SysUtils, Timing, Messages,
  BaseObjs, Graphics,DeviceModes;

type

  TWinBorder = (wbSingle, wbSizeable, wbNone, wbToolWin, wbSizeToolWin);
  TBorderIc = (biSysMenu, biMin, biMax, biHelp);
  TBorderIcs = set of TBorderIc;
  TCursor = -32768..32767;
  TWinPosition = (wpDesigned, wpDefault, wpCenter);

  TMsgEvent = procedure(Sender: TObject; var Msg: TMessage) of object;
  TCloseEvent = procedure(Sender: TObject; var CanClose: boolean) of object;
  TCreateEvent = TNotifyEvent;
  TRunEvent=TNotifyEvent;
  TRunProc=procedure;

  TMsgProc=procedure(var Msg: TMessage);

  TWindow = class(TBaseObj)
  private
    FCanvas: TCanvas;
    FLeft: LongInt;
    FTop: LongInt;
    FWidth: LongInt;
    FHeight: LongInt;
    FVisible: Boolean;
    FCaption: string;
    FColor: TColor;
    FIcon: TIcon;
    FBrush: TBrush;
    FBorder: TWinBorder;
    FIcons: TBorderIcs;
    FOnTop: boolean;
    FCursor: TCursor;
    FPosition: TWinPosition;
    FParent: THandle;
    FFullScreen:boolean;
    FPlace:TWindowPlacement;
    FH:LongInt;
    FW:LongInt;
    FBits:LongInt;
    FCanSetMode:boolean;
    FModeSet:boolean;

    FOnMessage: TMsgEvent;
    FOnMsgProc:TMsgProc;
    FOnClose: TCloseEvent;
    FOnDestroy: TCloseEvent;
    FOnCreate:TCreateEvent;
    FOnRun:TRunEvent;
    FOnRunProc:TRunProc;

    FWndClass: TWndClass;
    FWndClassName: string;
    FWndProc: TWndMethod;

    FClosing: boolean;
    FRunning: boolean;

    FActive:boolean;

  protected
    FHandle: THandle;

    procedure SetMetrics(idx: LongInt; Value: LongInt);
    procedure SetVisible(Value: boolean);
    procedure SetCaption(Value: string);
    procedure SetColor(Value: TColor);
    procedure SetIcon(Value: TIcon);
    procedure SetBrush(Value: TBrush);
    procedure SetBorder(Value: TWinBorder);
    procedure SetBorderIcs(Value: TBorderIcs);
    procedure SetOnTop(Value: boolean);
    procedure SetCursor(Value: TCursor);
    procedure SetPos(Value: TWinPosition);
    procedure SetParent(Value: THandle);
    procedure SetClientSize(idx:LongInt;Value:LongInt);

    function GetHandleValid: boolean;
    function GetClientSize(idx:LongInt):LongInt;

    procedure IconChange(Sender: TObject);
    procedure CloseMsg(Msg: TMessage);

  public
    constructor Create(AParent: THandle = 0); reintroduce;
    destructor Destroy; override;

    procedure Init; override;
    procedure Done;override;

    procedure SetBounds(ALeft, ATop, AWidth, AHeight: LongInt);
    procedure SetClientBounds(ALeft, ATop, AWidth, AHeight: LongInt);
    procedure CreateWnd;
    procedure CreateWndClass;
    procedure DefWndProc(var Message: TMessage);virtual;
    procedure Paint;
    procedure SetStyle;
    procedure DestroyWnd;
    procedure Run;
    procedure Stop;

    procedure SetFullScreen(AW,AH,ABits:LongInt);
    procedure SetWindowed;

    property Handle: THandle read FHandle;
    property Canvas: TCanvas read FCanvas;
    property Brush: TBrush read FBrush write SetBrush;
    property HandleValid: boolean read GetHandleValid;
    property Parent: THandle read FParent write SetParent;
    property Running: boolean read FRunning;
    property FullScreen:boolean read FFullScreen;

    property OnMsgProc: TMsgProc read FOnMsgProc write FOnMsgProc;
    property OnRunProc: TRunProc read FOnRunProc write FOnRunProc;

  published

    property Left: LongInt index 1 read FLeft write SetMetrics;
    property Top: LongInt index 2 read FTop write SetMetrics;
    property Width: LongInt index 3 read FWidth write SetMetrics;
    property Height: LongInt index 4 read FHeight write SetMetrics;
    property ClientWidth:LongInt index 1 read GetClientSize write SetClientSize;
    property ClientHeight:LongInt index 2 read GetClientSize write SetClientSize;
    property Visible: boolean read FVisible write SetVisible;
    property Caption: string read FCaption write SetCaption;
    property Color: TColor read FColor write SetColor;
    property Border: TWinBorder read FBorder write SetBorder;
    property BorderIcons: TBorderIcs read FIcons write SetBorderIcs;
    property OnTop: boolean read FOnTop write SetOnTop;
    property Icon: TIcon read FIcon write SetIcon;
    property Cursor: TCursor read FCursor write SetCursor;
    property Position: TWinPosition read FPosition write SetPos;
    property CanSetMode:boolean read FCanSetMode write FCanSetMode;
    property Active:boolean read FActive;

    property OnMessage: TMsgEvent read FOnMessage write FOnMessage;
    property OnClose:TCloseEvent read FOnClose write FOnClose;
    property OnDestroy:TCloseEvent read FOnDestroy write FOnDestroy;
    property OnCreate:TCreateEvent read FOnCreate write FOnCreate;
    property OnRun:TRunEvent read FOnRun write FOnRun;

  end;


const

  crUser = TCursor(1);
  crDefault = TCursor(0);
  crNone = TCursor(-1);
  crArrow = TCursor(-2);
  crCross = TCursor(-3);
  crIBeam = TCursor(-4);
  crSize = TCursor(-22);
  crSizeNESW = TCursor(-6);
  crSizeNS = TCursor(-7);
  crSizeNWSE = TCursor(-8);
  crSizeWE = TCursor(-9);
  crUpArrow = TCursor(-10);
  crHourGlass = TCursor(-11);
  crDrag = TCursor(-12);
  crNoDrop = TCursor(-13);
  crHSplit = TCursor(-14);
  crVSplit = TCursor(-15);
  crMultiDrag = TCursor(-16);
  crSQLWait = TCursor(-17);
  crNo = TCursor(-18);
  crAppStart = TCursor(-19);
  crHelp = TCursor(-20);
  crHandPoint = TCursor(-21);
  crSizeAll = TCursor(-22);

var

  CursorList: array[crSizeAll..crUser] of LongInt;
  ActiveWindow:TWindow=nil;
  CreatedWindows:TList;

function GetWin(AHandle:THandle):TWindow;


implementation

function GetWin(AHandle:THandle):TWindow;
var i:LongInt;
begin
 Result:=nil;
 for i:=0 to CreatedWindows.Count-1 do
  if AHandle=TWindow(CreatedWindows[i]).Handle then
   begin
    Result:=TWindow(CreatedWindows[i]);
    Break;
   end;


end;


constructor TWindow.Create(AParent: THandle);
begin
  inherited Create;
  ActiveWindow:=Self;
  CreatedWindows.Add(Self);
  Name := 'Window';
  Caption := 'Vampyre Application Main Window';
  FWndClassName := 'Vampyre_Window';
  FHandle := 0;
  FWndProc := DefWndProc;
  FVisible := true;
  FLeft := 100;
  FTop := 50;
  FWidth := 640;
  FHeight := 480;
  FIcon := TIcon.Create;
  FIcon.Width := GetSystemMetrics(SM_CXSMICON);
  FIcon.Height := GetSystemMetrics(SM_CYSMICON);
  FIcon.OnChange := IconChange;
  FCanvas := TCanvas.Create;
  FBrush := TBrush.Create;
  Color := RGB(48, 48, 48);
  FBorder := wbSingle;
  FIcons := [biSysMenu];
  FCursor := crArrow;
  FPosition := wpDesigned;
  FParent := AParent;
  FClosing := false;
  FFullScreen:=false;
  FCanSetMode:=false;
  FModeSet:=false;
end;

destructor TWindow.Destroy;
begin
  CreatedWindows.Remove(Self);
  DestroyWnd;
  FIcon.Free;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TWindow.Init;
begin
  inherited Init;
  CreateWnd;


end;

procedure TWindow.Stop;
begin
 FClosing:=true;

 inherited;
end;

procedure TWindow.Done;
begin
 if HandleValid then
 SendMessage(FHandle,WM_CLOSE,0,0);
 inherited;
end;


procedure TWindow.SetParent(Value: THandle);
begin
  FParent := Value;
  if not HandleValid then Exit;
  Windows.SetParent(FHandle, FParent);

end;

procedure TWindow.SetMetrics(idx: LongInt; Value: LongInt);
begin
  case idx of
    1: SetBounds(Value, FTop, FWidth, FHeight);
    2: SetBounds(FLeft, Value, FWidth, FHeight);
    3: SetBounds(FLeft, FTop, Value, FHeight);
    4: SetBounds(FLeft, FTop, FWidth, Value);
  end;
end;

procedure TWindow.SetBounds(ALeft, ATop, AWidth, AHeight: LongInt);
begin
  FLeft := ALeft;
  FTop := ATop;
  FWidth := AWidth;
  FHeight := AHeight;

  if not HandleValid then Exit;
  SetWindowPos(FHandle, 0, FLeft, FTop, FWidth, FHeight, SWP_NOZORDER or
    SWP_NOACTIVATE);

end;

procedure TWindow.SetClientBounds(ALeft, ATop, AWidth, AHeight: LongInt);
var dx,dy:LongInt;
begin
 dx:=0;
 dy:=0;

 if Inited then
 begin
 dx:=FWidth-ClientWidth;
 dy:=FHeight-ClientHeight;
 end;

 FLeft := ALeft;
 FTop := ATop;
 FWidth := AWidth+dx;
 FHeight := AHeight+dy;

 if not HandleValid then Exit;
  SetWindowPos(FHandle, 0, FLeft, FTop, FWidth, FHeight, SWP_NOZORDER or
    SWP_NOACTIVATE);


end;

procedure TWindow.SetPos(Value: TWinPosition);
var x, y: LongInt;
begin
  FPosition := Value;
  x := FLeft;
  y := FTop;
  case FPosition of
    wpDefault:
      begin
        x := LongInt(CW_USEDEFAULT); y := LongInt(CW_USEDEFAULT);
      end;
    wpCenter:
      begin
        x := (GetDevicecaps(GetDC(0), HORZRES) - FWidth) div 2;
        y := (GetDevicecaps(GetDC(0), VERTRES) - FHeight) div 2;
      end;
  end;

  if not HandleValid then Exit;
  SetWindowPos(FHandle, 0, x, y, FWidth, FHeight, SWP_NOZORDER or
    SWP_NOACTIVATE);


end;

procedure TWindow.SetVisible(Value: boolean);
begin
  FVisible := Value;
  if not HandleValid then Exit;
  if FVisible then
    ShowWindow(FHandle, SW_SHOWNORMAL)
  else
    ShowWindow(FHandle, SW_HIDE)

end;

procedure TWindow.SetColor(Value: TColor);
begin
  FColor := Value;
  FBrush.Color := FColor;

  if not HandleValid then Exit;
  SetClassLong(FHandle, GCL_HBRBACKGROUND, FBrush.Handle);
  SendMessage(FHandle, WM_ERASEBKGND, FCanvas.Handle, 0);

end;

procedure TWindow.SetCursor(Value: TCursor);
begin
  if (Value < crSizeAll) or (Value > crUser) then Value := 0;
  FCursor := Value;
  if not HandleValid then Exit;
   SetClassLong(FHandle,gcl_hcursor,CursorList[FCursor]);
end;

procedure TWindow.SetIcon(Value: TIcon);
begin
  FIcon.Assign(Value);
  IconChange(Self);

end;

procedure TWindow.SetBrush(Value: TBrush);
begin
  FBrush.Assign(Value);
end;

function TWindow.GetHandleValid: boolean;
begin
  Result := IsWindow(FHandle);
end;

procedure TWindow.IconChange(Sender: TObject);
begin
  SendMessage(FHandle, WM_SETICON, 1, FIcon.Handle);
end;

procedure TWindow.SetCaption(Value: string);
begin
  FCaption := Value;
  SetWindowText(FHandle, PChar(FCaption));

end;

procedure TWindow.SetBorder(Value: TWinBorder);
begin
  FBorder := Value;
  SetStyle;
end;

procedure TWindow.SetOnTop(Value: boolean);
begin
  FOnTop := Value;
  SetStyle;
end;

procedure TWindow.SetBorderIcs(Value: TBorderIcs);
begin
  FIcons := Value;
  SetStyle;
end;

procedure TWindow.SetClientSize(idx:LongInt;Value:LongInt);
var dx,dy:LongInt;
begin
 dx:=FWidth-ClientWidth;
 dy:=FHeight-ClientHeight;

 case idx of
 1:Width:=FWidth+dx;
 2:Height:=FHeight+dy;
 end;

end;

function TWindow.GetClientSize(idx:LongInt):LongInt;
var R:TRect;
begin
 Result:=0;
 if not HandleValid then Exit;
 GetClientRect(FHandle,R);
 case idx of
 1:Result:=R.Right;
 2:Result:=R.Bottom;
 end;
end;



procedure TWindow.SetStyle;
var
  Style: Dword;
  ExStyle: DWord;
begin
  Style := 0;
  ExStyle := WS_EX_APPWINDOW;

  case FBorder of
    wbSingle, wbToolWin: Style := WS_BORDER or WS_CAPTION;
    wbNone:
      begin
        Style := WS_POPUP; FIcons := [];
      end;
    wbSizeAble, wbSizeToolWin: Style := WS_CAPTION or WS_THICKFRAME;
  end;

  if FBorder in [wbToolWin, wbSizeToolWin] then
  begin
    ExStyle := ExStyle or WS_EX_TOOLWINDOW;
    FIcons := [biSysMenu];
  end;

  if biSysMenu in FIcons then Style := Style or WS_SYSMENU;
  if biMin in FIcons then Style := Style or WS_MINIMIZEBOX;
  if biMax in FIcons then Style := Style or WS_MAXIMIZEBOX;
  if biHelp in FIcons then Style := Style or WS_EX_CONTEXTHELP;

  if not HandleValid then Exit;

  if FOnTop then
    SetWindowPos(FHandle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOACTIVATE or SWP_NOSIZE
      or SWP_NOMOVE);

  if FFullScreen then Exit;

  SetWindowLong(FHandle, GWL_STYLE, Style);
  SetWindowLong(FHandle, GWL_EXSTYLE, ExStyle);
  SetVisible(Visible);

end;

procedure TWindow.CreateWndClass;
begin

  DestroyWnd;

  FWndClass.style := CS_VREDRAW or CS_HREDRAW or CS_DBLCLKS;
  FWndClass.lpfnWndProc := @DefWindowProc;
  FWndClass.hCursor := 0;
  FWndClass.hbrBackground := FBrush.Handle;
  FWndClass.hInstance := HInstance;
  FWndClass.lpszMenuName := nil;
  FWndClass.lpszClassName := PChar(FWndClassName);
  FWndClass.hIcon := FIcon.Handle;

  Windows.RegisterClass(FWndClass);


end;

procedure TWindow.CreateWnd;
var
  ExStyle: DWord;
  Style: DWord;
begin
  CreateWndClass;

  ExStyle := WS_EX_APPWINDOW;
  Style := WS_OVERLAPPED;


  FHandle := CreateWindowEx(ExStyle, PChar(FWndClassName), PChar(FCaption),
    Style, FLeft, FTop, FWidth, FHeight, FParent, 0,
    FWndClass.hInstance, nil);

  if not HandleValid then RaiseLastOSError;

  if Assigned(FWndProc) then
    SetWindowLong(FHandle, GWL_WNDPROC, Longint(MakeObjectInstance(FWndProc)));

  FCanvas.Handle := GetDC(FHandle);

  SetStyle;
  SetVisible(Visible);
  SetCursor(Cursor);
  SetPos(Position);
  UpdateWindow(FHandle);
  if FFullScreen then SetFullScreen(FW,FH,FBits);

  FClosing := false;

end;

procedure TWindow.DestroyWnd;
begin
  if HandleValid then
   Windows.DestroyWindow(FHandle);
end;

procedure TWindow.Paint;
begin
  if not HandleValid then Exit;
  if not FCanvas.HandleAllocated then Exit;

end;

procedure TWindow.DefWndProc(var Message: TMessage);
begin
  with Message do
  begin
    Result := 1;

    case Msg of
      WM_ACTIVATEAPP:
      FActive:=boolean(wParam);


      WM_SETCURSOR:
        begin
         if Cursor=crNone then
         Windows.SetCursor(0);
         Result := DefWindowProc(FHandle, Msg, WParam, LParam);
        end;
      WM_CLOSE: CloseMsg(Message);
      WM_DESTROY: CloseMsg(Message);

    else
      begin
       Result := DefWindowProc(FHandle, Msg, WParam, LParam);

      end;
    end;

    if Assigned(FOnMessage) then FOnMessage(Self, Message);
    if Assigned(FOnMsgProc) then FOnMsgProc( Message);
  end;
end;



procedure TWindow.Run;
var Msg: TMsg;
begin
  FRunning := true;
  while not FClosing do
  begin
    if not HandleValid then Break;

    if PeekMessage(Msg, 0, 0, 0, PM_REMOVE) then
    begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;

    if Assigned(FOnRun)and Active then
     FOnRun(Self);
    if Assigned(FOnRunProc)and Active then
     FOnRunProc;

  end;
  FRunning := false;
end;

procedure TWindow.CloseMsg(Msg: TMessage);
var CanClose: boolean;
begin
  CanClose := true;
  case Msg.Msg of
    WM_CLOSE:
      begin
        if Assigned(FOnClose) then
          FOnClose(Self, CanClose);
        if not CanClose then Exit;
        FClosing := True;
        DestroyWnd;
      end;


    WM_DESTROY:
      begin
        if Assigned(FOnDestroy) then
          FOnDestroy(Self, CanClose);
        if CanClose then
          PostQuitMessage(0);

      end;

  end;


end;

procedure TWindow.SetFullScreen(AW,AH,ABits:LongInt);
begin
 FPlace.Length:=SizeOf(FPlace);
 GetWindowPlacement(FHandle,@FPlace);
 FFullScreen:=true;
 FH:=AH;
 FW:=AW;
 FBits:=ABits;

 if not HandleValid then Exit;

 if FCanSetMode then
 begin
  SetMode(FindMode(FW,FH,FBits));
  FModeSet:=true;
 end;

 SetWindowLong(FHandle,GWL_STYLE,LongInt(WS_POPUP));
 SetWindowLong(FHandle,GWL_EXSTYLE,0);
 SetWindowPos(Fhandle,HWND_TOP,0,0,FW,FH,0);
 SetVisible(Visible);


end;

procedure TWindow.SetWindowed;
begin
 if not FFullScreen then Exit;

 FFullScreen:=false;

 if not HandleValid then Exit;

 if FModeSet then
 begin
 ResetMode;
 FModeSet:=false;
 end;
 SetStyle;
 SetBounds(FLeft,FTop,FWidth,FHeight);
 SetWindowPlacement(FHandle,@FPlace);

end;

procedure InitCursors;
const
  IDC_NODROP = PChar(32767);
  IDC_DRAG = PChar(32766);
  IDC_HSPLIT = PChar(32765);
  IDC_VSPLIT = PChar(32764);
  IDC_MULTIDRAG = PChar(32763);
  IDC_SQLWAIT = PChar(32762);
  IDC_HANDPT = PChar(32761);

  CursorMap: array[crSizeAll..crArrow] of PChar = (
    IDC_SIZEALL, IDC_HANDPT, IDC_HELP, IDC_APPSTARTING, IDC_NO, IDC_SQLWAIT,
    IDC_MULTIDRAG, IDC_VSPLIT, IDC_HSPLIT, IDC_NODROP, IDC_DRAG, IDC_WAIT,
    IDC_UPARROW, IDC_SIZEWE, IDC_SIZENWSE, IDC_SIZENS, IDC_SIZENESW, IDC_SIZE,
    IDC_IBEAM, IDC_CROSS, IDC_ARROW);

var
  I: LongInt;

begin
  CursorList[1] := 0;
  CursorList[0] := LoadCursor(0, IDC_ARROW);
  CursorList[-1] := 0;

  for I := Low(CursorMap) to High(CursorMap) do
  begin
    CursorList[I] := LoadCursor(0, CursorMap[I]);
  end;

end;




initialization

  InitCursors;
  CreatedWindows:=TList.Create;

finalization


  CreatedWindows.Free;

end.

