unit Archivers;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,ContNRS,BaseMgrs,
  ArcInfo,FileRec,FileDirs;

type

    TArchiver=class(TBaseObj)
     private
      FDescription:string;
      FArchives:TObjectList;
      FArcExts:TStringList;

     protected

       function GetArcInfo(Index:LongInt):TArcInfo;
       function GetArcCount:LongInt;

       procedure SearchArchives;virtual;
       procedure InitArcInfo(AName:string;var AI:TArcInfo);virtual;


     public
       constructor Create; override;
       destructor Destroy; override;

       function GetFile(AName:string):TFileRec;virtual;
       function GetFullPath(AName:string):string;virtual;
       function AddArc(Arc:TArcInfo):LongInt;virtual;

       function IsFileInArc(AName:string):boolean;virtual;

       property Archives[Index:LongInt]:TArcInfo read GetArcInfo;
       property ArcCount:LongInt read GetArcCount;
       property ArcExts:TStringList read FArcExts;

     published
      property Description:string read FDescription write FDescription;
     end;


TArchiverClass=class of TArchiver;


TArchiverInfo=class
            Name:string;
            Desc:string;
            Cls:TArchiverClass
           end;




procedure RegisterArchiver(Info:TArchiverInfo);
function CreateInfo(Name,Desc:string;Cls:TArchiverClass):TArchiverInfo;

var RegArchivers:TObjectList;


implementation


function IsSame(AI1,AI2:TArchiverInfo):boolean;
begin

 Result:=Iff(CompStr(AI1.Name,AI2.Name),True,False)and
 Iff(CompStr(AI1.Desc,AI2.Desc),True,False)and
 Iff(AI1.Cls=AI2.Cls,True,False);


end;

procedure RegisterArchiver(Info:TArchiverInfo);
var I:LongInt;
    Already:boolean;
begin
 if RegArchivers=nil then
  RegArchivers:=TObjectList.Create;

 Already:=false;
 for I:=0 to RegArchivers.Count-1 do
  if IsSame(TArchiverInfo(RegArchivers[I]),Info) then Already:=true;

 if not Already then
 RegArchivers.Add(Info);

end;

function CreateInfo(Name,Desc:string;Cls:TArchiverClass):TArchiverInfo;
var Inf:TArchiverInfo;
 begin
  Inf:=TArchiverInfo.Create;
  Inf.Name:=Name;
  Inf.Desc:=Desc;
  Inf.Cls:=Cls;
  Result:=Inf;
 end;



constructor TArchiver.Create;
begin
 inherited Create;
 FArcExts:=TSTringList.Create;
 FArchives:=TObjectList.Create;

end;

destructor TArchiver.Destroy;
begin

 FArcExts.Free;
 FArchives.Free;
 inherited Destroy;
end;


function TArchiver.GetArcInfo(Index:LongInt):TArcInfo;
begin
 Result:=nil;
 if Index in [0..ArcCount-1]then
  Result:=TArcInfo(FArchives[Index]);

end;

procedure TArchiver.SearchArchives;
 var I:LongInt;
     SR:TSearchRec;
     AI:TArcInfo;
begin

 for I:=0 to ArcDirs.Count-1 do
 begin
  if FindFirst(ArcDirs[I]+'*.?*',faAnyFile,SR) = 0 then
   begin
      repeat

       if StrInList(ExtractFileExt(SR.Name),ArcExts)then //found archive - now add it
        begin
         AI:=TArcInfo.Create;
         InitArcInfo(ArcDirs[I]+SR.Name,AI);
         AddArc(AI);
        end;

       until FindNext(sr) <> 0;
      FindClose(sr);
 end;
 end;

end;

function TArchiver.GetArcCount:LongInt;
begin
 Result:=FArchives.Count;
end;

function TArchiver.GetFile(AName:string):TFileRec;
begin
 Result:=nil;
end;

function TArchiver.GetFullPath(AName:string):string;
begin
 Result:='';
end;

function TArchiver.AddArc(Arc:TArcInfo):LongInt;
begin
 Result:=FArchives.Add(Arc);
end;


function TArchiver.IsFileInArc(AName:string):boolean;
begin
Result:=false;
end;

procedure TArchiver.InitArcInfo(AName:string;var AI:TArcInfo);
begin
end;


initialization



finalization
 RegArchivers.Free;


end.

