unit FileDirs;

interface


uses Windows,SysUtils,Classes,WinUtils;

type

   TFileType=(ftGraphic,ftSound,ftMusic,ftVideo,ftArchive,ftFont,ftUnknown);


const
 // game dirs relative to root dir

  DirGraphic='Data\Graphic\';
  DirMusic='Data\Music\';
  DirSound='Data\Sound\';
  DirVideo='Data\Video\';
  DirData='Data\';
  DirBin='Bin\';
  DirTexture=DirGraphic+'Textures\';
  DirFont=DirData+'Fonts\';

var
 // lists for storing supported file extensions

  ExtsGraphic:TStringList;
  ExtsMusic:TStringList;
  ExtsSound:TStringList;
  ExtsVideo:TStringList;
  ExtsArc:TStringList;
  ExtsFonts:TStringList;


  ArcDirs:TStringList;

// functions for retrieving full dir paths

function RootDir:string;
function DataDir:string;
function BinDir:string;
function GraphicDir:string;
function MusicDir:string;
function SoundDir:string;
function VideoDir:string;
function TextureDir:string;
function FontDir:string;


function GetFileType(AName:string):TFileType;


implementation


function RootDir:string;
begin
 Result:=DirUp(ExeDir)+'\';
end;

function DataDir:string;
begin
Result:=RootDir+DirData;
end;

function BinDir:string;
begin
Result:=RootDir+DirBin;
end;

function GraphicDir:string;
begin
Result:=RootDir+DirGraphic;
end;

function MusicDir:string;
begin
Result:=RootDir+DirMusic;
end;

function SoundDir:string;
begin
Result:=RootDir+DirSound;
end;

function VideoDir:string;
begin
Result:=RootDir+DirVideo;
end;

function TextureDir:string;
begin
Result:=RootDir+DirTexture;
end;

function FontDir:string;
begin
Result:=RootDir+DirFont;
end;


function GetFileType(AName:string):TFileType;
var I:LongInt;
    Found:boolean;
    Ext:string;
begin
 Result:=ftUnknown;
 Found:=false;
 Ext:=ExtractFileExt(AName);

 for I:=0 to ExtsGraphic.Count-1 do
  if CompStr(Ext,ExtsGraphic[I])then Found:=true;

 if Found then
 begin
  Result:=ftGraphic;
  Exit;
 end;

 for I:=0 to ExtsFonts.Count-1 do
  if CompStr(Ext,ExtsFonts[I])then Found:=true;

 if Found then
 begin
  Result:=ftFont;
  Exit;
 end;


 for I:=0 to ExtsVideo.Count-1 do
  if CompStr(Ext,ExtsVideo[I])then Found:=true;

 if Found then
 begin
  Result:=ftVideo;
  Exit;
 end;


 for I:=0 to ExtsMusic.Count-1 do
  if CompStr(Ext,ExtsMusic[I])then Found:=true;

 if Found then
 begin
  Result:=ftMusic;
  Exit;
 end;


 for I:=0 to ExtsSound.Count-1 do
  if CompStr(Ext,ExtsSound[I])then Found:=true;

 if Found then
 begin
  Result:=ftSound;
  Exit;
 end;



end;

initialization

// adding supported file extensions
// some are or will be supported by additional bpls

ExtsGraphic:=TStringList.Create;
ExtsGraphic.Add('.bmp');    // Windows bitmap
ExtsGraphic.Add('.dib');    // Device independent bitmap
ExtsGraphic.Add('.ico');    // Icon
ExtsGraphic.Add('.jpg');    // Joint Photographic Experts Group
ExtsGraphic.Add('.jpeg');
ExtsGraphic.Add('.jfif');
ExtsGraphic.Add('.gif');    // Graphic InterChange Format
ExtsGraphic.Add('.png');    // Portable Network Graphic
ExtsGraphic.Add('.tga');    // Truevision targa
ExtsGraphic.Add('.pcx');    // Zsoft paintbrush

ExtsFonts:=TStringList.Create;
ExtsFonts.Add('.ttf');    // raw True Type file
ExtsFonts.Add('.fon');    // font resource file
ExtsFonts.Add('.fnt');    // raw bitmap font file
ExtsFonts.Add('.fot');    // True Type resource file
ExtsFonts.Add('.vff');    // Vampyre Font File



ExtsMusic:=TStringList.Create;
ExtsMusic.Add('.mid');
ExtsMusic.Add('.mp3');
ExtsMusic.Add('.wav');
ExtsMusic.Add('.mod');
ExtsMusic.Add('.s3m');
ExtsMusic.Add('.it');
ExtsMusic.Add('.xm');
ExtsMusic.Add('.mo3');
ExtsMusic.Add('.wma');
ExtsMusic.Add('.ogg');


ExtsSound:=TStringList.Create;
ExtsSound.Add('.wav');
ExtsSound.Add('.mp3');
ExtsSound.Add('.ogg');

ExtsVideo:=TStringList.Create;
ExtsVideo.Add('.avi');
ExtsVideo.Add('.mpg');
ExtsVideo.Add('.wmv');
ExtsVideo.Add('.asf');

ExtsArc:=TStringList.Create;
ExtsArc.Add('.zip');  // zip archive
ExtsArc.Add('.vzf');  // Vampyre Zipped File
ExtsArc.Add('.jar');  // java file  -  zip format
ExtsArc.Add('.pk3');  // id soft file - zip format
ExtsArc.Add('.vdl');  // Vampyre data library - vamp archive
ExtsArc.Add('.rar');  // rar archive
ExtsArc.Add('.vrf');  // Vampyre rared file


ArcDirs:=TStringList.Create;
ArcDirs.Add(DataDir);
ArcDirs.Add(ExeDir);
ArcDirs.Add(ExeDir+'Data\');


finalization

ExtsGraphic.Free;
ExtsMusic.Free;
ExtsSound.Free;
ExtsVideo.Free;
ExtsArc.Free;
ExtsFonts.Free;

ArcDirs.Free;

end.
