unit RawArc;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Archivers,
  FileRec, FileDirs;

type

  TRawArc = class(TArchiver)
  private
    FFileList:TStringList;

    procedure BuildFileLists;

  protected

  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Init; override;
    procedure Done; override;

    function GetFile(AName: string): TFileRec; override;
    function GetFullPath(AName: string): string; override;
    function IsFileInArc(AName: string): boolean; override;

  published


  end;

const

  ArcName = 'RawFiles';
  ArcDesc = 'Raw Files IO Manager';


implementation

constructor TRawArc.Create;
begin
  inherited Create;
  Name := ArcName;
  Description := ArcDesc;
  Order := 10;
  FFileList:=TStringList.Create;

end;

destructor TRawArc.Destroy;
begin
  Done;

  FFileList.Free;
  inherited Destroy;
end;

procedure TRawArc.Init;
begin
  BuildFileLists;

  inherited Init;
end;

procedure TRawArc.Done;
begin
  inherited Done;
  FFileList.Clear;
end;

procedure TRawArc.BuildFileLists;
var I:LongInt;
    Sl:TStringList;
begin
Sl:=TStringList.Create;
for I:=0 to ArcDirs.Count-1 do
 begin
 Sl.Clear;
 WinUtils.AdvBuildFileList(ArcDirs[I]+'*',faAnyFile,Sl,[flFullNames, flRecursive]);
 FFileList.AddStrings(Sl);
 end;
Sl.Free;


end;

function TRawArc.GetFile(AName: string): TFileRec;
var FR: TFileRec;
    Path:string;

  procedure CreateFrom(AName: string);
  begin
    FR := TFileRec.Create(AName);
    FR.Stream.LoadFromFile(AName);

  end;

begin
  Result := nil;

  Path:=GetFullPath(AName);
  if FileExists(Path) then
  begin
  FR := TFileRec.Create(Path);
  FR.Stream.LoadFromFile(Path);
  Result:=FR;
  end;

end;

function TRawArc.GetFullPath(AName: string): string;
var I:LongInt;
begin

  for I:=0 to FFileList.Count -1 do
   if CompStr(ExtractFileName(AName),ExtractFileName(FFileList[I]))then
    begin
     Result:=FFileList[I];
     Exit;
    end;

  if FileExists(AName) then
  begin
    Result := AName;
    Exit;
  end;

  if FileExists(ExeDir + AName) then
  begin
    Result := ExeDir + AName;
    Exit;
  end;

  if FileExists(ExeDir + DirData + AName) then
  begin
    Result := ExeDir + DirData + AName;
    Exit;
  end;

  if FileExists(RootDir + AName) then
  begin
    Result := RootDir + AName;
    Exit;
  end;

  if FileExists(DataDir + AName) then
  begin
    Result := DataDir + AName;
    Exit;
  end;

  case GetFileType(AName) of
    ftGraphic:
      if FileExists(GraphicDir + AName) then
      begin
        Result := GraphicDir + AName;
        Exit;
      end;
    ftFont:
      if FileExists(FontDir + AName) then
      begin
        Result := FontDir + AName;
        Exit;
      end;
    ftMusic, ftSound:        // waves,mp3s,oggs  are both in sound and music
      begin
        if FileExists(MusicDir + AName) then
          Result := MusicDir + AName
        else
          if FileExists(SoundDir + AName) then
            Result := SoundDir + AName;
        Exit;
      end;
    ftVideo:
      if FileExists(VideoDir + AName) then
      begin
        Result := VideoDir + AName;
        Exit;
      end;



  end;


end;


function TRawArc.IsFileInArc(AName: string): boolean;
begin
  if not FileExists(GetFullPath(AName))then
    Result := false
  else
  begin
    Result := true;
  end;

end;

initialization

  Archivers.RegisterArchiver(CreateInfo(ArcName, ArcDesc, TRawArc));


finalization


end.

