unit ZipArc;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,Archivers,
     Zip,UnzDll,ZipDll,FileRec,FileDirs,ArcInfo;


type

   TZipArc=class(TArchiver)
   private
     FZM:TZipMaster;

   protected

     procedure SearchArchives;override;
     procedure InitArcInfo(AName:string;var AI:TArcInfo);override;

   public
     constructor Create; override;
     destructor Destroy; override;

     procedure Init;override;
     procedure Done;override;

     function GetFile(AName:string):TFileRec;override;
     function AddArc(Arc:TArcInfo):LongInt;override;
     function IsFileInArc(AName:string):boolean;override;


   published


   end;


const

  ArcName='ZipArchiver';
  ArcDesc='Zip Archives IO Manager';


implementation

constructor TZipArc.Create;
begin
 inherited Create;
 Name:=ArcName;
 Description:=ArcDesc;
 Order:=5;

 ArcExts.Add('.zip');
 ArcExts.Add('.vzf');
 ArcExts.Add('.jar');
 ArcExts.Add('.pk3');

end;

destructor TZipArc.Destroy;
begin
 Done;

 inherited Destroy;
end;

procedure TZipArc.Init;
begin
 Load_Zip_Dll;
 Load_Unz_Dll;
 FZM:=TZipMaster.Create;
 SearchArchives;

 inherited Init;
end;

procedure TZipArc.Done;
begin
 inherited Done;

 FZM.Free;
 UnLoad_Zip_Dll;
 UnLoad_Unz_Dll;
end;

procedure TZipArc.SearchArchives;
begin
 inherited;
end;

function TZipArc.GetFile(AName:string):TFileRec;
var I,J:LongInt;
    ArcN,FN:string;
    FR:TFileRec;
begin
 Result:=nil;
 ArcN:='';

 for I:=0 to ArcCount-1 do
  for J:=0 to Archives[I].Files.Count-1 do
   if CompStr(ExtractFilename(AName),ExtractFileName(Archives[I].Files[J]))
   then
   begin
    ArcN:=Archives[I].Name;
    FN:=Archives[I].Files[J];
    Break;
   end;

 if ArcN<>'' then
  begin

   if UnzDllHandle=0 then
    begin
     raise Exception.Create('Error in TZipArc.GetFile - UnzDll.dll not loaded!');
     Exit;
    end;
   FZM.ZipFileName:=ArcN;
   FZM.ExtractFileToStream(FN);
   FR:=TFileRec.Create(AName);
   FR.Stream.LoadFromStream(FZM.ZipStream);
   Result:=FR;

  end;

end;

function TZipArc.AddArc(Arc:TArcInfo):LongInt;
begin
 Result:=inherited AddArc(Arc);
end;

procedure TZipArc.InitArcInfo(AName:string;var AI:TArcInfo);
var I:LongInt;
    List:TStringList;
begin
 List:=TStringList.Create;
 FZM.ZipFileName:=AName;
 for I:=0 to FZM.ZipContents.Count-1 do
  List.Add(string(ZipDirEntry(FZM.ZipContents[I]^).FileName));

 AI.Init(AName,Name,FZM.ZipFileSize,List);

 List.Free;
end;

function TZipArc.IsFileInArc(AName:string):boolean;
var I,J:LongInt;
begin
Result:=false;
for I:=0 to ArcCount-1 do
 for J:=0 to Archives[I].Files.Count-1 do
   if CompStr(ExtractFilename(AName),ExtractFileName(Archives[I].Files[J]))
   then
   begin
   Result:=true;
   Exit;
  end;
end;



initialization

Archivers.RegisterArchiver(CreateInfo(ArcName, ArcDesc, TZipArc));



finalization


end.
