unit FontEngine;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Objs3D, Types3D,
  Utils3D, Colors, Textures, BaseMgrs, DrawingEngines,
  Engines,Images,Fonts,FontTypes;

type

  TFontEngine = class(TDrawingEngine)
  private
   FAFont:TFont;
   FCharSet:TFontChars;

  protected

    function GetFontI(Idx:LongInt):TFont;
    function GetFontCount:LongInt;
    function GetActFont:TFont;

    procedure SetActFont(Value:TFont);
    procedure SetCharSet(Value:TFontChars);

  public
    constructor Create(AOwner: TEngine); override;
    destructor Destroy; override;

    procedure Init;override;
    procedure Done;override;

    class function GetInstance: TFontEngine;

    function GetFont(AFile:string):TFont;

    property Font[Index:LongInt]:TFont read GetFontI;
    property FontCount:LongInt read GetFontCount;
    property ActiveFont:TFont read GetActFont write SetActFont;
    property ActiveCharSet:TFontChars read FCharSet write SetCharSet;

  published

  end;



implementation

uses GGraphic;

var FE:TFontEngine;

constructor TFontEngine.Create(AOwner: TEngine);
begin
  inherited Create(AOwner);
  FCharSet:=DefaultCharSet;

end;

destructor TFontEngine.Destroy;
begin
  Done;
  FE:=nil;
  inherited Destroy;
end;

procedure TFontEngine.Init;
var I:LongInt;
begin
 for I:=0 to FontCount-1 do
  Font[I].Init;

 inherited Init;
end;

procedure TFontEngine.Done;
var I:LongInt;
begin
 inherited Done;

 for I:=0 to FontCount-1 do
  Font[I].Init;

end;

class function TFontEngine.GetInstance: TFontEngine;
begin
  if FE = nil then
  begin
    FE := TFontEngine.Create(Graphic);
    if Graphic.Inited then
      FE.Init;

  end;
  Result := FE;

end;


function TFontEngine.GetFontCount:LongInt;
begin
 Result:=Objects.Count;
end;

function TFontEngine.GetFontI(Idx:LongInt):TFont;
begin
  Result := TFont(Objects[idx]);
end;

function TFontEngine.GetActFont:TFont;
begin
 Result:=FAFont;
 if Result=nil then
  begin
   if Font[0]=nil then Exit;
   ActiveFont:=Font[0];
   Result:=GetActFont;
  end;
end;

procedure TFontEngine.SetActFont(Value:TFont);
begin
 FAFont:=Value;
 if Inited and (not FAFont.Inited)then
  FAFont.Init;
end;

procedure TFontEngine.SetCharSet(Value:TFontChars);
begin
 if FCharSet=Value then Exit;
 FCharSet:=Value;
 ACharSet:=Value;
 Init;
end;

function TFontEngine.GetFont(AFile:string):TFont;
var F:TFont;
begin
 F:=TFont(Objects.Find(ChangeFileExt(ExtractFileName(AFile),'')));
 if F=nil then
 begin
  F:=TFont.Create;
  F.Load(AFile);
  if Inited then F.Init;
  Objects.Add(F);
 end;
Result:=F;

end;


end.

