unit FontFiles;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Objs3D, Types3D,
  Utils3D, Colors, Textures, BaseMgrs, Fonts, Chars, FileVFormat, FontTypes,
  Images,ImgUtils;


const

  FntEXT = 'VFF';
  FntID = 'VFF@';
  FntINFO = '<VAMPYRE FONT FILE>';

  VerHi = 0;
  VerLo = 4;

  sProps = 'Properties';
  sWidths = 'Char Widths';
  sImages = 'Char Images';

type

  EFontIO = class(Exception)
  end;

procedure SaveFont(F: TFont; AFile: string; Compress: boolean); overload;
procedure SaveFont(F: TFont; AStream: TStream; Compress: boolean); overload;
procedure ExportFont(F: TFont; AFile: string); overload;
procedure ExportFont(F: TFont; AStream: TStream); overload;

procedure LoadFont(F: TFont; AFile: string); overload;
procedure LoadFont(F: TFont; AStream: TStream); overload;
procedure ImportFont(F: TFont; AFile: string); overload;
procedure ImportFont(F: TFont; AStream: TStream); overload;

procedure ImportBmpFont(F:TFont;AFile:string;CharSize:Windows.TPoint;SpaceChars:Windows.TPoint;CharSet:TFontChars;CKey:TARGB=0);overload;
procedure ImportBmpFont(F:TFont;AStream:TStream;CharSize:Windows.TPoint;SpaceChars:Windows.TPoint;CharSet:TFontChars;CKey:TARGB=0);overload;

// used when importing different image formats from stream
//
var StreamFmt: TImgFormat;

implementation

procedure SaveFont(F: TFont; AFile: string; Compress: boolean);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AFile, fmCreate);
  try
    SaveFont(F, Stream, Compress);
  finally
    Stream.Free;
  end;
end;

procedure SaveFont(F: TFont; AStream: TStream; Compress: boolean);
var TF: TVFile;
  MS: TMemoryStream;
  Old: TFontStyles;
  L, I: LongInt;
begin

  Old := F.Style;

  if not F.Inited then Exit;
  if F.Chars.Char[0] = nil then Exit;
  if F.Chars.Char[0].Texture.Image = nil then
  begin
    F.Style := F.Style - [fsFreeImages];
    F.BuildChars;
  end;

  if F.Chars.Char[0].Texture.Image = nil then
  begin
    raise
      EFontIO.Create('EFontIOError - Error in FontFiles.SaveFont() - Chars can not be created.');
    Exit;
  end;

  TF := TVFile.Create;
  TF.Header.ID := FntID;
  TF.Header.Desc := FntINFO;
  TF.Header.VerHI := VerHI;
  TF.Header.VerLO := VerLO;
  TF.Header.Compress := Compress;

  MS := TMemoryStream.Create;

  try
    MS.Clear;
    WriteStr(MS, F.Name);
    L := F.Chars.Char[0].Texture.Image.Width;
    MS.WriteBuffer(L, SizeOf(L));
    WriteStr(MS, F.UsedCharSet);
    TF.AddData(sProps, MS);


    MS.Clear;
    for I := 0 to F.Chars.Count - 1 do
    begin
      MS.WriteBuffer(F.Chars.Char[I].Width, SizeOf(F.Chars.Char[I].Width));
    end;

    TF.AddData(sWidths, MS);

    MS.Clear;
    for I := 0 to F.Chars.Count - 1 do
    begin
      F.Chars.Char[I].Texture.Image.Save(MS);
    end;

    TF.AddData(sImages, MS);


    TF.SaveToStream(AStream);

  except
    raise
      EFontIO.Create('EFontIOError - Error in FontFiles.SaveFont() when saving to stream.')
  end;

  F.Style := Old;

  MS.Free;
  TF.Free;

end;


procedure ExportFont(F: TFont; AFile: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AFile, fmCreate);
  StreamFmt := FindFormat(ExtractFileExt(AFile));
  try
    ExportFont(F, Stream);
  finally
    Stream.Free;
  end;
  StreamFmt := nil;
end;


procedure ExportFont(F: TFont; AStream: TStream);
var Img: TImage;
  Old: TFontStyles;
  I, LineL: LongInt;
  Ch: TChar;
  Pos: LongInt;
begin

  Old := F.Style;

  if not F.Inited then Exit;
  if F.Chars.Char[0] = nil then Exit;
  if F.Chars.Char[0].Texture.Image = nil then
  begin
    F.Style := F.Style - [fsFreeImages];
    F.BuildChars;
  end;

  if F.Chars.Char[0].Texture.Image = nil then
  begin
    raise
      EFontIO.Create('EFontIOError - Error in FontFiles.ExportFont() - Chars can not be created.');
    Exit;
  end;

  Img := TImage.Create;
  Img.BitCount := 32;
  Img.Resize(F.Chars.Count * 3 + 1 + F.TextWidth(F.UsedCharSet,
    F.Chars.Char[0].Texture.Height), F.Chars.Char[0].Texture.Height);

  Pos := 0;
  for I := 0 to F.Chars.Count - 1 do
  begin
    Ch := F.Chars.Char[I];
    if Ch = nil then
    begin
      Pos := Pos + F.Chars.Char[0].Width;
      Continue;
    end;

    Img.Line(Pos, 0, Pos, Ch.Texture.Height, CharSeparatorCol);
    Pos := Pos + 1;

    Img.CopyRect(Ch.Texture.Image, Rect(Pos, 0, Pos + Ch.Width + 2,
      Ch.Texture.Height),
      (Ch.Texture.Width - Ch.Width) div 2 - 1, 0)
      ;

    Pos := Pos + Ch.Width + 2;



  end;

  Img.Line(Pos, 0, Pos, Ch.Texture.Height, CharSeparatorCol);

  if StreamFmt = nil then
    Img.Format := FindFormat('.tga')
  else
    Img.Format := StreamFmt;

  Img.Save(AStream);

  F.Style := Old;
  Img.Free;

end;

procedure LoadFont(F: TFont; AFile: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AFile, fmOpenRead	);
  try
    LoadFont(F, Stream);
  finally
    Stream.Free;
  end;
end;

procedure LoadFont(F: TFont; AStream: TStream);
var
  VDO: TVDataObj;
  I, TexSize: LongInt;
  Pos: Int64;
  TF: TVFile;
  CharSet: TFontChars;
  Widths: array of LongInt;
  Ch: TChar;
begin

  TF := TVFile.Create('', AStream);

  try
    Pos := TF.GetData(sProps, VDO);
    if Pos <> -1 then
    begin
      F.Name := ReadStr(VDO.Stream);
      VDO.Stream.ReadBuffer(TexSize, SizeOf(TexSize));
      CharSet := ReadStr(VDO.Stream);

      F.CharTextureSize := TexSize;
      F.UsedCharSet := CharSet;

      VDO.Free;
    end
    else
      raise EFontIO.Create('EFontIOError - Error in FontFiles.LoadFont().');


    Pos := TF.GetData(sWidths, VDO);
    if Pos <> -1 then
    begin
      SetLength(Widths, Length(CharSet));
      for I := 0 to Length(CharSet) - 1 do
        VDO.Stream.ReadBuffer(Widths[I], SizeOf(Widths[I]));
      VDO.Free;
    end
    else
      raise EFontIO.Create('EFontIOError - Error in FontFiles.LoadFont().');

    F.FreeChars;

    Pos := TF.GetData(sImages, VDO);
    if Pos <> -1 then
    begin
      for I := 0 to Length(CharSet) - 1 do
      begin
        Ch := TChar.Create(CharSet[I + 1]);
        Ch.Texture.Image.Load(VDO.Stream);
        Ch.Width := Widths[I];
        F.Chars.Add(Ch);
      end;

      VDO.Free;
    end
    else
      raise EFontIO.Create('EFontIOError - Error in FontFiles.LoadFont().');


  except
    raise EFontIO.Create('EFontIOError - Error in FontFiles.LoadFont().');
  end;

  TF.Free;

end;


procedure ImportFont(F: TFont; AFile: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AFile, fmOpenRead	);
  StreamFmt := FindFormat(ExtractFileExt(AFile));
  try
    ImportFont(F, Stream);
  finally
    Stream.Free;
  end;
  StreamFmt := nil;
end;

procedure ImportFont(F: TFont; AStream: TStream);
var SepCol, Fill: TARGB;
  Img: TImage;
  W, I, Pos, Prev: LongInt;
  Ch: TChar;

begin

  Img := TImage.Create();

  if StreamFmt = nil then
    Img.Format := FindFormat('.tga')
  else
    Img.Format := StreamFmt;

  Img.Load(AStream);

  SepCol := Img.PixelsARGB[0, 0];
  Fill := Img.PixelsARGB[1, 0];

  Prev := 0;
  Pos := 0;

  F.CharTextureSize := Img.Height;
  F.UsedCharSet := DefaultCharSet;
  F.FreeChars;

  for I := 1 to Img.Width - 1 do
  begin
    if Img.PixelsARGB[I, 0] = SepCol then
    begin
      W := I - Prev - 2;
    //////  WARNING - this works only for default charset
      Ch := TChar.Create(DefaultCharSet[Pos + 1]);
      Ch.Texture.Image.BitCount := Img.BitCount;
      Ch.Texture.Image.Resize(Img.Height, Img.Height);
      Ch.Width := W;
      Ch.Texture.Image.ClearARGB(Fill);
      Ch.Texture.Image.CopyRect(Img, Rect((Img.Height - W) div 2, 0, (Img.Height
        - W) div 2 + W, Img.Height),
        Prev + 1, 0);

      if not Img.HasAlpha then
       begin
        SwapColors(Ch.Texture.Image,Ch.Texture.Image.PixelsARGB[0,0],Textures.TextureColorKey);
       end;

      F.Chars.Add(Ch);

      Inc(Pos);
      Prev := I;
    end;
  end;

  Img.Free;

end;

procedure ImportBmpFont(F:TFont;AFile:string;CharSize:Windows.TPoint;SpaceChars:Windows.TPoint;CharSet:TFontChars;CKey:TARGB=0);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AFile, fmOpenRead	);
  StreamFmt := FindFormat(ExtractFileExt(AFile));
  try
    ImportBmpFont(F, Stream,CharSize,SpaceChars,CharSet,CKey);
  finally
    Stream.Free;
  end;
  StreamFmt := nil;
end;

procedure ImportBmpFont(F:TFont;AStream:TStream;CharSize:Windows.TPoint;SpaceChars:Windows.TPoint;CharSet:TFontChars;CKey:TARGB);
var Img: TImage;
  ALine:LongInt;
  I,J, Pos : LongInt;
  Ch: TChar;

begin

  Img := TImage.Create();

  if StreamFmt = nil then
    Img.Format := FindFormat('.tga')
  else
    Img.Format := StreamFmt;

  Img.Load(AStream);
  if Img.BitCount<24 then
   Img.BitCount:=24;

  Pos:=0;
  ALine:=0;

  F.UsedCharSet := CharSet;
  F.FreeChars;

  for J:=0 to Img.Height div (CharSize.Y+SpaceChars.y)-1 do
  for I:=0 to Img.Width div (CharSize.X+SpaceChars.X)-1 do
    begin
      if Pos+1>Length(CharSet) then
       begin Break;Break;end;

      Ch := TChar.Create(CharSet[Pos + 1]);
      Ch.Texture.Image.BitCount := Img.BitCount;
      Ch.Texture.Image.Resize(CharSize.X, CharSize.Y);
      Ch.Texture.Image.CopyRect(Img, Rect(0,0,CharSize.X,CharSize.Y),I*CharSize.X+I*SpaceChars.X,J*CharSize.Y+J*SpaceChars.Y);

      Ch.Texture.CheckSize;
      Ch.Width := Ch.Texture.Image.Width;

      if not Img.HasAlpha then
       begin
        if CKey=0 then
        SwapColors(Ch.Texture.Image,Ch.Texture.Image.PixelsARGB[0,0],Textures.TextureColorKey)
        else
        SwapColors(Ch.Texture.Image,SetA(CKey,0),Textures.TextureColorKey);
       end;

      F.Chars.Add(Ch);

      Inc(Pos);
    end;

  F.CharTextureSize := F.Chars.Char[0].Texture.Width;

  Img.Free;

end;





end.

