unit DrawBuffers;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Types3D,
 Colors, ContNRS,BaseMgrs,Textures;

const
 // drawcomplete flags
  DC_XYZ = 1;


type

  TDrawMode = (
    dmPoints,
    dmLineList,
    dmLineStrip,
    dmTriangleList,
    dmTriangleStrip,
    dmtriangleFan
    );

  TClearMode = (
    cmPoints,
    cmColors1,
    cmColors2,
    cmNormals,
    cmCoords1,
    cmCoords2,
    cmAll
    );

  TUsedField = (
    ufColors1,      // use diffuse color
    ufColors2,      // use specular color   -  WILL CHANGE
    ufNormals,      // use Normals
    ufCoords1,      // use Mapping Coords for Texture
    ufCoords2,      // use Mapping Coords for Tex2 (LightMap,Env Map,Bump Map,...);
    ufTexture,      // use Texture 1
    ufDeviceBuff,  // for non changing buffers - create vertex buffs and display lists
                   // do not uses with drawbuffers that change every frame
    ufAll
    );

  TUsedFields = set of TUsedField;

  TClearModes = set of TClearMode;

  TDrawBuffer = class(TBaseObj)
  private

  protected

  public
    Points: TList;
    Normals: TList;
    Colors1: TList;
    Coords1: TList;
    Colors2: TList;
    Coords2: TList;

    Texture:TTexture;


    Primitives: LongInt;
    Used: TUsedFields;
    Mode: TDrawMode;
    Ended: boolean;

    RenderData:Pointer;


    constructor Create; override;
    destructor Destroy; override;

    procedure ClearAll;

  published

  end;

  TDrawBufferMgr = class(TSortedMgr)
  private

  protected
    function GetBuff(Idx:LongInt):TDrawBuffer;
    procedure SetBuff(Idx:LongInt;Value:TDrawBuffer);


  public
    constructor Create(Owns: boolean = true); override;
    destructor Destroy; override;

    function Add(Buff:TDrawBuffer;AOrder:LongInt=0):TDrawBuffer;reintroduce;
    function Remove(Buff:TDrawBuffer):boolean;reintroduce;
    function Extract(Buff:TDrawBuffer):TDrawBuffer;reintroduce;
    function Delete(Idx:LongInt):boolean;reintroduce;

    procedure Draw;

    property Buffers[Idx:LongInt]:TDrawBuffer read GetBuff write SetBuff;default;

  published

  end;



implementation

uses GGraphic;

constructor TDrawBuffer.Create;
begin
  inherited Create;
  Ended := false;
  Used:=[ufColors1,ufCoords1,ufTexture];

  Points := TList.Create;
  Normals := TList.Create;
  Colors1 := TList.Create;
  Colors2 := TList.Create;
  Coords1 := TList.Create;
  Coords2 := TList.Create;

end;


destructor TDrawBuffer.Destroy;
begin
  ClearAll;
  Points.Free;
  Normals.Free;
  Colors1.Free;
  Colors2.Free;
  Coords1.Free;
  Coords2.Free;

  inherited Destroy;
end;


procedure TDrawBuffer.ClearAll();
var I: LongInt;
begin

  Renderer.FreeDrawBuffer(Self);

  for I := 0 to Points.Count - 1 do
    if Points[I] <> nil then FreeMem(Points[I]);

  for I := 0 to Normals.Count - 1 do
    if Normals[I] <> nil then FreeMem(Normals[I]);

  for I := 0 to Colors1.Count - 1 do
    if Colors1[I] <> nil then FreeMem(Colors1[I]);

  for I := 0 to Colors2.Count - 1 do
    if Colors2[I] <> nil then FreeMem(Colors2[I]);

  for I := 0 to Coords1.Count - 1 do
    if Coords1[I] <> nil then FreeMem(Coords1[I]);

  for I := 0 to Coords2.Count - 1 do
    if Coords2[I] <> nil then FreeMem(Coords2[I]);

  Points.Clear;
  Normals.Clear;
  Colors1.Clear;
  Colors2.Clear;
  Coords1.Clear;
  Coords2.Clear;

  Texture:=nil;
  Used:=[ufColors1,ufCoords1,ufTexture];
  Primitives:=0;
  Ended := false;

end;


/////////////// TDRAWBUFFERMGR

constructor TDrawBufferMgr.Create(Owns: boolean);
begin
 inherited Create(Owns);
end;

destructor TDrawBufferMgr.Destroy;
begin
 inherited Destroy;
end;

function TDrawBufferMgr.GetBuff(Idx:LongInt):TDrawBuffer;
begin
 Result:=TDrawBuffer(inherited GetObj(Idx));
end;

procedure TDrawBufferMgr.SetBuff(Idx:LongInt;Value:TDrawBuffer);
begin
// Objects[Idx]:=Value;
end;

function TDrawBufferMgr.Add(Buff:TDrawBuffer;AOrder:LongInt=0):TDrawBuffer;
begin
 Result:=nil;
 if Buff=nil then Exit;
 Buff.Order:=AOrder;
 inherited Add(Buff);
 Result:=TDrawBuffer(Objects[Count-1]);
end;

function TDrawBufferMgr.Remove(Buff:TDrawBuffer):boolean;
begin
 Result:=inherited Remove(Buff);
end;

function TDrawBufferMgr.Extract(Buff:TDrawBuffer):TDrawBuffer;
begin
 Result:=TDrawBuffer(inherited Extract(Buff));
end;

function TDrawBufferMgr.Delete(Idx:LongInt):boolean;
begin
 Result:=inherited Delete(Idx);
end;

procedure TDrawBufferMgr.Draw;
var I:LongInt;
begin
 for I:=0 to Count-1 do
  Renderer.DrawBuffer(Buffers[I]);
end;




end.

