unit GfxTypes;

interface

uses Windows, SysUtils, Classes,StrScan,BaseObjs,Types3D;

type


  TFeature = (
    gfLighting,
    gfAlphaBlend,
    gfStencil,
    gfFog,
    gfDither,
    gfAlphaTest,
    gfClipPlanes,
    gfVertexColors,
    gfZWrite,
    gfNormalizeNormals,

    gfZBuffer); // last value - do not change
  TFeatures = set of TFeature;

  TProjMode = (pmPerspective, pmOrthogonal);

  TSetDWord = //  to set various renderer dword values
    (sdAmbient,
    sdCullMode,
    sdShadeMode,
    sdFogColor,
    sdFogMode,
    sdTexFilter,
    sdZFunc,
    sdAlphaFunc,
    sdAlphaRef,
    sdStencilFunc,
    sdStencilRef,
    sdStencilMask,
    sdSrcBlend,
    sdDstBlend,
    sdTexAdrX,  // U ,S
    sdTexAdrY,  // V, T

    sdFillMode); // last value - do not change

  TDWordModes=set of TSetDword;

  TSetFloat = //  to set various renderer float values
    (sfFogStart,
    sfFogEnd,
    sfFogDensity,

    sfEnd
    );

  TFloatModes=set of TSetFloat;


 // for storing set values

  TFeatureSet=array[TFeature]of boolean;
  TDWordSet=array[TSetDWord]of DWord;
  TFloatSet=array[TSetFloat]of Single;


type
  TSetDWordValue = DWord;
  TCullMode=TSetDWordValue;
  TShadeMode=TSetDWordValue;
  TFogMode=TSetDWordValue;
  TFillMode=TSetDWordValue;

const
  // Cull Modes
  CULL_NONE = 0;
  CULL_CCW = 1;
  CULL_CW = 2;

  // Shade Modes
  SHADE_FLAT = 0;
  SHADE_GOURAUD = 1;


  // FogMode
  FOG_LINEAR = 0;
  FOG_EXP = 1;
  FOG_EXP2 = 2;

  // Fill Mode
  FILL_POINT = 0;
  FILL_WIREFRAME = 1;
  FILL_SOLID = 2;

  // Texture Filter
  TEXF_NEAREST=0;
  TEXF_BILINEAR=1;
  TEXF_TRILINEAR=2;

  // Compare Func - for Z-Buffer Func,Alph aFunc,Stencil Func
  CMP_NEVER=0;
  CMP_LESS=1;
  CMP_EQUAL=2;
  CMP_LESSEQUAL=3;
  CMP_GREATER=4;
  CMP_NOTEQUAL=5;
  CMP_GREATEREQUAL=6;
  CMP_ALWAYS=7;

  // Blending - for SrcBlend,DestBlend
  BLEND_ZERO=0;
  BLEND_ONE=1;
  BLEND_SRCCOLOR=2;
  BLEND_INVSRCCOLOR=3;
  BLEND_SRCALPHA=4;
  BLEND_INVSRCALPHA=5;
  BLEND_DSTALPHA=6;
  BLEND_INVDSTALPHA=7;
  BLEND_DSTCOLOR=8;
  BLEND_INVDSTCOLOR=9;
  BLEND_SRCALPHASAT=10;

  // TexAdress modes
  TEXADR_WRAP=0;
  TEXADR_MIRROR=1;
  TEXADR_CLAMP=2;

type

  TFrustumPlanes=(fpLeft,fpRight,fpTop,fpBottom,fpNear,fpFar);

  TFrustum=array[TFrustumPlanes]of TPlane;


  EBadParams = class(Exception)
  end;

  TRenderInitParams = class
  public
    Handle: THandle;
    Width: LongInt;
    Height: LongInt;
    ColorBits: byte;
    DepthBits: byte;
    StencilBits:byte;
    RefreshRate:LongInt;
    UseDepth: boolean;
    BackBuffers: byte;
    FullScreen: boolean;
    UseHardWare: boolean;
    UseVSync: boolean;
    UseHTL: boolean;
    MultiSamples: byte;

    procedure CopyTo(RIP: TRenderInitParams);
    procedure CopyFrom(RIP: TRenderInitParams);
    procedure SaveToStream(Stream: TStream);
    procedure LoadFromStream(Stream: TStream);

  end;



  TFullScreenMode=class(TBaseObj)
                   Width:LongInt;
                   Height:LongInt;
                   Bits:LongInt;
                   Refresh:LongInt;

                   constructor Create(W,H,B,R:LongInt);reintroduce;
                   procedure SetMode(W,H,B,R:LongInt);
                   procedure CopyTo(FSM: TFullScreenMode);
                   procedure CopyFrom(FSM: TFullScreenMode);
                   function GetStr:string;
                   procedure SetStr(Str:string);

                   end;


var

  AspectRatio: single = 1;



implementation



procedure TRenderInitParams.CopyTo(RIP: TRenderInitParams);
begin
  if RIP = nil then Exit;

  RIP.CopyFrom(Self);

end;

procedure TRenderInitParams.CopyFrom(RIP: TRenderInitParams);
begin
  if RIP = nil then Exit;
  Self.Handle := RIP.Handle;
  Self.Width := RIP.Width;
  Self.Height := RIP.Height;
  Self.ColorBits := RIP.ColorBits;
  Self.DepthBits := RIP.DepthBits;
  Self.StencilBits := RIP.StencilBits;
  Self.RefreshRate:=RIP.RefreshRate;
  Self.UseDepth := RIP.UseDepth;
  Self.BackBuffers := RIP.BackBuffers;
  Self.FullScreen := RIP.FullScreen;
  Self.UseHardWare := RIP.UseHardWare;
  Self.UseVSync := RIP.UseVSync;
  Self.UseHTL := RIP.UseHTL;
  Self.MultiSamples := RIP.MultiSamples;
end;

procedure TRenderInitParams.SaveToStream(Stream: TStream);
begin

end;

procedure TRenderInitParams.LoadFromStream(Stream: TStream);
begin
end;


constructor TFullScreenMode.Create(W,H,B,R:LongInt);
begin
 inherited Create;
 SetMode(W,H,B,R);
end;

procedure TFullScreenMode.SetMode(W,H,B,R:LongInt);
begin
 Width:=W;
 Height:=H;
 Bits:=B;
 Refresh:=R;
end;

procedure TFullScreenMode.CopyTo(FSM: TFullScreenMode);
begin
  if FSM = nil then Exit;

  FSM.CopyFrom(Self);


end;

procedure TFullScreenMode.CopyFrom(FSM: TFullScreenMode);
begin
Self.Width:=FSM.Width;
Self.Height:=FSM.Height;
Self.Bits:=FSM.Bits;
Self.Refresh:=FSM.Refresh;
end;

function TFullScreenMode.GetStr:string;
begin
 Result:=Format('%d x %d x %d @ %dHz',[Width,Height,Bits,Refresh]);
end;

procedure TFullScreenMode.SetStr(Str:string);
var Pars:TParsArr;
begin
if not StrScan.FStrScan(Str,'%I x %I x %I @ %IHz',Pars) then Exit;
Width:=Pars[0];
Height:=Pars[1];
Bits:=Pars[2];
Refresh:=Pars[3];
end;



end.

