unit ViewPorts;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,GfxTypes,
  Objs3D,Types3D,Utils3D,Cameras,GfxMessages;

type

    TViewPort=class(TBaseObj)
              private
               FProjection:TProjMode;

               FX:LongInt;       // in pixels
               FY:LongInt;       // in pixels
               FWidth:LongInt;   // in pixels
               FHeight:LongInt;  // in pixels

               FFov:single;
               FNear:single;
               FBack:single;
               FOrthoWidth:LongInt;
               FOrthoHeight:LongInt;
               FAspect:single;
          
               procedure OnRenderReset(MsgID:LongInt;Sender:TBaseObj;Data:Pointer);

              protected

                procedure SetX(Value:LongInt);virtual;
                procedure SetY(Value:LongInt);virtual;
                procedure SetWidth(Value:LongInt);virtual;
                procedure SetHeight(Value:LongInt);virtual;
                procedure SetProjection(Value:TProjMode);virtual;
                procedure SetFov(Value:single);virtual;
                procedure SetBack(Value:single);virtual;
                procedure SetNear(Value:single);virtual;
                procedure SetOrthoW(Value:LongInt);virtual;
                procedure SetOrthoH(Value:LongInt);virtual;
                procedure SetAspect(Value:single);virtual;


              public
                constructor Create; override;
                destructor Destroy; override;

                procedure Init;override;
                procedure Done;override;

                procedure Assign(Source:TBaseObj);override;
                procedure Configure(AX,AY,AWidth,AHeight:LongInt);virtual;
                procedure Orthogonal(AWidth,AHeight:LongInt);virtual;
                procedure Perspective(AFov,ANear,ABack:single;AAspect:single=0);overload;virtual;
                //procedure Perspective(AWidth,AHeight,ANear,ABack:single);overload;virtual;

                procedure ReConfigure;virtual;
                procedure ReOrthogonal;virtual;
                procedure RePerspective;virtual;
                procedure ReProjection;virtual;
                procedure Reset;virtual;

              published

                property X:LongInt read FX write SetX;
                property Y:LongInt read FY write SetY;
                property Width:LongInt read FWidth write SetWidth;
                property Height:LongInt read FHeight write SetHeight;
                property Projection:TProjMode read FProjection write SetProjection;
                property Fov:single read FFov write SetFov;
                property BackPlane:single read FBack write SetBack;
                property NearPlane:single read FNear write SetNear;
                property OrthoWidth:LongInt read FOrthoWidth write SetOrthoW;
                property OrthoHeight:LongInt read FOrthoHeight write SetOrthoH;
                property AspectRatio:single read FAspect write SetAspect;



              end;





implementation

uses GGraphic,GMessages;

constructor TViewPort.Create;
begin
 inherited Create;
 FProjection:=pmPerspective;
 FFov:=45;
 FBack:=1000;
 FNear:=10;
 FAspect:=1.333;
 FOrthoWidth:=800;
 FOrthoHeight:=600;
 FX:=0;
 FY:=0;
 FWidth:=800;
 FHeight:=600;

 Messages.AddHandler(MSG_GFX_RESET,0,OnRenderReset);
 Messages.AddHandler(MSG_GFX_WINDOWRESIZE,0,OnRenderReset);
 Messages.AddHandler(MSG_GFX_INIT,0,OnRenderReset);


end;

destructor TViewPort.Destroy;
begin
 Done;

 Messages.RemoveHandler(MSG_GFX_RESET,OnRenderReset);
 Messages.RemoveHandler(MSG_GFX_WINDOWRESIZE,OnRenderReset);
 Messages.RemoveHandler(MSG_GFX_INIT,OnRenderReset);


 inherited Destroy;
end;

procedure TViewPort.Init;
begin
 inherited Init;
 Reset;
end;

procedure TViewPort.Done;
begin
 inherited Done;
 
end;


procedure TViewPort.SetX(Value:LongInt);
begin
 FX:=Value;
 ReConfigure();
end;

procedure TViewPort.SetY(Value:LongInt);
begin
 FY:=Value;
 ReConfigure();
end;

procedure TViewPort.SetWidth(Value:LongInt);
begin
 FWidth:=Value;
 ReConfigure();
end;

procedure TViewPort.SetHeight(Value:LongInt);
begin
 FHeight:=Value;
 ReConfigure();
end;

procedure TViewPort.SetProjection(Value:TProjMode);
begin
 FProjection:=Value;
 ReProjection;
end;

procedure TViewPort.SetFov(Value:single);
begin
 FFov:=Value;
 if FProjection=pmPerspective then
  RePerspective;
end;

procedure TViewPort.SetBack(Value:single);
begin
 FBack:=Value;
 if FProjection=pmPerspective then
  RePerspective;
end;

procedure TViewPort.SetNear(Value:single);
begin
 FNear:=Value;
 if FNear<1 then
 FNear:=1;
 if FProjection=pmPerspective then
  RePerspective;
end;

procedure TViewPort.SetOrthoW(Value:LongInt);
begin
 FOrthoWidth:=Value;
 if FProjection=pmOrthogonal then
  ReOrthogonal;
end;

procedure TViewPort.SetOrthoH(Value:LongInt);
begin
 FOrthoHeight:=Value;
 if FProjection=pmOrthogonal then
  ReOrthogonal;

end;

procedure TViewPort.SetAspect(Value:single);
begin
 FAspect:=Value;
 if FProjection=pmPerspective then
  RePerspective;

end;

procedure TViewPort.Assign(Source:TBaseObj);
var V:TViewPort;
begin
 inherited Assign(Source);
 if not(Source is TViewPort) then Exit;
 V:=TViewPort(Source);


 Configure(V.X,V.Y,V.Width,V.Height);
 Orthogonal(V.OrthoWidth,V.OrthoHeight);
 Perspective(V.Fov,V.NearPlane,V.BackPlane);
 FProjection:=V.Projection;

end;

procedure TViewPort.Reset;
begin
 ReConfigure;
 ReProjection;

end;

procedure TViewPort.ReConfigure;
begin
 if Inited then
  Renderer.ConfigView(FX,FY,FWidth,FHeight);
end;

procedure TViewPort.Configure(AX,AY,AWidth,AHeight:LongInt);
begin
 FX:=AX;
 FY:=AY;
 FWidth:=AWidth;
 FHeight:=AHeight;

 ReConfigure;

end;

procedure TViewPort.Orthogonal(AWidth,AHeight:LongInt);
begin
 FProjection:=pmOrthogonal;
 FOrthoWidth:=AWidth;
 FOrthoHeight:=AHeight;
 ReOrthogonal;
end;

procedure TViewPort.Perspective(AFov,ANear,ABack:single;AAspect:single);
begin
  FProjection := pmPerspective;
  FFov := AFov;
  FBack := ABack;
  if ANear<1 then
   ANear:=1;
  FNear:=ANear;
  if AAspect>0.0001 then
   FAspect:=AAspect;
  RePerspective;
end;

{procedure TViewPort.Perspective(AWidth,AHeight,ANear,ABack:single);
begin


end;}

procedure TViewPort.ReOrthogonal;
begin
 Camera.SetIdentityView;
 if not Inited then Exit;
  Renderer.SetOrtho(FOrthoWidth,FOrthoHeight);

end;

procedure TViewPort.RePerspective;
begin
  if  not Inited then Exit;
  Renderer.SetPerspective(FFov,FNear,FBack,FAspect);

end;

procedure TViewPort.ReProjection;
begin
 if FProjection=pmOrthogonal then
  ReOrthogonal
 else
  RePerspective;

end;


procedure TViewPort.OnRenderReset(MsgID:LongInt;Sender:TBaseObj;Data:Pointer);
var R:Windows.TRect;
begin
 Reset;
 GetClientRect(Renderer.UsedParams.Handle,R);
 Configure(0,0,R.Right,R.Bottom);

end;



end.
