unit MsgEngine;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,Engines,MsgHandlers;


type

TMsgEngine = class(TEngine)
private
 FMessagesSent:LongInt;

protected

  function GetHandlersCount:LongInt;
  function GetHandlerProcsCount:LongInt;
  function GetSentID(MsgID:LongInt):LongInt;

public
  constructor Create(AOwner:TEngine=nil); override;
  destructor Destroy; override;

  procedure Init;override;
  procedure Done;override;

  class function GetInstance:TMsgEngine;

  procedure AddHandler(MsgID:LongInt;Priority:LongInt;Proc:TMsgHandlerProc);
  procedure RemoveHandler(MsgID:LongInt;Proc:TMsgHandlerProc);
  procedure SendMessage(MsgID:LongInt;Sender:TBaseObj;Data:Pointer=nil);

  function HandlerExists(MsgID:LongInt):boolean;
  function FindHandler(MsgID:LongInt):TMsgHandler;

  property HandlersCount:LongInt read GetHandlersCount;
  property HandlerProcsCount:LongInt read GetHandlerProcsCount;
  property MessagesSent:LongInt read FMessagesSent;
  property MessagesSentID[MsgID:LongInt]:LongInt read GetSentID;

published

end;



implementation

var ME:TMsgEngine;

constructor TMsgEngine.Create(AOwner:TEngine);
begin
 inherited Create(AOwner);
 FMessagesSent:=0;

 if ME<>nil then
   begin
   ME.Free;
   ME:=nil;
   end;

  ME:=Self;


  Init;
end;


destructor TMsgEngine.Destroy;
begin
 Done;
 inherited Destroy;
end;


procedure TMsgEngine.Init;
begin

 inherited Init;
end;

procedure TMsgEngine.Done;
begin
 inherited Done;

end;


class function TMsgEngine.GetInstance:TMsgEngine;
begin
 if ME=nil then
  Result:=TMsgEngine.Create(nil)
 else
  Result:=ME;
end;

function TMsgEngine.HandlerExists(MsgID:LongInt):boolean;
var I:LongInt;
begin
 Result:=false;
 for I:=0 to Objects.Count-1 do
  if MsgID=TMsgHandler(Objects[I]).MsgID then
   Result:=true;
end;

function TMsgEngine.FindHandler(MsgID:LongInt):TMsgHandler;
var I:LongInt;
begin
 Result:=nil;
 for I:=0 to Objects.Count-1 do
  if MsgID=TMsgHandler(Objects[I]).MsgID then
   Result:=TMsgHandler(Objects[I]);
end;



procedure TMsgEngine.AddHandler(MsgID:LongInt;Priority:LongInt;Proc:TMsgHandlerProc);
var H:TMsgHandler;
begin
 H:=FindHandler(MsgID);
 if H=nil then
   H:=TMsgHandler.Create(MsgID);

 H.AddHandlerProc(Proc,Priority);
 Objects.Add(TBaseObj(H));

end;

procedure TMsgEngine.RemoveHandler(MsgID:LongInt;Proc:TMsgHandlerProc);
var H:TMsgHandler;
begin
H:=FindHandler(MsgID);
if H=nil then Exit;
H.RemoveHandlerProc(Proc);
if H.Procs.Count=0 then
 Objects.Remove(TBaseObj(H));

end;

procedure TMsgEngine.SendMessage(MsgID:LongInt;Sender:TBaseObj;Data:Pointer=nil);
var H:TMsgHandler;
begin
Inc(FMessagesSent);
H:=FindHandler(MsgID);
if H=nil then Exit;
H.Perform(Sender,Data);
if Data<>nil then
 FreeMem(Data);
end;


function TMsgEngine.GetHandlersCount:LongInt;
begin
 Result:=Objects.Count;
end;

function TMsgEngine.GetHandlerProcsCount:LongInt;
var I:LongInt;
begin
 Result:=0;
 for I:=0 to Objects.Count-1 do
  Result:=Result+TMsgHandler(Objects[I]).Procs.Count;
end;

function TMsgEngine.GetSentID(MsgID:LongInt):LongInt;
var H:TMsgHandler;
begin
Result:=0;
H:=FindHandler(MsgID);
if H=nil then Exit;
Result:=H.Performed;
end;


initialization


finalization


end.
