unit MsgHandlers;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,BaseMgrs;

type

 TMsgHandlerProc=procedure (MsgID:LongInt;Sender:TBaseObj;Data:Pointer) of object;

 TProcContainer=class(TBaseObj)
        public
         Proc:TMsgHandlerProc;
         Order:LongInt;
        end;

 TMsgHandler = class(TObject)
 private
  FProcs:TSortedMgr;
  FMsgID:LongInt;
  FPerformed:LongInt;
 protected

 public
   constructor Create(AMsgID:LongInt);
   destructor Destroy;override;

   procedure Perform(Sender:TBaseObj;Data:Pointer);
   procedure AddHandlerProc(Proc:TMsgHandlerProc;Priority:Longint);
   procedure RemoveHandlerProc(Proc:TMsgHandlerProc);
   function ProcExists(Proc:TMsgHandlerProc):boolean;

   property MsgID:LongInt read FMsgID;
   property Procs:TSortedMgr read FProcs;
   property Performed:LongInt read FPerformed;

 published

 end;



implementation

constructor TMsgHandler.Create(AMsgID:LongInt);
begin
 inherited Create;
 FMsgID:=AMsgID;
 FProcs:=TSortedMgr.Create();
 FProcs.Sorting:=stDescending;
 FPerformed:=0;

end;

destructor TMsgHandler.Destroy;
begin
 FProcs.Free;
 inherited Destroy;
end;

function TMsgHandler.ProcExists(Proc:TMsgHandlerProc):boolean;
var I:LongInt;
begin
 Result:=false;
 for I:=0 to FProcs.Count-1 do
   if @TProcContainer(FProcs[I]).Proc=@Proc then
    Result:=true;

end;

procedure TMsgHandler.Perform(Sender:TBaseObj;Data:Pointer);
var I:LongInt;
begin
 for I:=0 to FProcs.Count-1 do
  if Assigned(TProcContainer(FProcs[I]).Proc)then
  TProcContainer(FProcs[I]).Proc(FMsgID,Sender,Data);
 Inc(FPerformed);
end;

procedure TMsgHandler.AddHandlerProc(Proc:TMsgHandlerProc;Priority:Longint);
var C:TProcContainer;
begin
  if ProcExists(Proc) then Exit;

 C:=TProcContainer.Create;
 C.Proc:=Proc;
 C.Order:=Priority;
 FProcs.Add(C);

end;

procedure TMsgHandler.RemoveHandlerProc(Proc:TMsgHandlerProc);
var I:LongInt;
begin
for I:=0 to FProcs.Count-1 do
 if @TProcContainer(FProcs[I]).Proc=@Proc then
  FProcs.Remove(FProcs[I]);


end;



end.
