unit Lights;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,Objs3D,Types3D,Utils3D,
     Colors;

     
type

  TLightType=(
  ltPoint,
  ltSpot,
  ltDirectional
  );

  TLightColor=record
    case Integer of
    0:(
    R:single;
    G:single;
    B:single;
    );
    1:(
    Arr:array[0..2]of single
    );
    end;

  TLight=class(TBase3DObj)
  private
   FLightType:TLightType;
   FAmbient:TLightColor;
   FDiffuse:TLightColor;
   FSpecular:TLightColor;
   FConstantAtt:single;
   FLinearAtt:single;
   FQuadraticAtt:single;
   FRange:single;
   FSpotFallOff:single;
   FSpotInnerCone:single;    //  angle of inner cone,can be 0..FSpotOuterCone
   FSpotOuterCone:single;    //  angle of outer cone
   FActive:boolean;
  protected
   procedure SetLightType(Value:TLightType);virtual;
   procedure SetRange(Value:single);virtual;
   procedure SetDiffuseColor(Value:TLightColor);virtual;
   procedure SetDiffVal(Idx:LongInt;Value:single);virtual;
   procedure SetAmbientColor(Value:TLightColor);virtual;
   procedure SetAmbVal(Idx:LongInt;Value:single);virtual;
   procedure SetSpecularColor(Value:TLightColor);virtual;
   procedure SetSpecVal(Idx:LongInt;Value:single);virtual;
   procedure SetAttenuations(Idx:LongInt;Value:single);virtual;
   procedure SetSpotVals(Idx:LongInt;Value:single);virtual;
   procedure SetActive(Value:boolean);virtual;

  public
    constructor Create; overload;override;
    constructor Create(ALType:TLightType;ARange:single);reintroduce;overload;virtual;

    destructor Destroy; override;

    procedure Init;override;
    procedure Done;override;
    procedure Update;override;
    procedure UpdateLight;virtual;
    procedure Draw;override;
 
    property LightType:TLightType read FLightType write SetLightType;
    property Range:single read FRange write SetRange;
    property DiffuseColor:TLightColor read FDiffuse write SetDiffuseColor;
    property DiffuseR:single index 1 read FDiffuse.R write SetDiffVal;
    property DiffuseG:single index 2 read FDiffuse.G write SetDiffVal;
    property DiffuseB:single index 3 read FDiffuse.B write SetDiffVal;
    property AmbientColor:TLightColor read FAmbient write SetAmbientColor;
    property AmbientR:single index 1 read FAmbient.R write SetAmbVal;
    property AmbientG:single index 2 read FAmbient.G write SetAmbVal;
    property AmbientB:single index 3 read FAmbient.B write SetAmbVal;
    property SpecularColor:TLightColor read FSpecular write SetSpecularColor;
    property SpecularR:single index 1 read FSpecular.R write SetSpecVal;
    property SpecularG:single index 2 read FSpecular.G write SetSpecVal;
    property SpecularB:single index 3 read FSpecular.B write SetSpecVal;
    property ConstantAttenuation:single index 1 read FConstantAtt write SetAttenuations;
    property LinearAttenuation:single index 2 read FLinearAtt write SetAttenuations;
    property QuadraticAttenuation:single index 3 read FQuadraticAtt write SetAttenuations;
    property SpotFallOff:single index 1 read FSpotFallOff write SetSpotVals;
    property SpotInnerConeAngle:single index 2 read FSpotInnerCone write SetSpotVals;
    property SpotOuterConeAngle:single index 3 read FSpotOuterCone write SetSpotVals;
    property Active:boolean read FActive write SetActive;


  published


  end;


function LightColor(R,G,B:single):TLightColor;
function LightFromARGB(ARGB:TARGB):TLightColor;
function ARGBFromLight(Light:TLightColor):TARGB;

implementation

function LightColor(R,G,B:single):TLightColor;
begin
 Result.R:=R;
 Result.G:=G;
 Result.B:=B;
end;

function LightFromARGB(ARGB:TARGB):TLightColor;
begin
 Result.R:=GetRS(ARGB);
 Result.G:=GetGS(ARGB);
 Result.B:=GetBS(ARGB);
end;

function ARGBFromLight(Light:TLightColor):TARGB;
var Rec:TARGBRec;
begin
Rec.A:=$FF;
Rec.R:=Round(ClampValue(Light.R,0,1)*255);
Rec.G:=Round(ClampValue(Light.G,0,1)*255);
Rec.B:=Round(ClampValue(Light.B,0,1)*255);
Result:=Rec.Color;
end;


constructor TLight.Create;
begin
 inherited Create;
 FLightType:=ltPoint;
 FAmbient:=LightFromARGB($FFFFFFFF);
 FDiffuse:=LightFromARGB($FFAAAAAA);
 FSpecular:=LightFromARGB($FF000000);;
 FConstantAtt:=1;
 FLinearAtt:=0;
 FQuadraticAtt:=0;
 FRange:=500;
 FSpotFallOff:=1;
 FSpotInnerCone:=45;
 FSpotOuterCone:=60;
 FActive:=false;
end;

constructor TLight.Create(ALType:TLightType;ARange:single);
begin
 Create;
 LightType:=ALType;
 Range:=ARange;
end;


destructor TLight.Destroy;
begin

 inherited Destroy;
end;

procedure TLight.Init;
begin
 inherited Init;
 UpdateLight;
end;

procedure TLight.Done;
begin
 inherited Done;

end;

procedure TLight.SetActive(Value:boolean);
begin
 FActive:=Value;
end;

procedure TLight.SetLightType(Value:TLightType);
begin
 FLightType:=Value;
 UpdateLight;
end;

procedure TLight.SetRange(Value:single);
begin
 FRange:=Value;
 UpdateLight;
end;


procedure TLight.SetDiffuseColor(Value:TLightColor);
begin
 FDiffuse:=Value;
 UpdateLight;
end;

procedure TLight.SetDiffVal(Idx:LongInt;Value:single);
begin
 case Idx of
 1:FDiffuse.R:=Value;
 2:FDiffuse.G:=Value;
 3:FDiffuse.B:=Value;
 end;
 UpdateLight;
end;

procedure TLight.SetAmbientColor(Value:TLightColor);
begin
 FAmbient:=Value;
 UpdateLight;
end;

procedure TLight.SetAmbVal(Idx:LongInt;Value:single);
begin
 case Idx of
 1:FAmbient.R:=Value;
 2:FAmbient.G:=Value;
 3:FAmbient.B:=Value;
 end;
 UpdateLight;
end;

procedure TLight.SetSpecularColor(Value:TLightColor);
begin
 FSpecular:=Value;
 UpdateLight;
end;

procedure TLight.SetSpecVal(Idx:LongInt;Value:single);
begin
 case Idx of
 1:FSpecular.R:=Value;
 2:FSpecular.G:=Value;
 3:FSpecular.B:=Value;
 end;
 UpdateLight;
end;

procedure TLight.SetAttenuations(Idx:LongInt;Value:single);
begin
 case Idx of
 1:FConstantAtt:=Value;
 2:FLinearAtt:=Value;
 3:FQuadraticAtt:=Value;
 end;

 UpdateLight;
end;

procedure TLight.SetSpotVals(Idx:LongInt;Value:single);
begin
 case Idx of
 1:FSpotFallOff:=Value;
 2:FSpotInnerCone:=Value;
 3:FSpotOuterCone:=Value;
 end;

 if FLightType=ltSpot then
 UpdateLight;
end;

procedure TLight.Update;
begin
 UpdateLight;
end;

procedure TLight.UpdateLight;
begin

end;

procedure TLight.Draw;
begin


end;


end.
