unit TexMapping;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Utils3D, Types3D,
  DrawBuffers, Objs3D;

type

  // Texture Coordinates Mapper for TGeoObjs
  // only for pregenerated coords - alters coords by offset,tiling,clamp,.. options

  TTexMapper = class
  private
    FOwner: TBaseObj;

      // if you process coords twice and invert=true then
    // mapper inverts inverted coords and then they are not inverted now // !!??!!
    // if you want to invert coords set invert=true then processcoords
    // and then switch invert back to false
    //
    // similar problem is witn SwapXY


    FXOffset: single;
    FYOffset: single;
    FXTiling: single;
    FYTiling: single;
    FXInvert: boolean;
    FYInvert: boolean;
    FXClamp: boolean;
    FYClamp: boolean;
    FXClampMin: single;
    FXClampMax: single;
    FYClampMin: single;
    FYClampMax: single;
    FSwapXY: boolean;

  protected
    procedure SetOwner(Value: TBaseObj);

  public


    constructor Create(AOwner: TBaseObj = nil);
    destructor Destroy; override;

    procedure ProcessBuffer(Buff: TDrawBuffer; UseCoords2: boolean = true);
    procedure ProcessList(List: TList);
    procedure ProcessArray(var Arr: array of TTexCoord);
    procedure ProcessPointer(Data: Pointer; Count: LongInt);
    procedure ProcessCoord(var C: TTexCoord); overload;
    procedure ProcessCoord(var pC: PTexCoord); overload;


    procedure SetInvert(cX, cY: boolean);
    procedure SetClamp(cX, cY: boolean);
    procedure SetClampValues(MinX, MinY, MaxX, MaxY: single);
    procedure SetOffset(cX, cY: single);
    procedure SetTiling(cX, cY: single);

    procedure Copy(TM:TTexMapper);

    property Owner: TBaseObj read FOwner write SetOwner;

  published

    property XOffset: single read FXOffset write FXOffset;
    property YOffset: single read FYOffset write FYOffset;
    property XTiling: single read FXTiling write FXTiling;
    property YTiling: single read FYTiling write FYTiling;
    property XInvert: boolean read FXInvert write FXInvert;
    property YInvert: boolean read FYInvert write FYInvert;
    property XClamp: boolean read FXClamp write FXClamp;
    property YClamp: boolean read FYClamp write FYClamp;
    property SwapXY: boolean read FSwapXY write FSwapXY;
    property XClampMin: single read FXClampMin write FXClampMin;
    property XClampMax: single read FXClampMax write FXClampMax;
    property YClampMin: single read FYClampMin write FYClampMin;
    property YClampMax: single read FYClampMax write FYClampMax;

  end;


implementation

constructor TTexMapper.Create(AOwner: TBaseObj);
begin
  inherited Create;
  FXOffset := 0;
  FYOffset := 0;
  FXTiling := 1;
  FYTiling := 1;
  FXInvert := false;
  FYInvert := false;
  FXClamp := false;
  FYClamp := false;
  FXClampMin := -1;
  FXClampMax := 1;
  FYClampMin := -1;
  FYClampMax := 1;
  FSwapXY := false;

  FOwner := AOwner;
end;

destructor TTexMapper.Destroy;
begin

  inherited Destroy;
end;

procedure TTexMapper.SetOwner(Value: TBaseObj);
begin
  if FOwner = Value then Exit;
  FOwner := Value;


end;

procedure TTexMapper.ProcessCoord(var C: TTexCoord);
begin
  if FSwapXY then
    Swap(C.X, C.Y);

  C.X := C.X * FXTiling + FXOffset;
  if FXInvert then
  begin
    C.X := -C.X
  end;

  if FXClamp then
    C.X := ClampValue(C.X, FXClampMin, FXClampMax);

  C.Y := C.Y * FYTiling + FYOffset;
  if FYInvert then
  begin
    C.Y := -C.Y
  end;

  if FYClamp then
    C.Y := ClampValue(C.Y, FYClampMin, FYClampMax);
 
end;

procedure TTexMapper.ProcessCoord(var pC: PTexCoord);
begin
  if FSwapXY then
    Swap(pC^.X, pC^.Y);

  pC^.X := pC^.X * FXTiling + FXOffset;
  if FXInvert then
  begin
    pC^.X := pC^.X
  end;

  if FXClamp then
    pC^.X := ClampValue(pC^.X, FXClampMin, FXClampMax);

  pC^.Y := pC^.Y * FYTiling + FYOffset;
  if FYInvert then
  begin
    pC^.Y := -pC^.Y
  end;

  if FYClamp then
    pC^.Y := ClampValue(pC^.Y, FYClampMin, FYClampMax);


end;

procedure TTexMapper.ProcessBuffer(Buff: TDrawBuffer; UseCoords2: boolean);
begin
  ProcessList(Buff.Coords1);
  if UseCoords2 then
    ProcessList(Buff.Coords2);

end;

procedure TTexMapper.ProcessList(List: TList);
var I: LongInt;
  pC: PTexCoord;
begin
  if List = nil then Exit;
  if List.Count = 0 then Exit;

  for I := 0 to List.Count - 1 do
    if List[I] <> nil then
    begin
      pC := PTexCoord(List[I]);
      ProcessCoord(pC);
    end;

end;

procedure TTexMapper.ProcessArray(var Arr: array of TTexCoord);
var I: LongInt;
begin
  for I := Low(Arr) to High(Arr) do
  begin
    ProcessCoord(Arr[I]);
  end;
end;

procedure TTexMapper.ProcessPointer(Data: Pointer; Count: LongInt);
var I: LongInt;
  pC: PTexCoord;
begin
  if Data = nil then Exit;
  pC := Data;
  for I := 0 to Count - 1 do
  begin
    ProcessCoord(pC);
    Inc(pC);
  end;
end;


procedure TTexMapper.SetInvert(cX, cY: boolean);
begin
  XInvert := cX;
  YInvert := cY;
end;

procedure TTexMapper.SetClamp(cX, cY: boolean);
begin
  XClamp := cX;
  YClamp := cY;
end;

procedure TTexMapper.SetClampValues(MinX, MinY, MaxX, MaxY: single);
begin
  XClampMin := MinX;
  YClampMin := MinY;
  XClampMax := MaxX;
  YClampMax := MaxY;
end;

procedure TTexMapper.SetOffset(cX, cY: single);
begin
  XOffset := cX;
  YOffset := cY;
end;

procedure TTexMapper.SetTiling(cX, cY: single);
begin
  XTiling := cX;
  YTiling := cY;
end;

procedure TTexMapper.Copy(TM:TTexMapper);
begin
 if TM=nil then Exit;
 FSwapXY:=TM.SwapXY;
 FXClamp:=TM.XClamp;
 FXClampMin:=TM.XClampMin;
 FXClampMax:=TM.XClampMax;
 FYClamp:=TM.YClamp;
 FYClampMin:=TM.YClampMin;
 FYClampMax:=TM.YClampMax;
 FXInvert:=TM.XInvert;
 FYInvert:=TM.YInvert;
 FXOffset:=TM.XOffset;
 FYOffset:=TM.YOffset;
 FXTiling:=TM.XTiling;
 FYTiling:=TM.YTiling;
end;





end.

