unit VisObjs;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Colors, Types3D,
  Utils3D, Objs3D, GfxTypes, DrawBuffers;

type

  TVisibleObj = class(TBase3DObj)
  private
    FVisible:boolean;
              //  FBoundingBox:TBoundingBox;

  protected
    procedure SetScale(Value: TPoint3S); override;
    procedure SetPosition(Value: TPoint3S); override;
    procedure SetAngle(Value: TPoint3S); override;
    procedure SetVisible(Value:boolean); virtual;



  public
    Bounds: TBoundingInfo;

    constructor Create; override;
    destructor Destroy; override;

    procedure Init; override;
    procedure Done; override;
    procedure DrawAlone; virtual; // do not draw children,restore world matrix
    procedure UpdateBounds; virtual;

    procedure Assign(Source: TBaseObj); override;
    procedure Show;
    procedure Hide;



  published

     property Visible:boolean read FVisible write SetVisible;

  end;


implementation

constructor TVisibleObj.Create;
begin
  inherited Create;

  Bounds := BoundInfo(NullPoint, NullPoint);

end;

destructor TVisibleObj.Destroy;
begin

  inherited Destroy;
end;

procedure TVisibleObj.Init;
begin
  inherited Init;

end;

procedure TVisibleObj.Done;
begin

  inherited Done;
end;

procedure TVisibleObj.SetPosition(Value:TPoint3S);
begin
 inherited SetPosition(Value);
 UpdateBounds;
end;

procedure TVisibleObj.SetAngle(Value:TPoint3S);
begin
 inherited SetAngle(Value);
 UpdateBounds;
end;

procedure TVisibleObj.SetScale(Value:TPoint3S);
begin
 inherited SetScale(Value);
 UpdateBounds;
end;

procedure TVisibleObj.SetVisible(Value:boolean);
begin
 FVisible:=Value;
end;

procedure TVisibleObj.UpdateBounds;
begin

end;

procedure TVisibleObj.DrawAlone;
begin
  if not InFrustum then Exit;
  SetUpWorld;
  Draw;
  RestoreWorld;
end;

procedure TVisibleObj.Assign(Source: TBaseObj);
var VO: TVisibleObj;
begin
  inherited Assign(Source);
  if not (Source is TVisibleObj) then Exit;
  VO := TVisibleObj(Source);
  Bounds:=VO.Bounds;
  Visible:=VO.Visible;

end;

procedure TVisibleObj.Show;
begin
 Visible:=true;
end;

procedure TVisibleObj.Hide;
begin
 Visible:=false;
end;



end.

