unit Faces;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,Objs3D,Types3D,
     Utils3D,Colors,Textures,BaseMgrs,Points;


type

    TFacePoints=array[0..2]of TFacePoint;

    TFaceCoords=array[0..2]of TTexCoord;

    TFaceColors=array[0..2]of PARGB; // for overroding point colors

    TFace=class
           public
            Points:TFacePoints;
            Coords:TFaceCoords;
            Normal:TPoint3S;
            Texture:TTexture;
            Colors:TFaceColors;

            Name:string;

//            constructor Create;
            destructor Destroy;override;

            procedure Draw;

            procedure SetColor(Idx:LongInt;Col:TARGB);
            procedure SetColors(Col:TARGB);
            procedure FreeColor(Idx:LongInt);
            procedure FreeColors;

            procedure CalcNormal;

           end;



implementation

uses GGraphic,DrawBuffers;

destructor TFace.Destroy;
begin
 FreeColors;
 inherited Destroy;
end;

procedure TFace.Draw;
var Cols:array[0..2]of TARGB;
begin

 if Colors[0]=nil then
 Cols[0]:=Points[0].Color
 else
 Cols[0]:=Colors[0]^;
 if Colors[1]=nil then
 Cols[1]:=Points[1].Color
 else
 Cols[1]:=Colors[1]^;
 if Colors[2]=nil then
 Cols[2]:=Points[2].Color
 else
 Cols[2]:=Colors[2]^;

 Renderer.DrawFace([Points[0].Pos,Points[1].Pos,Points[2].Pos],
  [Coords[0],Coords[1],Coords[2]],Cols,Texture,@Normal);

end;

procedure TFace.SetColors(Col:TARGB);
var I:LongInt;
begin
  for I:=0 to 2 do
   SetColor(I,Col);
end;

procedure TFace.SetColor(Idx:LongInt;Col:TARGB);
var PCol:PARGB;
begin
  FreeColor(Idx);
  GetMem(PCol,SizeOf(TARGB));
  PCol^:=Col;
  Colors[Idx]:=PCol;
end;

procedure TFace.FreeColors;
var I:LongInt;
begin
  for I:=0 to 2 do
   FreeColor(I);
end;

procedure TFace.FreeColor(Idx:LongInt);
begin
  if Colors[Idx]<>nil then
   FreeMem(Colors[Idx]);

 Colors[Idx]:=nil;
end;

procedure TFace.CalcNormal;
var S1,S2:TPoint3S;
begin
 if Points[0]=nil then Exit;
 if Points[1]=nil then Exit;
 if Points[2]=nil then Exit;

 S1 := SubPoint(Points[1].Pos, Points[0].Pos);
 S2 := SubPoint(Points[2].Pos, Points[0].Pos);

  Normal := VectorCross(S1, S2);

  Normalize(Normal);

end;



end.
