unit Walls;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,Objs3D,Types3D,
     Utils3D,Colors,Textures,BaseMgrs,Points,Faces,ContNRS;

type

    TWallAlign=(waStrip,waFan);


    TWall=class(TBaseObj)
           private
            FFaces:TObjectList;
            FAlign:TWallAlign;
            FTexture:TTexture;
            FColor:TARGB;

           protected

            procedure SetTexture(Value:TTexture);virtual;
            procedure SetColor(Value:TARGB);virtual;


           public

             constructor Create;override;
             destructor Destroy;override;

             procedure Draw;virtual;

             //           P1--------P3
             //           |         |
             //           P2--------P4
             procedure SetStripWall(P1,P2,P3,P4:TFacePoint);
             // must be called after SetStripWall
             //           U1V1--------U2V1
             //            |           |
             //           U1V2--------U2V2
             procedure SetStripCoords(U1,V1,U2,V2:single);


             procedure SetFanWall(Center:TFacePoint;Pts:array of TFacePoint);
             // X,Y-corner of wall;  Us,Vs - texture repeats after Us,Vs points
             procedure SetFanCoords(X,Y,UScale,VScale:single);

              // sets normals for all faces
             procedure SetNormal(N:TPoint3S);

             procedure SplitFace(F:TFace);

             property Texture:TTexture read FTexture write SetTexture;
             property Color:TARGB read FColor write SetColor;

             property Faces:TObjectList read FFaces; 

           published


    end;

    TWallMgr= class(TBaseMgr)
             protected
              procedure SetWall(Idx:LongInt;Value:TWall);
              function GetWall(Idx:LongInt):TWall;

             public
             property Objects[Index: LongInt]: TWall read GetWall write SetWall;default;
             end;


implementation

uses GPortal;

///////////////////////  TWALLMGR

procedure TWallMgr.SetWall(Idx:LongInt;Value:TWall);
begin
 SetObj(Idx,Value);
end;

function TWallMgr.GetWall(Idx:LongInt):TWall;
begin
 Result:=TWall(GetObj(Idx));
end;


//////////////////////  TWALL

constructor TWall.Create;
begin
 inherited Create;
 FFaces:=TObjectList.Create;
 FAlign:=waStrip;

end;

destructor TWall.Destroy;
begin
 Done;

 FFaces.Free;
 inherited Done;
end;

procedure TWall.SetTexture(Value:TTexture);
var I:LongInt;
begin
 FTexture:=Value;

 for I:=0 to FFaces.Count-1 do
  TFace(FFaces[I]).Texture:=FTexture;
end;

procedure TWall.SetColor(Value:TARGB);
var I:LongInt;
begin
 FColor:=Value;

 for I:=0 to FFaces.Count-1 do
  TFace(FFaces[I]).SetColors(FColor);
end;

procedure TWall.Draw;
var I:LongInt;
begin
 for I:=0 to FFaces.Count-1 do
  TFace(FFaces[I]).Draw;


end;


procedure TWall.SetStripWall(P1,P2,P3,P4:TFacePoint);
var F1,F2:TFace;
begin
 FAlign:=waStrip;

 FFaces.Clear;

 F1:=TFace.Create;
 F2:=TFace.Create;

 FFaces.Add(F1);
 FFaces.Add(F2);

 F1.Points[0]:=P1;
 F1.Points[1]:=P3;
 F1.Points[2]:=P2;

 F2.Points[0]:=P2;
 F2.Points[1]:=P3;
 F2.Points[2]:=P4;


 F1.CalcNormal;
 F2.CalcNormal;

end;

procedure TWall.SetStripCoords(U1,V1,U2,V2:single);
var F1,F2:TFace;
begin
 F1:=TFace(FFaces[0]);
 F2:=TFace(FFaces[1]);

 if (F1=nil)or(F2=nil)then Exit;

 F1.Coords[0]:=Coord(U1,V1);
 F1.Coords[1]:=Coord(U2,V1);
 F1.Coords[2]:=Coord(U1,V2);

 F2.Coords[0]:=Coord(U1,V2);
 F2.Coords[1]:=Coord(U2,V1);
 F2.Coords[2]:=Coord(U2,V2);

end;

procedure TWall.SetFanWall(Center:TFacePoint;Pts:array of TFacePoint);
var I:LongInt;
    F:TFace;
begin
FFaces.Clear;
FAlign:=waFan;

for I:=Low(Pts) to High(Pts) do
 begin
  F:=TFace.Create;
  F.Points[2]:=Pts[I];
  F.Points[1]:=Center;
  if I<>High(Pts) then
  F.Points[0]:=Pts[I+1]
  else
  F.Points[0]:=Pts[0];

  F.CalcNormal;
  FFaces.Add(F);

 end;

end;

procedure TWall.SetFanCoords(X,Y,UScale,VScale:single);
var I,J:LongInt;
    F:TFace;
begin

for I:=0 to FFaces.Count-1 do
 begin
  F:=TFace(FFaces[I]);
  for J:=0 to 2 do
   F.Coords[J]:=Coord((F.Points[J].X-X)/UScale,(F.Points[J].Y-Y)/VScale);
 end;


end;

procedure TWall.SetNormal(N:TPoint3S);
var I:LongInt;
begin
 for I:=0 to FFaces.Count-1 do
  TFace(FFaces[I]).Normal:=N;
end;

procedure TWall.SplitFace(F:TFace);
var mx,d1,d2,d3:single;
    NewP:TPoint3S;
    New:TFacePoint;
    dx,dy,dz:Double;
    F1,F2:TFace;
    I1,I2,I3:LongInt;
begin
 if FFaces.IndexOf(F)<0 then Exit;

 FFaces.Extract(F);

 d1:=PointDistance(F.Points[0].Pos,F.Points[1].Pos);
 d2:=PointDistance(F.Points[1].Pos,F.Points[2].Pos);
 d3:=PointDistance(F.Points[2].Pos,F.Points[0].Pos);

 if FAlign=waFan then
 begin
 mx:=Min(d1,d2);;
 mx:=Min(d3,mx);
 end;
 if FAlign=waStrip then
 begin
 mx:=Max(d1,d2);;
 mx:=Max(d3,mx);
 end;

 if mx=d1 then
  begin
   I1:=0;
   I2:=1;
   I3:=2
  end;
 if mx=d2 then
  begin
   I1:=1;
   I2:=2;
   I3:=0
  end;
 if mx=d3 then
  begin
   I1:=2;
   I2:=0;
   I3:=1
  end;

  NewP:=Point3S(F.Points[I1].Pos.X+(F.Points[I2].Pos.X-F.Points[I1].Pos.X)/2,
  F.Points[I1].Y+(F.Points[I2].Y-F.Points[I1].Y)/2,
  F.Points[I1].Z+(F.Points[I2].Z-F.Points[I1].Z)/2);


  New:=TFacePoint.Create(NewP.X,NewP.Y,NewP.Z);
  New.Color:=AvgColor(F.Points[I1].Color,F.Points[I2].Color);
  Portal.AddPoint(New);

  F1:=TFace.Create;
  F1.Points[0]:=F.Points[I1];
  F1.Points[1]:=New;
  F1.Points[2]:=F.Points[I3];
  F1.Texture:=F.Texture;
  F1.Coords[0]:=F.Coords[I1];
  F1.Coords[1]:=Coord((F.Coords[I1].X+F.Coords[I2].X)/2,(F.Coords[I1].Y+F.Coords[I2].Y)/2);
  F1.Coords[2]:=F.Coords[I3];

  F1.Normal:=F.Normal;

  F2:=TFace.Create;
  F2.Points[0]:=F.Points[I2];
  F2.Points[2]:=New;
  F2.Points[1]:=F.Points[I3];
  F2.Texture:=F.Texture;
  F2.Coords[0]:=F.Coords[I2];
  F2.Coords[2]:=Coord((F.Coords[I1].X+F.Coords[I2].X)/2,(F.Coords[I1].Y+F.Coords[I2].Y)/2);
  F2.Coords[1]:=F.Coords[I3];

  F2.Normal:=F.Normal;


  FFaces.Add(F1);
  FFaces.Add(F2);

  F.Free;
  F:=nil;

end;

end.
