unit Fog;

interface

uses Windows,SysUtils,Classes,WinUtils,BaseObjs,Objs3D,Types3D,Utils3D,
     GfxTypes,Colors;

type

   TFog=class(TBaseObj)
        private
         FMode:TFogMode;
         FColor:TARGB;
         FStart:single;
         FEnd:single;
         FDensity:single;
         FEnabled:boolean;

        protected

          procedure SetEnabled(Value:boolean);virtual;
          procedure SetColor(Value:TARGB);virtual;
          procedure SetMode(Value:TFogMode);virtual;
          procedure SetStart(Value:single);virtual;
          procedure SetEnd(Value:single);virtual;
          procedure SetDensity(Value:single);virtual;

        public
          constructor Create(AMode:TFogMode=0); reintroduce;virtual;
          destructor Destroy; override;

          procedure Init;override;

          procedure Assign(Source:TBaseObj);override;

        published

          property Enabled:boolean read FEnabled write SetEnabled;
          property Mode:TFogMode read FMode write SetMode;
          property Color:TARGB read FColor write SetColor;
          property FogStart:single read FStart write SetStart;
          property FogEnd:single read FEnd write SetEnd;
          property Density:single read FDensity write SetDensity;


        end;



implementation

uses GGraphic;

constructor TFog.Create(AMode:TFogMode);
begin
 inherited Create;
 FEnabled:=true;
 FMode:=AMode;
 FColor:=$FFFFFFFF;
 FStart:=4500;
 FEnd:=5000;
 FDensity:=0.1;
 end;

destructor TFog.Destroy;
begin
 Done;
 inherited Destroy;
end;

procedure TFog.Init;
begin
 inherited Init;
 SetEnabled(FEnabled);
 SetMode(FMode);
 SetColor(FColor);
 SetStart(FStart);
 SetEnd(FEnd);
 SetDensity(FDensity);
end;

procedure TFog.SetEnabled(Value:boolean);
begin
 FEnabled:=Value;
 if FEnabled then
  Graphic.Enable(gfFog)
 else
  Graphic.Disable(gfFog);
end;

procedure TFog.SetColor(Value:TARGB);
begin
 FColor:=Value;
 Renderer.SetDWord(sdFogColor,FColor);
end;

procedure TFog.SetMode(Value:TFogMode);
begin
 FMode:=Value;
 Renderer.SetDWord(sdFogMode,FMode);
end;

procedure TFog.SetStart(Value:single);
begin
 FStart:=Value;
 Renderer.SetFloat(sfFogStart,FStart);
end;

procedure TFog.SetEnd(Value:single);
begin
 FEnd:=Value;
 Renderer.SetFloat(sfFogEnd,FEnd);
end;

procedure TFog.SetDensity(Value:single);
begin
 FDensity:=Value;
 Renderer.SetFloat(sfFogDensity,FDensity);
end;

procedure TFog.Assign(Source:TBaseObj);
var F:TFog;
begin
 inherited Assign(Source);
 if not(Source is TFog) then Exit;
 F:=TFog(Source);

 Enabled:=F.Enabled;
 Mode:=F.Mode;
 Color:=F.Color;
 FogStart:=F.FogStart;
 FogEnd:=F.FogEnd;
 Density:=F.Density;

end;


end.
