unit SceneEngine;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Objs3D, Types3D, Utils3D,
  DrawingEngines, GfxTypes, GMessages, BaseMgrs, Textures, Fog, Engines, Sky,
  SceneTypes,Lighting;

type

  TSceneEngine = class(TDrawingEngine)
  private
    FSceneName: string;
    FFog: TFog;
    FSky: TSky;
    FLighting:TLighting;
    FEnabled:boolean;
    FRoot:TBase3DObj;

    procedure DefaultOpts;

  protected
    procedure SetSceneName(Value: string); virtual;
    procedure SetFog(Value: TFog); virtual;
    procedure SetSky(Value: TSky); virtual;
    procedure SetLighting(Value: TLighting); virtual;
    procedure SetEnabled(Value:boolean);virtual;

    function Get3DObj(Idx: LongInt): TBase3DObj; virtual;

  public
    Opts:TSceneOpts;

    constructor Create(AOwner: TEngine = nil); override;
    destructor Destroy; override;

    class function GetInstance: TSceneEngine;

    procedure Init; override;
    procedure Done; override;
    procedure Draw; override;
    procedure Update;override;

    function Add(AObj:TBase3DObj;AParent:TBase3DObj=nil):TBase3DObj;

    procedure Enable(Opt:TSceneOpt);
    function IsEnabled(Opt:TSceneOpt):boolean;
    procedure Disable(Opt:TSceneOpt);

    property AllObjects[Index: LongInt]: TBase3DObj read Get3DObj;
    property Root:TBase3DObj read FRoot;

    property Fog: TFog read FFog write SetFog;
    property Sky: TSky read FSky write SetSky;
    property Lighting:TLighting read FLighting write SetLighting;

  published

    property SceneName: string read FSceneName write SetSceneName;
    property Enabled:boolean read FEnabled write SetEnabled ;


  end;


implementation

uses GGraphic,SceneUpdate;

var SE: TSceneEngine;

constructor TSceneEngine.Create(AOwner: TEngine);
begin
  inherited Create(AOwner);
  DefaultOpts;
  Objects.OwnsObjects:=false;
  UpdatePeriod:=0;

  FFog := TFog.Create(FOG_LINEAR);
  FSky := TSky.Create(smSkyNone);
  FLighting:=TLighting.Create;

  FRoot:=TBase3DObj.Create;
  FRoot.Name:='Scene Root';
  FRoot.MoveAbs(0,0,0);


end;


destructor TSceneEngine.Destroy;
begin
  Done;
  FRoot.Free;
  FFog.Free;
  FSky.Free;
  FLighting.Free;
  SE:=nil;
  inherited Destroy;
end;

procedure TSceneEngine.Init;
begin
  inherited Init;
  FEnabled:=true;
  FRoot.Init;
  FFog.Init;
  FSky.Init;
  FLighting.Init;
end;

procedure TSceneEngine.Done;
begin
  inherited Done;
  FSky.Done;
  FFog.Done;
  FLighting.Done;
  FRoot.Done;
end;

procedure TSceneEngine.DefaultOpts;
begin
 Opts[soOptimizing]:=true;
 Opts[soCulling]:=true;
 Opts[soSorting]:=true;
 Opts[soAlphaLast]:=true;
 Opts[soAlphaSorting]:=true;
end;

procedure TSceneEngine.Enable(Opt:TSceneOpt);
begin
 Opts[Opt]:=true;
end;

function TSceneEngine.IsEnabled(Opt:TSceneOpt):boolean;
begin
 Result:=Opts[Opt];
end;

procedure TSceneEngine.Disable(Opt:TSceneOpt);
begin
 Opts[Opt]:=false;
end;


class function TSceneEngine.GetInstance: TSceneEngine;
begin
  if SE = nil then
  begin
    SE := TSceneEngine.Create(Graphic);
    if Graphic.Inited then
      SE.Init;

  end;
  Result := SE;
end;

procedure TSceneEngine.SetSceneName(Value: string);
begin
  FSceneName := Value;
end;

procedure TSceneEngine.SetEnabled(Value:boolean);
begin
 FEnabled:=Value;
end;

function TSceneEngine.Get3DObj(Idx: LongInt): TBase3DObj;
begin
  Result := nil;
  if ((Idx > -1) and (Idx < Objects.Count)) then
    Result := TBase3DObj(Objects[idx]);
end;

procedure TSceneEngine.SetFog(Value: TFog);
begin
  FFog.Assign(Value);
end;

procedure TSceneEngine.SetSky(Value: TSky);
begin
  FSky.Assign(Value);
end;

procedure TSceneEngine.SetLighting(Value: TLighting);
begin
 FLighting.Assign(Value);
end;

procedure TSceneEngine.Draw;
begin
 TryUpdate;
end;

procedure TSceneEngine.Update;
begin
  inherited Update;
  Sky.Draw;

  UpdateScene(Self);
end;

function TSceneEngine.Add(AObj:TBase3DObj;AParent:TBase3DObj=nil):TBase3DObj;
begin
// Result:=nil;
 //if AObj=nil then Exit;
 //Objects.Add(AObj);

 if AParent=nil then
  Root.AddChild(AObj)
 else
  AParent.AddChild(AObj);

 Result:=AObj;
end;


end.

