unit SceneUpdate;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Objs3D, Types3D, Renderers,
  Utils3D, Colors, Textures, BaseMgrs, SceneEngine, VisObjs, SceneTypes;

procedure UpdateScene(S: TSceneEngine);

implementation

uses GGraphic;

type

  TSorting = (stFrontToBack, stBackToFront);

var St: TStringList;


procedure UpdateScene(S: TSceneEngine);
var AObj: TBase3DObj;
  I: LongInt;
  Mgr: TBaseMgr;
  AlphaMgr: TBaseMgr;
  CamPos: TPoint3S;
  Ren: TRenderer;
  BackPlane: single;
  Mat:TMatrix;

  procedure QuickSort(M: TBaseMgr; iLo, iHi: LongInt; St: TSorting);
  var
    Lo, Hi: LongInt;
    Mid: single;
    T: TBaseObj;
  begin
    Lo := iLo;
    Hi := iHi;
    Mid := TBase3DObj(M[(Lo + Hi) div 2]).Dist;
    repeat
      if St = stFrontToBack then
      begin
        while TBase3DObj(M[Lo]).Dist < Mid do
          Inc(Lo);
        while TBase3DObj(M[Hi]).Dist > Mid do
          Dec(Hi);
      end
      else
      begin
        while TBase3DObj(M[Lo]).Dist > Mid do
          Inc(Lo);
        while TBase3DObj(M[Hi]).Dist < Mid do
          Dec(Hi);
      end;
      if Lo <= Hi then
      begin
        T := M[Lo];
        M[Lo] := M[Hi];
        M[Hi] := T;
        Inc(Lo);
        Dec(Hi);
      end;
    until Lo > Hi;
    if Hi > iLo then QuickSort(M, iLo, Hi, St);
    if Lo < iHi then QuickSort(M, Lo, iHi, St);
  end;


  procedure CheckVisual(Obj: TBase3DObj);
  var I: LongInt;
    AAObj: TBase3DObj;

    function InFrustum:boolean;
    begin
     Result:=false;
     if Abs(CamPos.X - Obj.WorldPos.X) > BackPlane then Exit;
     if Abs(CamPos.Y - Obj.WorldPos.Y) > BackPlane then Exit;
     if Abs(CamPos.Z - Obj.WorldPos.Z) > BackPlane then Exit;
     if PointDistance(CamPos,Obj.WorldPos)>BackPlane then Exit;
     if not Obj.InFrustum then Exit;


     Result:=true;
    end;

  begin
    if S.Opts[soCulling] then
    begin
      if InFrustum then
        Mgr.Add(Obj)
      else
        if Obj.HideChildren then
          Exit;
    end
    else
      Mgr.Add(Obj);

    for I := 0 to Obj.ChildCount - 1 do
      CheckVisual(Obj.Child[I]);

  end;

  procedure GetVisuals;
  begin
    AObj := S.Root;
    CamPos := Camera.Position;
    BackPlane := ViewPort.BackPlane;
    CheckVisual(AObj);
  end;

  procedure SplitAlpha;
  var I: LongInt;
    O: TBase3DObj;
  begin
    I := 0;
    while I < Mgr.Count do
    begin
      if TBase3DObj(Mgr[I]).HasAlpha then
      begin
        O := TBase3DObj(Mgr[I]);
        Mgr.Extract(O);
        AlphaMgr.Add(O);
      end
      else
        Inc(I);
    end;
  end;

  procedure Sort(M: TBaseMgr; St: TSorting);
  var I: LongInt;
  begin
    if M.Count < 2 then Exit;
    for I := 0 to M.Count - 1 do
      TBase3DObj(M[I]).CalcDist;

    QuickSort(M, 0, M.Count - 1, St);
  end;

  procedure UpdateVisuals(M: TBaseMgr);
  var I: LongInt;
    P: TBase3DObj;
    O: TBase3DObj;
  begin
    Ren.WorldIdentity;
    for I := 0 to M.Count - 1 do
    begin
      O := TBase3DObj(M[I]);
      O.TryUpdate;

      P := O.Parent;
      if (P = nil) or (P = S.Root) then
      begin
        O.SetUpWorld;
        O.Draw;
        O.RestoreWorld;
      end
      else
      begin
        Ren.WorldPush;
        O.AChild := nil;
        P.AChild := O;
     // get root parent, store child path(do not need to browse all children)
        while P.Parent <> nil do
        begin
          P.Parent.AChild := P;
          P := P.Parent;
        end;

     // go down the tree and multiply world

        Ren.WorldMultiply(P.WolrdMatrix);
        while P.AChild <> nil do
        begin
          P := P.AChild;
         Ren.WorldMultiply(P.WolrdMatrix);
        end;

     // finaly draw our Obj

        O.Draw;

        Ren.WorldPop;
      end;


    end;

  end;


begin
  if S.Opts[soOptimizing] then
  begin

    Mgr := TBaseMgr.Create(false);
    Mgr.UniqueObjects := not true;
    AlphaMgr := TBaseMgr.Create(false);
    AlphaMgr.UniqueObjects :=not  true;
    Ren := Renderer;
    GetVisuals;

    if S.Opts[soAlphaLast] then
      SplitAlpha;

    if S.Opts[soSorting] then
      Sort(Mgr, stFrontToBack);

    if S.Opts[soAlphaSorting] then
      Sort(AlphaMgr, stBackToFront);

    UpdateVisuals(Mgr);

    if S.Opts[soAlphaLast] then
      UpdateVisuals(AlphaMgr);

    Mgr.Free;
    AlphaMgr.Free;

  end
  else
    S.Root.DoDrawAndUpdate;
end;


end.

