unit Sky;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Objs3D, Types3D, Utils3D,
  SkyBase, SkyDome;

type

  TSkyMode = (smSkyNone,smSkyBox, smSkyPlane, smSkyDome, smSkyCustom);

  TSky = class(TBase3DObj)
  private
    FMode: TSkyMode;
    FSkyBox: TSkyBox;
    FSkyDome: TSkyDome;
    FSkyPlane: TSkyPlane;
    FSkyCustom: TSkyBase;

  protected

    function GetActSky: TSkyBase; virtual;
    procedure SetMode(Value:TSkyMode);virtual;

  public
    constructor Create(AMode: TSkyMode = smSkyBox); reintroduce; virtual;
    destructor Destroy; override;

    procedure Init; override;
    procedure Done; override;

    procedure Draw; override;
    procedure Update; override;

    property ActiveSky: TSkyBase read GetActSky;
    property SkyBox: TSkyBox read FSkyBox;
    property SkyPlane:TSkyPlane read FSkyPlane;
    property SkyMode:TSkyMode read FMode write SetMode;

  published

  end;



implementation

constructor TSky.Create(AMode: TSkyMode);
begin
  inherited Create;
  FMode := AMode;
  FSkyBox := TSkyBox.Create;
  FSkyDome := TSkyDome.Create;
  FSkyPlane := TSkyPlane.Create;
  FSkyCustom:=TSkyBase.Create;
  UpdatePeriod := 20;
end;

destructor TSky.Destroy;
begin
  Done;
  FSkyBox.Free;
  FSkyDome.Free;
  FSkyPlane.Free;
  inherited Destroy;
end;

procedure TSky.Init;
begin
  inherited Init;
  if ActiveSky<>nil then
  ActiveSky.Init;

end;

procedure TSky.Done;
begin
  inherited Done;
  if ActiveSky<>nil then
  ActiveSky.Done;

end;

function TSky.GetActSky: TSkyBase;
begin
  Result := nil;
  case FMode of
    smSkyBox: Result := FSkyBox;
    smSkyPlane: Result := FSkyPlane;
    smSkyDome: Result := FSkyDome;
    smSkyCustom: Result := FSkyCustom;
    smSkyNone:Result:=nil;
  end;
end;

procedure TSky.SetMode(Value:TSkyMode);
begin
 if ActiveSky<>nil then
 ActiveSky.Done;
 FMode:=Value;
 if ActiveSky<>nil then
 ActiveSky.Init;
end;

procedure TSky.Draw;
begin
  if not Inited then Exit;
  TryUpdate;
  if ActiveSky<>nil then
  ActiveSky.Draw;
end;

procedure TSky.Update;
begin
  if not Inited then Exit;
  if ActiveSky<>nil then
  ActiveSky.Update;
end;












end.

