unit Images;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, BaseMgrs,
  Graphics, DIB, ZLib, FileIOEngine, FileRec, FileDirs, Colors;




type

  TLines = array[Word] of Pointer; PLines = ^TLines;
  TLine8 = array[Word] of Byte; PLine8 = ^TLine8;
  TLine16 = array[Word] of Word; PLine16 = ^TLine16;
  TLine24 = array[Word] of TRGBRec; PLine24 = ^TLine24;
  TLine32 = array[Word] of TARGBRec; PLine32 = ^TLine32;
  TPixels8 = array[Word] of PLine8; PPixels8 = ^TPixels8;
  TPixels16 = array[Word] of PLine16; PPixels16 = ^TPixels16;
  TPixels24 = array[Word] of PLine24; PPixels24 = ^TPixels24;
  TPixels32 = array[Word] of PLine32; PPixels32 = ^TPixels32;

  EImageErr = class(Exception)
  end;

  TImgFormat = class;

  TImage = class(TBaseObj)
  private
    FDIB: TDIB;
    FOrig: TMemoryStream;
    FFormat: TImgFormat;
    FName: string;
    FStoreOrig: boolean;
    FTransparent: boolean;
    FTransColor: TARGB;
    FTransMode: TTransparentMode;
    FUseAlpha: boolean;
      // use alpha blits when drawing;uses alpha component when drawing lines,rects
    FAlphaFromFile: boolean; // loaded file was 32bit
    FAlpha: byte;
    FMultOnSet: boolean;
      // multiply alpha when loading or setting alpha channel;changes orig picture
    FGap: LongInt;
    FBpp: LongInt;
    FBiResize: boolean;

    procedure SetGap;
    function GetChannel(Idx: byte; GreyScale: boolean): TImage;
    procedure SetChannel(Idx: byte; Img: TImage; Val: byte);

  protected

    function GetBits: LongInt;
    function GetSize(idx: LongInt): LongInt;
    function GetEmpty: boolean;
    function GetCanvas: TCanvas;
    function GetRect: TRect;
    function GetTransColor: Dword;
    function GetHandle: THandle;
    function GetDC: THandle;
    function GetHasAlpha: boolean;
    function GetPBits: Pointer;
    function GetPixel(X, Y: LongInt): DWord;
    function GetPixelARGB(X, Y: LongInt): TARGB;
    function GetScanLine(Y: LongInt): Pointer;
    function GetCanRevert: boolean;
    function GetGap: LongInt;
    function GetColorT: TRGBQuads;
    function GetColorTP: PRGBQuads;
    function GetSizeB: LongInt;
    function GetWBytes: LongInt;

    procedure SetStoreOrig(Value: boolean);
    procedure SetSize(idx, Value: LOngInt);
    procedure SetBits(Value: LongInt);
    procedure SetAlpha(Value: byte);
    procedure SetMultOnSet(Value: boolean);
    procedure SetPixel(X, Y: LongInt; Col: DWord);
    procedure SetPixelARGB(X, Y: LongInt; Col: TARGB);
    procedure SetColorT(Value: TRGBQuads);

  public
    constructor Create(AName: string = ''); reintroduce; virtual;
    destructor Destroy; override;

    procedure Load(AName: string);overload;
    procedure Load(AStream: TStream);overload;
    procedure Save(AName: string = ''); overload;
    procedure Save(Stream: TStream); overload;
    procedure Revert;
    procedure Done; override;

    procedure Draw(Dest: TImage; X, Y: LongInt); overload;
    procedure Draw(Dest: HDC; X, Y: LongInt); overload;
    procedure DrawOn(Dest: TImage; X, Y: LOngInt); overload;
      // uses transparent,usealpha
    procedure DrawOn(Dest: TImage; DestR: TRect; SrcR: TRect); overload;
      // uses transparent,usealpha
    procedure DrawOn(Dest: HDC; X, Y: LongInt); overload;
      // uses transparent,usealpha
    procedure DrawOn(Dest: HDC; DestR: TRect; SrcR: TRect); overload;
      // uses transparent,usealpha
    procedure DrawStretch(Dest: TImage; DestR: TRect; SrcR: TRect); overload;
    procedure DrawTransparent(Dest: TImage; DestR: TRect; SrcR: TRect; Col: TARGB
      = 0); overload;
    procedure DrawAlpha(Dest: TImage; DestR: TRect; SrcR: TRect); overload;
    procedure DrawStretch(Dest: HDC; DestR: TRect; SrcR: TRect); overload;
    procedure DrawTransparent(Dest: HDC; DestR: TRect; SrcR: TRect; Col: TARGB =
      0); overload;
    procedure DrawAlpha(Dest: HDC; DestR: TRect; SrcR: TRect); overload;

    procedure Line(X1, Y1, X2, Y2: LongInt; Col: TARGB);
    procedure Ellipse(X, Y, XSize, YSize: LongInt; Col: TARGB);
    procedure Circle(X, Y, Size: LongInt; Col: TARGB);
    procedure Rectangle(DestR: TRect; Col: TARGB); overload;
    procedure Rectangle(X1, Y1, X2, Y2: LOngInt; Col: TARGB); overload;
    procedure FillRect(DestR: tRect; Col: TARGB); overload;
    procedure FillRect(X1, Y1, X2, Y2: LongInt; Col: TARGB); overload;

    function GetAlphaChannel: TImage;
    function GetRedChannel(GreyPal: boolean = false): TImage;
    function GetGreenChannel(GreyPal: boolean = false): TImage;
    function GetBlueChannel(GreyPal: boolean = false): TImage;

      // remove alpha loaded from png/tga ; change bitcount to 32
    procedure SetAlphaChannel(Alpha: byte); overload; // constant alpha
    procedure SetAlphaChannel(AName: string); overload; // from 8bit image file
    procedure SetAlphaChannel(Img: TImage); overload; // from 8bit image
    procedure MultiplyAlpha;
         //for proper alpha draw blending !!!!>do not call more than one per image


    procedure SetRedChannel(Red: byte); overload;
    procedure SetRedChannel(AName: string); overload;
    procedure SetRedChannel(Img: TImage); overload;
    procedure SetGreenChannel(Green: byte); overload;
    procedure SetGreenChannel(AName: string); overload;
    procedure SetGreenChannel(Img: TImage); overload;
    procedure SetBlueChannel(Blue: byte); overload;
    procedure SetBlueChannel(AName: string); overload;
    procedure SetBlueChannel(Img: TImage); overload;


    procedure ConvertBpp(Bpp: LongInt);
    procedure Resize(W, H: LongInt);
    procedure Assign(Source: TBaseObj); override;
    function Clone: TImage; // create new image - constructor
    procedure UpdatePal; // for 4/8 bit images ; call after colortable change
    function CreateScans: PLines; // for some procs in Imgutils
    procedure CopyRect(Src: TImage; DestR: TRect; sx, sy: LongInt);
    procedure ClearARGB(Col: TARGB); // clear image  use argbcolor
    procedure ClearDWord(Col: DWord); // direct value is used
    procedure Rotate(Angle: LongInt); // 90,180,270,360  degrees rotation

    property DIB: TDIB read FDIB;
    property StoreOrig: boolean read FStoreOrig write SetStoreOrig;

    property Format: TImgFormat read FFormat write FFormat;
      //fotmat of image loaded from file
    property Handle: THandle read GetHandle; //  hBITMAP
    property DC: THandle read GetDC;
    property Width: LongInt index 0 read GetSize write SetSize;
    property Height: LongInt index 1 read GetSize write SetSize;
    property BitCount: LongInt read GetBits write SetBits;
    property Empty: boolean read GetEmpty;
    property Canvas: TCanvas read GetCanvas;
    property Bounds: TRect read GetRect; // for fill rect,...
    property Transparent: boolean read FTransparent write FTransparent;
    property TransColor: TARGB read FTransColor write FTransColor;
    property TransMode: TTransparentMode read FTransMode write FTransMode;
    property UseAlpha: boolean read FUseAlpha write FUseAlpha;
    property AlphaFromFile: boolean read FAlphaFromFile;
    property HasAlpha: boolean read GetHasAlpha;
    property MasterAlpha: byte read FAlpha write FAlpha; //used in drawalpha
    property Bits: Pointer read GetPBits;
    property MultiplyAlphaOnSet: boolean read FMultOnSet write SetMultOnSet;
    property Pixels[X, Y: LongInt]: DWord read GetPixel write SetPixel;
      //to get/set direct values
    property PixelsARGB[X, Y: LongInt]: TARGB read GetPixelARGB write
    SetPixelARGB; //to get/set argb values
    property ScanLine[Y: LongInt]: Pointer read GetScanLine;
    property CanRevert: boolean read GetCanRevert;
    property FileName: string read FName write FName;
    property Gap: LongInt read GetGap;
    property ColorTable: TRGBQuads read GetColorT write SetColorT;
    property PColorTable: PRGBQuads read GetColorTP;
    property Size: LongInt read GetSizeB; // size of raw img data in bytes
    property WidthBytes: LongInt read GetWBytes;
    property BilinearResize: boolean read FBiResize write FBiResize;
      // use bilinear filter when resizing


  published

  end;

  TImgLoc = (ilFile, ilStream);

  TImgLoad = function(Path: string; Stream: TStream; Loc: TImgLoc): TDIB;
  TImgSave = procedure(DIB: TDIB; Path: string; Stream: TStream; Loc: TImgLoc);



  TImgFormat = class
  public
    Exts: TStringList;
    Desc: string;

    LoadFunc: TImgLoad;
    SaveFunc: TImgSave;

    destructor Destroy; override;

  end;


const
  sImageErr = 'Error in TImage';


var
  ImgFormats: TBaseMgr;
  ImageList: TBaseMgr;



procedure RegisterImgFormat(Exts: array of string; Desc: string; Load: TImgLoad;
  Save: TImgSave);
function FindFormat(Ext: string): TImgFormat;

implementation

uses
  ImgUtils,
  ImgBMP, //default supported image formats
  ImgJPG,
  ImgPNG,
  ImgGIF,
  ImgICO,
  ImgTGA;


var FE: TFileIOEngine;


function IsSame(IF1, IF2: TImgFormat): boolean;
begin
  Result := Iff(CompStr(IF1.Desc, IF2.Desc), True, False);
end;

procedure RegisterImgFormat(Exts: array of string; Desc: string; Load: TImgLoad;
  Save: TImgSave);
var I: LongInt;
  ImgF: TImgFormat;
  Already: boolean;
begin
  if ImgFormats = nil then
    ImgFormats := TBaseMgr.Create;

  ImgF := TImgFormat.Create;
  ImgF.Exts := StrListFromArr(Exts);
  ImgF.Desc := Desc;
  ImgF.LoadFunc := Load;
  ImgF.SaveFunc := Save;

  Already := false;
  for I := 0 to ImgFormats.Count - 1 do
    if IsSame(TImgFormat(ImgFormats[I]), ImgF) then Already := true;

  if not Already then
    ImgFormats.Add(TBaseObj(ImgF))
  else
    ImgF.Free;

end;

function FindFormat(Ext: string): TImgFormat;
var J, K: LongInt;
begin
  Result := nil;
  for J := 0 to ImgFormats.Count - 1 do
    for K := 0 to TImgFormat(ImgFormats[J]).Exts.Count - 1 do
      if CompStr(Ext, TImgFormat(ImgFormats[J]).Exts[K]) then
      begin
        Result := TImgFormat(ImgFormats[J]);
        Exit;
      end;
end;


destructor TImgFormat.Destroy;
begin
  Exts.Free;
  inherited Destroy;
end;


constructor TImage.Create(AName: string);
begin
  inherited Create;
  FStoreOrig := false;
  FAlpha := 255;
  FUseAlpha := true;
  FMultOnSet := false;
  ImageList.Add(Self);
  FOrig := TMemoryStream.Create;
  FBiResize := not true;
  FDIB := TDIB.Create;
  FDIB.SetSize(8, 8, 24);
  ImgUtils.AddNoise(Self, 255, false);
  Name := 'Image' + IntToStr(ImageList.IndexOf(Self));
  SetGap;
  if AName <> '' then
    Load(AName);

end;

destructor TImage.Destroy;
begin

  ImageList.Extract(Self);
  FOrig.Free;
  FDIB.Free;
  inherited Destroy;
end;

procedure TImage.SetStoreOrig(Value: boolean);
begin
  FStoreOrig := Value;
  if not Value then
    FOrig.Clear;
end;



procedure TImage.Load(AName: string);
var I: LongInt;
  Ext: string;
  FFile: TFileRec;
  OL, L: LongInt;
  Data: Pointer;
  InList: boolean;
  Back: TDIB;

  procedure PackOrig;
  begin
    if FFile = nil then Exit;
    FOrig.Clear;
    Data := nil;
    WriteStr(FOrig, FFile.Name);
    L := FFile.Size;
    ZLIB.CompressBuf(FFile.Data, L, Data, OL);
    FOrig.WriteBuffer(OL, SizeOf(OL));
    FOrig.WriteBuffer(Data^, OL);
    FreeMem(Data);
  end;


begin
  InList := false;
  FFile := nil;
  Back := TDIB.Create;
  Back.Assign(FDIB);

  for I := 0 to ImageList.Count - 1 do
    if TImage(ImageList[I]).FileName = AName then
    begin
      FDIB.Assign(TImage(ImageList[I]).FDIB);
      FFormat := TImage(ImageList[I]).FFormat;
      FName := AName;
      InList := true;
      Break;
    end;


  if not InList then
  begin

    FE := TFileIOEngine.GetInstance;
    FFile := FE.GetFile(AName);
    Ext := ExtractFileExt(AName);
    FDIB.Free;
    FDIB := nil;

    for I := 0 to ImgFormats.Count - 1 do
      if StrInList(Ext, TImgFormat(ImgFormats[I]).Exts) then
      begin
        FDIB := TImgFormat(ImgFormats[I]).LoadFunc('', FFile.Stream, ilStream);
        FFormat := TImgFormat(ImgFormats[I]);
        FName := AName;
        Break;
      end;

    if StoreOrig then
      PackOrig;

    FFile.Free;

    if FDIB = nil then
    begin
      MsgBox(sImageErr + '.Load - Unknown Image Format "' + ExtractFileExt(AName)
        + '"');
      FDIB := TDIB.Create;
      FDIB.Assign(Back);
    end;

    Back.Free;

    if FMultOnSet then
      MultiplyAlpha;

  end;

  if BitCount = 32 then
    FAlphaFromFile := true;

  SetGap;


end;

procedure TImage.Load(AStream: TStream);
begin

if FFormat=nil then Exit;

FDIB.Free;
FDIB := nil;
FDIB := FFormat.LoadFunc('', AStream, ilStream);

if FMultOnSet then
  MultiplyAlpha;

  if BitCount = 32 then
    FAlphaFromFile := true;

  SetGap;

end;

procedure TImage.Save(AName: string);
var UsedF: TImgFormat;
  Ext: string;
begin
  if Empty then Exit;
  if AName = '' then
    AName := FName;
  Ext := ExtractFileExt(AName);
  if Ext = '' then
    UsedF := FFormat
  else
    UsedF := FindFormat(Ext);

  FE := TFileIOEngine.GetInstance;

  if (UsedF = nil) then
  begin
    UsedF := FindFormat('.bmp');
    ChangeFileExt(AName, UsedF.Exts[0]);
  end;

  if (@UsedF.SaveFunc = nil) then
  begin
    UsedF := FindFormat('.bmp');
    ChangeFileExt(AName, UsedF.Exts[0]);
  end;

  if Ext = '' then
    AName := AName + ExtractFileExt(FName);

  UsedF.SaveFunc(FDIB, AName, nil, ilFile);

end;

procedure TImage.Save(Stream: TStream);
var UsedF: TImgFormat;
begin
  if FFormat = nil then Exit;
  if @FFormat.SaveFunc = nil then
    UsedF := FindFormat('.bmp')
  else
    UsedF := FFormat;

  UsedF.SaveFunc(FDIB, '', Stream, ilStream);

end;


procedure TImage.Revert;
var FFile: TFileRec;

  procedure UnpackOrig;
  var Size, CSize: LongInt;
    Data, CData: Pointer;
  begin
    FOrig.Position := 0;
    FFile.Name := ReadStr(FOrig);
    FOrig.ReadBuffer(CSize, SizeOf(CSize));
    GetMem(CData, CSize);
    FOrig.ReadBuffer(CData^, CSize);
    ZLIB.DecompressBuf(CData, CSize, 0, Data, Size);
    FFile.Stream.WriteBuffer(Data^, Size);
    FFile.Stream.Position := 0;
    FreeMem(CData);
    FreeMem(Data);
  end;


begin
  if FOrig.Size = 0 then Exit;

  FFile := TFileREc.Create();
  UnPackOrig;
  FDIB.Free;
  FDIB := FFormat.LoadFunc('', FFile.Stream, ilStream);

  FFile.Free;

  SetGap;

end;

function TImage.GetSize(idx: LongInt): LongInt;
begin
  Result := 0;
  if Empty then Exit;
  case idx of
    0: Result := FDIB.Width;
    1: Result := FDIB.Height;
  end;

end;


procedure TImage.SetSize(idx, Value: LOngInt);
var W, H: LongInt;
begin
  W := Width;
  H := Height;
  case idx of
    0: W := Value;
    1: H := Value;
  end;

  Resize(W, H);

end;

procedure TImage.SetBits(Value: LongInt);
begin
  ConvertBpp(Value);
end;

function TImage.GetBits: LongInt;
begin
  Result := 0;
  if Empty then Exit;
  Result := FDIB.BitCount;
end;

procedure TImage.SetMultOnSet(Value: boolean);
begin
  if FMultOnSet = Value then Exit;
  FMultOnSet := Value;
  if Value then
    MultiplyAlpha;

end;

procedure TImage.Resize(W, H: LongInt);
var D: TDIB;
begin
  if (W=Width)and(H=Height)then Exit;

  D := TDIB.Create;
  D.Assign(FDIB);
  D.SetSize(W, H, BitCount);
  D.Canvas.StretchDraw(D.Canvas.ClipRect, FDIB);
  FDIB.Assign(D);
  D.Free;
  SetGap;
  if BilinearResize then
    Blur(Self, BitCount, 1);
  //  QuickSoft(Self);
end;

procedure TImage.Assign(Source: TBaseObj);
begin
  if Source is TImage then
  begin
    FDIB.Assign(TImage(Source).DIB);
    FName := TImage(Source).FName;
    FFormat := TImage(Source).FFormat;
    FStoreOrig := TImage(Source).FStoreOrig;
    FOrig.LoadFromStream(TImage(Source).FOrig);
    FGap := TImage(Source).FGap;
    Self.FTransparent := TImage(Source).FTransparent;
    Self.FTransColor := TImage(Source).FTransColor;
    Self.FTransMode := TImage(Source).FTransMode;
    Self.FUseAlpha := TImage(Source).FUseAlpha;
    Self.FAlphaFromFile := TImage(Source).FAlphaFromFile;
    Self.FAlpha := TImage(Source).FAlpha;
    Self.FMultOnSet := TImage(Source).FMultOnSet;
    Self.FGap := TImage(Source).FGap;
    Self.FBpp := TImage(Source).FBpp;

  end
  else
    inherited Assign(Source);

end;

function TImage.Clone: TImage;
var Img: TImage;
begin
  Img := TImage.Create;
  Img.Assign(Self);
  Result := Img;

end;


function TImage.GetEmpty: boolean;
begin
  Result := FDIB.Empty;
end;

function TImage.GetPBits: Pointer;
begin
  Result := FDIB.pBits;
end;



function TImage.GetCanvas: TCanvas;
begin
  Result := nil;
  if Empty then Exit;
  Result := FDIB.Canvas;

end;

function TImage.GetRect: TRect;
begin
  Result := Rect(0, 0, Width, Height);
end;

function TImage.GetTransColor: Dword;
begin
  Result := 0;
  if Empty then Exit;
  if FTransMode = tmAuto then
    Result := FDIB.Pixels[0, Height - 1]
  else
    Result := FTransColor;

end;

function TImage.GetHandle: THandle;
begin
  Result := 0;
  if Empty then Exit;
  Result := FDIB.Handle;
end;

function TImage.GetDC: THandle;
begin
  Result := 0;
  if Empty then Exit;
  Result := Canvas.Handle;
end;

function TImage.GetHasAlpha: boolean;
begin
  Result := BitCount = 32;
end;

procedure TImage.SetAlpha(Value: byte);
begin
  FAlpha := Value;
end;

procedure TImage.ConvertBpp(Bpp: LongInt);
var TB: TBitmap;

begin
  if BitCount = Bpp then Exit;
  if (BitCount > 8) and (Bpp <= 8) then
  begin
    TB := TBitmap.Create;
    TB.PixelFormat := pf24bit;
    TB.Width := Width;
    TB.Height := height;
    TB.Canvas.Draw(0, 0, FDIB);
    case Bpp of
      4: TB.PixelFormat := pf4bit;
      8: TB.PixelFormat := pf8bit;
    end;
    FDIB.Assign(TB);
    TB.Free;
  end
  else
    FDIB.BitCount := Bpp;

  if BitCount = 32 then
    SetAlphaChannel($FF);

  SetGap;

end;


procedure TImage.DrawOn(Dest: TImage; X, Y: LongInt);
begin
  if Empty then Exit;
  DrawOn(Dest, Rect(X, Y, X + Width, Y + Height), Bounds);
end;

procedure TImage.DrawOn(Dest: TImage; DestR: TRect; SrcR: TRect);
begin
  if Empty then Exit;
  if (not Transparent) and ((not UseAlpha) or (not HasAlpha)) then
    DrawStretch(Dest, DestR, SrcR)
  else
    if (Transparent) and ((not UseAlpha) or (not HasAlpha)) then
      DrawTransparent(Dest, DestR, SrcR)
    else
      if UseAlpha and HasAlpha then
        DrawAlpha(Dest, DestR, SrcR);
end;

procedure TImage.DrawOn(Dest: HDC; X, Y: LongInt);
begin
  if Empty then Exit;
  DrawOn(Dest, Rect(X, Y, X + Width, Y + Height), Bounds);
end;

procedure TImage.DrawOn(Dest: HDC; DestR: TRect; SrcR: TRect);
begin
  if Empty then Exit;
  if (not Transparent) and ((not UseAlpha) or (not HasAlpha)) then
    DrawStretch(Dest, DestR, SrcR)
  else
    if (Transparent) and ((not UseAlpha) or (not HasAlpha)) then
      DrawTransparent(Dest, DestR, SrcR)
    else
      if UseAlpha and HasAlpha then
        DrawAlpha(Dest, DestR, SrcR);
end;


procedure TImage.Draw(Dest: TImage; X, Y: LongInt);
begin
  if Empty then Exit;
  BitBlt(Dest.DC, x, y, Width, Height, DC, 0, 0, SRCCOPY);
end;

procedure TImage.Draw(Dest: HDC; X, Y: LongInt);
begin
  if Empty then Exit;
  BitBlt(Dest, x, y, Width, Height, DC, 0, 0, SRCCOPY);
end;


procedure TImage.DrawStretch(Dest: TImage; DestR: TRect; SrcR: TRect);
var p: pointer;
begin
  if Empty then Exit;
  p := FDib.BitmapInfo;
  StretchDIBits(Dest.DC, DestR.Left, DestR.top, DestR.Right - DestR.Left,
    DestR.Right - DestR.Left,
    SrcR.Left, SrcR.Top, SrcR.Right - SrcR.Left, SrcR.Bottom - SrcR.Top, Bits,
    PBitmapInfo(p)^, 0, SRCCOPY)
end;



procedure TImage.DrawTransparent(Dest: TImage; DestR: TRect; SrcR: TRect; Col:
  TARGB);
begin
  if Empty then Exit;
  if Col = 0 then
    Col := GetTransColor;
  TransparentBlt(Dest.DC, DestR.Left, DestR.Top, DestR.Right - DestR.Left,
    DestR.Bottom - DestR.Top,
    DC, SrcR.Left, SrcR.Top, SrcR.Right - SrcR.Left, SrcR.Bottom - SrcR.Top,
    ARGBToBGR(Col));

end;

procedure TImage.DrawAlpha(Dest: TImage; DestR: TRect; SrcR: TRect);
var
  Blend: TBlendFunction;
  Img: TImage;
begin
  if Empty then Exit;
  Blend.BlendOp := 0;
  Blend.BlendFlags := $00;
  Blend.SourceConstantAlpha := FAlpha;
  Blend.AlphaFormat := Ord(HasAlpha);
  Img := Self;
  if HasAlpha and (not FMultOnSet) then
  begin
    Img := Clone;
    Img.MultiplyAlpha;
  end;
  AlphaBlend(Dest.DC, DestR.Left, DestR.Top, DestR.Right - DestR.Left,
    DestR.Bottom - DestR.Top,
    Img.DC, SrcR.Left, SrcR.Top, SrcR.Right - SrcR.Left, SrcR.Bottom - SrcR.Top,
    Blend);

  if Img <> Self then
    Img.Free;

end;

procedure TImage.DrawStretch(Dest: HDC; DestR: TRect; SrcR: TRect);
var p: pointer;
begin
  if Empty then Exit;
  p := FDib.BitmapInfo;
  StretchDIBits(Dest, DestR.Left, DestR.top, DestR.Right - DestR.Left,
    DestR.Right - DestR.Left,
    SrcR.Left, SrcR.Top, SrcR.Right - SrcR.Left, SrcR.Bottom - SrcR.Top, Bits,
    PBitmapInfo(p)^, 0, SRCCOPY)
end;


procedure TImage.DrawTransparent(Dest: HDC; DestR: TRect; SrcR: TRect; Col:
  TARGB);
begin
  if Empty then Exit;
  if Col = 0 then
    Col := GetTransColor;
  TransparentBlt(Dest, DestR.Left, DestR.Top, DestR.Right - DestR.Left,
    DestR.Bottom - DestR.Top,
    DC, SrcR.Left, SrcR.Top, SrcR.Right - SrcR.Left, SrcR.Bottom - SrcR.Top,
    ARGBToBGR(Col));

end;

procedure TImage.DrawAlpha(Dest: HDC; DestR: TRect; SrcR: TRect);
var
  Blend: TBlendFunction;
  Img: TImage;
begin
  if Empty then Exit;
  Blend.BlendOp := 0;
  Blend.BlendFlags := $00;
  Blend.SourceConstantAlpha := FAlpha;
  Blend.AlphaFormat := Ord(HasAlpha);
  Img := Self;
  if HasAlpha and (not FMultOnSet) then
  begin
    Img := Clone;
    Img.MultiplyAlpha;
  end;
  AlphaBlend(Dest, DestR.Left, DestR.Top, DestR.Right - DestR.Left, DestR.Bottom
    - DestR.Top,
    Img.DC, SrcR.Left, SrcR.Top, SrcR.Right - SrcR.Left, SrcR.Bottom - SrcR.Top,
    Blend);

  if Img <> Self then
    Img.Free;

end;


procedure TImage.MultiplyAlpha;
var
  pc: PARGBRec;
  x, y, i: Integer;
begin
  if Empty then Exit;
  if not HasAlpha then Exit;
  pc := Pointer(Bits);
  for y := 0 to Height - 1 do
  begin
    for x := 0 to Width - 1 do
    begin
      i := pc.a;
      if i = 0 then
      begin
        pc.b := 0;
        pc.g := 0;
        pc.r := 0;
      end
      else
        if i < 255 then
        begin
          pc.b := (pc.b * i) shr 8;
          pc.g := (pc.g * i) shr 8;
          pc.r := (pc.r * i) shr 8;
        end;
      Inc(pc);
    end;
  end;
end;

function TImage.GetScanLine(Y: LongInt): Pointer;
begin
  Result := FDIB.ScanLine[Y];
end;

function TImage.GetPixel(X, Y: LongInt): DWord;
begin
  Result := FDIB.Pixels[X, Y];
end;

procedure TImage.SetPixel(X, Y: LongInt; Col: DWord);
begin
  FDIB.Pixels[X, Y] := Col;
end;

function TImage.GetPixelARGB(X, Y: LongInt): TARGB;
var DW: DWord;
  r, g, b: byte;
  ct: PRGBQuads;
begin
  DW := FDIB.Pixels[X, Y];
  Result := 0;
  case BitCount of
    4, 8:
      begin
        ct := PColorTable;
        Result := ARGB($00, ct[DW].rgbRed, ct[DW].rgbGreen, ct[DW].rgbBlue);
      end;
    16:
      begin
        pfGetRGB(FDIB.PixelFormat, DW, R, G, B);
        Result := ARGB($00, R, G, B);
      end;
    24, 32: Result := DW;
  end;

end;

procedure TImage.SetPixelARGB(X, Y: LongInt; Col: TARGB);
begin
  case BitCount of
    24, 32: FDIB.Pixels[X, Y] := Col;
    16:
      begin
        FDIB.Pixels[X, Y] := pfRGB(FDIB.PixelFormat, Col shr 16, Col shr 8,
          Col);
      end;
    4, 8:
      FDIB.Pixels[X, Y] := ClosestColor(PColorTable, (1 shl BitCount) - 1,
        Col);
  end;

end;

function TImage.GetCanRevert: boolean;
begin
  Result := FOrig.Size > 0;
end;

function TImage.GetGap: LongInt;
begin
  if FBpp <> BitCount then
    SetGap;
  Result := FGap;

end;

procedure TImage.SetGap;
var BW, x: LongInt;
begin
  FBpp := BitCount;

  case BitCount of
    4:
      begin
        x := ((Width shl 2) + 7) and -8;
        BW := ((x + 31) and -32) shr 3;
        FGap := BW - (x shr 3);
      end;
    8:
      begin
        BW := (((Width shl 3) + 31) and -32) shr 3;
        FGap := BW - Width;
      end;
    16:
      begin
        BW := (((Width shl 4) + 31) and -32) shr 3;
        FGap := BW - (Width shl 1);
      end;
    24:
      begin
        BW := (((Width * 24) + 31) and -32) shr 3;
        FGap := BW - ((Width shl 1) + Width);
      end;
    32:
      begin
        FGap := 0;
      end;
  end;

end;


procedure TImage.UpdatePal;
begin
  FDIB.UpdatePalette;
end;

function TImage.GetColorT: TRGBQuads;
begin
  Result := FDIB.ColorTable;
end;

procedure TImage.SetColorT(Value: TRGBQuads);
begin
  FDIB.ColorTable := Value;
  UpdatePal;

end;

function TImage.GetColorTP: PRGBQuads;
begin
  Result := @FDIB.ColorTable;
end;

function TImage.GetSizeB: LongInt;
begin
  Result := FDIB.Size;
end;

function TImage.GetWBytes: LongInt;
begin
  Result := FDIB.WidthBytes;
end;

function TImage.CreateScans: PLines;
var Scans: PLines;
  x, i: LongInt;
begin
  Result := nil;
  if Empty then Exit;

  GetMem(Scans, Height shl 2);
  x := Integer(Bits);
  for i := 0 to Height - 1 do
  begin
    Scans[i] := Ptr(x);
    Inc(x, WidthBytes);
  end;
  Result := Scans;

end;

procedure TImage.CopyRect(Src: TImage; DestR: TRect; sx, sy: LongInt);
var
  yy, ix, iy, pc, sc, bb: LongInt;
  x, y, w, h: LongInt;
  ScanLines: PLines;
  wd: word;
  ct: PRGBQuads;
  r, g, b, idx: byte;
  ssy, ssx: LongInt;
  Pix: TDIBPixelFormat;
  SScanLines: PLines;
begin
  if Empty then Exit;
  if Src.Empty then Exit;
  x := DestR.Left;
  y := DestR.Top;
  w := DestR.Right - x;
  h := DestR.Bottom - y;
  ssy := sy;
  ssx := sx;
  yy := y;
  if Height > 0 then y := Height - h - y;
  if Src.Height > 0 then sy := Src.Height - h - sy;
  if x < 0 then
  begin
    Dec(sx, x);
    Inc(w, x);
    x := 0;
  end;
  if y < 0 then
  begin
    Dec(sy, y);
    Inc(h, y);
    y := 0;
  end;
  if sx < 0 then
  begin
    Dec(x, sx);
    Inc(w, sx);
    sx := 0;
  end;
  if sy < 0 then
  begin
    Dec(y, sy);
    Inc(h, sy);
    sy := 0;
  end;
  if (sx < Src.Width) and (sy < Src.Height) and (x < Width) and (y < Height)
    then
  begin
    if w + sx >= Src.Width then w := Src.Width - sx;
    if h + sy >= Src.Height then h := Src.Height - sy;
    if w + x >= Width then w := Width - x;
    if h + y >= Height then h := Height - y;
    if (BitCount >= 8) and (BitCount = Src.BitCount) then
    begin
      ScanLines := CreateScans;
      SScanLines := Src.CreateScans;
      bb := w;
      case BitCount of
        16:
          begin
            bb := w shl 1;
            x := x shl 1;
            sx := sx shl 1;
          end;
        24:
          begin
            bb := w * 3;
            x := x * 3;
            sx := sx * 3;
          end;
        32:
          begin
            bb := w shl 2;
            x := x shl 2;
            sx := sx shl 2;
          end;
      end;
      pc := Integer(Scanlines[y]) + x;
      sc := Integer(SScanlines[sy]) + sx;
      for iy := 0 to h - 1 do
      begin
        Move(Ptr(sc)^, Ptr(pc)^, bb);
        Inc(pc, WidthBytes);
        Inc(sc, Src.WidthBytes);
      end;
      FreeMem(ScanLines);
      FreeMem(SScanLines);
    end
    else
      if (BitCount in [16, 24, 8, 4]) and (Src.BitCount in [4, 16, 24, 8, 32])
        then
        Canvas.CopyRect(DestR, Src.Canvas, Rect(ssx, ssy, w, h))
      else
        if (BitCount = 32) and (Src.BitCount = 24) then
          for ix := 0 to w - 1 do
            for iy := 0 to h - 1 do
              Pixels[x + ix, yy + iy] := SetA(Src.Pixels[ssx + ix, ssy + iy],
                255)
        else
          if (BitCount = 32) and (Src.BitCount = 8) then
          begin
            ct := Src.PColorTable;
            for ix := 0 to w - 1 do
              for iy := 0 to h - 1 do
              begin
                idx := Src.Pixels[ssx + ix, ssy + iy];
                Pixels[x + ix, yy + iy] := ($FF shl 24) or (ct[idx].rgbRed shl
                  16)
                  or (ct[idx].rgbGreen shl 8) or (ct[idx].rgbBlue);
              end;
          end
          else
            if (BitCount = 32) and (Src.BitCount = 16) then
            begin
              Pix := MakeDIBPixelFormat(5, 5, 5);
              for ix := 0 to w - 1 do
                for iy := 0 to h - 1 do
                begin
                  wd := Src.Pixels[ssx + ix, ssy + iy];
                  pfGetRGB(Pix, wd, r, g, b);
                  Pixels[x + ix, yy + iy] := ($FF shl 24) or (R shl 16) or (G
                    shl
                    8) or (B);
                end
            end
  end;
end;


procedure TImage.ClearARGB(Col: TARGB);
begin
  case BitCount of
    4, 8:
      ClearDWord(ClosestColor(PColorTable, (1 shl BitCount) - 1, Col));
    16: ClearDWord(pfRGB(FDIB.PixelFormat, Col shr 16, Col shr 8, Col));
    24, 32: ClearDWord(Col);

  end;
end;


procedure TImage.ClearDWord(Col: DWord);
var
  ScanLines: PLines;
  I: LongInt;
  pc: PRGBRec;
begin
  ScanLines := CreateScans;
  if BitCount <= 4 then Col := Col or Col shl 4;
  if BitCount <= 8 then
  begin
    Col := Col or Col shl 8;
    Col := Col or Col shl 16;
  end
  else
    if BitCount = 16 then Col := Col or Col shl 16;
  if BitCount = 24 then
  begin
    pc := Pointer(Bits);
    for i := 0 to Width - 1 do
    begin
      pc^ := PRGBRec(@Col)^;
      Inc(pc);
    end;
    for i := 1 to Height - 1 do
      Move(Bits^, Scanlines[i]^, WidthBytes - Gap);
  end
  else
  begin
    if Size <> 0 then
      FillMem(Bits, Size, Col)
    else
      for i := 0 to Height - 1 do
        FillMem(Scanlines[i], WidthBytes - Gap, Col);
  end;
  FreeMem(ScanLines);
end;


function TImage.GetAlphaChannel: TImage;
begin
  Result := GetChannel(1, true);
end;

function TImage.GetRedChannel(GreyPal: boolean): TImage;
begin
  Result := GetChannel(2, GreyPal);
end;

function TImage.GetGreenChannel(GreyPal: boolean): TImage;
begin
  Result := GetChannel(3, GreyPal);
end;

function TImage.GetBlueChannel(GreyPal: boolean): TImage;
begin
  Result := GetChannel(4, GreyPal);
end;


function TImage.GetChannel(Idx: byte; GreyScale: boolean): TImage;
var Img: TImage;
  x, y: LongInt;
  pca: PARGBRec;
  pc: PRGBRec;
  pb: PByte;
  pw: PWord;


  function CreateColors: TRGBQuads;
  var I: LongInt;
    T: TRGBQuads;
  begin
    FillMem(@T, SizeOf(T), 0);
    for I := 0 to 255 do
    begin
      case Idx of
        2: T[I].rgbRed := I;
        3: T[I].rgbGreen := I;
        4: T[I].rgbBlue := I;
      end;
    end;
    Result := T;
  end;

begin
  Result := nil;
  if Empty then Exit;

  if BitCount <= 8 then
    Img := Clone
  else
  begin
    Img := Clone;
    Img.BitCount := 8;
    if (Idx = 1) or GreyScale then
      Img.ColorTable := GreyScaleColorTable
    else
      Img.ColorTable := CreateColors;

    case Idx of
      1:
        if HasAlpha then
        begin
          pca := Pointer(Bits);
          pb := Pointer(Img.Bits);

          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
            begin
              pb^ := pca^.a;
              Inc(pca);
              Inc(pb);
            end;
            Inc(pb, Img.Gap);
          end;
        end;
      2:
        case BitCount of
          32:
            begin
              pca := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pca^.r;
                  Inc(pca);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
              end;
            end;
          24:
            begin
              pc := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pc^.r;
                  Inc(pc);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
                pc := Ptr(Integer(pc) + Img.Gap);
              end;
            end;
          16:
            begin
              pw := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pfGetRValue(FDIB.PixelFormat, pw^);
                  Inc(pw);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
                pw := Ptr(Integer(pw) + Img.Gap);
              end;
            end;
        end;
      3:
        case BitCount of
          32:
            begin
              pca := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pca^.g;
                  Inc(pca);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
              end;
            end;
          24:
            begin
              pc := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pc^.g;
                  Inc(pc);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
                pc := Ptr(Integer(pc) + Img.Gap);
              end;
            end;
          16:
            begin
              pw := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pfGetGValue(FDIB.PixelFormat, pw^);
                  Inc(pw);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
                pw := Ptr(Integer(pw) + Img.Gap);
              end;
            end;
        end;
      4:
        case BitCount of
          32:
            begin
              pca := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pca^.b;
                  Inc(pca);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
              end;
            end;
          24:
            begin
              pc := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pc^.b;
                  Inc(pc);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
                pc := Ptr(Integer(pc) + Img.Gap);
              end;
            end;
          16:
            begin
              pw := Pointer(Bits);
              pb := Pointer(Img.Bits);

              for y := 0 to Height - 1 do
              begin
                for x := 0 to Width - 1 do
                begin
                  pb^ := pfGetBValue(FDIB.PixelFormat, pw^);
                  Inc(pw);
                  Inc(pb);
                end;
                Inc(pb, Img.Gap);
                pw := Ptr(Integer(pw) + Img.Gap);
              end;
            end;
        end;
    end;
  end;
  Result := Img;
end;

procedure TImage.SetAlphaChannel(Alpha: byte);
begin
  SetChannel(1, nil, Alpha);
end;

procedure TImage.SetAlphaChannel(AName: string);
var Img: TImage;
begin
  if Empty then Exit;
  Img := TImage.Create;
  Img.Load(AName);
  SetChannel(1, Img, 0);
  Img.Free;
end;


procedure TImage.SetAlphaChannel(Img: TImage);
begin
  SetChannel(1, Img, 0);
end;


procedure TImage.SetChannel(Idx: byte; Img: TImage; Val: byte);
var Temp: TImage;
  pca: PARGBRec;
  pc: PRGBRec;
  pw: PWord;
  pb: PByte;
  x, y: LongInt;
  r, g, b: byte;

begin
  if Empty then Exit;
  Temp := nil;
  if Img <> nil then
  begin
    Temp := Img.Clone;
    Temp.Resize(Width, Height);
    if Temp.BitCount <> 8 then
      Temp.BitCount := 8;
  end;

  case Idx of
    1:
      begin
        if BitCount <> 32 then
          BitCount := 32;

        if Temp = nil then
        begin
          pca := Pointer(Bits);
          for y := 0 to Height - 1 do
            for x := 0 to Width - 1 do
            begin
              pca^.a := Val;
              Inc(pca);
            end;
        end
        else
        begin
          pb := Pointer(Temp.Bits);
          pca := Pointer(Bits);
          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
            begin
              pca^.a := pb^;
              Inc(pca);
              Inc(pb);
            end;
            Inc(pb, Temp.Gap);
          end;
        end;

        if FMultOnSet then
          MultiplyAlpha;
      end;
    2:
      begin
        if Temp = nil then
        begin
          pca := Pointer(Bits);
          pc := Pointer(Bits);
          pw := Pointer(Bits);
          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
              case BitCount of
                32:
                  begin
                    pca^.r := Val;
                    Inc(pca);
                  end;
                24:
                  begin
                    pc^.r := Val;
                    Inc(pc);
                  end;
                16:
                  begin
                    pfGetRGB(FDIB.PixelFormat, pw^, r, g, b);
                    pw^ := pfRGB(FDIB.PixelFormat, Val, g, b);
                    Inc(pw);
                  end;
              end;
            case BitCount of
              32: pca := Ptr(Integer(pca) + Gap);
              24: pc := Ptr(Integer(pc) + Gap);
              16: pw := Ptr(Integer(pw) + Gap);
            end;
          end;
        end
        else
        begin
          pca := Pointer(Bits);
          pc := Pointer(Bits);
          pw := Pointer(Bits);
          pb := Pointer(Temp.Bits);

          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
            begin
              case BitCount of
                32:
                  begin
                    pca^.r := pb^;
                    Inc(pca);
                  end;
                24:
                  begin
                    pc^.r := pb^;
                    Inc(pc);
                  end;
                16:
                  begin
                    pfGetRGB(FDIB.PixelFormat, pw^, r, g, b);
                    pw^ := pfRGB(FDIB.PixelFormat, pb^, g, b);
                    Inc(pw);
                  end;
              end;
              Inc(pb);
            end;
            case BitCount of
              32: pca := Ptr(Integer(pca) + Gap);
              24: pc := Ptr(Integer(pc) + Gap);
              16: pw := Ptr(Integer(pw) + Gap);
            end;
            Inc(pb, Temp.Gap);
          end;
        end;
      end;
    3:
      begin
        if Temp = nil then
        begin
          pca := Pointer(Bits);
          pc := Pointer(Bits);
          pw := Pointer(Bits);
          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
              case BitCount of
                32:
                  begin
                    pca^.g := Val;
                    Inc(pca);
                  end;
                24:
                  begin
                    pc^.g := Val;
                    Inc(pc);
                  end;
                16:
                  begin
                    pfGetRGB(FDIB.PixelFormat, pw^, r, g, b);
                    pw^ := pfRGB(FDIB.PixelFormat, r, Val, b);
                    Inc(pw);
                  end;
              end;
            case BitCount of
              32: pca := Ptr(Integer(pca) + Gap);
              24: pc := Ptr(Integer(pc) + Gap);
              16: pw := Ptr(Integer(pw) + Gap);
            end;
          end;
        end
        else
        begin
          pca := Pointer(Bits);
          pc := Pointer(Bits);
          pw := Pointer(Bits);
          pb := Pointer(Temp.Bits);
          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
            begin
              case BitCount of
                32:
                  begin
                    pca^.g := pb^;
                    Inc(pca);
                  end;
                24:
                  begin
                    pc^.g := pb^;
                    Inc(pc);
                  end;
                16:
                  begin
                    pfGetRGB(FDIB.PixelFormat, pw^, r, g, b);
                    pw^ := pfRGB(FDIB.PixelFormat, r, pb^, b);
                    Inc(pw);
                  end;
              end;
              Inc(pb);
            end;
            case BitCount of
              32: pca := Ptr(Integer(pca) + Gap);
              24: pc := Ptr(Integer(pc) + Gap);
              16: pw := Ptr(Integer(pw) + Gap);
            end;
            Inc(pb, Temp.Gap);
          end;
        end;
      end;
    4:
      begin
        if Temp = nil then
        begin
          pca := Pointer(Bits);
          pc := Pointer(Bits);
          pw := Pointer(Bits);
          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
              case BitCount of
                32:
                  begin
                    pca^.b := Val;
                    Inc(pca);
                  end;
                24:
                  begin
                    pc^.b := Val;
                    Inc(pc);
                  end;
                16:
                  begin
                    pfGetRGB(FDIB.PixelFormat, pw^, r, g, b);
                    pw^ := pfRGB(FDIB.PixelFormat, r, g, Val);
                    Inc(pw);
                  end;
              end;
            case BitCount of
              32: pca := Ptr(Integer(pca) + Gap);
              24: pc := Ptr(Integer(pc) + Gap);
              16: pw := Ptr(Integer(pw) + Gap);
            end;
          end;
        end
        else
        begin
          pca := Pointer(Bits);
          pc := Pointer(Bits);
          pw := Pointer(Bits);
          pb := Pointer(Temp.Bits);
          for y := 0 to Height - 1 do
          begin
            for x := 0 to Width - 1 do
            begin
              case BitCount of
                32:
                  begin
                    pca^.b := pb^;
                    Inc(pca);
                  end;
                24:
                  begin
                    pc^.b := pb^;
                    Inc(pc);
                  end;
                16:
                  begin
                    pfGetRGB(FDIB.PixelFormat, pw^, r, g, b);
                    pw^ := pfRGB(FDIB.PixelFormat, r, g, pb^);
                    Inc(pw);
                  end;
              end;
              Inc(pb);
            end;
            case BitCount of
              32: pca := Ptr(Integer(pca) + Gap);
              24: pc := Ptr(Integer(pc) + Gap);
              16: pw := Ptr(Integer(pw) + Gap);
            end;
            Inc(pb, Temp.Gap);
          end;
        end;
      end;
  end;
  Temp.Free;
end;

procedure TImage.SetRedChannel(Red: byte);
begin
  SetChannel(2, nil, Red);
end;

procedure TImage.SetRedChannel(AName: string);
var Img: TImage;
begin
  if Empty then Exit;
  Img := TImage.Create;
  Img.Load(AName);
  SetChannel(2, Img, 0);
  Img.Free;
end;

procedure TImage.SetRedChannel(Img: TImage);
begin
  SetChannel(2, Img, 0);
end;

procedure TImage.SetGreenChannel(Green: byte);
begin
  SetChannel(3, nil, Green);
end;

procedure TImage.SetGreenChannel(AName: string);
var Img: TImage;
begin
  if Empty then Exit;
  Img := TImage.Create;
  Img.Load(AName);
  SetChannel(3, Img, 0);
  Img.Free;
end;

procedure TImage.SetGreenChannel(Img: TImage);
begin
  SetChannel(3, Img, 0);
end;

procedure TImage.SetBlueChannel(Blue: byte);
begin
  SetChannel(4, nil, Blue);
end;

procedure TImage.SetBlueChannel(AName: string);
var Img: TImage;
begin
  if Empty then Exit;
  Img := TImage.Create;
  Img.Load(AName);
  SetChannel(4, Img, 0);
  Img.Free;
end;

procedure TImage.SetBlueChannel(Img: TImage);
begin
  SetChannel(4, Img, 0);
end;

procedure TImage.Line(x1, y1, x2, y2: LongInt; Col: TARGB);
begin
  ImgUtils.Line(Self, x1, y1, x2, y2, Col, UseAlpha);
end;

procedure TImage.Ellipse(X, Y, XSize, YSize: LongInt; Col: TARGB);
var xx, mx1, mx2, my1, my2: LongInt;
  aq, bq, dx, dy, r, rx, ry: LongInt;
begin
  case BitCount of
    32:
      if not UseAlpha then
        Col := SetA(Col, 255);
    24: Col := SetA(Col, 0);
    16: col := pfRGB(FDIB.PixelFormat, Col shr 16, Col shr 8, Col);
    4, 8: col := ClosestColor(PColorTable, (1 shl BitCount) - 1, Col);
  end;

  Pixels[x + xsize, y] := col;
  Pixels[x - xsize, y] := col;
  mx1 := x - xsize;
  mx2 := x + xsize;
  my1 := y;
  my2 := y;

  aq := xsize * xsize;
  bq := ysize * ysize;
  dx := aq shl 1;
  dy := bq shl 1;
  r := xsize * bq;
  rx := r shl 1;
  ry := 0;
  xx := xsize;

  while xx > 0 do
  begin
    if r > 0 then
    begin
      inc(my1);
      dec(my2);
      inc(ry, dx);
      dec(r, ry);
    end;
    if r <= 0 then
    begin
      dec(xx);
      inc(mx1);
      dec(mx2);
      dec(rx, dy);
      inc(r, rx);
    end;
    Pixels[mx1, my1] := col;
    Pixels[mx1, my2] := col;
    Pixels[mx2, my1] := col;
    Pixels[mx2, my2] := col;
  end;
end;

procedure TImage.Circle(X, Y, Size: LongInt; Col: TARGB);
begin
  Ellipse(X, Y, Size, Size, Col);
end;

procedure TImage.Rectangle(DestR: TRect; Col: TARGB);
begin
  Rectangle(DestR.Left, DestR.Top, DestR.Right, DestR.Bottom, Col);
end;

procedure TImage.Rectangle(X1, Y1, X2, Y2: LOngInt; Col: TARGB);
begin
  ImgUtils.PolyLine(Self, [Point(X1, Y1), Point(X2, Y1), Point(X2, Y2),
    Point(X1, Y2), Point(X1, Y1)], Col, UseAlpha);
end;

procedure TImage.FillRect(DestR: tRect; Col: TARGB);
begin
  FillRect(DestR.Left, DestR.Top, DestR.Right, DestR.Bottom, Col);
end;


procedure TImage.FillRect(X1, Y1, X2, Y2: LongInt; Col: TARGB);
var
  w, h: LongInt;
  iy, pc: LongInt;
  ScanLines: PLines;
begin

  if X1 > X2 then Swap(X1, X2);
  if X1 < 0 then X1 := 0;
  if X2 > Width then X2 := Width;

  if Y1 > Y2 then Swap(Y1, Y2);
  if Y1 < 0 then Y1 := 0;
  if Y2 > Height then Y2 := Height;

  if BitCount < 32 then
  begin
    Canvas.Brush.Color := ARGBToBGR(Col);
    Canvas.FillRect(Rect(x1, y1, x2, y2));
  end
  else
  begin
    if not UseAlpha then
      Col := SetA(Col, 255);
    w := x2 - x1;
    h := y2 - y1;
    y1 := Height - h - y1;
    ScanLines := CreateScans;
    w := w shl 2;
    x1 := x1 shl 2;
    pc := LongInt(Scanlines[y1]) + x1;
    for iy := 0 to h - 1 do
    begin
      FillMem(Ptr(pc), w, Col);
      Inc(pc, WidthBytes);
    end;
    FreeMem(ScanLines);

  end;
end;


procedure TImage.Done;
begin
  inherited Done;
end;

procedure TImage.Rotate(Angle: LongInt);
var

  bpp: LongInt;
  X, Y, Z: LongInt;
  way: LongInt;
  TempDIB: TDIB;

begin
  if Angle > 359 then
    Angle := Angle mod 360;

  if Angle < 359 then
    Angle := Angle mod 360;

  if Angle = 0 then Exit;
  if Abs(Angle) < 45 then Exit;

  if not IntInArray(Angle, [-360, -270, -180, -90, 90, 180, 270, 360]) then
    Exit;

  TempDib := TDIB.Create;
  bpp := BitCount div 8;

  Way := Angle div Abs(Angle);
  case Abs(Angle) of
    180:
      begin
        TempDib.SetSize(Width, Height, BitCount);
        for x := 0 to Width - 1 do
          for y := 0 to Height - 1 do
              TempDIB.Pixels[Width-1 - x, Height-1 - y] :=
                FDIB.Pixels[x, y];
        FDIB.Assign(TempDIB);
      end;
    270, 90:
      begin
        if Abs(Angle) = 270 then Way := -Way;
        TempDib.SetSize(Height, Width, BitCount);

        if way = -1 then
        begin
          for x := 0 to Width - 1 do
            for y := 0 to Height - 1 do
                TempDIB.Pixels[Height-1 - y, x] :=
                  FDIB.Pixels[x, y];
        end
        else
        begin
          for x := 0 to Width - 1 do
            for y := 0 to Height - 1 do
              TempDIB.Pixels[y, Width-1 - x] :=
                FDIB.Pixels[x, y];
        end;
        FDIB.Assign(TempDIB);
      end;
  end;
  TempDIB.Free;
end;







initialization
  ImageList := TBaseMgr.Create();
finalization

  ImageList.Free;
  ImgFormats.Free;

end.

