unit ImgUtils;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Images,
  Graphics, Colors, DIB;


type

  TMatrix5x5 = array[0..24] of SmallInt;

  TImgFilter = record
    Matrix: TMatrix5x5;
    Divisor: LongInt;
    Bias: LongInt;
  end;

  TLut = array[Byte] of Byte; PLut = ^TLut;
  TWLut = array[Word] of Word; PWLut = ^TWLut;
  TSLut = array[Word] of Integer; PSLut = ^TSLut;
  PSaturationLut = ^TSaturationLut;
  TSaturationLut = record
    Grays: array[0..767] of Integer;
    Alpha: array[Byte] of Word;
  end;

  TFilterMode = (fmNormal, fmMix50, fmMix25, fmMix75);

  TLightSource = record
    X, Y: Integer;
    Size1, Size2: Integer;
    Color: TARGB;
  end;

  TLightArray = array of TLightsource;

procedure Greyscale(Img: TImage; ABitCount: LongInt); //bpp 4,8,16,24,32
procedure Blur(Img: TImage; ABitCount: LongInt; Radius: LongInt);
  //bpp 4,8,16,24,32
procedure Negative(Img: TImage); //bpp 4,8,16,24,32
procedure Mirror(Img: TImage; MirrorX, MirrorY: Boolean); //bpp 4,8,16,24,32
procedure ApplyFilter(Img: TImage; Filter: TImgFilter; FilterAlpha: boolean =
  false); //bpp 4,8,16,24,32
procedure AddNoise(Img: TImage; Amount: Byte; Mono: Boolean; AlphaNoise: boolean
  = false); //8,16,24,32 bpp
function CountColors(Img: TImage): DWord; //bpp 4,8,16,24,32
procedure ShiftImage(Img: TImage; xAmount, yAmount: LongInt); //all bpp
procedure QuickSoft(Img: TImage; FilterAlpha: boolean = false); // 8,24,32 bpp
procedure QuickSharp(Img: TImage; FilterAlpha: boolean = false); // 8,24,32 bpp
procedure QuickEmboss(Img: TImage; FilterAlpha: boolean = false); // 8,24,32 bpp
procedure DrawTiles(Src, Dst: TImage; TW: longInt = 0; TH: LongInt = 0);
  // 4,8,16,24,32 bpp
function ClosestColor(Pal: PRGBQuads; Max: LongInt; col: TARGB): Byte;
procedure Mosaic(Img: TImage; xAmount, yAmount: LongInt); // 8,16,24,32 bpp
procedure DrawLightmap(Img: TImage; Size: LongInt; UseAlpha: boolean = false);
  // 8,16,24,32 bpp
function CreateLightMap(Size: LongInt; Col: TARGB = $FFFFFFFF; UseAlpha: boolean
  = false): TImage; // result is 8bpp
procedure ApplyLut(Img: TImage; RLut, GLut, BLut: TLut); // 4,8,24,32 bpp
procedure Gamma(Img: TImage; r, g, b: Extended); // 4,8,24,32 bpp
procedure Addition(Img: TImage; r, g, b: Integer); // 4,8,24,32 bpp
procedure Contrast(Img: TImage; r, g, b: Integer); // 4,8,24,32 bpp
procedure Lightness(Img: TImage; r, g, b: Integer); // 4,8,24,32 bpp
procedure ApplySaturationLut(Img: TImage; Lut: TSaturationLut); // 4,8,24,32 bpp
procedure Saturation(Img: TImage; Amount: Integer);
  // 4,8,24,32 bpp; orig saturation =255
procedure AutoContrast(Img: TImage);
procedure GradientFill(Img: TImage; DestR: TRect; tl, tr, bl, br: TARGB;
  UseAlpha: boolean = false); //8,24,32 bpp
procedure Line(Img: TImage; x1, y1, x2, y2: Integer; col: TARGB; UseAlpha:
  boolean = false); // 8,16,24,32 bpp
procedure Polyline(Img: TImage; pnts: array of TPoint; col: TARGB; UseAlpha:
  boolean = false); // 8,16,24,32 bpp

// 24,32 bpp drawing functions
procedure DrawAdditive(Dest, Src: TImage; X, Y: LongInt; SrcR: TRect);
procedure DrawDarken(Dest, Src: TImage; X, Y: LongInt; SrcR: TRect);
procedure DrawShadow(Dest, Src: TImage; X, Y: LongInt; SrcR: TRect;
  FilterMode: TFilterMode);
procedure DrawTranslucent(Dest, Src: Images.TImage; X, Y: LongInt; SrcR: TRect;
  Col: TARGB);
procedure FilterLine(Dest: TImage; X1, Y1, X2, Y2: Integer; Col: TARGB;
  FilterMode: TFilterMode);
procedure FilterRect(Dest: TImage; DestR: TRect; Col: TARGB;
  FilterMode: TFilterMode);
procedure InitLight(Count, Detail: Integer);
procedure DrawLights(Dest: TImage; FLight: TLightArray;
  AmbientLight: TARGB);
procedure DrawTransparent(Dest, Src: Timage; X, Y: LongInt; SrcR: TRect;
  Col: TARGB);


procedure AddImage(Dest, Src: TImage; UseAlpha: boolean = false);
procedure SubImage(Dest, Src: TImage; UseAlpha: boolean = false);
procedure MulImage(Dest, Src: TImage; UseAlpha: boolean = false);
procedure OrImage(Dest, Src: TImage; UseAlpha: boolean = false);
procedure AndImage(Dest, Src: TImage; UseAlpha: boolean = false);
procedure DarkestImage(Dest, Src: TImage; UseAlpha: boolean = false);
procedure LightestImage(Dest, Src: TImage; UseAlpha: boolean = false);

procedure SwapColors(Img: TImage; Col, NewCol: TARGB; UseAlpha: boolean =
  false);
procedure FastBlur(Img: TImage; UseAlpha: boolean = false);
procedure SwapRGB(Img: TImage); // for 24,32 bit
function GreyscaleColorTable: TRGBQuads;


const

  ifSoften: TImgFilter = (Matrix:
    (1, 1, 1, 1, 1,
    1, 2, 2, 2, 1,
    1, 2, 0, 2, 1,
    1, 2, 2, 2, 1,
    1, 1, 1, 1, 1); Divisor: 24; Bias: 0);

  ifSoftenMore: TImgFilter = (Matrix:
    (2, 2, 2, 2, 2,
    2, 4, 4, 4, 2,
    2, 4, 0, 4, 2,
    2, 4, 4, 4, 2,
    2, 2, 2, 2, 2); Divisor: 64; Bias: 0);

  ifBlur: TImgFilter = (Matrix:
    (2, 2, 2, 2, 2,
    2, 3, 3, 3, 2,
    2, 3, 10, 3, 2,
    2, 3, 3, 3, 2,
    2, 2, 2, 2, 2); Divisor: 66; Bias: 0);

  ifLaPlace: TImgFilter = (Matrix: // find all  filter in psp
    (0, 0, 0, 0, 0,
    0, -1, -1, -1, 0,
    0, -1, 8, -1, 0,
    0, -1, -1, -1, 0,
    0, 0, 0, 0, 0); Divisor: 1; Bias: 0);

  ifFindEdges: TImgFilter = (Matrix:
    (0, 0, 0, 0, 0,
    0, 1, 1, 1, 0,
    0, 1, -2, 1, 0,
    0, -1, -1, -1, 0,
    0, 0, 0, 0, 0); Divisor: 1; Bias: 0);

  ifSharpen: TImgFilter = (Matrix:
    (-1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1,
    -1, -1, 32, 1, -1,
    -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1); Divisor: 10; Bias: 0);

  ifEmboss: TImgFilter = (Matrix:
    (0, 0, 0, 0, 0,
    0, 1, 0, 1, 0,
    0, 0, 0, 0, 0,
    0, 1, 0, -2, 0,
    0, 0, 0, 0, 0); Divisor: 1; Bias: 0);

  ifHardEmboss: TImgFilter = (Matrix:
    (3, 0, 0, 0, 3,
    0, 1, 0, 1, 0,
    0, 0, 0, 0, 0,
    0, 1, 0, -2, 0,
    3, 0, 0, 0, -6); Divisor: 4; Bias: 0);

  ifLinear3: TImgFilter = (Matrix:
    (0, 0, 0, 0, 0,
    0, 1, 1, 1, 0,
    0, 1, 1, 1, 0,
    0, 1, 1, 1, 0,
    0, 0, 0, 0, 0); Divisor: 9; Bias: 0);

  ifSoftenLess: TImgFilter = (Matrix:
    (0, 0, 0, 0, 0,
    0, 0, 1, 0, 0,
    0, 1, 2, 1, 0,
    0, 0, 1, 0, 0,
    0, 0, 0, 0, 0); Divisor: 6; Bias: 0);

  ifLinear5: TImgFilter = (Matrix:
    (1, 1, 1, 1, 1,
    1, 1, 1, 1, 1,
    1, 1, 1, 1, 1,
    1, 1, 1, 1, 1,
    1, 1, 1, 1, 1); Divisor: 25; Bias: 0);

  ifLighten: TImgFilter = (Matrix:
    (0, 0, 0, 0, 0,
    0, 0, 0, 0, 0,
    0, 0, 2, 0, 0,
    0, 0, 0, 0, 0,
    0, 0, 0, 0, 0); Divisor: 1; Bias: 0);

  ifDarken: TImgFilter = (Matrix:
    (0, 0, 0, 0, 0,
    0, 0, 0, 0, 0,
    0, 0, 1, 0, 0,
    0, 0, 0, 0, 0,
    0, 0, 0, 0, 0); Divisor: 2; Bias: 0);

implementation

function Set255(Clr: LongInt): LongInt;
asm
  MOV  EAX,Clr
  CMP  EAX,254
  JG   @SETHI
  CMP  EAX,1
  JL   @SETLO
  RET
  @SETHI:
  MOV  EAX,255
  RET
@SETLO:
  MOV  EAX,0
end;


function ClosestColor(Pal: PRGBQuads; Max: LongInt; col: TARGB): Byte;
var
  n: Byte;
  pc: PARGBRec;
  c: TRGBRec;
  i, x, d: LongInt;
begin
  c := RGBRec(Col);
  x := 765; n := 0;
  pc := Pointer(Pal);
  for i := 0 to Max do
  begin
    if pc.b > c.b then
      d := pc.b - c.b
    else
      d := c.b - pc.b;
    if pc.g > c.g then
      Inc(d, pc.g - c.g)
    else
      Inc(d, c.g - pc.g);
    if pc.r > c.r then
      Inc(d, pc.r - c.r)
    else
      Inc(d, c.r - pc.r);
    if d < x then
    begin
      x := d;
      n := i;
    end;
    Inc(pc);
  end;
  Result := n;
end;


procedure Greyscale(Img: TImage; ABitCount: LongInt);
var
  YTblR, YTblG, YTblB: array[0..255] of Byte;
  i, j, x, y: LongInt;
  c: DWORD;
  R, G, B: Byte;
  Temp: TDIB;
  DestP, SrcP: Pointer;
  P: PByte;
begin
  if Img.Empty then exit;

  with Img.DIB do
  begin


    Temp := TDIB.Create;
    try
      Temp.Assign(Img.DIB);
      SetSize(Width, Height, ABitCount);

      if ABitCount <= 8 then
      begin
        FillChar(ColorTable, SizeOf(ColorTable), 0);
        for i := 0 to (1 shl ABitCount) - 1 do
        begin
          j := i * (1 shl (8 - ABitCount));
          j := j or (j shr ABitCount);
          ColorTable[i] := RGBQuad(j, j, j);
        end;
        UpdatePalette;
      end;

      for i := 0 to 255 do
      begin
        YTblR[i] := Trunc(0.3588 * i);
        YTblG[i] := Trunc(0.4020 * i);
        YTblB[i] := Trunc(0.2392 * i);
      end;

      c := 0;

      try
        for y := 0 to Height - 1 do
        begin
          DestP := ScanLine[y];
          SrcP := Temp.ScanLine[y];

          for x := 0 to Width - 1 do
          begin
            case Temp.BitCount of
              4:
                begin
                  with Temp.ColorTable[(PArrayByte(SrcP)[X shr 1] and Mask4[X
                    and
                      1]) shr Shift4[X and 1]] do
                    c := YTblR[rgbRed] + YTblG[rgbGreen] + YTblB[rgbBlue];
                end;
              8:
                begin
                  with Temp.ColorTable[PByte(SrcP)^] do
                    c := YTblR[rgbRed] + YTblG[rgbGreen] + YTblB[rgbBlue];
                  Inc(PByte(SrcP));
                end;
              16:
                begin
                  pfGetRGB(Temp.NowPixelFormat, PWord(SrcP)^, R, G, B);
                  c := YTblR[R] + YTblR[G] + YTblR[B];
                  Inc(PWord(SrcP));
                end;
              24:
                begin
                  with PBGR(SrcP)^ do
                    c := YTblR[R] + YTblG[G] + YTblB[B];
                  Inc(PBGR(SrcP));
                end;
              32:
                begin
                  pfGetRGB(Temp.NowPixelFormat, PDWORD(SrcP)^, R, G, B);
                  c := YTblR[R] + YTblR[G] + YTblR[B];
                  Inc(PDWORD(SrcP));
                end;
            end;

            case BitCount of
              4:
                begin
                  P := @PArrayByte(DestP)[X shr 1];
                  P^ := (P^ and Mask4n[X and 1]) or ((c shr 4) shl Shift4[X and
                    1]);
                end;
              8:
                begin
                  PByte(DestP)^ := c;
                  Inc(PByte(DestP));
                end;
              16:
                begin
                  PWord(DestP)^ := pfRGB(NowPixelFormat, c, c, c);
                  Inc(PWord(DestP));
                end;
              24:
                begin
                  with PBGR(DestP)^ do
                  begin
                    R := c;
                    G := c;
                    B := c;
                  end;
                  Inc(PBGR(DestP));
                end;
              32:
                begin
                  PDWORD(DestP)^ := pfRGB(NowPixelFormat, c, c, c);
                  Inc(PDWORD(DestP));
                end;
            end;
          end;

        end;
      finally
      end;
    finally
      Temp.Free;
    end;
  end;
end;

procedure Blur(Img: TImage; ABitCount: LongInt; Radius: LongInt);
type
  TAve = record
    cR, cG, cB: DWORD;
    c: DWORD;
  end;
  TArrayAve = array[0..0] of TAve;

var
  Temp: TDIB;

  procedure AddAverage(Y, XCount: LongInt; var Ave: TArrayAve);
  var
    X: LongInt;
    SrcP: Pointer;
    AveP: ^TAve;
    R, G, B: Byte;
  begin
    case Temp.BitCount of
      4:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            with Temp.ColorTable[(PByte(LongInt(SrcP) + X shr 1)^ and Mask4[x
              and
                1]) shr Shift4[x and 1]], AveP^ do
            begin
              Inc(cR, rgbRed);
              Inc(cG, rgbGreen);
              Inc(cB, rgbBlue);
              Inc(c);
            end;
            Inc(AveP);
          end;
        end;
      8:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            with Temp.ColorTable[PByte(SrcP)^], AveP^ do
            begin
              Inc(cR, rgbRed);
              Inc(cG, rgbGreen);
              Inc(cB, rgbBlue);
              Inc(c);
            end;
            Inc(PByte(SrcP));
            Inc(AveP);
          end;
        end;
      16:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            pfGetRGB(Temp.NowPixelFormat, PWord(SrcP)^, R, G, B);
            with AveP^ do
            begin
              Inc(cR, R);
              Inc(cG, G);
              Inc(cB, B);
              Inc(c);
            end;
            Inc(PWord(SrcP));
            Inc(AveP);
          end;
        end;
      24:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            with PBGR(SrcP)^, AveP^ do
            begin
              Inc(cR, R);
              Inc(cG, G);
              Inc(cB, B);
              Inc(c);
            end;
            Inc(PBGR(SrcP));
            Inc(AveP);
          end;
        end;
      32:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            pfGetRGB(Temp.NowPixelFormat, PDWORD(SrcP)^, R, G, B);
            with AveP^ do
            begin
              Inc(cR, R);
              Inc(cG, G);
              Inc(cB, B);
              Inc(c);
            end;
            Inc(PDWORD(SrcP));
            Inc(AveP);
          end;
        end;
    end;
  end;

  procedure DeleteAverage(Y, XCount: LongInt; var Ave: TArrayAve);
  var
    X: LongInt;
    SrcP: Pointer;
    AveP: ^TAve;
    R, G, B: Byte;
  begin
    case Temp.BitCount of
      4:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            with Temp.ColorTable[(PByte(LongInt(SrcP) + X shr 1)^ and Mask4[x
              and
                1]) shr Shift4[x and 1]], AveP^ do
            begin
              Dec(cR, rgbRed);
              Dec(cG, rgbGreen);
              Dec(cB, rgbBlue);
              Dec(c);
            end;
            Inc(AveP);
          end;
        end;
      8:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            with Temp.ColorTable[PByte(SrcP)^], AveP^ do
            begin
              Dec(cR, rgbRed);
              Dec(cG, rgbGreen);
              Dec(cB, rgbBlue);
              Dec(c);
            end;
            Inc(PByte(SrcP));
            Inc(AveP);
          end;
        end;
      16:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            pfGetRGB(Temp.NowPixelFormat, PWord(SrcP)^, R, G, B);
            with AveP^ do
            begin
              Dec(cR, R);
              Dec(cG, G);
              Dec(cB, B);
              Dec(c);
            end;
            Inc(PWord(SrcP));
            Inc(AveP);
          end;
        end;
      24:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            with PBGR(SrcP)^, AveP^ do
            begin
              Dec(cR, R);
              Dec(cG, G);
              Dec(cB, B);
              Dec(c);
            end;
            Inc(PBGR(SrcP));
            Inc(AveP);
          end;
        end;
      32:
        begin
          SrcP := Pointer(LongInt(Temp.TopPBits) + Y * Temp.NextLine);
          AveP := @Ave;
          for x := 0 to XCount - 1 do
          begin
            pfGetRGB(Temp.NowPixelFormat, PDWORD(SrcP)^, R, G, B);
            with AveP^ do
            begin
              Dec(cR, R);
              Dec(cG, G);
              Dec(cB, B);
              Dec(c);
            end;
            Inc(PDWORD(SrcP));
            Inc(AveP);
          end;
        end;
    end;
  end;

  procedure Blur_Radius_Other;
  var
    FirstX, LastX, FirstX2, LastX2, FirstY, LastY: LongInt;
    x, y, x2, y2, jx, jy: LongInt;
    Ave: TAve;
    AveX: ^TArrayAve;
    DestP: Pointer;
    P: PByte;
  begin
    GetMem(AveX, Img.Width * SizeOf(TAve));
    try
      FillChar(AveX^, Img.Width * SizeOf(TAve), 0);

      FirstX2 := -1;
      LastX2 := -1;
      FirstY := -1;
      LastY := -1;

      x := 0;
      for x2 := -Radius to Radius do
      begin
        jx := x + x2;
        if (jx >= 0) and (jx < Img.Width) then
        begin
          if FirstX2 = -1 then FirstX2 := jx;
          if LastX2 < jx then LastX2 := jx;
        end;
      end;

      y := 0;
      for y2 := -Radius to Radius do
      begin
        jy := y + y2;
        if (jy >= 0) and (jy < Img.Height) then
        begin
          if FirstY = -1 then FirstY := jy;
          if LastY < jy then LastY := jy;
        end;
      end;

      for y := FirstY to LastY do
        AddAverage(y, Temp.Width, AveX^);

      for y := 0 to Img.Height - 1 do
      begin
        DestP := Img.DIB.ScanLine[y];

        {  The average is updated.  }
        if y - FirstY = Radius + 1 then
        begin
          DeleteAverage(FirstY, Temp.Width, AveX^);
          Inc(FirstY);
        end;

        if LastY - y = Radius - 1 then
        begin
          Inc(LastY);
          if LastY >= Img.Height then LastY := Img.Height - 1;
          AddAverage(LastY, Temp.Width, AveX^);
        end;

        {  The average is calculated again.  }
        FirstX := FirstX2;
        LastX := LastX2;

        FillChar(Ave, SizeOf(Ave), 0);
        for x := FirstX to LastX do
          with AveX[x] do
          begin
            Inc(Ave.cR, cR);
            Inc(Ave.cG, cG);
            Inc(Ave.cB, cB);
            Inc(Ave.c, c);
          end;

        for x := 0 to Img.Width - 1 do
        begin
          {  The average is updated.  }
          if x - FirstX = Radius + 1 then
          begin
            with AveX[FirstX] do
            begin
              Dec(Ave.cR, cR);
              Dec(Ave.cG, cG);
              Dec(Ave.cB, cB);
              Dec(Ave.c, c);
            end;
            Inc(FirstX);
          end;

          if LastX - x = Radius - 1 then
          begin
            Inc(LastX);
            if LastX >= Img.Width then LastX := Img.Width - 1;
            with AveX[LastX] do
            begin
              Inc(Ave.cR, cR);
              Inc(Ave.cG, cG);
              Inc(Ave.cB, cB);
              Inc(Ave.c, c);
            end;
          end;

          {  The average is written.  }
          case Img.BitCount of
            4:
              begin
                P := @PArrayByte(DestP)[X shr 1];
                with Ave do
                  P^ := (P^ and Mask4n[X and 1]) or (((((cR + cG + cB) div c)
                    div
                    3) shr 4) shl Shift4[X and 1]);
              end;
            8:
              begin
                with Ave do
                  PByte(DestP)^ := ((cR + cG + cB) div c) div 3;
                Inc(PByte(DestP));
              end;
            16:
              begin
                with Ave do
                  PWORD(DestP)^ := pfRGB(Img.DIB.NowPixelFormat, cR div c, cG
                    div
                    c, cB div c);
                Inc(PWORD(DestP));
              end;
            24:
              begin
                with PBGR(DestP)^, Ave do
                begin
                  R := cR div c;
                  G := cG div c;
                  B := cB div c;
                end;
                Inc(PBGR(DestP));
              end;
            32:
              begin
                with Ave do
                  PDWORD(DestP)^ := pfRGB(Img.DIB.NowPixelFormat, cR div c, cG
                    div c, cB div c);
                Inc(PDWORD(DestP));
              end;
          end;
        end;

      end;
    finally
      FreeMem(AveX);
    end;
  end;

var
  i, j: LongInt;
begin
  if Img.Empty or (Radius = 0) then Exit;

  Radius := Abs(Radius);

  with Img.DIB do
  try
    Temp := TDIB.Create;
    try
      Temp.Assign(Img.DIB);
      SetSize(Width, Height, ABitCount);

      if ABitCount <= 8 then
      begin
        FillChar(ColorTable, SizeOf(ColorTable), 0);
        for i := 0 to (1 shl ABitCount) - 1 do
        begin
          j := i * (1 shl (8 - ABitCount));
          j := j or (j shr ABitCount);
          ColorTable[i] := RGBQuad(j, j, j);
        end;
        UpdatePalette;
      end;

      Blur_Radius_Other;
    finally
      Temp.Free;
    end;
  finally
  end;
end;



procedure Negative(Img: TImage);
var
  i, i2: LongInt;
  P: Pointer;
begin
  if Img.Empty then exit;

  with Img.DIB do
  begin
    if BitCount <= 8 then
    begin
      for i := 0 to 255 do
        with ColorTable[i] do
        begin
          rgbRed := 255 - rgbRed;
          rgbGreen := 255 - rgbGreen;
          rgbBlue := 255 - rgbBlue;
        end;
      UpdatePalette;
    end
    else
    begin
      P := PBits;
      i2 := Size;
      asm
      mov ecx,i2
      mov eax,P
      mov edx,ecx

    @@qword_skip:
      shr ecx,2
      jz @@dword_skip

      dec ecx
    @@dword_loop:
      not dword ptr [eax+ecx*4]
      dec ecx
      jnl @@dword_loop

      mov ecx,edx
      shr ecx,2
    //  add eax,ecx*4

    @@dword_skip:
      mov ecx,edx
      and ecx,3
      jz @@byte_skip

      dec ecx
    @@loop_byte:
      not byte ptr [eax+ecx]
      dec ecx
      jnl @@loop_byte

    @@byte_skip:
      end;
    end;
  end;
end;


procedure Mirror(Img: TImage; MirrorX, MirrorY: Boolean);
var
  x, y, Width2, c: LongInt;
  P1, P2, TempBuf: Pointer;
begin
  if Img.Empty then exit;
  if (not MirrorX) and (not MirrorY) then Exit;
  with Img.DIB do
  begin

    if (not MirrorX) and (MirrorY) then
    begin
      GetMem(TempBuf, WidthBytes);
      try
        try
          for y := 0 to Height shr 1 - 1 do
          begin
            P1 := ScanLine[y];
            P2 := ScanLine[Height - y - 1];

            Move(P1^, TempBuf^, WidthBytes);
            Move(P2^, P1^, WidthBytes);
            Move(TempBuf^, P2^, WidthBytes);

          end;
        finally
        end;
      finally
        FreeMem(TempBuf, WidthBytes);
      end;
    end
    else
      if (MirrorX) and (not MirrorY) then
      begin
        Width2 := Width shr 1;

        try
          for y := 0 to Height - 1 do
          begin
            P1 := ScanLine[y];

            case BitCount of
              4:
                begin
                  for x := 0 to Width2 - 1 do
                  begin
                    c := Pixels[x, y];
                    Pixels[x, y] := Pixels[Width - x - 1, y];
                    Pixels[Width - x - 1, y] := c;
                  end;
                end;
              8:
                begin
                  P2 := Pointer(LongInt(P1) + Width - 1);
                  for x := 0 to Width2 - 1 do
                  begin
                    PByte(@c)^ := PByte(P1)^;
                    PByte(P1)^ := PByte(P2)^;
                    PByte(P2)^ := PByte(@c)^;
                    Inc(PByte(P1));
                    Dec(PByte(P2));
                  end;
                end;
              16:
                begin
                  P2 := Pointer(LongInt(P1) + (Width - 1) * 2);
                  for x := 0 to Width2 - 1 do
                  begin
                    PWord(@c)^ := PWord(P1)^;
                    PWord(P1)^ := PWord(P2)^;
                    PWord(P2)^ := PWord(@c)^;
                    Inc(PWord(P1));
                    Dec(PWord(P2));
                  end;
                end;
              24:
                begin
                  P2 := Pointer(LongInt(P1) + (Width - 1) * 3);
                  for x := 0 to Width2 - 1 do
                  begin
                    PBGR(@c)^ := PBGR(P1)^;
                    PBGR(P1)^ := PBGR(P2)^;
                    PBGR(P2)^ := PBGR(@c)^;
                    Inc(PBGR(P1));
                    Dec(PBGR(P2));
                  end;
                end;
              32:
                begin
                  P2 := Pointer(LongInt(P1) + (Width - 1) * 4);
                  for x := 0 to Width2 - 1 do
                  begin
                    PDWORD(@c)^ := PDWORD(P1)^;
                    PDWORD(P1)^ := PDWORD(P2)^;
                    PDWORD(P2)^ := PDWORD(@c)^;
                    Inc(PDWORD(P1));
                    Dec(PDWORD(P2));
                  end;
                end;
            end;

          end;
        finally
        end;
      end
      else
        if (MirrorX) and (MirrorY) then
        begin
          try
            for y := 0 to Height shr 1 - 1 do
            begin
              P1 := ScanLine[y];
              P2 := ScanLine[Height - y - 1];

              case BitCount of
                4:
                  begin
                    for x := 0 to Width - 1 do
                    begin
                      c := Pixels[x, y];
                      Pixels[x, y] := Pixels[Width - x - 1, Height - y - 1];
                      Pixels[Width - x - 1, Height - y - 1] := c;
                    end;
                  end;
                8:
                  begin
                    P2 := Pointer(LongInt(P2) + Width - 1);
                    for x := 0 to Width - 1 do
                    begin
                      PByte(@c)^ := PByte(P1)^;
                      PByte(P1)^ := PByte(P2)^;
                      PByte(P2)^ := PByte(@c)^;
                      Inc(PByte(P1));
                      Dec(PByte(P2));
                    end;
                  end;
                16:
                  begin
                    P2 := Pointer(LongInt(P2) + (Width - 1) * 2);
                    for x := 0 to Width - 1 do
                    begin
                      PWord(@c)^ := PWord(P1)^;
                      PWord(P1)^ := PWord(P2)^;
                      PWord(P2)^ := PWord(@c)^;
                      Inc(PWord(P1));
                      Dec(PWord(P2));
                    end;
                  end;
                24:
                  begin
                    P2 := Pointer(LongInt(P2) + (Width - 1) * 3);
                    for x := 0 to Width - 1 do
                    begin
                      PBGR(@c)^ := PBGR(P1)^;
                      PBGR(P1)^ := PBGR(P2)^;
                      PBGR(P2)^ := PBGR(@c)^;
                      Inc(PBGR(P1));
                      Dec(PBGR(P2));
                    end;
                  end;
                32:
                  begin
                    P2 := Pointer(LongInt(P2) + (Width - 1) * 4);
                    for x := 0 to Width - 1 do
                    begin
                      PDWORD(@c)^ := PDWORD(P1)^;
                      PDWORD(P1)^ := PDWORD(P2)^;
                      PDWORD(P2)^ := PDWORD(@c)^;
                      Inc(PDWORD(P1));
                      Dec(PDWORD(P2));
                    end;
                  end;
              end;

            end;
          finally
          end;
        end;
  end;
end;


procedure ApplyFilter(Img: TImage; Filter: TImgFilter; FilterAlpha: boolean);

type

  TScans24 = array[0..4] of PLine24;

  TScans32 = array[0..4] of PLine32;

  TScans16 = array[0..4] of PWordArray;



var
  Temp: TImage;
  Src: TImage;
  x, y, j, k: LongInt;
  d: LongInt;
  O24: PLine24;
  Scans24: TScans24;
  O32: PLine32;
  Scans32: TScans32;
  PixFormat: TDIBPixelFormat;
  Scans16: TScans16;
  O16: PWordArray;
  r, g, b: byte;
  da, dr, db, dg: LongInt;
  DScans: PLines;
  SScans: PLines;
  R24: TRGBRec;

  procedure SetCorners;
  var
    I: LongInt;
  begin
    for I := 0 to 1 do
    begin
      Temp.Pixels[I, I] := Src.Pixels[I, I];
      Temp.Pixels[Temp.Width - (I + 1), I] := Src.Pixels[Src.Width - (I + 1),
        I];

      Temp.Pixels[I, Temp.Height - (I + 1)] := Src.Pixels[I, Src.Height - (I +
        1)];
      Temp.Pixels[Temp.Width - (I + 1), Temp.Height - (I + 1)] :=
        Src.Pixels[Src.Width - (I + 1), Src.Height - (I + 1)];
    end;

  end;

  procedure SetLines;
  var I: LongInt;
  begin
    for I := 0 to 1 do
    begin
 // Copy bottom line to  top - trying to remain seamless
      Temp.Canvas.CopyRect(RECT(I + 1, I, Temp.Width - (I + 1), I + 1),
        Src.Canvas,
        RECT(0, Src.Height - (I + 1), Src.Width, Src.Height - (I)));
  // Copy top line to bottom
      Temp.Canvas.CopyRect(RECT(I + 1, Temp.Height - (I + 1), Temp.Width - (I +
        1), Temp.Height - I),
        Src.Canvas, RECT(0, I, Src.Width, I + 1));
  // Copy left to right
      Temp.Canvas.CopyRect(RECT(Temp.Width - (I + 1), I + 1, Temp.Width - I,
        Temp.Height - (I + 1)),
        Src.Canvas, RECT(I, 0, I + 1, Src.Height));
  // Copy right to left
      Temp.Canvas.CopyRect(RECT(I, I + 1, I + 1, Temp.Height - (I + 1)),
        Src.Canvas, RECT(Src.Width - (I + 1), 0, Src.Width - I, Src.Height));

    end;
  end;


begin
  if Img.Empty then Exit;
  Src := Img;

  Temp := TImage.Create;
  Temp.DIB.SetSize(Src.Width + 4, Src.Height + 4, Src.BitCount);
//  Temp.BitCount := Src.BitCount;
  Src.Draw(Temp, 2, 2);

 // for seamless pattern
  SetCorners;
  SetLines;

  DScans := Temp.CreateScans;
  SScans := Src.CreateScans;

  with Filter do
    case Src.BitCount of
      4: // convert to 24bit ,apply filter,then back to 4 bit
        begin
          Img.BitCount := 24;
          ApplyFilter(Img, Filter, FilterAlpha);
          Img.BitCount := 4;
        end;
      8: // convert to 24bit ,apply filter,then back to 8 bit
        begin
          Img.BitCount := 24;
          ApplyFilter(Img, Filter, FilterAlpha);
          Img.BitCount := 8;
        end;
      16:
        begin
          PixFormat := MakeDIBPixelFormat(5, 5, 5);
          for y := 0 to Src.Height - 1 do
          begin
            Scans16[0] := dScans[y];
            Scans16[1] := dScans[y + 1];
            Scans16[2] := dScans[y + 2];
            Scans16[3] := dScans[y + 3];
            Scans16[4] := dScans[y + 4];
            O16 := SScans[y];
            for x := 0 to Src.Width - 1 do
            begin
              dr := 0;
              dg := 0;
              db := 0;
              for j := 1 to 5 do
                for k := 1 to 5 do
                begin
                  pfGetRGB(PixFormat, Scans16[j - 1, x + k - 1], r, g, b);
                  dr := dr + r * Matrix[(j - 1) * 5 + k - 1];
                  dg := dg + g * Matrix[(j - 1) * 5 + k - 1];
                  db := db + b * Matrix[(j - 1) * 5 + k - 1];
                end;
              dr := Set255((dr div Divisor) + Bias);
              dg := Set255((dg div Divisor) + Bias);
              db := Set255((db div Divisor) + Bias);
              O16[x] := pfRGB(PixFormat, dr, dg, db);
            end;
          end;
        end;
      24:
        begin
          for y := 0 to Src.Height - 1 do
          begin
            Scans24[0] := dScans[y];
            Scans24[1] := dScans[y + 1];
            Scans24[2] := dScans[y + 2];
            Scans24[3] := dScans[y + 3];
            Scans24[4] := dScans[y + 4];
            O24 := SScans[y];
            for x := 0 to Src.Width - 1 do
            begin

              dr := 0;
              dg := 0;
              db := 0;
              for j := 1 to 5 do
                for k := 1 to 5 do
                begin
                  dr := dr + Scans24[j - 1, x + k - 1].R * Matrix[(j - 1) * 5 + k
                    - 1];
                  dg := dg + Scans24[j - 1, x + k - 1].G * Matrix[(j - 1) * 5 + k
                    - 1];
                  db := db + Scans24[j - 1, x + k - 1].B * Matrix[(j - 1) * 5 + k
                    - 1];
                end;
              O24[x].R := Set255(dr div Divisor + Bias);
              O24[x].G := Set255(dg div Divisor + Bias);
              O24[x].B := Set255(db div Divisor + Bias);
            end;
          end;

        end;
      32:
        begin

          for y := 0 to Src.Height - 1 do
          begin

            Scans32[0] := dScans[y];
            Scans32[1] := dScans[y + 1];
            Scans32[2] := dScans[y + 2];
            Scans32[3] := dScans[y + 3];
            Scans32[4] := dScans[y + 4];
            O32 := SScans[y];

            for x := 0 to Src.Width - 1 do
            begin
              dr := 0;
              dg := 0;
              db := 0;
              da := 0;
              for j := 1 to 5 do
                for k := 1 to 5 do
                begin
                  dr := dr + Scans32[j - 1, x + k - 1].R * Matrix[(j - 1) * 5 + k
                    - 1];
                  dg := dg + Scans32[j - 1, x + k - 1].G * Matrix[(j - 1) * 5 + k
                    - 1];
                  db := db + Scans32[j - 1, x + k - 1].B * Matrix[(j - 1) * 5 + k
                    - 1];
                  if FilterAlpha then
                    da := da + Scans32[j - 1, x + k - 1].A * Matrix[(j - 1) * 5
                      + k - 1];
                end;
              O32[x].R := Set255(dr div Divisor + Bias);
              O32[x].G := Set255(dg div Divisor + Bias);
              O32[x].B := Set255(db div Divisor + Bias);
              if FilterAlpha then
                O32[x].A := Set255(da div Divisor + Bias);

            end;
          end;
        end;

    end;

  Temp.Free;
  FreeMem(DScans);
  FreeMem(SScans);


end;


procedure AddNoise8(Img: TImage; Amount: LongInt);
var
  x, y, i, a: LongInt;
  pb: PByte;
begin
  i := Amount shr 1;
  pb := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 1 do
    begin
      a := pb^ + Random(Amount) - i;
      if a > 255 then
        pb^ := 255
      else
        if a < 0 then
          pb^ := 0
        else
          pb^ := a;
      Inc(pb);
    end;
    Inc(pb, Img.Gap);
  end;
end;

procedure AddNoise16(Img: TImage; Amount: LongInt; Mono: Boolean);
var
  z, a, i, x, y: LongInt;
  r, g, b: byte;
  Pix: TDIBPixelFormat;
  pw: PWord;
  wd: Word;
begin
  i := (Amount shr 1) + 1;
  Pix := MakeDIBPixelFormat(5, 5, 5);
  pw := Pointer(Img.Bits);
  if Mono then
  begin
    for y := 0 to Img.Height - 1 do
    begin
      for x := 0 to Img.Width - 1 do
      begin
        a := Random(Amount) - i;
        pfGetRGB(Pix, pw^, r, g, b);
        z := r + a;
        if z > 255 then
          z := 255
        else
          if z < 0 then z := 0;
        r := z;
        z := g + a;
        if z > 255 then
          z := 255
        else
          if z < 0 then z := 0;
        g := z;
        z := b + a;
        if z > 255 then
          z := 255
        else
          if z < 0 then z := 0;
        b := z;
        wd := pfRGB(Pix, r, g, b);
        pw^ := wd; Inc(pw);
      end;
      pw := Ptr(LongInt(pw) + Img.Gap);
    end;
  end
  else
  begin
    for y := 0 to Img.Height - 1 do
    begin
      for x := 0 to Img.Width - 1 do
      begin
        pfGetRGB(Pix, pw^, r, g, b);
        z := r + Random(Amount) - i;
        if z > 255 then
          z := 255
        else
          if z < 0 then z := 0;
        r := z;
        z := g + Random(Amount) - i;
        if z > 255 then
          z := 255
        else
          if z < 0 then z := 0;
        g := z;
        z := b + Random(Amount) - i;
        if z > 255 then
          z := 255
        else
          if z < 0 then z := 0;
        b := z;
        wd := pfRGB(Pix, r, g, b);
        pw^ := wd; Inc(pw);
      end;
      pw := Ptr(LongInt(pw) + Img.Gap);
    end;
  end;
end;

procedure AddNoise24(Img: TImage; Amount: LongInt; Mono: Boolean);
var
  i, z, a, x, y: LongInt;
  pc: PRGBRec;
begin
  i := Amount shr 1;
  pc := Pointer(Img.Bits);
  if Mono then
  begin
    for y := 0 to Img.Height - 1 do
    begin
      for x := 0 to Img.Width - 1 do
      begin
        a := Random(Amount) - i;
        z := pc.b + a;
        if z > 255 then
          pc.b := 255
        else
          if z < 0 then
            pc.b := 0
          else
            pc.b := z;
        z := pc.g + a;
        if z > 255 then
          pc.g := 255
        else
          if z < 0 then
            pc.g := 0
          else
            pc.g := z;
        z := pc.r + a;
        if z > 255 then
          pc.r := 255
        else
          if z < 0 then
            pc.r := 0
          else
            pc.r := z;
        Inc(pc);
      end;
      pc := Ptr(LongInt(pc) + Img.Gap);
    end;
  end
  else
  begin
    for y := 0 to Img.Height - 1 do
    begin
      for x := 0 to Img.Width - 1 do
      begin
        z := pc.b + Random(Amount) - i;
        if z > 255 then
          pc.b := 255
        else
          if z < 0 then
            pc.b := 0
          else
            pc.b := z;
        z := pc.g + Random(Amount) - i;
        if z > 255 then
          pc.g := 255
        else
          if z < 0 then
            pc.g := 0
          else
            pc.g := z;
        z := pc.r + Random(Amount) - i;
        if z > 255 then
          pc.r := 255
        else
          if z < 0 then
            pc.r := 0
          else
            pc.r := z;
        Inc(pc);
      end;
      pc := Ptr(LongInt(pc) + Img.Gap);
    end;
  end;
end;

procedure AddNoise32(Img: TImage; Amount: LongInt; Mono: Boolean; AlphaNoise:
  boolean);
var
  s, a, z, x, y: LongInt;
  pa: PARGBRec;
begin
  s := Amount shr 1;
  pa := Pointer(Img.Bits);
  if Mono then
  begin
    for y := 0 to Img.Height - 1 do
    begin
      for x := 0 to Img.Width - 1 do
      begin
        a := Random(Amount) - s;
        z := pa.b + a;
        if z > 255 then
          pa.b := 255
        else
          if z < 0 then
            pa.b := 0
          else
            pa.b := z;
        z := pa.g + a;
        if z > 255 then
          pa.g := 255
        else
          if z < 0 then
            pa.g := 0
          else
            pa.g := z;
        z := pa.r + a;
        if z > 255 then
          pa.r := 255
        else
          if z < 0 then
            pa.r := 0
          else
            pa.r := z;
        if AlphaNoise then
          z := pa.a + a;
        if z > 255 then
          pa.a := 255
        else
          if z < 0 then
            pa.a := 0
          else
            pa.a := z;
        Inc(pa);
      end;
    end;
  end
  else
  begin
    for y := 0 to Img.Height - 1 do
    begin
      for x := 0 to Img.Width - 1 do
      begin
        z := pa.b + Random(Amount) - s;
        if z > 255 then
          pa.b := 255
        else
          if z < 0 then
            pa.b := 0
          else
            pa.b := z;
        z := pa.g + Random(Amount) - s;
        if z > 255 then
          pa.g := 255
        else
          if z < 0 then
            pa.g := 0
          else
            pa.g := z;
        z := pa.r + Random(Amount) - s;
        if z > 255 then
          pa.r := 255
        else
          if z < 0 then
            pa.r := 0
          else
            pa.r := z;
        z := pa.a + Random(Amount) - s;
        if z > 255 then
          pa.a := 255
        else
          if z < 0 then
            pa.a := 0
          else
            pa.a := z;
        Inc(pa);
      end;
    end;
  end;
end;

procedure AddNoise(Img: TImage; Amount: Byte; Mono: Boolean; AlphaNoise:
  boolean);
begin
  if Img.Empty then Exit;
  case Img.BitCount of
    8: AddNoise8(Img, Amount);
    16: AddNoise16(Img, Amount, Mono);
    24: AddNoise24(Img, Amount, Mono);
    32: AddNoise32(Img, Amount, Mono, AlphaNoise);
  end;
end;

function Count4(Img: TImage): LongInt;
var
  pb, pc: PByte;
  x, y, w: LongInt;
  Check: array[0..15] of Byte;
begin
  Result := 0;
  FillChar(Check, SizeOf(Check), 0);
  pb := Pointer(Img.Bits);
  w := (Img.Width div 2) - 1;
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to w do
    begin
      pc := @Check[pb^ shr 4];
      if pc^ = 0 then
      begin
        Inc(Result);
        pc^ := 1;
      end;
      pc := @Check[pb^ and 15];
      if pc^ = 0 then
      begin
        Inc(Result);
        pc^ := 1;
      end;
      if Result = 16 then Exit;
      Inc(pb);
    end;
    Inc(pb, Img.Gap);
  end;
end;

function Count8(Img: TImage): LongInt;
var
  x, y: LongInt;
  pb, pc: PByte;
  Check: array[Byte] of Byte;
begin
  Result := 0;
  FillChar(Check, SizeOf(Check), 0);
  pb := Pointer(img.bits);
  for y := 0 to img.height - 1 do
  begin
    for x := 0 to img.width - 1 do
    begin
      pc := @Check[pb^];
      if pc^ = 0 then
      begin
        Inc(Result);
        pc^ := 1;
      end;
      if Result = 256 then Exit;
      Inc(pb);
    end;
    Inc(pb, Img.Gap);
  end;
end;

function Count16(Img: TImage): LongInt;
var
  pw: PWord;
  pc: PByte;
  x, y: LongInt;
  Check: array[Word] of Byte;
begin
  Result := 0;
  FillChar(Check, SizeOf(Check), 0);
  pw := Pointer(img.bits);
  for y := 0 to img.height - 1 do
  begin
    for x := 0 to img.width - 1 do
    begin
      pc := @Check[pw^];
      if pc^ = 0 then
      begin
        Inc(Result);
        pc^ := 1;
      end;
      Inc(pw);
    end;
    pw := Ptr(LongInt(pw) + Img.Gap);
  end;
end;

function Count24(Img: TImage): LongInt;
type
  PCheck = ^TCheck;
  TCheck = array[Byte, Byte, 0..31] of Byte;
var
  pb: PByte;
  pc: PRGBRec;
  Check: PCheck;
  x, y, c: LongInt;
begin
  Result := 0;
  New(Check);
  FillChar(Check^, SizeOf(TCheck), 0);
  pc := Pointer(img.bits);
  for y := 0 to img.height - 1 do
  begin
    for x := 0 to img.width - 1 do
    begin
      pb := @Check[pc.r, pc.g, pc.b shr 3];
      c := 1 shl (pc.b and 7);
      if (c and pb^) = 0 then
      begin
        Inc(Result);
        pb^ := pb^ or c;
      end;
      Inc(pc);
    end;
    pc := Ptr(LongInt(pc) + Img.Gap);
  end;
  Dispose(Check);
end;

function Count32(Img: TImage): LongInt;
type
  PCheck = ^TCheck;
  TCheck = array[Byte, Byte, 0..31] of Byte;
var
  pb: PByte;
  pc: PARGBRec;
  i, c: LongInt;
  Check: PCheck;
begin
  Result := 0;
  New(Check);
  FillChar(Check^, SizeOf(TCheck), 0);
  pc := Pointer(img.bits);
  for i := 0 to (Img.DIB.Size shr 2) - 1 do
  begin
    pb := @Check[pc.r, pc.g, pc.b shr 3];
    c := 1 shl (pc.b and 7);
    if (c and pb^) = 0 then
    begin
      Inc(Result);
      pb^ := pb^ or c;
    end;
    Inc(pc);
  end;
  Dispose(Check);
end;

function CountColors(Img: TImage): DWord;
begin
  Result := 0;
  if Img.Empty then Exit;
  case Img.BitCount of
    4: Result := Count4(Img);
    8: Result := Count8(Img);
    16: Result := Count16(Img);
    24: Result := Count24(Img);
    32: Result := Count32(Img);
  else
    Result := 0;
  end;
end;


procedure ShiftImage(Img: TImage; xAmount, yAmount: LongInt);
var
  y, Size: LongInt;
  ScanLines: PLines;
  Line, Buf: PLine8;
begin
  if Img.Empty then Exit;
  ScanLines := Img.CreateScans;
  if yAmount <> 0 then
  begin
    if yAmount < 0 then yAmount := Img.Height - (-yAmount mod Img.Height);
    if yAmount > Img.Height then yAmount := yAmount mod Img.Height;
    if Img.Size <> 0 then
    begin
      Size := yAmount * Img.WidthBytes;
      GetMem(Buf, Size);
      Move(Img.Bits^, Buf^, Size);
      Move(Scanlines[yAmount]^, Img.Bits^, (Img.Height - yAmount) *
        Img.WidthBytes);
      Move(Buf^, Scanlines[Img.Height - yAmount]^, Size);
      FreeMem(Buf);
    end
    else
    begin
      Size := Img.WidthBytes - Img.Gap;
      GetMem(Buf, yAmount * Size);
      Line := Pointer(Buf);
      for y := 0 to yAmount - 1 do
      begin
        Move(Scanlines[y]^, Line^, Size);
        Line := Ptr(LongInt(Line) + Size);
      end;
      for y := yAmount to Img.Height - 1 do
        Move(Scanlines[y]^, Scanlines[LongInt(y) - yAmount]^, Size);
      Line := Pointer(Buf);
      for y := Img.Height - yAmount to Img.Height - 1 do
      begin
        Move(Line^, Scanlines[y]^, Size);
        Line := Ptr(LongInt(Line) + Size);
      end;
      FreeMem(Buf);
    end;
  end;
  if xAmount <> 0 then
  begin
    if xAmount < 0 then xAmount := Img.Width - (-xAmount mod Img.Width);
    if xAmount > Img.Width then xAmount := xAmount mod Img.Width;
    Size := xAmount * (Img.BitCount div 8);
    GetMem(Buf, Size);
    Line := Img.Bits;
    for y := 0 to Img.Height - 1 do
    begin
      Move(Line^, Buf^, Size);
      Move(Ptr(LongInt(Line) + Size)^, Line^, (Img.WidthBytes - Img.Gap) -
        Size);
      Move(Buf^, Ptr(LongInt(Line) + ((Img.WidthBytes - Img.Gap) - Size))^,
        Size);
      Line := Ptr(LongInt(Line) + Img.WidthBytes);
    end;
    FreeMem(Buf);
  end;
  FreeMem(ScanLines);
end;


procedure QuickSoft8(Img: TImage);
var
  a, b, c: PByte;
  x, y: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 1);
  c := Ptr(LongInt(b) + 1);
  for y := 0 to Img.Height - 1 do
  begin
    a^ := (a^ + b^) shr 1;
    for x := 0 to Img.Width - 3 do
    begin
      b^ := (a^ + c^) shr 1;
      Inc(a); Inc(b); Inc(c);
    end;
    b^ := (b^ + a^) shr 1;
    Inc(a, Img.Gap + 2);
    Inc(b, Img.Gap + 2);
    Inc(c, Img.Gap + 2);
  end;
end;

procedure QuickSoft24(Img: TImage);
var
  a, b, c: PRGBRec;
  x, y: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 3);
  c := Ptr(LongInt(b) + 3);
  for y := 0 to Img.Height - 1 do
  begin
    a.b := (a.b + b.b) shr 1;
    a.g := (a.g + b.g) shr 1;
    a.r := (a.r + b.r) shr 1;
    for x := 0 to Img.Width - 3 do
    begin
      b.b := (a.b + c.b) shr 1;
      b.g := (a.g + c.g) shr 1;
      b.r := (a.r + c.r) shr 1;
      Inc(a); Inc(b); Inc(c);
    end;
    b.b := (b.b + a.b) shr 1;
    b.g := (b.g + a.g) shr 1;
    b.r := (b.r + a.r) shr 1;
    a := Ptr(LongInt(a) + Img.Gap + 6);
    b := Ptr(LongInt(b) + Img.Gap + 6);
    c := Ptr(LongInt(c) + Img.Gap + 6);
  end;
end;

procedure QuickSoft32(Img: TImage; FilterAlpha: boolean);
var
  a, b, c: PARGBRec;
  x, y: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 4);
  c := Ptr(LongInt(b) + 4);
  for y := 0 to Img.Height - 1 do
  begin
    a.b := (a.b + b.b) shr 1;
    a.g := (a.g + b.g) shr 1;
    a.r := (a.r + b.r) shr 1;
    if FilterAlpha then
      a.a := (a.a + b.a) shr 1;
    for x := 0 to Img.Width - 3 do
    begin
      b.b := (a.b + c.b) shr 1;
      b.g := (a.g + c.g) shr 1;
      b.r := (a.r + c.r) shr 1;
      if FilterAlpha then
        b.a := (a.a + c.a) shr 1;
      Inc(a); Inc(b); Inc(c);
    end;
    b.b := (b.b + a.b) shr 1;
    b.g := (b.g + a.g) shr 1;
    b.r := (b.r + a.r) shr 1;
    if FilterAlpha then
      b.a := (b.a + a.a) shr 1;
    a := Ptr(LongInt(a) + Img.Gap + 8);
    b := Ptr(LongInt(b) + Img.Gap + 8);
    c := Ptr(LongInt(c) + Img.Gap + 8);
  end;
end;

procedure QuickSoft(Img: TImage; FilterAlpha: boolean);
begin
  if Img.Empty then Exit;
  case Img.BitCount of
    8: QuickSoft8(Img);
    24: QuickSoft24(Img);
    32: QuickSoft32(Img, FilterAlpha);
  end;
end;

procedure QuickSharp8(Img: TImage);
var
  a, b, c: PByte;
  x, y, i: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 1);
  c := Ptr(LongInt(b) + 1);
  for y := 0 to Img.Height - 1 do
  begin
    i := ((a^ shl 2) - (a^ + b^)) div 2;
    if i > 255 then
      a^ := 255
    else
      if i < 0 then
        a^ := 0
      else
        a^ := i;
    for x := 0 to Img.Width - 3 do
    begin
      i := ((b^ shl 2) - (a^ + c^)) div 2;
      if i > 255 then
        b^ := 255
      else
        if i < 0 then
          b^ := 0
        else
          b^ := i;
      Inc(a); Inc(b); Inc(c);
    end;
    i := ((b^ shl 2) - (b^ + a^)) div 2;
    if i > 255 then
      b^ := 255
    else
      if i < 0 then
        b^ := 0
      else
        b^ := i;
    Inc(a, Img.Gap + 2);
    Inc(b, Img.Gap + 2);
    Inc(c, Img.Gap + 2);
  end;
end;

procedure QuickSharp24(Img: TImage);
var
  a, b, c: PRGBRec;
  x, y, i: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 3);
  c := Ptr(LongInt(b) + 3);
  for y := 0 to Img.Height - 1 do
  begin
    i := ((a.b shl 2) - (a.b + b.b)) div 2;
    if i > 255 then
      a.b := 255
    else
      if i < 0 then
        a.b := 0
      else
        a.b := i;
    i := ((a.g shl 2) - (a.g + b.g)) div 2;
    if i > 255 then
      a.g := 255
    else
      if i < 0 then
        a.g := 0
      else
        a.g := i;
    i := ((a.r shl 2) - (a.r + b.r)) div 2;
    if i > 255 then
      a.r := 255
    else
      if i < 0 then
        a.r := 0
      else
        a.r := i;
    for x := 0 to Img.Width - 3 do
    begin
      i := ((b.b shl 2) - (a.b + c.b)) div 2;
      if i > 255 then
        b.b := 255
      else
        if i < 0 then
          b.b := 0
        else
          b.b := i;
      i := ((b.g shl 2) - (a.g + c.g)) div 2;
      if i > 255 then
        b.g := 255
      else
        if i < 0 then
          b.g := 0
        else
          b.g := i;
      i := ((b.r shl 2) - (a.r + c.r)) div 2;
      if i > 255 then
        b.r := 255
      else
        if i < 0 then
          b.r := 0
        else
          b.r := i;
      Inc(a); Inc(b); Inc(c);
    end;
    i := ((b.b shl 2) - (b.b + a.b)) div 2;
    if i > 255 then
      b.b := 255
    else
      if i < 0 then
        b.b := 0
      else
        b.b := i;
    i := ((b.g shl 2) - (b.g + a.g)) div 2;
    if i > 255 then
      b.g := 255
    else
      if i < 0 then
        b.g := 0
      else
        b.g := i;
    i := ((b.r shl 2) - (b.r + a.r)) div 2;
    if i > 255 then
      b.r := 255
    else
      if i < 0 then
        b.r := 0
      else
        b.r := i;
    a := Ptr(LongInt(a) + Img.Gap + 6);
    b := Ptr(LongInt(b) + Img.Gap + 6);
    c := Ptr(LongInt(c) + Img.Gap + 6);
  end;
end;

procedure QuickSharp32(Img: TImage; FilterAlpha: boolean);
var
  a, b, c: PARGBRec;
  x, y, i: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 4);
  c := Ptr(LongInt(b) + 4);
  for y := 0 to Img.Height - 1 do
  begin
    i := ((a.b shl 2) - (a.b + b.b)) div 2;
    if i > 255 then
      a.b := 255
    else
      if i < 0 then
        a.b := 0
      else
        a.b := i;
    i := ((a.g shl 2) - (a.g + b.g)) div 2;
    if i > 255 then
      a.g := 255
    else
      if i < 0 then
        a.g := 0
      else
        a.g := i;
    i := ((a.r shl 2) - (a.r + b.r)) div 2;
    if i > 255 then
      a.r := 255
    else
      if i < 0 then
        a.r := 0
      else
        a.r := i;
    if FilterAlpha then
    begin
      i := ((a.a shl 2) - (a.a + b.a)) div 2;
      if i > 255 then
        a.a := 255
      else
        if i < 0 then
          a.a := 0
        else
          a.a := i;
    end;
    for x := 0 to Img.Width - 3 do
    begin
      i := ((b.b shl 2) - (a.b + c.b)) div 2;
      if i > 255 then
        b.b := 255
      else
        if i < 0 then
          b.b := 0
        else
          b.b := i;
      i := ((b.g shl 2) - (a.g + c.g)) div 2;
      if i > 255 then
        b.g := 255
      else
        if i < 0 then
          b.g := 0
        else
          b.g := i;
      i := ((b.r shl 2) - (a.r + c.r)) div 2;
      if i > 255 then
        b.r := 255
      else
        if i < 0 then
          b.r := 0
        else
          b.r := i;
      if FilterAlpha then
      begin
        i := ((b.a shl 2) - (a.a + c.a)) div 2;
        if i > 255 then
          b.a := 255
        else
          if i < 0 then
            b.a := 0
          else
            b.a := i;
      end;
      Inc(a); Inc(b); Inc(c);
    end;
    i := ((b.b shl 2) - (b.b + a.b)) div 2;
    if i > 255 then
      b.b := 255
    else
      if i < 0 then
        b.b := 0
      else
        b.b := i;
    i := ((b.g shl 2) - (b.g + a.g)) div 2;
    if i > 255 then
      b.g := 255
    else
      if i < 0 then
        b.g := 0
      else
        b.g := i;
    i := ((b.r shl 2) - (b.r + a.r)) div 2;
    if i > 255 then
      b.r := 255
    else
      if i < 0 then
        b.r := 0
      else
        b.r := i;
    if FilterAlpha then
    begin
      i := ((b.a shl 2) - (b.a + a.a)) div 2;
      if i > 255 then
        b.a := 255
      else
        if i < 0 then
          b.a := 0
        else
          b.a := i;
    end;
    a := Ptr(LongInt(a) + Img.Gap + 8);
    b := Ptr(LongInt(b) + Img.Gap + 8);
    c := Ptr(LongInt(c) + Img.Gap + 8);
  end;
end;

procedure QuickSharp(Img: TImage; FilterAlpha: boolean);
begin
  if Img.Empty then Exit;
  case Img.BitCount of
    8: QuickSharp8(Img);
    24: QuickSharp24(Img);
    32: QuickSharp32(Img, FilterAlpha);
  end;
end;

procedure QuickEmboss8(Img: TImage);
var
  a, b: PByte;
  x, y: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 1);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 2 do
    begin
      a^ := (b^ + (a^ xor LongInt(-1)) and $FF) shr 1;
      Inc(a); Inc(b);
    end;
    a^ := (a^ + (a^ xor LongInt(-1)) and $FF) shr 1;
    Inc(a, Img.Gap + 1);
    Inc(b, Img.Gap + 1);
  end;
end;

procedure QuickEmboss24(Img: TImage);
var
  a, b: PRGBRec;
  x, y: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 3);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 2 do
    begin
      a.b := (b.b + (a.b xor $FFFFFFFF) and $FF) shr 1;
      a.g := (b.g + (a.g xor $FFFFFFFF) and $FF) shr 1;
      a.r := (b.r + (a.r xor $FFFFFFFF) and $FF) shr 1;
      Inc(a); Inc(b);
    end;
    a.b := (a.b + (a.b xor $FFFFFFFF) and $FF) shr 1;
    a.g := (a.g + (a.g xor $FFFFFFFF) and $FF) shr 1;
    a.r := (a.r + (a.r xor $FFFFFFFF) and $FF) shr 1;
    a := Ptr(LongInt(a) + Img.Gap + 3);
    b := Ptr(LongInt(b) + Img.Gap + 3);
  end;
end;

procedure QuickEmboss32(Img: TImage; FilterAlpha: boolean);
var
  a, b: PARGBRec;
  x, y: LongInt;
begin
  a := Pointer(Img.Bits);
  b := Ptr(LongInt(a) + 4);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 2 do
    begin
      a.b := (b.b + (a.b xor $FFFFFFFF) and $FF) shr 1;
      a.g := (b.g + (a.g xor $FFFFFFFF) and $FF) shr 1;
      a.r := (b.r + (a.r xor $FFFFFFFF) and $FF) shr 1;
      if FilterAlpha then
        a.a := (b.a + (a.a xor $FFFFFFFF) and $FF) shr 1;
      Inc(a); Inc(b);
    end;
    a.b := (a.b + (a.b xor $FFFFFFFF) and $FF) shr 1;
    a.g := (a.g + (a.g xor $FFFFFFFF) and $FF) shr 1;
    a.r := (a.r + (a.r xor $FFFFFFFF) and $FF) shr 1;
    if FilterAlpha then
      a.a := (a.a + (a.a xor $FFFFFFFF) and $FF) shr 1;
    a := Ptr(LongInt(a) + Img.Gap + 4);
    b := Ptr(LongInt(b) + Img.Gap + 4);
  end;
end;

procedure QuickEmboss(Img: TImage; FilterAlpha: boolean);
begin
  if Img.Empty then Exit;
  case Img.BitCount of
    8: QuickEmboss8(Img);
//    16: QuickEmboss16(Img);
    24: QuickEmboss24(Img);
    32: QuickEmboss32(Img, FilterAlpha);
  end;
end;

procedure DrawTiles(Src, Dst: TImage; TW, TH: LongInt);
var
  Img: TImage;
  wd, hd: LongInt;
begin
  if (TW = 0) or (TH = 0) then
    Img := Src
  else
  begin
    Img := Src.Clone;
    Img.Resize(TW, TH);
  end;
  wd := Img.Width;
  hd := Img.Height;
  Dst.CopyRect(Img, Rect(0, 0, Img.Width, Img.Height), 0, 0);
  while wd < Dst.Width do
  begin
    Dst.CopyRect(Dst, Rect(wd, 0, wd shl 1, Img.Height), 0, 0);
    Inc(wd, wd);
  end;
  while hd < Dst.Height do
  begin
    Dst.CopyRect(Dst, Rect(0, hd, Dst.Width, hd shl 1), 0, 0);
    Inc(hd, hd);
  end;
  if Img <> Src then
    Img.Free;
end;

procedure Mosaic8(Img: TImage; xAmount, yAmount: LongInt);
var
  Delta, tx, ty, ix, iy, cx, cy, x, y: LongInt;
  Line: PLine8;
  pc: PByte;
  ScanLines: PLines;
  Pixels8: PPixels8;
  tc: Byte;
begin
  ScanLines := Img.CreateScans;
  Pixels8 := Pointer(ScanLines);
  xAmount := Abs(xAmount);
  yAmount := Abs(yAmount);
  if (xAmount = 0) or (yAmount = 0) then Exit;
  ix := (xAmount shr 1) + (xAmount and 1);
  iy := (yAmount shr 1) + (yAmount and 1);
  y := 0;
  while y < Img.Height do
  begin
    x := 0;
    cy := y + iy;
    if cy >= Img.Height then
      Line := Scanlines[Img.Height - 1]
    else
      Line := Scanlines[cy];
    if y + yAmount - 1 > Img.Height - 1 then
      ty := Img.Height - y
    else
      ty := yAmount;
    while x < Img.Width do
    begin
      cx := x + ix;
      if cx >= Img.Width then
        tc := Line[Img.Width - 1]
      else
        tc := Line[cx];
      if x + xAmount - 1 > Img.Width - 1 then
        tx := Img.Width - x
      else
        tx := xAmount;
      Delta := LongInt(Img.WidthBytes) - tx;
      pc := @Pixels8[y, x];
      for cy := 1 to ty do
      begin
        for cx := 1 to tx do
        begin
          pc^ := tc;
          Inc(pc);
        end;
        Inc(pc, Delta);
      end;
      Inc(x, xAmount);
    end;
    Inc(y, yAmount);
  end;
  FreeMem(ScanLines);
end;

procedure Mosaic16(Img: TImage; xAmount, yAmount: LongInt);
var
  Delta, tx, ty, ix, iy, cx, cy, x, y: LongInt;
  Line: PLine16;
  ScanLines: PLines;
  Pixels16: PPixels16;
  pc: PWord;
  tc: Word;
begin
  ScanLines := Img.CreateScans;
  Pixels16 := Pointer(ScanLines);
  xAmount := Abs(xAmount);
  yAmount := Abs(yAmount);
  if (xAmount = 0) or (yAmount = 0) then Exit;
  ix := (xAmount shr 1) + (xAmount and 1);
  iy := (yAmount shr 1) + (yAmount and 1);
  y := 0;
  while y < Img.Height do
  begin
    x := 0; cy := y + iy;
    if cy >= Img.Height then
      Line := Scanlines[Img.Height - 1]
    else
      Line := Scanlines[cy];
    if y + yAmount - 1 > Img.Height - 1 then
      ty := Img.Height - y
    else
      ty := yAmount;
    while x < Img.Width do
    begin
      cx := x + ix;
      if cx >= Img.Width then
        tc := Line[Img.Width - 1]
      else
        tc := Line[cx];
      if x + xAmount - 1 > Img.Width - 1 then
        tx := Img.Width - x
      else
        tx := xAmount;
      Delta := LongInt(Img.WidthBytes) - tx * 2; pc := @Pixels16[y, x];
      for cy := 1 to ty do
      begin
        for cx := 1 to tx do
        begin
          pc^ := tc;
          Inc(pc);
        end;
        pc := Ptr(LongInt(pc) + Delta);
      end;
      Inc(x, xAmount);
    end;
    Inc(y, yAmount);
  end;
  FreeMem(ScanLines);
end;

procedure Mosaic24(Img: TImage; xAmount, yAmount: LongInt);
var
  Delta, tx, ty, ix, iy, cx, cy, x, y: LongInt;
  Line: PLine24;
  ScanLines: PLines;
  Pixels24: PPixels24;
  pc: PRGBRec;
  tc: TRGBRec;
begin
  ScanLines := Img.CreateScans;
  Pixels24 := Pointer(ScanLines);
  xAmount := Abs(xAmount);
  yAmount := Abs(yAmount);
  if (xAmount = 0) or (yAmount = 0) then Exit;
  ix := (xAmount shr 1) + (xAmount and 1);
  iy := (yAmount shr 1) + (yAmount and 1);
  y := 0;
  while y < Img.Height do
  begin
    x := 0; cy := y + iy;
    if cy >= Img.Height then
      Line := Scanlines[Img.Height - 1]
    else
      Line := Scanlines[cy];
    if y + yAmount - 1 > Img.Height - 1 then
      ty := Img.Height - y
    else
      ty := yAmount;
    while x < Img.Width do
    begin
      cx := x + ix;
      if cx >= Img.Width then
        tc := Line[Img.Width - 1]
      else
        tc := Line[cx];
      if x + xAmount - 1 > Img.Width - 1 then
        tx := Img.Width - x
      else
        tx := xAmount;
      Delta := LongInt(Img.WidthBytes) - tx * 3;
      pc := @Pixels24[y, x];
      for cy := 1 to ty do
      begin
        for cx := 1 to tx do
        begin
          pc^ := tc;
          Inc(pc);
        end;
        pc := Ptr(LongInt(pc) + Delta);
      end;
      Inc(x, xAmount);
    end;
    Inc(y, yAmount);
  end;
  FreeMem(ScanLines);
end;

procedure Mosaic32(Img: TImage; xAmount, yAmount: LongInt);
var
  Delta, tx, ty, ix, iy, cx, cy, x, y: LongInt;
  Line: PLine32;
  pc: PARGBRec;
  tc: TARGBRec;
  ScanLines: PLines;
  Pixels32: PPixels32;
begin
  ScanLines := Img.CreateScans;
  Pixels32 := Pointer(ScanLines);
  xAmount := Abs(xAmount);
  yAmount := Abs(yAmount);
  if (xAmount = 0) or (yAmount = 0) then Exit;
  ix := (xAmount shr 1) + (xAmount and 1);
  iy := (yAmount shr 1) + (yAmount and 1);
  y := 0;
  while y < Img.Height do
  begin
    x := 0; cy := y + iy;
    if cy >= Img.Height then
      Line := Scanlines[Img.Height - 1]
    else
      Line := Scanlines[cy];
    if y + yAmount - 1 > Img.Height - 1 then
      ty := Img.Height - y
    else
      ty := yAmount;
    while x < Img.Width do
    begin
      cx := x + ix;
      if cx >= Img.Width then
        tc := Line[Img.Width - 1]
      else
        tc := Line[cx];
      if x + xAmount - 1 > Img.Width - 1 then
        tx := Img.Width - x
      else
        tx := xAmount;
      Delta := LongInt(Img.WidthBytes) - tx * 4;
      pc := @Pixels32[y, x];
      for cy := 1 to ty do
      begin
        for cx := 1 to tx do
        begin
          pc^ := tc;
          Inc(pc);
        end;
        pc := Ptr(LongInt(pc) + Delta);
      end;
      Inc(x, xAmount);
    end;
    Inc(y, yAmount);
  end;
  FreeMem(ScanLines);
end;

procedure Mosaic(Img: TImage; xAmount, yAmount: LongInt);
begin
  case Img.BitCount of
    8: Mosaic8(Img, xAmount, yAmount);
    16: Mosaic16(Img, xAmount, yAmount);
    24: Mosaic24(Img, xAmount, yAmount);
    32: Mosaic32(Img, xAmount, yAmount);
  end;
end;

procedure DrawLightmap8(Img: TImage; Size: LongInt);
var
  x, y, yy, f, r, i: LongInt;
  pb: PByte;
begin
  r := Size shr 1;
  f := Round(65536 / (Size / ((256 / (Size / 2)) * 2)));
  pb := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    yy := (y - r) * (y - r);
    for x := 0 to Img.Width - 1 do
    begin
      i := ((x - r) * (x - r) + yy) * f shr 16;
      if i > 255 then i := 255;
      i := i xor -1;
      pb^ := i;
      Inc(pb);
    end;
    Inc(pb, Img.Gap);
  end;
end;

procedure DrawLightmap16(Img: TImage; Size: LongInt);
begin
  Img.BitCount := 24;
  DrawLightMap(Img, Size);
  Img.BitCount := 16;
end;

procedure DrawLightmap24(Img: TImage; Size: LongInt);
var
  x, y, yy, f, r, i: LongInt;
  pc: PRGBRec;
begin
  r := Size shr 1;
  f := Round(65536 / (Size / ((256 / (Size / 2)) * 2)));
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    yy := (y - r) * (y - r);
    for x := 0 to Img.Width - 1 do
    begin
      i := ((x - r) * (x - r) + yy) * f shr 16;
      if i > 255 then i := 255;
      i := i xor -1;
      pc.b := i;
      pc.g := i;
      pc.r := i;
      Inc(pc);
    end;
    pc := Ptr(LongInt(pc) + Img.Gap);
  end;
end;

procedure DrawLightmap32(Img: TImage; Size: LongInt; UseAlpha: boolean);
var
  x, y, yy, f, r, i: LongInt;
  pc: PARGBRec;
begin
  r := Size shr 1;
  f := Round(65536 / (Size / ((256 / (Size / 2)) * 2)));
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    yy := (y - r) * (y - r);
    for x := 0 to Img.Width - 1 do
    begin
      i := ((x - r) * (x - r) + yy) * f shr 16;
      if i > 255 then i := 255;
      i := i xor -1;
      pc.b := i;
      pc.g := i;
      pc.r := i;
      if UseAlpha then
        pc.A := i;
      Inc(pc);
    end;
    pc := Ptr(LongInt(pc) + Img.Gap);
  end;
end;

procedure DrawLightmap(Img: TImage; Size: LongInt; UseAlpha: boolean);
begin
  case Img.BitCount of
    8: DrawLightmap8(Img, Size);
    16: DrawLightmap16(Img, Size);
    24: DrawLightmap24(Img, Size);
    32: DrawLightmap32(Img, Size, UseAlpha);
  end;
end;

function CreateLightMap(Size: LongInt; Col: TARGB; UseAlpha: boolean): TImage;
var Img: TImage;
  I: LongInt;
  a, r, g, b: byte;
  pc: PARGBRec;
begin
  Img := TImage.Create;
  Img.BitCount := 8;
  Img.Resize(Size, Size);
  DrawLightMap(Img, Size);
  Img.ColorTable := DIB.GreyscaleColorTable;

  if (Col <> $FFFFFFFF) or (Col <> $00FFFFFF) then
  begin
    pc := Pointer(Img.PColorTable);
    GetARGB(Col, a, r, g, b);
    for I := 0 to 255 do
    begin
      pc.R := Set255(round(r / 255 * I));
      pc.G := Set255(round(g / 255 * I));
      pc.B := Set255(round(b / 255 * I));
      pc.A := 0;
      Inc(pc);
    end;
    Img.UpdatePal;
  end;

  Result := Img;
end;

procedure ApplyLut8(Img: TImage; RLut, GLut, BLut: TLut);
var
  i: Integer;
  pa: PARGBRec;
begin
  pa := Pointer(Img.PColorTable);
  for i := 0 to (1 shl Img.BitCount) - 1 do
  begin
    pa.b := BLut[pa.b];
    pa.g := GLut[pa.g];
    pa.r := RLut[pa.r];
    Inc(pa);
  end;
  Img.UpdatePal;
end;

procedure ApplyLut24(Img: TImage; RLut, GLut, BLut: TLut);
var
  pc: PRGBRec;
  x, y: Integer;
begin
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 1 do
    begin
      pc.b := BLut[pc.b];
      pc.g := GLut[pc.g];
      pc.r := RLut[pc.r];
      Inc(pc);
    end;
    pc := Ptr(Integer(pc) + Img.Gap);
  end;
end;

procedure ApplyLut32(Img: TImage; RLut, GLut, BLut: TLut);
var
  pc: PARGBRec;
  x, y: Integer;
begin
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 1 do
    begin
      pc.b := BLut[pc.b];
      pc.g := GLut[pc.g];
      pc.r := RLut[pc.r];
      Inc(pc);
    end;
    pc := Ptr(Integer(pc) + Img.Gap);
  end;
end;

procedure ApplyLut(Img: TImage; RLut, GLut, BLut: TLut);
begin
  case Img.BitCount of
    4, 8: ApplyLut8(Img, RLut, GLut, BLut);
    24: ApplyLut24(Img, RLut, GLut, BLut);
    32: ApplyLut32(Img, RLut, GLut, BLut);
  end;
end;

procedure FillLut(var Lut: TLut; x1, y1, x2, y2: Byte);
var
  x, n, i, ii: Integer;
begin
  n := x2 - x1;
  if n <> 0 then
  begin
    i := y1 shl 16;
    ii := ((y2 - y1 + 1) shl 16) div n;
    for x := x1 to x2 do
    begin
      Lut[x] := i shr 16;
      Inc(i, ii);
    end;
  end;
end;

function ContrastLut(Amount: Integer): TLut;
begin
  if Amount < 0 then
    FillLut(Result, 0, -Amount, 255, 255 + Amount)
  else
  begin
    if Amount > 255 then Amount := 255;
    FillChar(Result, Amount, 0);
    FillLut(Result, Amount, 0, 255 - Amount, 255);
    FillChar(Result[256 - Amount], Amount, 255);
  end;
end;

function LightnessLut(Amount: Integer): TLut;
begin
  if Amount < 0 then
    FillLut(Result, 0, 0, 255, 255 + Amount)
  else
  begin
    if Amount > 255 then Amount := 255;
    FillLut(Result, 0, Amount, 255, 255);
  end;
end;

function AdditionLut(Amount: Integer): TLut;
var
  i, x: Integer;
begin
  if Amount < 0 then
  begin
    if Amount < -255 then Amount := -255;
    FillChar(Result, -Amount, 0);
    x := 0;
    for i := -Amount to 255 do
    begin
      Result[i] := x;
      Inc(x);
    end;
  end
  else
    if Amount > 0 then
    begin
      if Amount > 255 then Amount := 255;
      x := Amount;
      for i := 0 to 255 - Amount do
      begin
        Result[i] := x;
        Inc(x);
      end;
      FillChar(Result[256 - Amount], Amount, 255);
    end
    else
      for i := 0 to 255 do
        Result[i] := i;
end;

function GammaLut(Amount: Extended): TLut;
var
  i, z: Integer;
  y: Extended;
begin
  if Amount > 0 then
  begin
    Result[0] := 0;
    y := 1 / Amount;
    for i := 1 to 255 do
    begin
      z := Round(255 * Exp(y * Ln(i / 255)));
      if z > 255 then
        z := 255
      else
        if z < 0 then z := 0;
      Result[i] := z;
    end;
  end;
end;

function MergeLuts(Luts: array of TLut): TLut;
var
  x, y, z: Integer;
begin
  x := Low(Luts);
  for y := 0 to 255 do
    Result[y] := Luts[x, y];
  for z := x + 1 to High(Luts) do
    for y := 0 to 255 do
      Result[y] := Luts[z, Result[y]];
end;

procedure Gamma(Img: TImage; r, g, b: Extended);
begin
  ApplyLut(Img,
    GammaLut(r),
    GammaLut(g),
    GammaLut(b));
end;


procedure Addition(Img: TImage; r, g, b: Integer);
begin
  ApplyLut(Img,
    AdditionLut(r),
    AdditionLut(g),
    AdditionLut(b));
end;

procedure Contrast(Img: TImage; r, g, b: Integer);
begin
  ApplyLut(Img,
    ContrastLut(r),
    ContrastLut(g),
    ContrastLut(b));
end;

procedure Lightness(Img: TImage; r, g, b: Integer);
begin
  ApplyLut(Img,
    LightnessLut(r),
    LightnessLut(g),
    LightnessLut(b));
end;


function SaturationLut(Amount: Integer): TSaturationLut;
var
  x, y, i: Integer;
begin
  x := 0;
  for i := 1 to 256 do
    Result.Alpha[i - 1] := (i * Amount) shr 8;
  for i := 1 to 256 do
  begin
    y := i - Result.Alpha[i - 1];
    Result.Grays[x] := y; Inc(x);
    Result.Grays[x] := y; Inc(x);
    Result.Grays[x] := y; Inc(x);
  end;
end;

procedure ApplySaturationLut8(Img: TImage; Lut: TSaturationLut);
var
  g, z, c: Integer;
  pa: PARGBRec;
begin
  pa := Pointer(Img.PColorTable);
  for c := 0 to (1 shl Img.BitCount) - 1 do
  begin
    g := Lut.Grays[pa.b + pa.g + pa.r];
    z := Lut.Alpha[pa.b] + g;
    if z > 255 then
      z := 255
    else
      if z < 0 then z := 0; pa.b := z;
    z := Lut.Alpha[pa.g] + g;
    if z > 255 then
      z := 255
    else
      if z < 0 then z := 0; pa.g := z;
    z := Lut.Alpha[pa.r] + g;
    if z > 255 then
      z := 255
    else
      if z < 0 then z := 0; pa.r := z;
    Inc(pa);
  end;
  Img.UpdatePal;
end;

procedure ApplySaturationLut24(Img: TImage; Lut: TSaturationLut);
var
  x, y, g, z: Integer;
  pc: PRGBRec;
begin
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 1 do
    begin
      g := Lut.Grays[pc.b + pc.g + pc.r];
      z := Lut.Alpha[pc.b] + g;
      if z > 255 then
        z := 255
      else
        if z < 0 then z := 0; pc.b := z;
      z := Lut.Alpha[pc.g] + g;
      if z > 255 then
        z := 255
      else
        if z < 0 then z := 0; pc.g := z;
      z := Lut.Alpha[pc.r] + g;
      if z > 255 then
        z := 255
      else
        if z < 0 then z := 0; pc.r := z;
      Inc(pc);
    end;
    pc := Ptr(Integer(pc) + Img.Gap);
  end;
end;

procedure ApplySaturationLut32(Img: TImage; Lut: TSaturationLut);
var
  g, x, y, z: Integer;
  pc: PARGBRec;
begin
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 1 do
    begin
      g := Lut.Grays[pc.b + pc.g + pc.r];
      z := Lut.Alpha[pc.b] + g;
      if z > 255 then
        z := 255
      else
        if z < 0 then z := 0; pc.b := z;
      z := Lut.Alpha[pc.g] + g;
      if z > 255 then
        z := 255
      else
        if z < 0 then z := 0; pc.g := z;
      z := Lut.Alpha[pc.r] + g;
      if z > 255 then
        z := 255
      else
        if z < 0 then z := 0; pc.r := z;
      Inc(pc);
    end;
    pc := Ptr(Integer(pc) + Img.Gap);
  end;
end;

procedure ApplySaturationLut(Img: TImage; Lut: TSaturationLut);
begin
  case Img.BitCount of
    4, 8: ApplySaturationLut8(Img, Lut);
    24: ApplySaturationLut24(Img, Lut);
    32: ApplySaturationLut32(Img, Lut);
  end;
end;

procedure Saturation(Img: TImage; Amount: Integer);
begin
  ApplySaturationLut(Img, SaturationLut(Amount));
end;

procedure AutoContrast8(Img: TImage);
var
  hr, hg, hb, lr, lg, lb: Byte;
  r, g, b: TLut;
  pa: PARGBRec;
  i: Integer;
begin
  hr := 0; hg := 0; hb := 0;
  lr := 255; lg := 255; lb := 255;
  pa := Pointer(Img.PColorTable);
  for i := 0 to (1 shl Img.BitCount) - 1 do
  begin
    if pa.b > hb then hb := pa.b;
    if pa.b < lb then lb := pa.b;
    if pa.g > hg then hg := pa.g;
    if pa.g < lg then lg := pa.g;
    if pa.r > hr then hr := pa.r;
    if pa.r < lr then lr := pa.r;
    Inc(pa);
  end;
  if ((lr or lg or lb) <> 0) or ((hr and hg and hb) <> 255) then
  begin
    FillLut(r, lr, 0, hr, 255);
    FillLut(g, lg, 0, hg, 255);
    FillLut(b, lb, 0, hb, 255);
    ApplyLut8(Img, r, g, b);
  end;
end;

procedure AutoContrast24(Img: TImage);
var
  hr, hg, hb, lr, lg, lb: Byte;
  r, g, b: TLut;
  x, y: Integer;
  pc: PRGBRec;
begin
  hr := 0; hg := 0; hb := 0;
  lr := 255; lg := 255; lb := 255;
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 1 do
    begin
      if pc.b > hb then hb := pc.b;
      if pc.b < lb then lb := pc.b;
      if pc.g > hg then hg := pc.g;
      if pc.g < lg then lg := pc.g;
      if pc.r > hr then hr := pc.r;
      if pc.r < lr then lr := pc.r;
      Inc(pc);
    end;
    pc := Ptr(Integer(pc) + Img.Gap);
  end;
  if ((lr or lg or lb) <> 0) or ((hr and hg and hb) <> 255) then
  begin
    FillLut(r, lr, 0, hr, 255);
    FillLut(g, lg, 0, hg, 255);
    FillLut(b, lb, 0, hb, 255);
    ApplyLut24(Img, r, g, b);
  end;
end;

procedure AutoContrast32(Img: TImage);
var
  hr, hg, hb, lr, lg, lb: Byte;
  r, g, b: TLut;
  pc: PARGBRec;
  x, y: Integer;
begin
  hr := 0; hg := 0; hb := 0;
  lr := 255; lg := 255; lb := 255;
  pc := Pointer(Img.Bits);
  for y := 0 to Img.Height - 1 do
  begin
    for x := 0 to Img.Width - 1 do
    begin
      if pc.b > hb then hb := pc.b;
      if pc.b < lb then lb := pc.b;
      if pc.g > hg then hg := pc.g;
      if pc.g < lg then lg := pc.g;
      if pc.r > hr then hr := pc.r;
      if pc.r < lr then lr := pc.r;
      Inc(pc);
    end;
    pc := Ptr(Integer(pc) + Img.Gap);
  end;
  if ((lr or lg or lb) <> 0) or ((hr and hg and hb) <> 255) then
  begin
    FillLut(r, lr, 0, hr, 255);
    FillLut(g, lg, 0, hg, 255);
    FillLut(b, lb, 0, hb, 255);
    ApplyLut32(Img, r, g, b);
  end;
end;

procedure AutoContrast(Img: TImage);
begin
  case Img.BitCount of
    4, 8: AutoContrast8(Img);
    24: AutoContrast24(Img);
    32: AutoContrast32(Img);
  end;
end;


procedure GradientFill8(Img: TImage; DestR: TRect; tl, tr, bl, br: Byte);
var
  Pixels8: PPixels8;
  pc: PByte;
  ne, nw, se, sw: byte;
  x1, x2, y1, y2,
    xc, yc, t, t2, z, iz, p2, p, dx, xx: Integer;
begin
  Pixels8 := PPixels8(Img.CreateScans);
  ne := br;
  nw := bl;
  se := tl;
  sw := tr;
  x1 := DestR.Left;
  x2 := DestR.Right;
  y1 := DestR.Top;
  y2 := DestR.Bottom;
  z := 0;
  iz := 65536;
  if x2 <> x1 then
    t := 65536 div (x2 - x1)
  else
    t := 0;
  if y2 <> y1 then
    t2 := 65536 div (y2 - y1)
  else
    t2 := 0;
  dx := x2 - x1;
  for yc := y1 to y2 - 1 do
  begin
    xx := ((nw * iz + se * z) shr 16);
    p := xx shl 16;
    p2 := (((ne * iz + sw * z) shr 16) - xx) * t;
    pc := @Pixels8[yc, x1];
    for xc := 0 to dx - 1 do
    begin
      Inc(p, p2);
      pc^ := p shr 16;
      Inc(pc);
    end;
    Inc(z, t2);
    Dec(iz, t2);
  end;
  FreeMem(Pixels8);
end;

procedure GradientFill24(Img: TImage; DestR: TRect; tl, tr, bl, br: TARGB);
var
  pc: PRGBRec;
  Pixels24: PPixels24;
  nw, ne, sw, se: TRGBRec;
  x1, x2, y1, y2: LongInt;
  xc, yc, t, t2, z, iz, rp,
    rp2, gp, gp2, bp, bp2, dx, xx: Integer;
begin
  ne := RGBRec(br);
  nw := RGBRec(bl);
  se := RGBRec(tl);
  sw := RGBRec(tr);
  x1 := DestR.Left;
  x2 := DestR.Right;
  y1 := DestR.Top;
  y2 := DestR.Bottom;
  Pixels24 := PPixels24(Img.CreateScans);
  z := 0;
  iz := 65536;
  if x2 <> x1 then
    t := 65536 div (x2 - x1)
  else
    t := 0;
  if y2 <> y1 then
    t2 := 65536 div (y2 - y1)
  else
    t2 := 0;
  dx := x2 - x1;
  for yc := y1 to y2 - 1 do
  begin
    xx := ((nw.r * iz + se.r * z) shr 16);
    rp := xx shl 16;
    rp2 := (((ne.r * iz + sw.r * z) shr 16) - xx) * t;
    xx := ((nw.g * iz + se.g * z) shr 16);
    gp := xx shl 16;
    gp2 := (((ne.g * iz + sw.g * z) shr 16) - xx) * t;
    xx := ((nw.b * iz + se.b * z) shr 16);
    bp := xx shl 16;
    bp2 := (((ne.b * iz + sw.b * z) shr 16) - xx) * t;
    pc := @Pixels24[yc, x1];
    for xc := 0 to dx - 1 do
    begin
      Inc(bp, bp2);
      Inc(gp, gp2);
      Inc(rp, rp2);
      pc.b := bp shr 16;
      pc.g := gp shr 16;
      pc.r := rp shr 16;
      Inc(pc);
    end;
    Inc(z, t2);
    Dec(iz, t2);
  end;
  FreeMem(Pixels24);
end;

procedure GradientFill32(Img: TImage; DestR: TRect; tl, tr, bl, br: TARGB;
  UseAlpha: boolean);
var
  Pixels32: PPixels32;
  pc: PARGBRec;
  ne, nw, sw, se: TARGBRec;
  xc, yc, t, t2, z, iz, rp,
    rp2, gp, gp2, bp, bp2,
    x1, x2, y1, y2,
    ap, ap2,
    dx, xx: LongInt;
begin
  ne := ARGBRec(br);
  nw := ARGBRec(bl);
  se := ARGBRec(tl);
  sw := ARGBRec(tr);
  x1 := DestR.Left;
  x2 := DestR.Right;
  y1 := DestR.Top;
  y2 := DestR.Bottom;
  Pixels32 := PPixels32(Img.CreateScans);
  z := 0;
  iz := 65536;
  if x2 <> x1 then
    t := 65536 div (x2 - x1)
  else
    t := 0;
  if y2 <> y1 then
    t2 := 65536 div (y2 - y1)
  else
    t2 := 0;
  dx := x2 - x1;
  for yc := y1 to y2 - 1 do
  begin
    xx := ((nw.r * iz + se.r * z) shr 16);
    rp := xx shl 16;
    rp2 := (((ne.r * iz + sw.r * z) shr 16) - xx) * t;
    xx := ((nw.g * iz + se.g * z) shr 16);
    gp := xx shl 16;
    gp2 := (((ne.g * iz + sw.g * z) shr 16) - xx) * t;
    xx := ((nw.b * iz + se.b * z) shr 16);
    bp := xx shl 16;
    bp2 := (((ne.b * iz + sw.b * z) shr 16) - xx) * t;
    xx := ((nw.a * iz + se.a * z) shr 16);
    ap := xx shl 16;
    ap2 := (((ne.a * iz + sw.a * z) shr 16) - xx) * t;
    pc := @Pixels32[yc, x1];
    for xc := 0 to dx - 1 do
    begin
      Inc(bp, bp2);
      Inc(gp, gp2);
      Inc(rp, rp2);
      Inc(ap, ap2);
      pc.b := bp shr 16;
      pc.g := gp shr 16;
      pc.r := rp shr 16;
      if UseAlpha then
        pc.a := ap shr 16;
      Inc(pc);
    end;
    Inc(z, t2);
    Dec(iz, t2);
  end;
  FreeMem(Pixels32);
end;

procedure GradientFill(Img: TImage; DestR: TRect; tl, tr, bl, br: TARGB;
  UseAlpha: boolean);
begin
  case Img.BitCount of
    8:
      GradientFill8(Img, DestR,
        ClosestColor(Img.PColorTable, 255, tl),
        ClosestColor(Img.PColorTable, 255, tr),
        ClosestColor(Img.PColorTable, 255, bl),
        ClosestColor(Img.PColorTable, 255, br));
    24: GradientFill24(Img, DestR, tl, tr, bl, br);
    32: GradientFill32(Img, DestR, tl, tr, bl, br, UseAlpha);
  end;
end;



procedure Line(Img: TImage; x1, y1, x2, y2: Integer; col: TARGB; UseAlpha:
  boolean);
var i, s, d1x, d1y, d2x, d2y, u, v, m, n: Longint;
begin
  case Img.BitCount of
    4, 8: Col := ClosestColor(Img.PColorTable, (1 shl Img.BitCount) - 1, col);
    16: Col := pfRGB(Img.DIB.PixelFormat, Col shr 16, Col shr 8, Col);
    24: Col := SetA(Col, 0);
    32:
      if not UseAlpha then
        Col := SetA(Col, 255);
  end;

  u := x2 - x1;
  v := y2 - y1;
  d1x := SGN(u);
  d1y := SGN(v);
  d2x := SGN(u);
  d2y := 0;
  m := ABS(u);
  n := ABS(v);
  if not (M > N) then
  begin
    d2x := 0;
    d2y := SGN(v);
    m := ABS(v);
    n := ABS(u);
  end;
  s := m shr 1;
  for i := 0 to m do
  begin
    img.pixels[x1, y1] := col;
    s := s + n;
    if not (s < m) then
    begin
      s := s - m;
      x1 := x1 + d1x;
      y1 := y1 + d1y;
    end
    else
    begin
      x1 := x1 + d2x;
      y1 := y1 + d2y;
    end;
  end;

end;

procedure Polyline(Img: TImage; pnts: array of TPoint; col: TARGB; UseAlpha:
  boolean);
var
  n, i: Integer;
begin
  n := High(pnts) + 1;
  for i := 0 to n - 1 do
    Line(Img, pnts[i].x, pnts[i].y, pnts[(i + 1) mod n].x, pnts[(i + 1) mod
      n].y, col, UseAlpha);

end;


procedure DrawAdditive(Dest, Src: TImage; X, Y: LongInt; SrcR: TRect);
var
  i, j, Wid: integer;
  p1, p2: pByte;
begin
  if SrcR.Left > SrcR.Right then Swap(SrcR.Left, SrcR.Right);
  if SrcR.Left < 0 then SrcR.Left := 0;
  if SrcR.Right > Src.Width then SrcR.Right := Src.Width;

  if SrcR.Top > SrcR.Bottom then Swap(SrcR.Top, SrcR.Bottom);
  if SrcR.Top < 0 then SrcR.Top := 0;
  if SrcR.Bottom > Src.Height then SrcR.Bottom := Src.Height;

  if X < 0 then X := 0;
  if X > Dest.Width then X := Dest.Width;
  if Y < 0 then Y := 0;
  if Y > Dest.Height then Y := Dest.Height;

  if X + (SrcR.Right - SrcR.Left) > Dest.Width then
    SrcR.Right := Dest.Width - X + SrcR.Left;

  if Y + (SrcR.Bottom - SrcR.Top) > Dest.Height then
    SrcR.Bottom := Dest.Height - Y + SrcR.Top;

  if (Dest.BitCount = 32) and (Src.BitCount = 24) then
  begin
    Src.BitCount := 32;
    DrawAdditive(Dest, Src, X, Y, SrcR);
    Src.BitCount := 24;
  end
  else
    if (Dest.BitCount = 24) and (Src.BitCount = 32) then
    begin
      Dest.BitCount := 32;
      DrawAdditive(Dest, Src, X, Y, SrcR);
      Dest.BitCount := 24;
    end
    else
      if (Dest.BitCount = 24) and (Src.BitCount = 24) then
      begin
        Wid := (SrcR.Right - SrcR.Left) shl 1 + (SrcR.Right - SrcR.Left);
        for i := 1 to (SrcR.Bottom - SrcR.Top) - 1 do
        begin
          p1 := Dest.Scanline[i + Y];
          p2 := Src.Scanline[i + SrcR.Top];
          inc(p1, X shl 1 + X + 3);
          inc(p2, SrcR.Left * 3 + 3);
          for j := 3 to Wid - 4 do
          begin
            inc(p1^, (255 - p1^) * p2^ shr 8);
            inc(p1);
            inc(p2);
          end;
        end;
      end
      else
        if (Dest.BitCount = 32) and (Src.BitCount = 32) then
        begin
          for i := 1 to (SrcR.Bottom - SrcR.Top) - 1 do
          begin
            p1 := Dest.Scanline[i + Y];
            p2 := Src.Scanline[i + SrcR.Top];
            inc(p1, X shl 2 + 4);
            inc(p2, (SrcR.Bottom - SrcR.Left) shl 2 + 4);
            for j := 3 to SrcR.Right - SrcR.Left - 4 do
            begin
              inc(p1^, (255 - p1^) * p2^ shr 8);
              inc(p1);
              inc(p2);
              inc(p1^, (255 - p1^) * p2^ shr 8);
              inc(p1);
              inc(p2);
              inc(p1^, (255 - p1^) * p2^ shr 8);
              inc(p1, 2);
              inc(p2, 2);
            end;
          end;
        end;
end;


procedure DrawDarken(Dest, Src: TImage; X, Y: LongInt; SrcR: TRect);
var
  i, j: integer;
  p1, p2: pByte;

begin
  if SrcR.Left > SrcR.Right then Swap(SrcR.Left, SrcR.Right);
  if SrcR.Left < 0 then SrcR.Left := 0;
  if SrcR.Right > Src.Width then SrcR.Right := Src.Width;

  if SrcR.Top > SrcR.Bottom then Swap(SrcR.Top, SrcR.Bottom);
  if SrcR.Top < 0 then SrcR.Top := 0;
  if SrcR.Bottom > Src.Height then SrcR.Bottom := Src.Height;

  if X < 0 then X := 0;
  if X > Dest.Width then X := Dest.Width;
  if Y < 0 then Y := 0;
  if Y > Dest.Height then Y := Dest.Height;

  if X + (SrcR.Right - SrcR.Left) > Dest.Width then
    SrcR.Right := Dest.Width - X + SrcR.Left;

  if Y + (SrcR.Bottom - SrcR.Top) > Dest.Height then
    SrcR.Bottom := Dest.Height - Y + SrcR.Top;


  if (Dest.BitCount = 32) and (Src.BitCount = 24) then
  begin
    Src.BitCount := 32;
    DrawDarken(Dest, Src, X, Y, SrcR);
    Src.BitCount := 24;
  end
  else
    if (Dest.BitCount = 24) and (Src.BitCount = 32) then
    begin
      Dest.BitCount := 32;
      DrawDarken(Dest, Src, X, Y, SrcR);
      Dest.BitCount := 24;
    end
    else
      if (Dest.BitCount = 24) and (Src.BitCount = 24) then
      begin
        for i := 1 to SrcR.Bottom - SrcR.Top - 1 do
        begin
          p1 := Dest.Scanline[i + Y];
          p2 := Src.Scanline[i + SrcR.Top];
          inc(p1, 3 * X + 3);
          inc(p2, (SrcR.Right - SrcR.Left) * 3 + 3);
          for j := 1 to SrcR.Right - SrcR.Left - 1 do
          begin
            p1^ := (p2^ * p1^) shr 8;
            inc(p1);
            inc(p2);
            p1^ := (p2^ * p1^) shr 8;
            inc(p1);
            inc(p2);
            p1^ := (p2^ * p1^) shr 8;
            inc(p1);
            inc(p2);
          end;
        end;
      end
      else
        if (Dest.BitCount = 32) and (Src.BitCount = 32) then
        begin
          for i := 1 to SrcR.Bottom - SrcR.Top - 1 do
          begin
            p1 := Dest.Scanline[i + Y];
            p2 := Src.Scanline[i + SrcR.Top];
            inc(p1, 4 * X + 4);
            inc(p2, (SrcR.Right - SrcR.Left) * 4 + 4);
            for j := 1 to SrcR.Right - SrcR.Left - 1 do
            begin
              p1^ := (p2^ * p1^) shr 8;
              inc(p1);
              inc(p2);
              p1^ := (p2^ * p1^) shr 8;
              inc(p1);
              inc(p2);
              p1^ := (p2^ * p1^) shr 8;
              inc(p1, 2);
              inc(p2, 2);
            end;
          end;
        end;
end;


procedure DrawShadow(Dest, Src: TImage; X, Y: LongInt; SrcR: TRect;
  FilterMode: TFilterMode);
var
  i, j: Integer;
  p1, p2: PByte;

begin
  if SrcR.Left > SrcR.Right then Swap(SrcR.Left, SrcR.Right);
  if SrcR.Left < 0 then SrcR.Left := 0;
  if SrcR.Right > Src.Width then SrcR.Right := Src.Width;

  if SrcR.Top > SrcR.Bottom then Swap(SrcR.Top, SrcR.Bottom);
  if SrcR.Top < 0 then SrcR.Top := 0;
  if SrcR.Bottom > Src.Height then SrcR.Bottom := Src.Height;

  if X < 0 then X := 0;
  if X > Dest.Width then X := Dest.Width;
  if Y < 0 then Y := 0;
  if Y > Dest.Height then Y := Dest.Height;

  if X + (SrcR.Right - SrcR.Left) > Dest.Width then
    SrcR.Right := Dest.Width - X + SrcR.Left;

  if Y + (SrcR.Bottom - SrcR.Top) > Dest.Height then
    SrcR.Bottom := Dest.Height - Y + SrcR.Top;

  if (Dest.BitCount = 32) and (Src.BitCount = 24) then
  begin
    Src.BitCount := 32;
    DrawDarken(Dest, Src, X, Y, SrcR);
    Src.BitCount := 24;
  end
  else
    if (Dest.BitCount = 24) and (Src.BitCount = 32) then
    begin
      Dest.BitCount := 32;
      DrawDarken(Dest, Src, X, Y, SrcR);
      Dest.BitCount := 24;
    end
    else
      if (Dest.BitCount = 24) and (Src.BitCount = 24) then
      begin
        for i := 1 to SrcR.Bottom - SrcR.Top - 1 do
        begin
          p1 := Dest.Scanline[i + Y];
          p2 := Src.Scanline[i + SrcR.Top];
          inc(p1, 3 * (X + 1));
          inc(p2, 3 * (SrcR.Right - SrcR.Left + 1));
          for j := 1 to SrcR.Right - SrcR.Left - 1 do
          begin
            if (p2^ = 0) then
            begin
              case FilterMode of
                fmNormal, fmMix50:
                  begin
                    p1^ := p1^ shr 1; // Blue
                    inc(p1);
                    p1^ := p1^ shr 1; // Green
                    inc(p1);
                    p1^ := p1^ shr 1; // Red
                    inc(p1);
                  end;
                fmMix25:
                  begin
                    p1^ := p1^ - p1^ shr 2; // Blue
                    inc(p1);
                    p1^ := p1^ - p1^ shr 2; // Green
                    inc(p1);
                    p1^ := p1^ - p1^ shr 2; // Red
                    inc(p1);
                  end;
                fmMix75:
                  begin
                    p1^ := p1^ shr 2; // Blue
                    inc(p1);
                    p1^ := p1^ shr 2; // Green
                    inc(p1);
                    p1^ := p1^ shr 2; // Red
                    inc(p1);
                  end;
              end;
            end
            else
              inc(p1, 3); // Not in the loop...
            inc(p2, 3);
          end;
        end;
      end
      else
        if (Dest.BitCount = 32) and (Src.BitCount = 32) then
        begin
          for i := 1 to SrcR.Bottom - SrcR.Top - 1 do
          begin
            p1 := Dest.Scanline[i + Y];
            p2 := Src.Scanline[i + SrcR.Top];
            inc(p1, 4 * (X + 1));
            inc(p2, 4 * (SrcR.Right - SrcR.Left + 1));
            for j := 1 to SrcR.Right - SrcR.Left - 1 do
            begin
              if (p2^ = 0) then
              begin
                case FilterMode of
                  fmNormal, fmMix50:
                    begin
                      p1^ := p1^ shr 1; // Blue
                      inc(p1);
                      p1^ := p1^ shr 1; // Green
                      inc(p1);
                      p1^ := p1^ shr 1; // Red
                      inc(p1, 2);
                    end;
                  fmMix25:
                    begin
                      p1^ := p1^ - p1^ shr 2; // Blue
                      inc(p1);
                      p1^ := p1^ - p1^ shr 2; // Green
                      inc(p1);
                      p1^ := p1^ - p1^ shr 2; // Red
                      inc(p1, 2);
                    end;
                  fmMix75:
                    begin
                      p1^ := p1^ shr 2; // Blue
                      inc(p1);
                      p1^ := p1^ shr 2; // Green
                      inc(p1);
                      p1^ := p1^ shr 2; // Red
                      inc(p1, 2);
                    end;
                end;
              end
              else
                inc(p1, 4); // Not in the loop...
              inc(p2, 4);
            end;
          end;
        end;
end;



procedure DrawTranslucent(Dest, Src: Images.TImage; X, Y: LongInt; SrcR: TRect;
  Col: TARGB);
var
  i, j: Integer;
  k1, k2: Integer;
  n: DWord;
  p1, p2: PByteArray;
  Startk1, Startk2: Integer;
  StartY: Integer;
  EndY: Integer;
  DestStartY: Integer;
begin
  if SrcR.Left > SrcR.Right then Swap(SrcR.Left, SrcR.Right);
  if SrcR.Left < 0 then SrcR.Left := 0;
  if SrcR.Right > Src.Width then SrcR.Right := Src.Width;

  if SrcR.Top > SrcR.Bottom then Swap(SrcR.Top, SrcR.Bottom);
  if SrcR.Top < 0 then SrcR.Top := 0;
  if SrcR.Bottom > Src.Height then SrcR.Bottom := Src.Height;

  if X < 0 then X := 0;
  if X > Dest.Width then X := Dest.Width;
  if Y < 0 then Y := 0;
  if Y > Dest.Height then Y := Dest.Height;

  if X + (SrcR.Right - SrcR.Left) > Dest.Width then
    SrcR.Right := Dest.Width - X + SrcR.Left;

  if Y + (SrcR.Bottom - SrcR.Top) > Dest.Height then
    SrcR.Bottom := Dest.Height - Y + SrcR.Top;

  DestStartY := Y - SrcR.Top;
  StartY := SrcR.Top;
  EndY := SrcR.Bottom;
  if (StartY + DestStartY < 0) then
    StartY := -DestStartY;
  if (EndY + DestStartY > Dest.Height) then
    EndY := Dest.Height - DestStartY;

  if (StartY < 0) then
    StartY := 0;
  if (EndY > Src.Height) then
    EndY := Src.Height;

  if (Dest.BitCount = 32) and (Src.BitCount = 24) then
  begin
    Src.BitCount := 32;
    DrawDarken(Dest, Src, X, Y, SrcR);
    Src.BitCount := 24;
  end
  else
    if (Dest.BitCount = 24) and (Src.BitCount = 32) then
    begin
      Dest.BitCount := 32;
      DrawDarken(Dest, Src, X, Y, SrcR);
      Dest.BitCount := 24;
    end
    else
      if (Dest.BitCount = 24) and (Src.BitCount = 24) then
      begin
        Col := SetA(Col, 0);
        Startk1 := 3 * SrcR.Left;
        Startk2 := 3 * X;
        for j := StartY to EndY - 1 do
        begin
          p1 := Dest.Scanline[j + DestStartY];
          p2 := Src.Scanline[j];

          k1 := Startk1;
          k2 := Startk2;

          for i := SrcR.Left to SrcR.Right - 1 do
          begin
            n := (p2[k1] shl 16) + (p2[k1 + 1] shl 8) + p2[k1 + 2];
            if not (n = Col) then
            begin
              p1[k2] := (p1[k2] + p2[k1]) shr 1;
              p1[k2 + 1] := (p1[k2 + 1] + p2[k1 + 1]) shr 1;
              p1[k2 + 2] := (p1[k2 + 2] + p2[k1 + 2]) shr 1;
            end;
            k1 := k1 + 3;
            k2 := k2 + 3;
          end;
        end;
      end
      else
        if (Dest.BitCount = 32) and (Src.BitCount = 32) then
        begin
          Col := SetA(Col, 0);

          Startk1 := 4 * SrcR.Left;
          Startk2 := 4 * X;
          for j := StartY to EndY - 1 do
          begin
            p1 := Dest.Scanline[j + DestStartY];
            p2 := Src.Scanline[j];
            k1 := Startk1;
            k2 := Startk2;
            for i := SrcR.Left to SrcR.Right - 1 do
            begin
              n := (p2[k1] shl 16) + (p2[k1 + 1] shl 8) + p2[k1 + 2];
              if not (n = Col) then
              begin
                p1[k2] := (p1[k2] + p2[k1]) shr 1;
                p1[k2 + 1] := (p1[k2 + 1] + p2[k1 + 1]) shr 1;
                p1[k2 + 2] := (p1[k2 + 2] + p2[k1 + 2]) shr 1;
              end;
              k1 := k1 + 4;
              k2 := k2 + 4;
            end;
          end;
        end;
end;


procedure FilterLine(Dest: TImage; X1, Y1, X2, Y2: Integer; Col: TARGB;
  FilterMode: TFilterMode);
var
  i, j: integer;
  t: TARGB;
  A: byte;
  r1, g1, b1, r2, g2, b2: integer;
begin
  if not (Dest.BitCount in [24, 32]) then Exit;
  j := ROUND(Sqrt(Sqr(ABS(X2 - X1)) + Sqr(ABS(Y2 - Y1))));
  if j < 1 then Exit;

  A := 0;

  r1 := GetR(Col);
  g1 := GetG(Col);
  b1 := GetB(Col);

  for i := 0 to j do
  begin
    t := Dest.Pixels[X1 + ((X2 - X1) * i div j), Y1 + ((Y2 - Y1) * i div j)];
    r2 := GetR(t);
    g2 := GetG(t);
    b2 := GetB(t);
    if Dest.BitCount = 32 then
      A := GetA(T);
    case FilterMode of
      fmNormal:
        t := ARGB(A, r1 + (((256 - r1) * r2) shr 8),
          g1 + (((256 - g1) * g2) shr 8),
          b1 + (((256 - b1) * b2) shr 8));
      fmMix25:
        t := ARGB(A, (r1 + r2 * 3) shr 2, (g1 + g2 * 3) shr 2, (b1 + b2 *
          3) shr 2);
      fmMix50: t := ARGB(A, (r1 + r2) shr 1, (g1 + g2) shr 1, (b1 + b2) shr 1);
      fmMix75:
        t := ARGB(A, (r1 * 3 + r2) shr 2, (g1 * 3 + g2) shr 2, (b1 * 3 +
          b2) shr 2);
    end;
    Dest.Pixels[X1 + ((X2 - X1) * i div j), Y1 + ((Y2 - Y1) * i div j)] := t;
  end;
end;

procedure FilterRect(Dest: TImage; DestR: TRect; Col: TARGB;
  FilterMode: TFilterMode);
var
  i, j, r, g, b, C1: integer;
  p1, p2, p3: pByte;
begin
  r := GetR(Col);
  g := GetG(Col);
  b := GetB(Col);

  case Dest.BitCount of
    24:
      for i := 0 to DestR.Bottom - DestR.Top - 1 do
      begin
        p1 := Dest.Scanline[i + DestR.Top];
        Inc(p1, (3 * DestR.Left));
        for j := 0 to DestR.Right - DestR.Left - 1 do
        begin
          case FilterMode of
            fmNormal:
              begin
                p2 := p1;
                Inc(p2);
                p3 := p2;
                Inc(p3);
                C1 := (p1^ + p2^ + p3^) div 3;

                p1^ := (C1 * b) shr 8;
                Inc(p1);
                p1^ := (C1 * g) shr 8;
                Inc(p1);
                p1^ := (C1 * r) shr 8;
                Inc(p1);
              end;
            fmMix25:
              begin
                p1^ := (3 * p1^ + b) shr 2;
                Inc(p1);
                p1^ := (3 * p1^ + g) shr 2;
                Inc(p1);
                p1^ := (3 * p1^ + r) shr 2;
                Inc(p1);
              end;
            fmMix50:
              begin
                p1^ := (p1^ + b) shr 1;
                Inc(p1);
                p1^ := (p1^ + g) shr 1;
                Inc(p1);
                p1^ := (p1^ + r) shr 1;
                Inc(p1);
              end;
            fmMix75:
              begin
                p1^ := (p1^ + 3 * b) shr 2;
                Inc(p1);
                p1^ := (p1^ + 3 * g) shr 2;
                Inc(p1);
                p1^ := (p1^ + 3 * r) shr 2;
                Inc(p1);
              end;
          end;
        end;
      end;
    32:
      for i := 0 to DestR.Bottom - DestR.Top - 1 do
      begin
        p1 := Dest.Scanline[i + DestR.Top];
        Inc(p1, (4 * DestR.Left));
        for j := 0 to DestR.Right - DestR.Left - 1 do
        begin
          case FilterMode of
            fmNormal:
              begin
                p2 := p1;
                Inc(p2);
                p3 := p2;
                Inc(p3);
                C1 := (p1^ + p2^ + p3^) div 3;

                p1^ := (C1 * b) shr 8;
                Inc(p1);
                p1^ := (C1 * g) shr 8;
                Inc(p1);
                p1^ := (C1 * r) shr 8;
                Inc(p1, 2);
              end;
            fmMix25:
              begin
                p1^ := (3 * p1^ + b) shr 2;
                Inc(p1);
                p1^ := (3 * p1^ + g) shr 2;
                Inc(p1);
                p1^ := (3 * p1^ + r) shr 2;
                Inc(p1, 2);
              end;
            fmMix50:
              begin
                p1^ := (p1^ + b) shr 1;
                Inc(p1);
                p1^ := (p1^ + g) shr 1;
                Inc(p1);
                p1^ := (p1^ + r) shr 1;
                Inc(p1, 2);
              end;
            fmMix75:
              begin
                p1^ := (p1^ + 3 * b) shr 2;
                Inc(p1);
                p1^ := (p1^ + 3 * g) shr 2;
                Inc(p1);
                p1^ := (p1^ + 3 * r) shr 2;
                Inc(p1, 2);
              end;
          end;
        end;
      end;
  end;
end;


procedure DrawTransparent(Dest, Src: Timage; X, Y: LongInt; SrcR: TRect;
  Col: TARGB);
var
  i, j: Integer;
  k1, k2: Integer;
  n: DWord;
  p1, p2: PByteArray;

  Startk1, Startk2: Integer;

  StartY: Integer;
  EndY: Integer;

  DestStartY: Integer;
begin
  if SrcR.Left > SrcR.Right then Swap(SrcR.Left, SrcR.Right);
  if SrcR.Left < 0 then SrcR.Left := 0;
  if SrcR.Right > Src.Width then SrcR.Right := Src.Width;

  if SrcR.Top > SrcR.Bottom then Swap(SrcR.Top, SrcR.Bottom);
  if SrcR.Top < 0 then SrcR.Top := 0;
  if SrcR.Bottom > Src.Height then SrcR.Bottom := Src.Height;

  if X < 0 then X := 0;
  if X > Dest.Width then X := Dest.Width;
  if Y < 0 then Y := 0;
  if Y > Dest.Height then Y := Dest.Height;

  if X + (SrcR.Right - SrcR.Left) > Dest.Width then
    SrcR.Right := Dest.Width - X + SrcR.Left;

  if Y + (SrcR.Bottom - SrcR.Top) > Dest.Height then
    SrcR.Bottom := Dest.Height - Y + SrcR.Top;


  if (Dest.BitCount = 32) and (Src.BitCount = 24) then
  begin
    Src.BitCount := 32;
    DrawTransparent(Dest, Src, X, Y, SrcR, Col);
    Src.BitCount := 24;

  end
  else
    if (Dest.BitCount = 24) and (Src.BitCount = 32) then
    begin
      Dest.BitCount := 32;
      DrawTransparent(Dest, Src, X, Y, SrcR, Col);
      Dest.BitCount := 24;
    end
    else
      if Dest.BitCount = Src.BitCount then
      begin

        Startk1 := (Dest.BitCount shr 8) * SrcR.Left;
        Startk2 := (Dest.BitCount shr 8) * X;

        DestStartY := Y - SrcR.Top;

        StartY := SrcR.Top;
        EndY := SrcR.Bottom;

        if (StartY + DestStartY < 0) then
          StartY := -DestStartY;
        if (EndY + DestStartY > Dest.Height) then
          EndY := Dest.Height - DestStartY;

        if (StartY < 0) then
          StartY := 0;
        if (EndY > Src.Height) then
          EndY := Src.Height;


        for j := StartY to EndY - 1 do
        begin

          p1 := Dest.Scanline[j + DestStartY];
          p2 := Src.Scanline[j];

          k1 := Startk1;
          k2 := Startk2;

          for i := SrcR.Left to SrcR.Right - 1 do
          begin
            n := (p2[k1] shl 16) + (p2[k1 + 1] shl 8) + p2[k1 + 2];

            if not (n = Col) then
            begin
              p1[k2] := p2[k1];
              p1[k2 + 1] := p2[k1 + 1];
              p1[k2 + 2] := p2[k1 + 2];
            end;

            k1 := k1 + (Dest.BitCount shr 8);
            k2 := k2 + (Dest.BitCount shr 8);
          end;
        end;
      end;
end;



















//////////////////  LIGHTING -- EXPERIMENTAL  //////////////


var
  FLUTDist: array[0..255, 0..255] of Integer;
  LG_COUNT: LongInt = 1;
  LG_DETAIL: Integer = 10;

function PosValue(Value: integer): integer;
begin
  if Value < 0 then
    result := 0
  else
    result := Value;
end;


procedure InitLight(Count, Detail: Integer);
var
  i, j: Integer;
begin
  LG_COUNT := Count;
  LG_DETAIL := Detail;

  for i := 0 to 255 do // Build Lightning LUT
    for j := 0 to 255 do
      FLUTDist[i, j] := ROUND(Sqrt(Sqr(i * 10) + Sqr(j * 10)));
end;

procedure DrawLights(Dest: TImage; FLight: TLightArray;
  AmbientLight: TARGB);
var
  fc, ii, i, j, l, m, n, o, q, D1, D2, R, G, B, AR, AG, AB: integer;
  P: array[0..32767] of PByteArray;
begin
  AR := GetR(AmbientLight);
  AG := GetG(AmbientLight);
  AB := GetB(AmbientLight);
  if LG_DETAIL = 1 then
    fc := 1
  else
    fc := 0;

  for i := ((Dest.Height) div (LG_DETAIL + 1)) - fc downto 1 do
  begin
    for o := 0 to LG_DETAIL do
    begin
      ii := ((LG_DETAIL + 1) * i - o);
//    if ii>Dest.Height-1 then ii:=Dest.Height-1;
  //  if ii<0 then ii:=0;
      P[o] := Dest.ScanLine[ii];
    end;

    for j := (Dest.Width div (LG_DETAIL + 1)) - fc downto 1 do
    begin
      R := AR;
      G := AG;
      B := AB;

      for l := LG_COUNT - 1 downto 0 do // Check the lightsources
      begin
        D1 := ABS(j * (LG_DETAIL + 1) - FLight[l].X) div FLight[l].Size1;
        D2 := ABS(i * (LG_DETAIL + 1) - FLight[l].Y) div FLight[l].Size2;
        if D1 > 255 then D1 := 255;
        if D2 > 255 then D2 := 255;

        m := 255 - FLUTDist[D1, D2];
        if m < 0 then m := 0;

        Inc(R, (PosValue(GetR(FLight[l].Color) - R) * m shr 8));
        Inc(G, (PosValue(GetG(FLight[l].Color) - G) * m shr 8));
        Inc(B, (PosValue(GetB(FLight[l].Color) - B) * m shr 8));
      end;

      for q := LG_DETAIL downto 0 do
      begin
        n := 3 * (j * (LG_DETAIL + 1) - q);

        for o := LG_DETAIL downto 0 do
        begin
          P[o][n] := (P[o][n] * B) shr 8;
          P[o][n + 1] := (P[o][n + 1] * G) shr 8;
          P[o][n + 2] := (P[o][n + 2] * R) shr 8;
        end;
      end;
    end;
  end;


end;


procedure AddImage(Dest, Src: TImage; UseAlpha: boolean);
var Temp2, Temp: TImage;
  X, Y: LongInt;
  pD32, pS32: pARGBRec;
  pD24, pS24: pRGBRec;

begin
  if (Dest = nil) or (Src = nil) then Exit;
  Temp := Src.Clone;

  Temp.Resize(Dest.Width, Dest.Height);
  Temp.BitCount := Dest.BitCount;

  case Dest.BitCount of
    8:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        AddImage(Temp2, Temp);
        Temp2.BitCount := 8;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    16:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        AddImage(Temp2, Temp);
        Temp2.BitCount := 16;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    24:
      begin
        pD24 := Dest.Bits;
        pS24 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD24.R := Set255(pD24.R + pS24.R);
            pD24.G := Set255(pD24.G + pS24.G);
            pD24.B := Set255(pD24.B + pS24.B);

            Inc(pD24);
            Inc(pS24);
          end;
      end;
    32:
      begin
        pD32 := Dest.Bits;
        pS32 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD32.R := Set255(pD32.R + pS32.R);
            pD32.G := Set255(pD32.G + pS32.G);
            pD32.B := Set255(pD32.B + pS32.B);
            if UseAlpha then
              pD32.A := Set255(pD32.A + pS32.A);

            Inc(pD32);
            Inc(pS32);
          end;
      end;


  end;
  Temp.Free;

end;

procedure SubImage(Dest, Src: TImage; UseAlpha: boolean);
var Temp2, Temp: TImage;
  X, Y: LongInt;
  pD32, pS32: pARGBRec;
  pD24, pS24: pRGBRec;

begin
  if (Dest = nil) or (Src = nil) then Exit;
  Temp := Src.Clone;

  Temp.Resize(Dest.Width, Dest.Height);
  Temp.BitCount := Dest.BitCount;

  case Dest.BitCount of
    8:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        SubImage(Temp2, Temp);
        Temp2.BitCount := 8;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    16:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        SubImage(Temp2, Temp);
        Temp2.BitCount := 16;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    24:
      begin
        pD24 := Dest.Bits;
        pS24 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD24.R := Set255(pD24.R - pS24.R);
            pD24.G := Set255(pD24.G - pS24.G);
            pD24.B := Set255(pD24.B - pS24.B);

            Inc(pD24);
            Inc(pS24);
          end;
      end;
    32:
      begin
        pD32 := Dest.Bits;
        pS32 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD32.R := Set255(pD32.R - pS32.R);
            pD32.G := Set255(pD32.G - pS32.G);
            pD32.B := Set255(pD32.B - pS32.B);
            if UseAlpha then
              pD32.A := Set255(pD32.A - pS32.A);

            Inc(pD32);
            Inc(pS32);
          end;
      end;


  end;
  Temp.Free;
end;

procedure MulImage(Dest, Src: TImage; UseAlpha: boolean);
var Temp2, Temp: TImage;
  X, Y: LongInt;
  pD32, pS32: pARGBRec;
  pD24, pS24: pRGBRec;

begin
  if (Dest = nil) or (Src = nil) then Exit;
  Temp := Src.Clone;

  Temp.Resize(Dest.Width, Dest.Height);
  Temp.BitCount := Dest.BitCount;

  case Dest.BitCount of
    8:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        MulImage(Temp2, Temp);
        Temp2.BitCount := 8;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    16:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        MulImage(Temp2, Temp);
        Temp2.BitCount := 16;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    24:
      begin
        pD24 := Dest.Bits;
        pS24 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD24.R := Set255(pD24.R * pS24.R);
            pD24.G := Set255(pD24.G * pS24.G);
            pD24.B := Set255(pD24.B * pS24.B);

            Inc(pD24);
            Inc(pS24);
          end;
      end;
    32:
      begin
        pD32 := Dest.Bits;
        pS32 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD32.R := Set255(pD32.R * pS32.R);
            pD32.G := Set255(pD32.G * pS32.G);
            pD32.B := Set255(pD32.B * pS32.B);
            if UseAlpha then
              pD32.A := Set255(pD32.A * pS32.A);

            Inc(pD32);
            Inc(pS32);
          end;
      end;


  end;
  Temp.Free;
end;

procedure OrImage(Dest, Src: TImage; UseAlpha: boolean);
var Temp2, Temp: TImage;
  X, Y: LongInt;
  pD32, pS32: pARGBRec;
  pD24, pS24: pRGBRec;

begin
  if (Dest = nil) or (Src = nil) then Exit;
  Temp := Src.Clone;

  Temp.Resize(Dest.Width, Dest.Height);
  Temp.BitCount := Dest.BitCount;

  case Dest.BitCount of
    8:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        OrImage(Temp2, Temp);
        Temp2.BitCount := 8;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    16:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        OrImage(Temp2, Temp);
        Temp2.BitCount := 16;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    24:
      begin
        pD24 := Dest.Bits;
        pS24 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD24.R := Set255(pD24.R or pS24.R);
            pD24.G := Set255(pD24.G or pS24.G);
            pD24.B := Set255(pD24.B or pS24.B);

            Inc(pD24);
            Inc(pS24);
          end;
      end;
    32:
      begin
        pD32 := Dest.Bits;
        pS32 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD32.R := Set255(pD32.R or pS32.R);
            pD32.G := Set255(pD32.G or pS32.G);
            pD32.B := Set255(pD32.B or pS32.B);
            if UseAlpha then
              pD32.A := Set255(pD32.A or pS32.A);

            Inc(pD32);
            Inc(pS32);
          end;
      end;


  end;
  Temp.Free;
end;

procedure AndImage(Dest, Src: TImage; UseAlpha: boolean);
var Temp2, Temp: TImage;
  X, Y: LongInt;
  pD32, pS32: pARGBRec;
  pD24, pS24: pRGBRec;

begin
  if (Dest = nil) or (Src = nil) then Exit;
  Temp := Src.Clone;

  Temp.Resize(Dest.Width, Dest.Height);
  Temp.BitCount := Dest.BitCount;

  case Dest.BitCount of
    8:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        AndImage(Temp2, Temp);
        Temp2.BitCount := 8;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    16:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        AndImage(Temp2, Temp);
        Temp2.BitCount := 16;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    24:
      begin
        pD24 := Dest.Bits;
        pS24 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD24.R := Set255(pD24.R and pS24.R);
            pD24.G := Set255(pD24.G and pS24.G);
            pD24.B := Set255(pD24.B and pS24.B);

            Inc(pD24);
            Inc(pS24);
          end;
      end;
    32:
      begin
        pD32 := Dest.Bits;
        pS32 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD32.R := Set255(pD32.R and pS32.R);
            pD32.G := Set255(pD32.G and pS32.G);
            pD32.B := Set255(pD32.B and pS32.B);
            if UseAlpha then
              pD32.A := Set255(pD32.A and pS32.A);

            Inc(pD32);
            Inc(pS32);
          end;
      end;


  end;
  Temp.Free;
end;

procedure DarkestImage(Dest, Src: TImage; UseAlpha: boolean);
var Temp2, Temp: TImage;
  X, Y: LongInt;
  pD32, pS32: pARGBRec;
  pD24, pS24: pRGBRec;

begin
  if (Dest = nil) or (Src = nil) then Exit;
  Temp := Src.Clone;

  Temp.Resize(Dest.Width, Dest.Height);
  Temp.BitCount := Dest.BitCount;

  case Dest.BitCount of
    8:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        DarkestImage(Temp2, Temp);
        Temp2.BitCount := 8;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    16:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        DarkestImage(Temp2, Temp);
        Temp2.BitCount := 16;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    24:
      begin
        pD24 := Dest.Bits;
        pS24 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD24.R := Min(pD24.R, pS24.R);
            pD24.G := Min(pD24.G, pS24.G);
            pD24.B := Min(pD24.B, pS24.B);

            Inc(pD24);
            Inc(pS24);
          end;
      end;
    32:
      begin
        pD32 := Dest.Bits;
        pS32 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD32.R := Min(pD32.R, pS32.R);
            pD32.G := Min(pD32.G, pS32.G);
            pD32.B := Min(pD32.B, pS32.B);
            if UseAlpha then
              pD32.A := Min(pD32.A, pS32.A);

            Inc(pD32);
            Inc(pS32);
          end;
      end;


  end;
  Temp.Free;
end;

procedure LightestImage(Dest, Src: TImage; UseAlpha: boolean);
var Temp2, Temp: TImage;
  X, Y: LongInt;
  pD32, pS32: pARGBRec;
  pD24, pS24: pRGBRec;

begin
  if (Dest = nil) or (Src = nil) then Exit;
  Temp := Src.Clone;

  Temp.Resize(Dest.Width, Dest.Height);
  Temp.BitCount := Dest.BitCount;

  case Dest.BitCount of
    8:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        LightestImage(Temp2, Temp);
        Temp2.BitCount := 8;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    16:
      begin
        Temp2 := Dest.Clone;
        Temp2.BitCount := 24;
        LightestImage(Temp2, Temp);
        Temp2.BitCount := 16;
        Dest.Assign(Temp2);
        Temp2.Free;
      end;
    24:
      begin
        pD24 := Dest.Bits;
        pS24 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD24.R := max(pD24.R, pS24.R);
            pD24.G := max(pD24.G, pS24.G);
            pD24.B := max(pD24.B, pS24.B);

            Inc(pD24);
            Inc(pS24);
          end;
      end;
    32:
      begin
        pD32 := Dest.Bits;
        pS32 := Temp.Bits;

        for X := 0 to Dest.Width - 1 do
          for Y := 0 to Dest.Height - 1 do
          begin
            pD32.R := max(pD32.R, pS32.R);
            pD32.G := max(pD32.G, pS32.G);
            pD32.B := max(pD32.B, pS32.B);
            if UseAlpha then
              pD32.A := max(pD32.A, pS32.A);

            Inc(pD32);
            Inc(pS32);
          end;
      end;


  end;
  Temp.Free;



end;


procedure SwapColors(Img: TImage; Col, NewCol: TARGB; UseAlpha: boolean);
var X, Y: LongInt;
  pB: pByte;
  pD32: pARGBRec;
  pD24: pRGBRec;
  c24: TRGBRec;
  c32: TARGBRec;
begin
  if (Img = nil) then Exit;

  case Img.BitCount of
    8:
      begin
        pB := Img.Bits;

        for X := 0 to Img.Width - 1 do
          for Y := 0 to Img.Height - 1 do
          begin
            if pB^ = NewCol then
              pB^ := NewCol;

            Inc(pB);
          end;
      end;
    16:
      begin
        Img.BitCount := 24;
        SwapColors(Img, Col, NewCol);
        Img.BitCount := 16;
      end;
    24:
      begin
        pD24 := Img.Bits;
        c24 := RGBRec(Col);

        for X := 0 to Img.Width - 1 do
          for Y := 0 to Img.Height - 1 do
          begin
            if (c24.R = pD24.R) and
              (c24.G = pD24.G) and
              (c24.B = pD24.B) then
              pD24^ := RGBRec(NewCol);

            Inc(pD24);
          end;
      end;
    32:
      begin
        pD32 := Img.Bits;
        c32 := ARGBRec(Col);

        for X := 0 to Img.Width - 1 do
          for Y := 0 to Img.Height - 1 do
          begin
            if UseAlpha then
            begin
              if pD32.Color = NewCol then
                pD32.Color := NewCol;
            end
            else
            begin
              if (c32.R = pD32.R) and
                (c32.G = pD32.G) and
                (c32.B = pD32.B) then
                pD32.Color := SetA(pD32.A, NewCol);
            end;

            Inc(pD32);
          end;
      end;
  end;
end;

procedure FastBlur(Img: TImage; UseAlpha: boolean);
type
  TScans24 = array[0..2] of PLine24;
  TScans32 = array[0..2] of PLine32;
var
  X, Y: LongInt;
  pD32: pARGBRec;
  pD24: pRGBRec;
  Temp: TImage;
  R, G, B, A: LongInt;
  DScans: PLines;
  SScans: PLines;
  Scans24: TScans24;
  Scans32: TScans32;

begin
  Temp := TImage.Create();
  Temp.BitCount := Img.BitCount;
  Temp.Resize(Img.Width + 2, Img.Height + 2);
  Temp.ClearDWord(Img.Pixels[0, 0]);
  Img.Draw(Temp, 1, 1);

  DScans := Img.CreateScans;
  SScans := Temp.CreateScans;

  case Img.BitCount of
    24:
      begin
        pD24 := Img.Bits;
        for Y := 0 to Temp.Height - 3 do
        begin
          Scans24[0] := SScans[y];
          Scans24[1] := SScans[y + 1];
          Scans24[2] := SScans[y + 2];

          for X := 0 to Temp.Width - 3 do
          begin
            R := Scans24[1, X].R + Scans24[1, X + 1].R * 2 + Scans24[1, X + 2].R
              +
              Scans24[0, X + 1].R + Scans24[2, X + 1].R;
            G := Scans24[1, X].G + Scans24[1, X + 1].G * 2 + Scans24[1, X + 2].G
              +
              Scans24[0, X + 1].G + Scans24[2, X + 1].G;
            B := Scans24[1, X].B + Scans24[1, X + 1].B * 2 + Scans24[1, X + 2].B
              +
              Scans24[0, X + 1].B + Scans24[2, X + 1].B;

            pD24.R := Set255(R div 6);
            pD24.G := Set255(G div 6);
            pD24.B := Set255(B div 6);

            Inc(pD24);
          end;
        end;
      end;
    32:
      begin
        pD32 := Img.Bits;
        for Y := 0 to Temp.Height - 3 do
        begin
          Scans32[0] := SScans[y];
          Scans32[1] := SScans[y + 1];
          Scans32[2] := SScans[y + 2];

          for X := 0 to Temp.Width - 3 do
          begin
            R := Scans32[1, X].R + Scans32[1, X + 1].R * 2 + Scans32[1, X + 2].R
              +
              Scans32[0, X + 1].R + Scans32[2, X + 1].R;
            G := Scans32[1, X].G + Scans32[1, X + 1].G * 2 + Scans32[1, X + 2].G
              +
              Scans32[0, X + 1].G + Scans32[2, X + 1].G;
            B := Scans32[1, X].B + Scans32[1, X + 1].B * 2 + Scans32[1, X + 2].B
              +
              Scans32[0, X + 1].B + Scans32[2, X + 1].B;
            A := Scans32[1, X].A + Scans32[1, X + 1].A * 2 + Scans32[1, X + 2].A
              +
              Scans32[0, X + 1].A + Scans32[2, X + 1].A;

            pD32.R := Set255(R div 6);
            pD32.G := Set255(G div 6);
            pD32.B := Set255(B div 6);
            if USeAlpha then
              pD32.A := Set255(A div 6);
            Inc(pD32);
          end;
        end;
      end;

  end;

  Temp.Free;
  FreeMem(DScans);
  FreeMem(SScans);

end;


procedure SwapRGB(Img: TImage);
var
  PRGBA: PARGBRec;
  PRGB: PRGBRec;
  X, Y: LongInt;
  B: byte;
begin
   case Img.BitCount of
    24:
      begin
        PRGB := Img.Bits;
        for X := 0 to Img.Width - 1 do
          for Y := 0 to Img.Height - 1 do
          begin
            b := PRGB.R;
            PRGB.R := PRGB.B;
            PRGB.B := b;
            Inc(PRGB);
          end;
      end;
    32:
      begin
        PRGBA := Img.Bits;
        for X := 0 to Img.Width - 1 do
          for Y := 0 to Img.Height - 1 do
          begin
            b := PRGBA.R;
            PRGBA.R := PRGBA.B;
            PRGBA.B := b;
            Inc(PRGBA);
          end;
      end;
  end;
end;


function GreyscaleColorTable: TRGBQuads;
var
  i: Integer;
begin
  for i:=0 to 255 do
    with Result[i] do
    begin
      rgbRed := i;
      rgbGreen := i;
      rgbBlue := i;
      rgbReserved := 0;
    end;
end;


end.

