unit TextureMgrs;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Textures, BaseMgrs;

type

  TTextureMgr = class(TNamedMgr)
  private

  protected

  public
    constructor Create(Owns: boolean = true); override;
    destructor Destroy; override;

     // returns reference to tex in list, if not exists load tex from file
    function GetTexture(AName: string): TTexture; virtual;

     // diferences on GetTex - this not change tex detail(=size)
    function GetPicture(AName: string): TTexture; virtual;

     // adds created texture to list, returns Tex parameter ( for  Tex1:=Mgr.AddTexture(CreateTexture('Wall','walls\01.jpg',1,16));
    function AddTexture(Tex: TTexture): TTexture; virtual;

    // adds textures named in stinglist, returns added textures count
    function AddTextures(List:TStringList):LongInt;virtual;

     // returns reference to tex in list if exists otherwise returns nil
    function FindTexture(AName: string): TTexture; virtual;

     // returns true if texture is in list
    function IsTexture(AName: string): boolean; virtual;

      // frees texture from list and memory , returns true if found in list
     function DeleteTexture(AName:string):boolean;overload;virtual;

    // frees texture from list and memory , returns true if found in list
    function DeleteTexture(Tex:TTexture):boolean;overload;virtual;

    // frees textures named in stinglist, returns freed textures count
    function DeleteTextures(List:TStringList):LongInt;virtual;

  published

  end;



implementation

constructor TTextureMgr.Create;
begin
  inherited Create;

end;

destructor TTextureMgr.Destroy;
begin

  inherited Destroy;
end;

function TTextureMgr.GetTexture(AName: string): TTexture;
var Tex:TTexture;
begin
 Tex:=FindTexture(AName);
 if Tex=nil then
 begin
  Tex:=TTexture.Create(AName);
  Tex.Init;
  Add(Tex);
 end;
Result:=Tex;
end;

function TTextureMgr.GetPicture(AName: string): TTexture;
var Tex:TTexture;
begin
 Tex:=FindTexture(AName);
 if Tex=nil then
 begin
  Tex:=TTexture.Create(AName);
  Tex.Detail:=1;
  Tex.Init;
  Add(Tex);
 end;
Result:=Tex;
end;


function TTextureMgr.AddTexture(Tex: TTexture): TTexture;
begin
 Result:=Tex;
 Add(Tex);
 if not Tex.Inited then
  Tex.Init;

end;

function TTextureMgr.FindTexture(AName: string): TTexture;
var I:LongInt;
begin
 Result:=nil;

  for I:=0 to Count-1 do
   if CompStr(AName,TTexture(Objects[i]).Name)
    then
     begin
      Result:=TTexture(Objects[i]);
      Exit;
     end;

end;

function TTextureMgr.IsTexture(AName: string): boolean;
var I:LongInt;
begin
  Result := false;
  for I := 0 to Count - 1 do
    if CompStr(AName, TTexture(Objects[i]).Name) then
    begin
      Result := true;
      Exit;
    end;

end;

function TTextureMgr.DeleteTexture(AName:string):boolean;
begin
 Result:=DeleteTexture(FindTexture(AName));
end;

function TTextureMgr.DeleteTexture(Tex:TTexture):boolean;
begin
 Result:=false;
 if Tex=nil then Exit;
 Extract(Tex);
 Tex.Free;
 Tex:=nil;

 Result:=true;

end;

function TTextureMgr.AddTextures(List:TStringList):LongInt;
var I,Cnt:LongInt;
begin
 Result:=0;
 if List=nil then Exit;

 Cnt:=Count;

 for I:=0 to List.Count-1 do
  AddTexture(CreateTexture(List[I]));

 Result:=Count-Cnt;

end;


function TTextureMgr.DeleteTextures(List:TStringList):LongInt;
var I,Cnt:LongInt;
begin
 Result:=0;
 if List=nil then Exit;

 Cnt:=Count;

 for I:=0 to List.Count-1 do
  DeleteTexture(List[I]);

 Result:=Count-Cnt;

end;


end.

