unit DemoTools;

interface

uses Windows, SysUtils, Classes, WinUtils, BaseObjs, Objs3D, Types3D, Utils3D,
  Images, ImgUtils, GGraphic, Timers, Graphics,Textures,DIB,Colors,KeyBoard;



type

  TOnShut = procedure of object;

  TShutTimer = class
  private
    FTimer: TTimer;
    FShutTime: DWord;
    FOnShut: TOnShut;
    FOnShutProc: procedure;
    FStart: DWord;
    FElapsed: DWord;
    procedure OnTimer(Sender: TObject);
    procedure CallExitProc;

  protected

  public
    constructor Create(AShutTime: DWord);
    destructor Destroy; override;
    procedure SetTime(Time:DWord);
    procedure Start;
    procedure Pause;
    procedure Resume;
    procedure Stop;

  published

  end;

  TImgText = class
  private
    FPos: TPoint2I;
    FTex: TTexture;
    FCaption: string;
    FFont: TFont;

  protected

  public
    constructor Create;
    destructor Destroy; override;

    procedure Draw;
    procedure SetCaption(Value: string);
    procedure SetFont(Name: string; Size: LongInt; Col: TARGB);
    procedure SetPos(X, Y: LongInt);

  published

  end;

  TInput=class
      private
       FRelPos:Windows.TPoint;
       FOldPos:Windows.TPoint;
      protected

      public
        constructor Create;
        destructor Destroy; override;

        function GetMouseXRel:LongInt;
        function GetMouseXAbs:LongInt;
        function GetMouseYRel:LongInt;
        function GetMouseYAbs:LongInt;

        function IsKey(vkey:LongInt):boolean;
        procedure Update;
      published

      end;

var

  ShutTimer:TShutTimer;
  ImgText:TImgText;
  Input:TInput;

procedure TakeScreenShot(DC:HDC;W,H:LongInt);

implementation

uses ImgJPG;

procedure TakeScreenShot(DC:HDC;W,H:LongInt);
var Img: TImage;
  I: LongInt;
begin
  JPEGSaveQuality := 95;

  Img := TImage.Create();
  Img.Resize(W,H);
  Img.BitCount := 24;

  StretchBlt(Img.Canvas.Handle, 0,0,W,H,DC,0,0,W,H,SRCCOPY);

  I := 0;
  while FileExists(ExeDir + 'Shot' + IntToStr(I) + '.jpg') do
    Inc(I);
  Img.Save(ExeDir + 'Shot' + IntToStr(I) + '.jpg');
  Img.Free;

end;


constructor TShutTimer.Create(AShutTime: DWord);
begin
  inherited Create;
  FTimer := TTimer.Create(nil);
  FShutTime := AShutTime;
  FTimer.OnTimer := OnTimer;
  FTimer.Enabled := false;
  FTimer.Interval := 100;
end;


destructor TShutTimer.Destroy;
begin
  FTimer.Free;

end;

procedure TShutTimer.SetTime(Time:DWord);
begin
FShutTime:=Time;
end;


procedure TShutTimer.OnTimer(Sender: TObject);
begin
  Inc(FElapsed, FTimer.Interval);
  if FElapsed > FShutTime then
    CallExitProc;
end;

procedure TShutTimer.CallExitProc;
begin
  if Assigned(FOnShut) then
    FOnShut;
  if Assigned(FOnShutProc) then
    FOnShutProc;

  Halt(0)

end;

procedure TShutTimer.Start;
begin
  FElapsed := 0;
  FTimer.Enabled := true;
end;

procedure TShutTimer.Pause;
begin
  FTimer.Enabled := not true;
end;

procedure TShutTimer.Resume;
begin
  FTimer.Enabled := true;
end;

procedure TShutTimer.Stop;
begin
  FElapsed := 0;
  FTimer.Enabled := not true;
end;


constructor TImgText.Create;
begin
 inherited Create;
 FTex:=TTexture.Create;
 FTex.ClearAfterInit:=false;
 FTex.ColorKey:=$FF000000;
 FTex.UseMipMaps:=true;
 FTex.Detail:=1;
 FFont:=TFont.Create;
 FFont.Style:=[fsBold];
 fFont.Color:=clWhite;
 FCaption:='Text';
 FPos:=TPoint2I(Point(10,10));

end;

destructor TImgText.Destroy;
begin
 FFont.Free;
 FTex.Free;
 inherited Destroy;
end;

procedure TImgText.Draw;
begin
if FTex<>nil then
if FTex.DevTex<>nil then
Graphic.RenderTexture(FPos.X,FPos.Y,FTex);

end;

procedure TImgText.SetCaption(Value: string);
var Img:TImage;
    Db:TDIB;
    Can:TCanvas;
    TW,TH:LongInt;

begin
FCaption:=Value;
Img:=FTex.Image;
Db:=Img.DIB;
Can:=Db.Canvas;
Can.Font.Assign(FFont);
TW:=Can.TextWidth(FCaption);
TH:=Can.TextHeight(FCaption);
Db.SetSize(TW+5,TH+2,24);
Img.ClearARGB($FF000000);
SetBkMode(Can.Handle,Transparent);
Can.TextOut(1,1,FCaption);
Img.Resize(Img.Width*2,Img.Height*2);
Blur(Img,24,1);
Img.Resize(Img.Width div 2,Img.Height div 2);


FTex.Init;

end;

procedure TImgText.SetFont(Name: string; Size: LongInt; Col: TARGB);
begin
FFont.Name:=Name;
FFont.Size:=Size;
FFont.Color:=ARGBToBGR(Col);
end;

procedure TImgText.SetPos(X, Y: LongInt);
begin
FPos.X:=X;
FPos.Y:=Y;
end;



constructor TInput.Create;
begin
 inherited Create;
 FOldPos:=Point(100,100);
end;

destructor TInput.Destroy;
begin

  inherited Destroy;;

end;

function TInput.GetMouseXRel:LongInt;
begin
Result:=FRelPos.X;
end;


function TInput.GetMouseXAbs:LongInt;
var P:Windows.TPoint;
begin
GetCursorPos(P);
Result:=P.X;
end;

function TInput.GetMouseYRel:LongInt;
begin
Result:=FRelPos.Y;

end;

function TInput.GetMouseYAbs:LongInt;
var P:Windows.TPoint;
begin
GetCursorPos(P);
Result:=P.Y;
end;

function TInput.IsKey(vkey:LongInt):boolean;
begin
 Result:=KeyBoard.IsKeyDown(vkey);
end;

procedure TInput.Update;
var P:Windows.TPoint;
begin
GetCursorPos(P);
FRelPos.X:=(FOldPos.X+P.X)div 2-100;
FRelPos.Y:=(FOldPos.Y+P.Y)div 2-100;
FOldPos:=P;
SetCursorPos(100, 100);
end;

initialization
  ShutTimer:=TShutTimer.Create(10000);
  ImgText:=TImgText.Create;
  Input:=TInput.Create;


finalization
{  ShutTimer.Free;
  ImgText.Free;
  Input.Free;
 }


end.

