///////////////////////////////////////////////////////
// VAMPYRE PCX LOADER
///////////////////////  by Galfar


//////////////////////////////////////////////////////
// contains only base functions for load/save
/////////////////////////////////////////////////////
//  can only load  8bit or 24bit PCX Images
//  no saving functions are available
// it is so damnable format - i dont want have nothing more with this
/////////////////////////////////////////////////////

unit PCXImage;

interface

uses Windows, Graphics, Classes, SysUtils, DIB;


type

  TPCXFile = class
  private
    FDIB: TDIB;

  protected
    procedure Draw(ACanvas: TCanvas; const Rect: TRect);

    function GetHeight: Integer;
    function GetWidth: Integer;
    procedure SetHeight(Value: Integer);
    procedure SetWidth(Value: Integer);
    function GetBits: byte;
    procedure SetBits(Value: byte);


  public

    destructor Destroy;override;

    procedure LoadFromFile(FileName: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(Stream: TStream);

  published

    property Width: LongInt read GetWidth write SetWidth;
    property Height: LongInt read GetWidth write SetWidth;
    property BitCount: byte read GetBits write SetBits;

  end;


  TPCXImage = class(TGraphic)
  protected
    procedure Draw(ACanvas: TCanvas; const Rect: TRect); override;
    function GetEmpty: Boolean; override;
    function GetHeight: Integer; override;
    function GetWidth: Integer; override;
    function GetBits: byte;
    procedure SetHeight(Value: Integer); override;
    procedure SetWidth(Value: Integer); override;
  public
    Image: TPCXFile;

    constructor Create; override;
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
    procedure LoadFromStream(Stream: TStream); override;
    procedure SaveToStream(Stream: TStream); override;
    procedure LoadFromClipboardFormat(AFormat: Word; AData: THandle;
      APalette: HPALETTE); override;
    procedure SaveToClipboardFormat(var AFormat: Word; var AData: THandle;
      var APalette: HPALETTE); override;

    property BitCount: byte read GetBits;

  end;


  TPCXTriple = record
    r, g, b: byte;
  end;

  TPCXColors256 = array[0..255] of TPCXTriple;
  TPCXColors16 = array[0..15] of TPCXTriple;

  TPCXHeader = record
    Maker: byte;
    Version: byte;
    Encoding: byte;
    Bpp: byte;
    Xmn, Ymn: smallint;
    Xmx, Ymx: smallint;
    HRes, VRes: smallint;
    CMap: TPCXColors16;
    Reserved: byte;
    NPlanes: byte;
    NBpl: smallint;
    PalType: smallint;
  end;



implementation

function Iff(const Condition: Boolean; const TruePart, FalsePart: Integer): Integer; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

destructor TPCXFile.Destroy;
begin
 FDIB.Free;
 inherited Destroy;
end;


procedure TPCXFile.LoadFromFile(FileName: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TPCXFile.LoadFromStream(Stream: TStream);
var w, h, tot, cnt, I, ii, iii, x: LongInt;
  Header: TPCXHeader;
  Bits, data: byte;
  Img, S: TMemoryStream;
  Pal256: TPCXColors256;
  r, g, b: byte;
  Tri: TPCXTriple;

  function GetByte: byte;
  begin
    Stream.ReadBuffer(Result, 1);
  end;


begin

  with Stream do
  begin

    Stream.Read(Header, Sizeof(Header));

    W := Header.Xmx - Header.Xmn + 1;
    H := Header.Ymx - Header.Ymn + 1;

    Bits := 0;
    with Header do
      case Header.Bpp of
        1:
          if NPlanes = 4 then Bits := 4;
        8:
          case NPlanes of
            1: Bits := 8;
            3: Bits := 24;
          end;
      end;

    if not (Bits in [4, 8, 24]) then Exit;

    Tot := Header.NPlanes * Header.NBpl;

    Seek(128, 0);

    Img := TMemoryStream.Create;


    case Bits of
      8:
        for I := 0 to H - 1 do
        begin

          x := 0;
          while x < Tot do
          begin
            Data := GetByte;
            if Data and $C0 = $C0 then
            begin
              cnt := Data and $3F;
              Data := GetByte;
            end
            else
              cnt := 1;

            for II := 0 to cnt - 1 do
            begin
              Img.WriteBuffer(data, 1);
              inc(x);
            end;

          end;



        end;
      24:
        begin
        // working now but unusable  -  warey slowwwwwwwwwwwwwwww


          for I := 0 to H * 3 - 1 do
          begin

            x := 0;
            while x < Header.NBpl do
            begin
              Data := GetByte;
              if Data and $C0 = $C0 then
              begin
                cnt := Data and $3F;
                Data := GetByte;
              end
              else
                cnt := 1;

              for II := 0 to cnt - 1 do
              begin
                Img.WriteBuffer(data, 1);
                inc(x);
              end;

            end;
          end;


          S := TMemoryStream.Create;
          Img.Position := 0;

          for I := 0 to h - 2 do
          begin
            III := I * 3 * (w);
            for II := 0 to w - 1 do
            begin


              Img.ReadBuffer(r, 1);
              Img.Position := Img.Position + w;
              Img.ReadBuffer(g, 1);
              Img.Position := Img.Position + w;
              Img.ReadBuffer(b, 1);
              Img.Position := III;
              Inc(III);


              Tri.r := b;
              Tri.g := g;
              Tri.b := r;

              S.WriteBuffer(Tri, 3);

            end;

          end;

          Img.LoadFromStream(s);
          S.Free;


        end;
    end;


    FDIB.Free;
    FDIB := TDIB.Create;
    FDIB.SetSize(w, h, Bits);
    Move(Img.memory^, FDIB.PBits^, Img.Size);
    FDIB.Mirror(false, true);

    case Bits of
      8:
        begin
          ReadBuffer(b, 1);
          ReadBuffer(Pal256, SizeOf(Pal256));


          for I := 0 to 255 do
          begin
            FDIB.ColorTable[I].rgbRed := Pal256[I].r;
            FDIB.ColorTable[I].rgbGreen := Pal256[I].g;
            FDIB.ColorTable[I].rgbBlue := Pal256[I].b;
            FDIB.ColorTable[I].rgbReserved := 0;
          end;


          FDIB.UpdatePalette;


        end;
      24:
        begin


        end;

    end;

  // FDIB.SaveToFile('umajokenta.dib');
    Img.Free;


  end;
end;

procedure TPCXFile.SaveToFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TPCXFile.SaveToStream(Stream: TStream);
begin
  with Stream do
  begin

   //  ni
   raise Exception.Create('ERROR!Saving images in PCX format is not yet implemented!');
  end;
end;


procedure TPCXFile.Draw(ACanvas: TCanvas; const Rect: TRect);
begin
  if Assigned(FDIB) then
    FDIB.Draw(ACanvas, Rect);

end;



function TPCXFile.GetHeight: Integer;
begin
  Result := Iff(FDIB <> nil, FDIB.Height, 0);
end;


function TPCXFile.GetWidth: Integer;
begin
  Result := Iff(FDIB <> nil, FDIB.Width, 0);
end;

procedure TPCXFile.SetHeight(Value: Integer);
begin
  if (FDIB <> nil) then
    FDIB.Height := Value;

end;

procedure TPCXFile.SetWidth(Value: Integer);
begin
  if (FDIB <> nil) then
    FDIB.Width := Value;

end;

function TPCXFile.GetBits: byte;
begin
  Result := Iff(FDIB <> nil, FDIB.BitCount, 0);
end;


procedure TPCXFile.SetBits(Value: byte);
begin
  if (FDIB <> nil) and (Value in [16, 24, 32]) then
    FDIB.BitCount := Value;

end;

///////////////// TPCX IMAGE  ///////////////////////


constructor TPCXImage.Create;
begin
  inherited;
  Image := TPCXFile.Create;
end;

destructor TPCXImage.Destroy;
begin
  Image.Free;
  inherited;
end;

procedure TPCXImage.Assign(Source: TPersistent);
var TempMS: TMemoryStream;
begin
  if (Source is TBitmap) or (Source is TDIB) then
  begin
    if Image.FDIB = nil then
      Image.FDIB := TDIB.Create;
    Image.FDIB.Assign(Source);
  end
  else
    if Source is TPCXImage then
    begin
      TempMS := TMemoryStream.Create;
      TPCXImage(Source).SaveToStream(TempMS);
      Self.LoadFromStream(TempMS);
      TempMS.Free;

    end
    else

      inherited Assign(Source);
end;

procedure TPCXImage.Draw(ACanvas: TCanvas; const Rect: TRect);
begin
  Image.Draw(ACanvas, Rect);
end;

function TPCXImage.GetEmpty: Boolean;
begin
  if (Image <> nil) and (Image.FDIB <> nil) then
    Result := False
  else
    Result := True;
end;

function TPCXImage.GetHeight: Integer;
begin
  Result := Image.Height;
end;

function TPCXImage.GetWidth: Integer;
begin
  Result := Image.Width;
end;

function TPCXImage.GetBits: byte;
begin
  Result := Image.BitCount;
end;


procedure TPCXImage.LoadFromClipboardFormat(AFormat: Word; AData: THandle;
  APalette: HPALETTE);
begin
  raise
    Exception.Create('Culov!! Clipboard(ble) is not supported by Vampyre SoftWorks');
end;

procedure TPCXImage.LoadFromFile(const Filename: string);
begin
  Image.LoadFromFile(Filename);
end;

procedure TPCXImage.LoadFromStream(Stream: TStream);
begin
  Image.LoadFromStream(Stream);
end;

procedure TPCXImage.SaveToClipboardFormat(var AFormat: Word;
  var AData: THandle; var APalette: HPALETTE);
begin
  raise Exception.Create('Cannot save a TPCXImage to the Clipboard');
end;

procedure TPCXImage.SaveToFile(const Filename: string);
begin
  Image.SaveToFile(Filename);
end;

procedure TPCXImage.SaveToStream(Stream: TStream);
begin

  Image.SaveToStream(Stream);
end;

procedure TPCXImage.SetHeight(Value: Integer);
begin
  Image.Height:=Value;
end;


procedure TPCXImage.SetWidth(Value: Integer);
begin
  Image.Width:=Value;

end;

initialization

 TPicture.RegisterFileFormat('pcx', 'ZSoft Paintbrush', TPCXImage);


end.

