///////////////////////////////////////////////////////
// VAMPYRE TGA LOADER
///////////////////////  by Galfar


//////////////////////////////////////////////////////
// contains only base functions for load/save
/////////////////////////////////////////////////////
// supports loading of TGA format 1,2,9,10
// 8,16,24,32 bit Targas saved uncompressed or with RLE
//
// can save only uncompressed  8/24/32 bit images
//////////////////////////////////////////////////////

unit TGAImage;

interface

uses Windows, Graphics, Classes, SysUtils, DIB;


type

  TTGAFile = class
  private
    FDIB: TDIB;

  protected
    procedure Draw(ACanvas: TCanvas; const Rect: TRect);

    function GetHeight: Integer;
    function GetWidth: Integer;
    procedure SetHeight(Value: Integer);
    procedure SetWidth(Value: Integer);
    function GetBits: byte;
    procedure SetBits(Value: byte);


  public

    destructor Destroy;override;

    procedure LoadFromFile(FileName: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(Stream: TStream);

  published

    property Width: LongInt read GetWidth write SetWidth;
    property Height: LongInt read GetHeight write SetHeight;
    property BitCount: byte read GetBits write SetBits;
    property DIB:TDIB read FDIB;
  end;


  TTGAImage = class(TGraphic)
  protected
    procedure Draw(ACanvas: TCanvas; const Rect: TRect); override;
    function GetEmpty: Boolean; override;
    function GetHeight: Integer; override;
    function GetWidth: Integer; override;
    function GetBits: byte;
    procedure SetHeight(Value: Integer); override;
    procedure SetWidth(Value: Integer); override;
  public
    Image: TTGAFile;

    constructor Create; override;
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
    procedure LoadFromStream(Stream: TStream); override;
    procedure SaveToStream(Stream: TStream); override;
    procedure LoadFromClipboardFormat(AFormat: Word; AData: THandle;
      APalette: HPALETTE); override;
    procedure SaveToClipboardFormat(var AFormat: Word; var AData: THandle;
      var APalette: HPALETTE); override;

    property BitCount: byte read GetBits;

  end;




implementation


function Iff(const Condition: Boolean; const TruePart, FalsePart: Integer): Integer; overload;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

destructor TTGAFile.Destroy;
begin
 FDIB.Free;
 inherited Destroy;
end;


procedure TTGAFile.LoadFromFile(FileName: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TTGAFile.LoadFromStream(Stream: TStream);
type
  TArr = array[0..3] of byte;
  TPal = array[0..255, 0..2] of byte;

var w, h, fentry, entries: word;
  TGAFormat, TGAColors,inv: byte;
  TempData: pointer;
  SizeToLoad: cardinal;
  cnt: LongInt;
  cpixel, I: LongInt;
  hdr, esize: byte;
  S: TMemoryStream;
  Pal: TPal;
  Pix: ^TArr;
  Bytespp: byte;
begin

  with Stream do
  begin
//    Seek(2, soFromBeginning);
    ReadBuffer(TGAFormat, 2);
    ReadBuffer(TGAFormat, 2);
    ReadBuffer(fentry, 2);
    ReadBuffer(entries, 2);
    ReadBuffer(esize, 1); //  color map entry size
    ReadBuffer(esize, 1); //  color map entry size


//    Seek(12, soFromBeginning);
    ReadBuffer(w, 2);

    ReadBuffer(w, 2);
    ReadBuffer(h, 2);
    ReadBuffer(TGAColors, 1);
    ReadBuffer(inv, 1);



    if TGAFormat in [1, 2, 9, 10] then
    begin
      // Format 1:  uncompressed indexed-colors
      // Format 2:  uncompressed RGB
      // Format 9:  RLE compressed indexed-colors
      // Format 10: RLE compressed RGB


      SizeToLoad := TGAColors div 8;
      Bytespp := SizeToLoad;

      SizeToLoad := SizeToLoad * w * h;


      FDIB.Free;
      FDIB := TDIB.Create;


      FDIB.SetSize(w, h, TGAColors);

      if TGAFormat in [1, 9] then
      begin
        ReadBuffer(Pal, SizeOf(Pal));

        for I := 0 to 255 do
        begin
          FDIB.ColorTable[I].rgbRed := Pal[I, 2];
          FDIB.ColorTable[I].rgbGreen := Pal[I, 1];
          FDIB.ColorTable[I].rgbBlue := Pal[I, 0];
          FDIB.ColorTable[I].rgbReserved := 0;

        end;
        FDIB.UpdatePalette;

      end;

      case TGAFormat of
        1, 2:
          begin

            GetMem(tempData, SizeToLoad);
            ReadBuffer(tempData^, SizeToLoad);
            Move(tempData^, FDIB.PBits^, SizeToLoad);
            FreeMem(tempData);
          end;
        9, 10:
          begin
            cnt := w * h;
            GetMem(Pix, Bytespp);
            cpixel := 0;
            S := TMemoryStream.Create;
            while cpixel < cnt do
            begin
              hdr := 0;
              ReadBuffer(hdr, 1);
              if hdr < 128 then
              begin
                Inc(hdr);

                for I := 0 to hdr - 1 do
                begin
                  ReadBuffer(Pix^, Bytespp);

                  S.WriteBuffer(Pix^[0], 1);

                  if Bytespp > 1 then
                  begin
                    S.WriteBuffer(Pix^[1], 1);
                    S.WriteBuffer(Pix^[2], 1);
                  end;
                  if Bytespp > 3 then
                    S.WriteBuffer(Pix^[3], 1);

                  Inc(cpixel);

                end;
              end
              else
              begin
                Dec(hdr, 127);

                ReadBuffer(Pix^, Bytespp);
                for I := 0 to hdr - 1 do
                begin

                  S.WriteBuffer(Pix^[0], 1);
                  if Bytespp > 1 then
                  begin
                    S.WriteBuffer(Pix^[1], 1);
                    S.WriteBuffer(Pix^[2], 1);
                  end;
                  if Bytespp > 3 then
                    S.WriteBuffer(Pix^[3], 1);

                  Inc(cpixel);

                end;

              end;

            end;

            FreeMem(Pix);
            Move(S.Memory^, FDIB.PBits^, S.Size);
            S.Free;
          end;
      end;


      if inv>10 then FDIB.Mirror(false,true);
    end;
  end;

end;

procedure TTGAFile.SaveToFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename,fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TTGAFile.SaveToStream(Stream: TStream);
var b: Byte;
  w, wx, wy: word;

  procedure WritePal;
  type TPal = array[0..255, 0..2] of byte;
  var I: LongInt;
    Pal: TPal;
  begin
    for I := 0 to 255 do
    begin
      Pal[I, 2] := FDIB.ColorTable[I].rgbRed;
      Pal[I, 1] := FDIB.ColorTable[I].rgbGreen;
      Pal[I, 0] := FDIB.ColorTable[I].rgbBlue;
    end;
    Stream.WriteBuffer(Pal, SizeOf(Pal));

  end;

begin
  with Stream do
  begin
    b := 0;
    WriteBuffer(b, 1);

    b := 0;
    if BitCount = 8 then b := 1;
    WriteBuffer(b, 1);

    b := 2;
    if BitCount = 8 then b := 1;
    WriteBuffer(b, 1);

    w := 0;
    WriteBuffer(w, 2);


    w := 0;
    if BitCount = 8 then w := 256;
    WriteBuffer(w, 2);

    b := 0;
    if BitCount = 8 then b := 24;
    WriteBuffer(b, 1);

    w := 0;
    WriteBuffer(w, 2);

    w := 0;
    WriteBuffer(w, 2);

    wx := Width;
    wy := Height;
    WriteBuffer(wx, 2);

    WriteBuffer(wy, 2);

    w := BitCount;
    WriteBuffer(w, 2);

    b := 0;
    WriteBuffer(b, 0);

    if BitCount = 8 then
      WritePal;

    WriteBuffer(FDIB.PBits^, (BitCount div 8) * Width * Height);



  end;
end;


procedure TTGAFile.Draw(ACanvas: TCanvas; const Rect: TRect);
begin
  if Assigned(FDIB) then
    FDIB.Draw(ACanvas, Rect);

end;



function TTGAFile.GetHeight: Integer;
begin
  Result := Iff(FDIB <> nil, FDIB.Height, 0);
end;


function TTGAFile.GetWidth: Integer;
begin
  Result := Iff(FDIB <> nil, FDIB.Width, 0);
end;

procedure TTGAFile.SetHeight(Value: Integer);
begin
  if (FDIB <> nil) then
    FDIB.Height := Value;

end;

procedure TTGAFile.SetWidth(Value: Integer);
begin
  if (FDIB <> nil) then
    FDIB.Width := Value;

end;

function TTGAFile.GetBits: byte;
begin
  Result := Iff(FDIB <> nil, FDIB.BitCount, 0);
end;


procedure TTGAFile.SetBits(Value: byte);
begin
  if (FDIB <> nil) and (Value in [8,16, 24, 32]) then
    FDIB.BitCount := Value;

end;

///////////////// TTGA IMAGE  ///////////////////////


constructor TTGAImage.Create;
begin
  inherited;
  Image := TTGAFile.Create;
end;

destructor TTGAImage.Destroy;
begin
  Image.Free;
  inherited;
end;

procedure TTGAImage.Assign(Source: TPersistent);
begin
  if (Source is TBitmap) or (Source is TDIB)
  or(Source is TTGAImage)or(Source is TIcon) then
  begin
    if Image.FDIB = nil then
      Image.FDIB := TDIB.Create;
    Image.FDIB.Assign(Source);
  end
    else

      inherited Assign(Source);
end;

procedure TTGAImage.Draw(ACanvas: TCanvas; const Rect: TRect);
begin
  Image.Draw(ACanvas, Rect);
end;

function TTGAImage.GetEmpty: Boolean;
begin
  if (Image <> nil) and (Image.FDIB <> nil) then
    Result := False
  else
    Result := True;
end;

function TTGAImage.GetHeight: Integer;
begin
  Result := Image.Height;
end;

function TTGAImage.GetWidth: Integer;
begin
  Result := Image.Width;
end;

function TTGAImage.GetBits: byte;
begin
  Result := Image.BitCount;
end;


procedure TTGAImage.LoadFromClipboardFormat(AFormat: Word; AData: THandle;
  APalette: HPALETTE);
begin
  raise
    Exception.Create('Culov!! Clipboard(ble) is not supported by Vampyre SoftWorks');
end;

procedure TTGAImage.LoadFromFile(const Filename: string);
begin
  Image.LoadFromFile(Filename);
end;

procedure TTGAImage.LoadFromStream(Stream: TStream);
begin
  Image.LoadFromStream(Stream);
end;

procedure TTGAImage.SaveToClipboardFormat(var AFormat: Word;
  var AData: THandle; var APalette: HPALETTE);
begin
  raise Exception.Create('Cannot save a TTGAImage to the Clipboard');
end;

procedure TTGAImage.SaveToFile(const Filename: string);
begin
  Image.SaveToFile(Filename);
end;

procedure TTGAImage.SaveToStream(Stream: TStream);
begin

  Image.SaveToStream(Stream);
end;

procedure TTGAImage.SetHeight(Value: Integer);
begin
  raise Exception.Create('Cannot set height on a TTGAImage');
end;


procedure TTGAImage.SetWidth(Value: Integer);
begin
  raise Exception.Create('Cannot set width on a TTGAImage');
end;


initialization

 TPicture.RegisterFileFormat('tga', 'TrueVision Targa', TTGAImage);


end.

