{*****************************************************************************}
{  								              }
{	Copyright (c) 2002  Vampyre SoftWorks			              }
{								              }
{								              }
{                                                                             }
{*****************************************************************************}
{								              }
{	DICEMAIN.PAS						              }
{	Multi-sided dice for generating random numbers		              }
{								              }
{	Created by:						              }
{	Marek "Mar" "Galfar"  Mauder  [pentar@seznam.cz]	              }
{								              }
{	Last Modified:						              }
{	18.03.2002						              }
{								              }
{*****************************************************************************}


unit DiceMain;


interface


uses
  Classes;

type

  TDiceKind = (dkD2, dkD3, dkD4, dkD6, dkD8, dkD10, dkD12, dkD20, dkD24,
    dkD100);

  TDices = set of TDiceKind;

  TDice = class(TObject)
  public
    constructor Create; virtual;

    {

     ALower,AUpper - for d6 should be 1 and 6
     ACount - number of throws
     AMod - modifier for each dice

     ADices - dices types to be thrown
     AThrows - number of throws for each dice in ADices
     AMods - modifier for each dice in ADices

    }
    function Roll(ALower, AUpper: LongInt): LongInt; overload;
    function Roll(ACount, ALower, AUpper: LongInt): LongInt; overload;
    function Roll(ACount, AMod, ALower, AUpper: LongInt): LongInt; overload;
    function Roll(ADices: array of TDiceKind): LongInt; overload;
    function Roll(ADices: array of TDiceKind; AThrows: array of LongInt):
      LongInt; overload;
    function Roll(ADices: array of TDiceKind; AThrows: array of LongInt;
      AMods: array of LongInt): LongInt; overload;


    function D2: LongInt;
    function D3: LongInt;
    function D4: LongInt;
    function D5: LongInt;
    function D6: LongInt;
    function D8: LongInt;
    function D10: LongInt;
    function D12: LongInt;
    function D20: LongInt;
    function D24: LongInt;
    function D100: LongInt;

  end;


var Dice: TDice;


implementation


procedure Swap(var a, b: Longint);
var
  c: Longint;
begin
  c := a;
  a := b;
  b := c;
end;

procedure Check(var ALower: LongInt; var AUpper: LongInt);
begin
  if ALower > AUpper then
    Swap(ALower, AUpper);
end;


constructor TDice.Create;
begin
  inherited Create;
  Randomize;
end;


function TDice.D10: LongInt;
begin
  Result := Roll(1, 10);
end;


function TDice.D100: LongInt;
begin
  Result := Roll(1, 100);
end;


function TDice.D12: LongInt;
begin
  Result := Roll(1, 12);
end;


function TDice.D2: LongInt;
begin
  Result := Roll(1, 2);
end;


function TDice.D20: LongInt;
begin
  Result := Roll(1, 20);
end;


function TDice.D24: LongInt;
begin
  Result := Roll(1, 24);
end;


function TDice.D3: LongInt;
begin
  Result := Roll(1, 3);
end;


function TDice.D4: LongInt;
begin
  Result := Roll(1, 4);
end;


function TDice.D5: LongInt;
begin
  Result := Roll(1, 5);
end;


function TDice.D6: LongInt;
begin
  Result := Roll(1, 6);
end;


function TDice.D8: LongInt;
begin
  Result := Roll(1, 8);
end;


function TDice.Roll(ALower, AUpper: LongInt): LongInt;
begin
  Check(ALower, AUpper);
  Result := Random((AUpper + 1) - ALower) + ALower;
end;


function TDice.Roll(ACount, ALower, AUpper: LongInt): LongInt;
var I: LongInt;
begin
  Result := 0;
  for I := 0 to ACount - 1 do
    Result := Result + Roll(ALower, AUpper);

end;


function TDice.Roll(ACount, AMod, ALower, AUpper: LongInt): LongInt;
var I: LongInt;
begin
  Result := 0;
  for I := 0 to ACount - 1 do
    Result := Result + Roll(ALower, AUpper) + AMod;

end;


function TDice.Roll(ADices: array of TDiceKind): LongInt;
var
  I,
    Val: LongInt;
begin
  Result := 0;
  Val := 0;
  for I := Low(ADices) to High(ADices) do
  begin
    case ADices[I] of
      dkD2: Val := D2;
      dkD3: Val := D3;
      dkD4: Val := D4;
      dkD6: Val := D6;
      dkD8: Val := D8;
      dkD10: Val := D10;
      dkD12: Val := D12;
      dkD20: Val := D20;
      dkD24: Val := D24;
      dkD100: Val := D100;
    end;
    Result := Result + Val;
  end;

end;


function TDice.Roll(ADices: array of TDiceKind; AThrows: array of LongInt):
  LongInt;
var
  I, J,
    Val: LongInt;
begin
  Result := 0;
  if High(ADices) <> High(AThrows) then Exit;

  Val := 0;
  for I := Low(ADices) to High(ADices) do
    for J := 0 to AThrows[I] - 1 do
    begin
      case ADices[I] of
        dkD2: Val := D2;
        dkD3: Val := D3;
        dkD4: Val := D4;
        dkD6: Val := D6;
        dkD8: Val := D8;
        dkD10: Val := D10;
        dkD12: Val := D12;
        dkD20: Val := D20;
        dkD24: Val := D24;
        dkD100: Val := D100;
      end;
      Result := Result + Val;
    end;

end;


function TDice.Roll(ADices: array of TDiceKind; AThrows: array of LongInt;
  AMods: array of LongInt): LongInt;
var
  I, J,
    Val: LongInt;
begin
  Result := 0;
  if (High(ADices) <> High(AThrows)) or
    (High(ADices) <> High(AMods)) or
    (High(AMods) <> High(AThrows)) then Exit;

  Val := 0;
  for I := Low(ADices) to High(ADices) do
    for J := 0 to AThrows[I] - 1 do
    begin
      case ADices[I] of
        dkD2: Val := D2;
        dkD3: Val := D3;
        dkD4: Val := D4;
        dkD6: Val := D6;
        dkD8: Val := D8;
        dkD10: Val := D10;
        dkD12: Val := D12;
        dkD20: Val := D20;
        dkD24: Val := D24;
        dkD100: Val := D100;
      end;
      Result := Result + Val + AMods[I];
    end;

end;





initialization
  Dice := TDice.Create;


finalization
  Dice.Free;



end.

